@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2006, 2013, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2017, MariaDB
+   Copyright (c) 2010, 2013, Monty Program Ab.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 #include <welcome_copyright_notice.h> /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-#define VER "1.4"
+#define VER "1.3a"
 
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
@@ -36,8 +36,6 @@
 # endif
 #endif
 
-static int phase = 0;
-static int phases_total = 6;
 static char mysql_path[FN_REFLEN];
 static char mysqlcheck_path[FN_REFLEN];
 
@@ -47,16 +45,12 @@ static my_bool opt_not_used, opt_silent;
 static uint my_end_arg= 0;
 static char *opt_user= (char*)"root";
 
-static my_bool upgrade_from_mysql;
-
 static DYNAMIC_STRING ds_args;
 static DYNAMIC_STRING conn_args;
 
 static char *opt_password= 0;
 static char *opt_plugin_dir= 0, *opt_default_auth= 0;
 
-static char *cnf_file_path= 0, defaults_file[FN_REFLEN + 32];
-
 static my_bool tty_password= 0;
 
 static char opt_tmpdir[FN_REFLEN] = "";
@@ -113,7 +107,6 @@ static struct my_option my_long_options[]=
    &opt_force, &opt_force, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"host", 'h', "Connect to host.", 0,
    0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-#define PASSWORD_OPT 12
   {"password", 'p',
    "Password to use when connecting to server. If password is not given,"
    " it's solicited on the tty.", &opt_password,&opt_password,
@@ -147,22 +140,21 @@ static struct my_option my_long_options[]=
 #include <sslopt-longopts.h>
   {"tmpdir", 't', "Directory for temporary files.",
    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"upgrade-system-tables", 's', "Only upgrade the system tables in the mysql database. Tables in other databases are not checked or touched.",
+  {"upgrade-system-tables", 's', "Only upgrade the system tables "
+   "do not try to upgrade the data.",
    &opt_systables_only, &opt_systables_only, 0,
    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-#define USER_OPT (array_elements(my_long_options) - 6)
   {"user", 'u', "User for login if not current user.", &opt_user,
    &opt_user, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"verbose", 'v', "Display more output about the process; Using it twice will print connection argument; Using it 3 times will print out all CHECK, RENAME and ALTER TABLE during the check phase.",
+  {"verbose", 'v', "Display more output about the process.",
    &opt_not_used, &opt_not_used, 0, GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
   {"version", 'V', "Output version information and exit.", 0, 0, 0,
    GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"version-check", 'k',
-   "Run this program only if its \'server version\' "
-   "matches the version of the server to which it's connecting. "
-   "Note: the \'server version\' of the program is the version of the MariaDB "
-   "server with which it was built/distributed.",
-   &opt_version_check, &opt_version_check, 0,
+  {"version-check", 'k', "Run this program only if its \'server version\' "
+   "matches the version of the server to which it's connecting, (enabled by "
+   "default); use --skip-version-check to avoid this check. Note: the \'server "
+   "version\' of the program is the version of the MySQL server with which it "
+   "was built/distributed.", &opt_version_check, &opt_version_check, 0,
    GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
   {"write-binlog", OPT_WRITE_BINLOG, "All commands including those, "
    "issued by mysqlcheck, are written to the binary log.",
@@ -184,13 +176,10 @@ static const char *load_default_groups[]=
 static void free_used_memory(void)
 {
   /* Free memory allocated by 'load_defaults' */
-  if (defaults_argv)
-    free_defaults(defaults_argv);
+  free_defaults(defaults_argv);
 
   dynstr_free(&ds_args);
   dynstr_free(&conn_args);
-  if (cnf_file_path)
-    my_delete(cnf_file_path, MYF(MY_WME));
 }
 
 
@@ -242,32 +231,31 @@ static void verbose(const char *fmt, ...)
   this way we pass the same arguments on to mysql and mysql_check
 */
 
-static void add_one_option_cmd_line(DYNAMIC_STRING *ds,
-                                    const struct my_option *opt,
-                                    const char* arg)
+static void add_one_option(DYNAMIC_STRING* ds,
+                           const struct my_option *opt,
+                           const char* argument)
+
 {
-  dynstr_append(ds, "--");
-  dynstr_append(ds, opt->name);
-  if (arg)
+  const char* eq= NullS;
+  const char* arg= NullS;
+  if (opt->arg_type != NO_ARG)
   {
-    dynstr_append(ds, "=");
-    dynstr_append_os_quoted(ds, arg, NullS);
+    eq= "=";
+    switch (opt->var_type & GET_TYPE_MASK) {
+    case GET_STR:
+      arg= argument;
+      break;
+    case GET_BOOL:
+      arg= (*(my_bool *)opt->value) ? "1" : "0";
+      break;
+    default:
+      die("internal error at %s: %d",__FILE__, __LINE__);
+    }
   }
+  dynstr_append_os_quoted(ds, "--", opt->name, eq, arg, NullS);
   dynstr_append(ds, " ");
 }
 
-static void add_one_option_cnf_file(DYNAMIC_STRING *ds,
-                                    const struct my_option *opt,
-                                    const char* arg)
-{
-  dynstr_append(ds, opt->name);
-  if (arg)
-  {
-    dynstr_append(ds, "=");
-    dynstr_append_os_quoted(ds, arg, NullS);
-  }
-  dynstr_append(ds, "\n");
-}
 
 static my_bool
 get_one_option(int optid, const struct my_option *opt,
@@ -298,17 +286,16 @@ get_one_option(int optid, const struct my_option *opt,
   case 'p':
     if (argument == disabled_my_option)
       argument= (char*) "";			/* Don't require password */
+    tty_password= 1;
     add_option= FALSE;
     if (argument)
     {
       /* Add password to ds_args before overwriting the arg with x's */
-      add_one_option_cnf_file(&ds_args, opt, argument);
+      add_one_option(&ds_args, opt, argument);
       while (*argument)
         *argument++= 'x';                       /* Destroy argument */
       tty_password= 0;
     }
-    else
-      tty_password= 1;
     break;
 
   case 't':
@@ -355,18 +342,18 @@ get_one_option(int optid, const struct my_option *opt,
   case OPT_SHARED_MEMORY_BASE_NAME: /* --shared-memory-base-name */
   case OPT_PLUGIN_DIR:                          /* --plugin-dir */
   case OPT_DEFAULT_AUTH:                        /* --default-auth */
-    add_one_option_cmd_line(&conn_args, opt, argument);
+    add_one_option(&conn_args, opt, argument);
     break;
   }
 
   if (add_option)
   {
     /*
-      This is an option that is accepted by mysql_upgrade just so
+      This is an option that is accpted by mysql_upgrade just so
       it can be passed on to "mysql" and "mysqlcheck"
       Save it in the ds_args string
     */
-    add_one_option_cnf_file(&ds_args, opt, argument);
+    add_one_option(&ds_args, opt, argument);
   }
   return 0;
 }
@@ -383,9 +370,6 @@ static int run_command(char* cmd,
   FILE *res_file;
   int error;
 
-  if (opt_verbose >= 4)
-    puts(cmd);
-
   if (!(res_file= popen(cmd, "r")))
     die("popen(\"%s\", \"r\") failed", cmd);
 
@@ -429,8 +413,11 @@ static int run_tool(char *tool_path, DYNAMIC_STRING *ds_res, ...)
 
   while ((arg= va_arg(args, char *)))
   {
-    /* Options should already be os quoted */
-    dynstr_append(&ds_cmdline, arg);
+    /* Options should be os quoted */
+    if (strncmp(arg, "--", 2) == 0)
+      dynstr_append_os_quoted(&ds_cmdline, arg, NullS);
+    else
+      dynstr_append(&ds_cmdline, arg);
     dynstr_append(&ds_cmdline, " ");
   }
 
@@ -572,7 +559,8 @@ static int run_query(const char *query, DYNAMIC_STRING *ds_res,
 
   ret= run_tool(mysql_path,
                 ds_res,
-                defaults_file,
+                "--no-defaults",
+                ds_args.str,
                 "--database=mysql",
                 "--batch", /* Turns off pager etc. */
                 force ? "--force": "--skip-force",
@@ -644,7 +632,7 @@ static int get_upgrade_info_file_name(char* name)
 /*
   Read the content of mysql_upgrade_info file and
   compare the version number form file against
-  version number which mysql_upgrade was compiled for
+  version number wich mysql_upgrade was compiled for
 
   NOTE
   This is an optimization to avoid running mysql_upgrade
@@ -748,113 +736,37 @@ static void print_conn_args(const char *tool_name)
   in the server using "mysqlcheck --check-upgrade .."
 */
 
-static int run_mysqlcheck_upgrade(my_bool mysql_db_only)
+static int run_mysqlcheck_upgrade(void)
 {
-  const char *what= mysql_db_only ? "mysql database" : "tables";
-  const char *arg1= mysql_db_only ? "--databases" : "--all-databases";
-  const char *arg2= mysql_db_only ? "mysql" : "--skip-database=mysql";
-  int retch;
-  if (opt_systables_only && !mysql_db_only)
-  {
-    verbose("Phase %d/%d: Checking and upgrading %s... Skipped",
-            ++phase, phases_total, what);
-    return 0;
-  }
-  verbose("Phase %d/%d: Checking and upgrading %s", ++phase, phases_total, what);
-  print_conn_args("mysqlcheck");
-  retch= run_tool(mysqlcheck_path,
-                  NULL, /* Send output from mysqlcheck directly to screen */
-                  defaults_file,
-                  "--check-upgrade",
-                  "--auto-repair",
-                  !opt_silent || opt_verbose >= 1 ? "--verbose" : "",
-                  opt_verbose >= 2 ? "--verbose" : "",
-                  opt_verbose >= 3 ? "--verbose" : "",
-                  opt_silent ? "--silent": "",
-                  opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
-                  arg1, arg2,
-                  "2>&1",
-                  NULL);
-  return retch;
-}
-
-#define EVENTS_STRUCT_LEN 7000
-
-static my_bool is_mysql()
-{
-  my_bool ret= TRUE;
-  DYNAMIC_STRING ds_events_struct;
-
-  if (init_dynamic_string(&ds_events_struct, NULL,
-                          EVENTS_STRUCT_LEN, EVENTS_STRUCT_LEN))
-    die("Out of memory");
-
-  if (run_query("show create table mysql.event",
-                &ds_events_struct, FALSE) ||
-      strstr(ds_events_struct.str, "IGNORE_BAD_TABLE_OPTIONS") != NULL)
-    ret= FALSE;
-  else
-    verbose("MySQL upgrade detected");
-
-  dynstr_free(&ds_events_struct);
-  return(ret);
-}
-
-static int run_mysqlcheck_views(void)
-{
-  const char *upgrade_views="--process-views=YES";
-  if (upgrade_from_mysql)
-  {
-    /*
-      this has to ignore opt_systables_only, because upgrade_from_mysql
-      is determined by analyzing systables. if we honor opt_systables_only
-      here, views won't be fixed by subsequent mysql_upgrade runs
-    */
-    upgrade_views="--process-views=UPGRADE_FROM_MYSQL";
-    verbose("Phase %d/%d: Fixing views from mysql", ++phase, phases_total);
-  }
-  else if (opt_systables_only)
-  {
-    verbose("Phase %d/%d: Fixing views... Skipped", ++phase, phases_total);
-    return 0;
-  }
-  else
-    verbose("Phase %d/%d: Fixing views", ++phase, phases_total);
-
+  verbose("Phase 2/3: Checking and upgrading tables");
   print_conn_args("mysqlcheck");
   return run_tool(mysqlcheck_path,
                   NULL, /* Send output from mysqlcheck directly to screen */
-                  defaults_file,
-                  "--all-databases", "--repair",
-                  upgrade_views,
-                  "--skip-process-tables",
-                  opt_verbose ? "--verbose": "",
+                  "--no-defaults",
+                  ds_args.str,
+                  "--check-upgrade",
+                  "--all-databases",
+                  "--auto-repair",
+                  !opt_silent || opt_verbose ? "--verbose": "",
                   opt_silent ? "--silent": "",
                   opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
                   "2>&1",
                   NULL);
 }
 
+
 static int run_mysqlcheck_fixnames(void)
 {
-  if (opt_systables_only)
-  {
-    verbose("Phase %d/%d: Fixing table and database names ... Skipped",
-            ++phase, phases_total);
-    return 0;
-  }
-  verbose("Phase %d/%d: Fixing table and database names",
-          ++phase, phases_total);
+  verbose("Phase 1/3: Fixing table and database names");
   print_conn_args("mysqlcheck");
   return run_tool(mysqlcheck_path,
                   NULL, /* Send output from mysqlcheck directly to screen */
-                  defaults_file,
+                  "--no-defaults",
+                  ds_args.str,
                   "--all-databases",
                   "--fix-db-names",
                   "--fix-table-names",
-                  opt_verbose >= 1 ? "--verbose" : "",
-                  opt_verbose >= 2 ? "--verbose" : "",
-                  opt_verbose >= 3 ? "--verbose" : "",
+                  opt_verbose ? "--verbose": "",
                   opt_silent ? "--silent": "",
                   opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
                   "2>&1",
@@ -867,7 +779,6 @@ static const char *expected_errors[]=
   "ERROR 1060", /* Duplicate column name */
   "ERROR 1061", /* Duplicate key name */
   "ERROR 1054", /* Unknown column */
-  "ERROR 1290", /* RR_OPTION_PREVENTS_STATEMENT */
   0
 };
 
@@ -932,9 +843,7 @@ static int run_sql_fix_privilege_tables(void)
   if (init_dynamic_string(&ds_result, "", 512, 512))
     die("Out of memory");
 
-  verbose("Phase %d/%d: Running 'mysql_fix_privilege_tables'",
-          ++phase, phases_total);
-
+  verbose("Phase 3/3: Running 'mysql_fix_privilege_tables'...");
   /*
     Individual queries can not be executed independently by invoking
     a forked mysql client, because the script uses session variables
@@ -945,8 +854,7 @@ static int run_sql_fix_privilege_tables(void)
         query_ptr++
       )
   {
-    if (strcasecmp(*query_ptr, "flush privileges;\n"))
-      dynstr_append(&ds_script, *query_ptr);
+    dynstr_append(&ds_script, *query_ptr);
   }
 
   run_query(ds_script.str,
@@ -1042,7 +950,7 @@ static int check_version_match(void)
 
 int main(int argc, char **argv)
 {
-  char self_name[FN_REFLEN + 1];
+  char self_name[FN_REFLEN];
 
   MY_INIT(argv[0]);
 
@@ -1050,7 +958,7 @@ int main(int argc, char **argv)
   if (GetModuleFileName(NULL, self_name, FN_REFLEN) == 0)
 #endif
   {
-    strmake_buf(self_name, argv[0]);
+    strncpy(self_name, argv[0], FN_REFLEN);
   }
 
   if (init_dynamic_string(&ds_args, "", 512, 256) ||
@@ -1072,32 +980,26 @@ int main(int argc, char **argv)
   {
     opt_password= get_tty_password(NullS);
     /* add password to defaults file */
-    add_one_option_cnf_file(&ds_args, &my_long_options[PASSWORD_OPT], opt_password);
-    DBUG_ASSERT(strcmp(my_long_options[PASSWORD_OPT].name, "password") == 0);
+    dynstr_append_os_quoted(&ds_args, "--password=", opt_password, NullS);
+    dynstr_append(&ds_args, " ");
   }
   /* add user to defaults file */
-  add_one_option_cnf_file(&ds_args, &my_long_options[USER_OPT], opt_user);
-  DBUG_ASSERT(strcmp(my_long_options[USER_OPT].name, "user") == 0);
-
-  cnf_file_path= strmov(defaults_file, "--defaults-file=");
-  {
-    int fd= create_temp_file(cnf_file_path, opt_tmpdir[0] ? opt_tmpdir : NULL,
-                             "mysql_upgrade-", O_CREAT | O_WRONLY, MYF(MY_FAE));
-    if (fd < 0)
-      die(NULL);
-    my_write(fd, USTRING_WITH_LEN( "[client]\n"), MYF(MY_FAE));
-    my_write(fd, (uchar*)ds_args.str, ds_args.length, MYF(MY_FAE));
-    my_close(fd, MYF(0));
-  }
+  dynstr_append_os_quoted(&ds_args, "--user=", opt_user, NullS);
+  dynstr_append(&ds_args, " ");
 
   /* Find mysql */
   find_tool(mysql_path, IF_WIN("mysql.exe", "mysql"), self_name);
 
-  /* Find mysqlcheck */
-  find_tool(mysqlcheck_path, IF_WIN("mysqlcheck.exe", "mysqlcheck"), self_name);
-
-  if (opt_systables_only && !opt_silent)
-    printf("The --upgrade-system-tables option was used, user tables won't be touched.\n");
+  if (!opt_systables_only)
+  {
+    /* Find mysqlcheck */
+    find_tool(mysqlcheck_path, IF_WIN("mysqlcheck.exe", "mysqlcheck"), self_name);
+  }
+  else
+  {
+    if (!opt_silent)
+      printf("The --upgrade-system-tables option was used, databases won't be touched.\n");
+  }
 
   /*
     Read the mysql_upgrade_info file to check if mysql_upgrade
@@ -1108,37 +1010,32 @@ int main(int argc, char **argv)
     printf("This installation of MySQL is already upgraded to %s, "
            "use --force if you still need to run mysql_upgrade\n",
            MYSQL_SERVER_VERSION);
-    goto end;
+    die(NULL);
   }
 
   if (opt_version_check && check_version_match())
     die("Upgrade failed");
 
-  upgrade_from_mysql= is_mysql();
-
   /*
     Run "mysqlcheck" and "mysql_fix_privilege_tables.sql"
   */
-  if (run_mysqlcheck_upgrade(TRUE) ||
-      run_mysqlcheck_views() ||
-      run_sql_fix_privilege_tables() ||
-      run_mysqlcheck_fixnames() ||
-      run_mysqlcheck_upgrade(FALSE))
+  if ((!opt_systables_only &&
+       (run_mysqlcheck_fixnames() || run_mysqlcheck_upgrade())) ||
+      run_sql_fix_privilege_tables())
+  {
+    /*
+      The upgrade failed to complete in some way or another,
+      significant error message should have been printed to the screen
+    */
     die("Upgrade failed" );
-
-  verbose("Phase %d/%d: Running 'FLUSH PRIVILEGES'", ++phase, phases_total);
-  if (run_query("FLUSH PRIVILEGES", NULL, TRUE))
-    die("Upgrade failed" );
-
+  }
   verbose("OK");
 
   /* Create a file indicating upgrade has been performed */
   create_mysql_upgrade_info_file();
 
-  DBUG_ASSERT(phase == phases_total);
-
-end:
   free_used_memory();
   my_end(my_end_arg);
   exit(0);
 }
+
