@@ -1,6 +1,6 @@
 /*
-   Copyright (c) 2000, 2014, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2016, MariaDB
+   Copyright (c) 2000, 2012, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2012, Monty Program Ab.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,8 +38,8 @@ char ex_var_names[MAX_MYSQL_VAR][FN_REFLEN];
 ulonglong last_values[MAX_MYSQL_VAR];
 static int interval=0;
 static my_bool option_force=0,interrupted=0,new_line=0,
-               opt_compress= 0, opt_local= 0, opt_relative= 0, opt_verbose= 0,
-               opt_vertical= 0, tty_password= 0, opt_nobeep;
+               opt_compress=0, opt_relative=0, opt_verbose=0, opt_vertical=0,
+               tty_password= 0, opt_nobeep;
 static my_bool debug_info_flag= 0, debug_check_flag= 0;
 static uint tcp_port = 0, option_wait = 0, option_silent=0, nr_iterations;
 static uint opt_count_iterations= 0, my_end_arg;
@@ -71,7 +71,6 @@ extern "C" my_bool get_one_option(int optid, const struct my_option *opt,
                                   char *argument);
 static my_bool sql_connect(MYSQL *mysql, uint wait);
 static int execute_commands(MYSQL *mysql,int argc, char **argv);
-static char **mask_password(int argc, char ***argv);
 static int drop_db(MYSQL *mysql,const char *db);
 extern "C" sig_handler endprog(int signal_number);
 static void nice_time(ulong sec,char *buff);
@@ -102,12 +101,9 @@ enum commands {
   ADMIN_PING,             ADMIN_EXTENDED_STATUS, ADMIN_FLUSH_STATUS,
   ADMIN_FLUSH_PRIVILEGES, ADMIN_START_SLAVE,     ADMIN_STOP_SLAVE,
   ADMIN_START_ALL_SLAVES, ADMIN_STOP_ALL_SLAVES,
-  ADMIN_FLUSH_THREADS,    ADMIN_OLD_PASSWORD,    ADMIN_FLUSH_BINARY_LOG,
-  ADMIN_FLUSH_ENGINE_LOG, ADMIN_FLUSH_ERROR_LOG, ADMIN_FLUSH_GENERAL_LOG,
-  ADMIN_FLUSH_RELAY_LOG,  ADMIN_FLUSH_SLOW_LOG,
+  ADMIN_FLUSH_THREADS,    ADMIN_OLD_PASSWORD,    ADMIN_FLUSH_SLOW_LOG,
   ADMIN_FLUSH_TABLE_STATISTICS, ADMIN_FLUSH_INDEX_STATISTICS,
   ADMIN_FLUSH_USER_STATISTICS, ADMIN_FLUSH_CLIENT_STATISTICS,
-  ADMIN_FLUSH_USER_RESOURCES,
   ADMIN_FLUSH_ALL_STATUS, ADMIN_FLUSH_ALL_STATISTICS
 };
 static const char *command_names[]= {
@@ -119,10 +115,9 @@ static const char *command_names[]= {
   "ping",                 "extended-status",     "flush-status",
   "flush-privileges",     "start-slave",         "stop-slave",
   "start-all-slaves", "stop-all-slaves",
-  "flush-threads", "old-password", "flush-binary-log", "flush-engine-log",
-  "flush-error-log", "flush-general-log", "flush-relay-log", "flush-slow-log",
+  "flush-threads", "old-password", "flush-slow-log",
   "flush-table-statistics", "flush-index-statistics",
-  "flush-user-statistics", "flush-client-statistics", "flush-user-resources",
+  "flush-user-statistics", "flush-client-statistics",
   "flush-all-status", "flush-all-statistics",
   NullS
 };
@@ -164,9 +159,6 @@ static struct my_option my_long_options[] =
    NO_ARG, 0, 0, 0, 0, 0, 0},
   {"host", 'h', "Connect to host.", &host, &host, 0, GET_STR,
    REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"local", 'l', "Local command, don't write to binlog.",
-   &opt_local, &opt_local, 0, GET_BOOL, NO_ARG, 0, 0, 0,
-   0, 0, 0},
   {"no-beep", 'b', "Turn off beep on error.", &opt_nobeep,
    &opt_nobeep, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0}, 
   {"password", 'p',
@@ -242,6 +234,8 @@ my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 	       char *argument)
 {
+  int error = 0;
+
   switch(optid) {
   case 'c':
     opt_count_iterations= 1;
@@ -289,8 +283,8 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
     break;
   case '?':
   case 'I':					/* Info */
-    usage();
-    exit(0);
+    error++;
+    break;
   case OPT_CHARSETS_DIR:
 #if MYSQL_VERSION_ID > 32300
     charsets_dir = argument;
@@ -301,15 +295,20 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
                                     opt->name);
     break;
   }
+  if (error)
+  {
+    usage();
+    exit(1);
+  }
   return 0;
 }
 
 
 int main(int argc,char *argv[])
 {
-  int error= 0, temp_argc;
+  int error= 0;
   MYSQL mysql;
-  char **commands, **save_argv, **temp_argv;
+  char **commands, **save_argv;
 
   MY_INIT(argv[0]);
   mysql_init(&mysql);
@@ -317,12 +316,8 @@ int main(int argc,char *argv[])
   if ((error= load_defaults("my",load_default_groups,&argc,&argv)))
     goto err1;
   save_argv = argv;				/* Save for free_defaults */
-
   if ((error=handle_options(&argc, &argv, my_long_options, get_one_option)))
     goto err2;
-  temp_argv= mask_password(argc, &argv);
-  temp_argc= argc;
-
   if (debug_info_flag)
     my_end_arg= MY_CHECK_ERROR | MY_GIVE_INFO;
   if (debug_check_flag)
@@ -333,7 +328,7 @@ int main(int argc,char *argv[])
     usage();
     exit(1);
   }
-  commands = temp_argv;
+  commands = argv;
   if (tty_password)
     opt_password = get_tty_password(NullS);
 
@@ -480,13 +475,6 @@ int main(int argc,char *argv[])
   }                                             /* got connection */
 
   mysql_close(&mysql);
-  temp_argc--;
-  while(temp_argc >= 0)
-  {
-    my_free(temp_argv[temp_argc]);
-    temp_argc--;
-  }
-  my_free(temp_argv);
 err2:
   mysql_library_end();
   my_free(opt_password);
@@ -624,18 +612,6 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
   */
 
   struct my_rnd_struct rand_st;
-  char buff[FN_REFLEN + 20];
-
-  if (opt_local)
-  {
-    sprintf(buff, "set local sql_log_bin=0");
-    if (mysql_query(mysql, buff))
-    {
-      my_printf_error(0, "SET LOCAL SQL_LOG_BIN=0 failed; error: '%-.200s'",
-                      error_flags, mysql_error(mysql));
-      return -1;
-    }
-  }
 
   for (; argc > 0 ; argv++,argc--)
   {
@@ -643,6 +619,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     switch ((command= find_type(argv[0],&command_typelib,FIND_TYPE_BASIC))) {
     case ADMIN_CREATE:
     {
+      char buff[FN_REFLEN+20];
       if (argc < 2)
       {
 	my_printf_error(0, "Too few arguments to create", error_flags);
@@ -920,56 +897,6 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       }
       break;
     }
-    case ADMIN_FLUSH_BINARY_LOG:
-    {
-      if (mysql_query(mysql, "flush binary logs"))
-      {
-        my_printf_error(0, "flush failed; error: '%s'", error_flags,
-                        mysql_error(mysql));
-        return -1;
-      }
-      break;
-    }
-    case ADMIN_FLUSH_ENGINE_LOG:
-    {
-      if (mysql_query(mysql,"flush engine logs"))
-      {
-        my_printf_error(0, "flush failed; error: '%s'", error_flags,
-                        mysql_error(mysql));
-        return -1;
-      }
-      break;
-    }
-    case ADMIN_FLUSH_ERROR_LOG:
-    {
-      if (mysql_query(mysql, "flush error logs"))
-      {
-        my_printf_error(0, "flush failed; error: '%s'", error_flags,
-                        mysql_error(mysql));
-        return -1;
-      }
-      break;
-    }
-    case ADMIN_FLUSH_GENERAL_LOG:
-    {
-      if (mysql_query(mysql, "flush general logs"))
-      {
-        my_printf_error(0, "flush failed; error: '%s'", error_flags,
-                        mysql_error(mysql));
-        return -1;
-      }
-      break;
-    }
-    case ADMIN_FLUSH_RELAY_LOG:
-    {
-      if (mysql_query(mysql, "flush relay logs"))
-      {
-        my_printf_error(0, "flush failed; error: '%s'", error_flags,
-                        mysql_error(mysql));
-        return -1;
-      }
-      break;
-    }
     case ADMIN_FLUSH_SLOW_LOG:
     {
       if (mysql_query(mysql,"flush slow logs"))
@@ -1040,16 +967,6 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       }
       break;
     }
-    case ADMIN_FLUSH_USER_RESOURCES:
-    {
-      if (mysql_query(mysql, "flush user_resources"))
-      {
-        my_printf_error(0, "flush failed; error: '%s'", error_flags,
-                        mysql_error(mysql));
-        return -1;
-      }
-      break;
-    }
     case ADMIN_FLUSH_CLIENT_STATISTICS:
     {
       if (mysql_query(mysql,"flush client_statistics"))
@@ -1299,50 +1216,6 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
   return 0;
 }
 
-/**
-   @brief Masking the password if it is passed as command line argument.
-
-   @details It works in Linux and changes cmdline in ps and /proc/pid/cmdline,
-            but it won't work for history file of shell.
-            The command line arguments are copied to another array and the
-            password in the argv is masked. This function is called just after
-            "handle_options" because in "handle_options", the agrv pointers
-            are altered which makes freeing of dynamically allocated memory
-            difficult. The password masking is done before all other operations
-            in order to minimise the time frame of password visibility via cmdline.
-
-   @param argc            command line options (count)
-   @param argv            command line options (values)
-
-   @return temp_argv      copy of argv
-*/
-
-static char **mask_password(int argc, char ***argv)
-{
-  char **temp_argv;
-  if (!argc)
-    return NULL;
-
-  temp_argv= (char **)(my_malloc(sizeof(char *) * argc, MYF(MY_WME)));
-  argc--;
-  while (argc > 0)
-  {
-    temp_argv[argc]= my_strdup((*argv)[argc], MYF(MY_FAE));
-    if (find_type((*argv)[argc - 1],&command_typelib, FIND_TYPE_BASIC) == ADMIN_PASSWORD ||
-        find_type((*argv)[argc - 1],&command_typelib, FIND_TYPE_BASIC) == ADMIN_OLD_PASSWORD)
-    {
-      char *start= (*argv)[argc];
-      while (*start)
-        *start++= 'x';
-      start= (*argv)[argc];
-      if (*start)
-        start[1]= 0;                         /* Cut length of argument */
-     }
-    argc--;
-  }
-  temp_argv[argc]= my_strdup((*argv)[argc], MYF(MY_FAE));
-  return(temp_argv);
-}
 
 static void print_version(void)
 {
@@ -1373,18 +1246,12 @@ static void usage(void)
   flush-index-statistics  Flush index statistics\n\
   flush-logs              Flush all logs\n\
   flush-privileges        Reload grant tables (same as reload)\n\
-  flush-binary-log        Flush binary log\n\
-  flush-engine-log        Flush engine log(s)\n\
-  flush-error-log         Flush error log\n\
-  flush-general-log       Flush general log\n\
-  flush-relay-log         Flush relay log\n\
   flush-slow-log          Flush slow query log\n\
-  flush-status            Clear status variables\n\
+  flush-status		  Clear status variables\n\
   flush-table-statistics  Clear table statistics\n\
   flush-tables            Flush all tables\n\
   flush-threads           Flush the thread cache\n\
   flush-user-statistics   Flush user statistics\n\
-  flush-user-resources    Flush user resources\n\
   kill id,id,...	Kill mysql threads");
 #if MYSQL_VERSION_ID >= 32200
   puts("\
@@ -1651,10 +1518,8 @@ static my_bool get_pidfile(MYSQL *mysql, char *pidfile)
 
   if (mysql_query(mysql, "SHOW VARIABLES LIKE 'pid_file'"))
   {
-    my_printf_error(mysql_errno(mysql),
-                    "The query to get the server's pid file failed,"
-                    " error: '%s'. Continuing.", error_flags,
-                    mysql_error(mysql));
+    my_printf_error(0, "query failed; error: '%s'", error_flags,
+		    mysql_error(mysql));
   }
   result = mysql_store_result(mysql);
   if (result)
