@@ -75,12 +75,11 @@ typedef int my_socket;
 
 #include "my_list.h" /* for LISTs used in 'MYSQL' and 'MYSQL_STMT' */
 
-extern unsigned int mariadb_deinitialize_ssl;
 extern unsigned int mysql_port;
 extern char *mysql_unix_port;
 
-#define CLIENT_NET_READ_TIMEOUT		(365*24*3600)	/* Timeout on read */
-#define CLIENT_NET_WRITE_TIMEOUT	(365*24*3600)	/* Timeout on write */
+#define CLIENT_NET_READ_TIMEOUT		365*24*3600	/* Timeout on read */
+#define CLIENT_NET_WRITE_TIMEOUT	365*24*3600	/* Timeout on write */
 
 #define IS_PRI_KEY(n)	((n) & PRI_KEY_FLAG)
 #define IS_NOT_NULL(n)	((n) & NOT_NULL_FLAG)
@@ -283,7 +282,7 @@ typedef struct st_mysql
 
   /* session-wide random string */
   char	        scramble[SCRAMBLE_LENGTH+1];
-  my_bool       auto_local_infile;
+  my_bool unused1;
   void *unused2, *unused3, *unused4, *unused5;
 
   LIST  *stmts;                     /* list of all statements */
@@ -865,12 +864,6 @@ my_socket STDCALL mysql_get_socket(const MYSQL *mysql);
 unsigned int STDCALL mysql_get_timeout_value(const MYSQL *mysql);
 unsigned int STDCALL mysql_get_timeout_value_ms(const MYSQL *mysql);
 
-/********************************************************************
-  mysql_net_ functions - low-level API to MySQL protocol
-*********************************************************************/
-unsigned long STDCALL mysql_net_read_packet(MYSQL *mysql);
-unsigned long STDCALL mysql_net_field_length(unsigned char **packet);
-
 /* status return codes */
 #define MYSQL_NO_DATA        100
 #define MYSQL_DATA_TRUNCATED 101
