@@ -329,8 +329,6 @@ enum enum_server_command
 */
 #define SERVER_STATUS_IN_TRANS_READONLY 8192
 
-#define SERVER_STATUS_ANSI_QUOTES       32768
-
 
 /**
   Server status flags that must be cleared when starting
@@ -388,7 +386,7 @@ typedef struct st_net {
   char save_char;
   char net_skip_rest_factor;
   my_bool thread_specific_malloc;
-  unsigned char compress;
+  my_bool compress;
   my_bool unused3; /* Please remove with the next incompatible ABI change. */
   /*
     Pointer to query object in query cache, do not equal NULL (0) for
@@ -543,8 +541,7 @@ my_bool	net_write_command(NET *net,unsigned char command,
 			  const unsigned char *header, size_t head_len,
 			  const unsigned char *packet, size_t len);
 int	net_real_write(NET *net,const unsigned char *packet, size_t len);
-unsigned long my_net_read_packet(NET *net, my_bool read_from_server);
-#define my_net_read(A) my_net_read_packet((A), 0)
+unsigned long my_net_read(NET *net);
 
 #ifdef MY_GLOBAL_INCLUDED
 void my_net_set_write_timeout(NET *net, uint timeout);
@@ -620,17 +617,14 @@ void scramble_323(char *to, const char *message, const char *password);
 my_bool check_scramble_323(const unsigned char *reply, const char *message,
                            unsigned long *salt);
 void get_salt_from_password_323(unsigned long *res, const char *password);
-#if MYSQL_VERSION_ID < 100100
 void make_password_from_salt_323(char *to, const unsigned long *salt);
-#endif
+
 void make_scrambled_password(char *to, const char *password);
 void scramble(char *to, const char *message, const char *password);
 my_bool check_scramble(const unsigned char *reply, const char *message,
                        const unsigned char *hash_stage2);
 void get_salt_from_password(unsigned char *res, const char *password);
-#if MYSQL_VERSION_ID < 100100
 void make_password_from_salt(char *to, const unsigned char *hash_stage2);
-#endif
 char *octet2hex(char *to, const char *str, unsigned int len);
 
 /* end of password.c */
