@@ -38,14 +38,28 @@ mysql_mutex_t LOCK_localtime_r;
 #ifdef _MSC_VER
 static void install_sigabrt_handler();
 #endif
+#ifdef TARGET_OS_LINUX
+
+/*
+  Dummy thread spawned in my_thread_global_init() below to avoid
+  race conditions in NPTL pthread_exit code.
+*/
+
+static pthread_handler_t
+nptl_pthread_exit_hack_handler(void *arg __attribute((unused)))
+{
+  /* Do nothing! */
+  pthread_exit(0);
+  return 0;
+}
+
+#endif /* TARGET_OS_LINUX */
 
 
 static uint get_thread_lib(void);
 
 /** True if @c my_thread_global_init() has been called. */
 static my_bool my_thread_global_init_done= 0;
-/* True if THR_KEY_mysys is created */
-my_bool my_thr_key_mysys_exists= 0;
 
 
 /*
@@ -169,20 +183,11 @@ my_bool my_thread_global_init(void)
     return 0;
   my_thread_global_init_done= 1;
 
-  /*
-    THR_KEY_mysys is deleted in my_end() as DBUG libraries are using it even
-    after my_thread_global_end() is called.
-    my_thr_key_mysys_exist is used to protect against application like QT
-    that calls my_thread_global_init() + my_thread_global_end() multiple times
-    without calling my_init() + my_end().
-  */
-  if (!my_thr_key_mysys_exists &&
-      (pth_ret= pthread_key_create(&THR_KEY_mysys, NULL)) != 0)
+  if ((pth_ret= pthread_key_create(&THR_KEY_mysys, NULL)) != 0)
   {
     fprintf(stderr, "Can't initialize threads: error %d\n", pth_ret);
     return 1;
   }
-  my_thr_key_mysys_exists= 1;
 
   /* Mutex used by my_thread_init() and after my_thread_destroy_mutex() */
   my_thread_init_internal_mutex();
@@ -192,6 +197,33 @@ my_bool my_thread_global_init(void)
 
   thd_lib_detected= get_thread_lib();
 
+#ifdef TARGET_OS_LINUX
+  /*
+    BUG#24507: Race conditions inside current NPTL pthread_exit()
+    implementation.
+
+    To avoid a possible segmentation fault during concurrent
+    executions of pthread_exit(), a dummy thread is spawned which
+    initializes internal variables of pthread lib. See bug description
+    for a full explanation.
+
+    TODO: Remove this code when fixed versions of glibc6 are in common
+    use.
+  */
+  if (thd_lib_detected == THD_LIB_NPTL)
+  {
+    pthread_t       dummy_thread;
+    pthread_attr_t  dummy_thread_attr;
+
+    pthread_attr_init(&dummy_thread_attr);
+    pthread_attr_setdetachstate(&dummy_thread_attr, PTHREAD_CREATE_JOINABLE);
+
+    if (pthread_create(&dummy_thread,&dummy_thread_attr,
+                       nptl_pthread_exit_hack_handler, NULL) == 0)
+      (void)pthread_join(dummy_thread, NULL);
+  }
+#endif /* TARGET_OS_LINUX */
+
   my_thread_init_common_mutex();
 
   return 0;
@@ -383,8 +415,6 @@ void my_thread_end(void)
       mysql_cond_signal(&THR_COND_threads);
     mysql_mutex_unlock(&THR_LOCK_threads);
 
-    /* Trash variable so that we can detect false accesses to my_thread_var */
-    tmp->init= 2;
     TRASH(tmp, sizeof(*tmp));
     free(tmp);
   }
