@@ -20,7 +20,6 @@ mysqld_ld_preload=
 mysqld_ld_library_path=
 flush_caches=0
 numa_interleave=0
-unsafe_my_cnf=0
 
 # Initial logging status: error log is not open, and not using syslog
 logging=init
@@ -29,8 +28,6 @@ syslog_tag=
 user='@MYSQLD_USER@'
 pid_file=
 err_log=
-err_log_base=
-skip_err_log=0
 
 syslog_tag_mysqld=mysqld
 syslog_tag_mysqld_safe=mysqld_safe
@@ -130,18 +127,6 @@ my_which ()
   return $ret  # Success
 }
 
-find_in_bin() {
-  if test -x "$MY_BASEDIR_VERSION/bin/$1"
-  then
-    echo "$MY_BASEDIR_VERSION/bin/$1"
-  elif test -x "@bindir@/$1"
-  then
-    echo "@bindir@/$1"
-  else
-    echo "$1"
-  fi
-}
-
 log_generic () {
   priority="$1"
   shift
@@ -150,7 +135,7 @@ log_generic () {
   echo "$msg"
   case $logging in
     init) ;;  # Just echo the message, don't save it anywhere
-    file) echo "$msg" | "$helper" "$user" log "$err_log" ;;
+    file) echo "$msg" >> "$err_log" ;;
     syslog) logger -t "$syslog_tag_mysqld_safe" -p "$priority" "$*" ;;
     *)
       echo "Internal program error (non-fatal):" \
@@ -170,7 +155,7 @@ log_notice () {
 eval_log_error () {
   cmd="$1"
   case $logging in
-    file) cmd="$cmd 2>&1 | "`shell_quote_string "$helper"`" $user log "`shell_quote_string "$err_log"` ;;
+    file) cmd="$cmd >> "`shell_quote_string "$err_log"`" 2>&1" ;;
     syslog)
       # mysqld often prefixes its messages with a timestamp, which is
       # redundant when logging to syslog (which adds its own timestamp)
@@ -204,17 +189,21 @@ shell_quote_string() {
   echo "$1" | sed -e 's,\([^a-zA-Z0-9/_.=-]\),\\\1,g'
 }
 
-check_executable_location() {
-  if test "$unsafe_my_cnf" = 1 -a "$unrecognized_handling" != collect; then
-    log_error "Cannot accept $1 from a config file, when my.cnf is in the datadir"
-    exit 1
-  fi
-}
-
 parse_arguments() {
+  # We only need to pass arguments through to the server if we don't
+  # handle them here.  So, we collect unrecognized options (passed on
+  # the command line) into the args variable.
+  pick_args=
+  if test "$1" = PICK-ARGS-FROM-ARGV
+  then
+    pick_args=1
+    shift
+  fi
+
   for arg do
     val=`echo "$arg" | sed -e "s;--[^=]*=;;"`
     case "$arg" in
+      --crash[-_]script=*) CRASH_SCRIPT="$val" ;;
       # these get passed explicitly to mysqld
       --basedir=*) MY_BASEDIR_VERSION="$val" ;;
       --datadir=*|--data=*) DATADIR="$val" ;;
@@ -223,32 +212,23 @@ parse_arguments() {
       --user=*) user="$val"; SET_USER=1 ;;
       --log[-_]basename=*|--hostname=*|--loose[-_]log[-_]basename=*)
         pid_file="$val.pid";
-	err_log_base="$val";
+	err_log="$val.err";
 	;;
 
       # these might have been set in a [mysqld_safe] section of my.cnf
       # they are added to mysqld command line to override settings from my.cnf
-      --skip[-_]log[-_]error)
-        err_log=;
-        skip_err_log=1;
-        ;;
-      --log[-_]error=*)
-        err_log="$val";
-        skip_err_log=0;
-        ;;
+      --log[-_]error=*) err_log="$val" ;;
       --port=*) mysql_tcp_port="$val" ;;
       --socket=*) mysql_unix_port="$val" ;;
 
       # mysqld_safe-specific options - must be set in my.cnf ([mysqld_safe])!
       --core[-_]file[-_]size=*) core_file_size="$val" ;;
-      --ledir=*) check_executable_location "$arg" ; ledir="$val" ;;
-      --malloc[-_]lib=*) check_executable_location "$arg"; set_malloc_lib "$val" ;;
-      --crash[-_]script=*) check_executable_location "$arg"; crash_script="$val" ;;
-      --mysqld=*) check_executable_location "$arg"; MYSQLD="$val" ;;
+      --ledir=*) ledir="$val" ;;
+      --malloc[-_]lib=*) set_malloc_lib "$val" ;;
+      --mysqld=*) MYSQLD="$val" ;;
       --mysqld[-_]version=*)
         if test -n "$val"
         then
-          check_executable_location "$arg"
           MYSQLD="mysqld-$val"
           PLUGIN_VARIANT="/$val"
         else
@@ -269,10 +249,11 @@ parse_arguments() {
       --help) usage ;;
 
       *)
-        case "$unrecognized_handling" in
-          collect) append_arg_to_args "$arg" ;;
-          complain) log_error "unknown option '$arg'" ;;
-        esac
+        if test -n "$pick_args"
+        then
+          append_arg_to_args "$arg"
+        fi
+        ;;
     esac
   done
 }
@@ -407,8 +388,15 @@ set_malloc_lib() {
 # First, try to find BASEDIR and ledir (where mysqld is)
 #
 
-MY_PWD=`dirname $0`
-MY_PWD=`cd "$MY_PWD"/.. && pwd`
+if echo '@pkgdatadir@' | grep '^@prefix@' > /dev/null
+then
+  relpkgdata=`echo '@pkgdatadir@' | sed -e 's,^@prefix@,,' -e 's,^/,,' -e 's,^,./,'`
+else
+  # pkgdatadir is not relative to prefix
+  relpkgdata='@pkgdatadir@'
+fi
+
+MY_PWD=`pwd`
 # Check for the directories we would expect from a binary release install
 if test -n "$MY_BASEDIR_VERSION" -a -d "$MY_BASEDIR_VERSION"
 then
@@ -424,16 +412,16 @@ then
   else
     ledir="$MY_BASEDIR_VERSION/bin"
   fi
-elif test -x "$MY_PWD/bin/mysqld"
+elif test -f "$relpkgdata"/english/errmsg.sys -a -x "$MY_PWD/bin/mysqld"
 then
   MY_BASEDIR_VERSION="$MY_PWD"		# Where bin, share and data are
   ledir="$MY_PWD/bin"			# Where mysqld is
 # Check for the directories we would expect from a source install
-elif test -x "$MY_PWD/libexec/mysqld"
+elif test -f "$relpkgdata"/english/errmsg.sys -a -x "$MY_PWD/libexec/mysqld"
 then
   MY_BASEDIR_VERSION="$MY_PWD"		# Where libexec, share and var are
   ledir="$MY_PWD/libexec"		# Where mysqld is
-elif test -x "$MY_PWD/sbin/mysqld"
+elif test -f "$relpkgdata"/english/errmsg.sys -a -x "$MY_PWD/sbin/mysqld"
 then
   MY_BASEDIR_VERSION="$MY_PWD"		# Where sbin, share and var are
   ledir="$MY_PWD/sbin"			# Where mysqld is
@@ -443,8 +431,6 @@ else
   ledir='@libexecdir@'
 fi
 
-helper=`find_in_bin mysqld_safe_helper`
-print_defaults=`find_in_bin my_print_defaults`
 
 #
 # Second, try to find the data directory
@@ -482,7 +468,6 @@ IGNORING $DATADIR/my.cnf"
     log_error "WARNING: Found $DATADIR/my.cnf
 The data directory is a deprecated location for my.cnf, please move it to
 $MY_BASEDIR_VERSION/my.cnf"
-    unsafe_my_cnf=1
     MYSQL_HOME=$DATADIR
   else
     MYSQL_HOME=$MY_BASEDIR_VERSION
@@ -490,15 +475,34 @@ $MY_BASEDIR_VERSION/my.cnf"
 fi
 export MYSQL_HOME
 
+
+# Get first arguments from the my.cnf file, groups [mysqld] and [mysqld_safe]
+# and then merge with the command line arguments
+if test -x "$MY_BASEDIR_VERSION/bin/my_print_defaults"
+then
+  print_defaults="$MY_BASEDIR_VERSION/bin/my_print_defaults"
+elif test -x `dirname $0`/my_print_defaults
+then
+  print_defaults="`dirname $0`/my_print_defaults"
+elif test -x ./bin/my_print_defaults
+then
+  print_defaults="./bin/my_print_defaults"
+elif test -x @bindir@/my_print_defaults
+then
+  print_defaults="@bindir@/my_print_defaults"
+elif test -x @bindir@/mysql_print_defaults
+then
+  print_defaults="@bindir@/mysql_print_defaults"
+else
+  print_defaults="my_print_defaults"
+fi
+
 append_arg_to_args () {
   args="$args "`shell_quote_string "$1"`
 }
 
 args=
 
-# Get first arguments from the my.cnf file, groups [mysqld] and [mysqld_safe]
-# and then merge with the command line arguments
-
 SET_USER=2
 parse_arguments `$print_defaults $defaults --loose-verbose --mysqld`
 if test $SET_USER -eq 2
@@ -506,16 +510,8 @@ then
   SET_USER=0
 fi
 
-# If arguments come from [mysqld_safe] section of my.cnf
-# we complain about unrecognized options
-unrecognized_handling=complain
 parse_arguments `$print_defaults $defaults --loose-verbose mysqld_safe safe_mysqld mariadb_safe`
-
-# We only need to pass arguments through to the server if we don't
-# handle them here.  So, we collect unrecognized options (passed on
-# the command line) into the args variable.
-unrecognized_handling=collect
-parse_arguments "$@"
+parse_arguments PICK-ARGS-FROM-ARGV "$@"
 
 
 #
@@ -527,7 +523,7 @@ if [ -n "${PLUGIN_DIR}" ]; then
   plugin_dir="${PLUGIN_DIR}"
 else
   # Try to find plugin dir relative to basedir
-  for dir in lib64/mysql/plugin lib64/plugin lib/mysql/plugin lib/plugin
+  for dir in lib/mysql/plugin lib/plugin
   do
     if [ -d "${MY_BASEDIR_VERSION}/${dir}" ]; then
       plugin_dir="${MY_BASEDIR_VERSION}/${dir}"
@@ -554,11 +550,6 @@ then
   fi
 fi
 
-if [ $skip_err_log -eq 1 ]
-then
-  append_arg_to_args "--skip-log-error"
-fi
-
 if [ -n "$err_log" -o $want_syslog -eq 0 ]
 then
   if [ -n "$err_log" ]
@@ -582,16 +573,7 @@ then
       * ) err_log="$DATADIR/$err_log" ;;
     esac
   else
-    if [ -n "$err_log_base" ]
-    then
-      err_log=$err_log_base.err
-      case "$err_log" in
-        /* ) ;;
-        * ) err_log="$DATADIR/$err_log" ;;
-      esac
-    else
-      err_log=$DATADIR/`@HOSTNAME@`.err
-    fi
+    err_log=$DATADIR/`@HOSTNAME@`.err
   fi
 
   append_arg_to_args "--log-error=$err_log"
@@ -600,13 +582,17 @@ then
   then
     # User explicitly asked for syslog, so warn that it isn't used
     log_error "Can't log to error log and syslog at the same time.  Remove all --log-error configuration options for --syslog to take effect."
-    want_syslog=0
   fi
 
   # Log to err_log file
   log_notice "Logging to '$err_log'."
   logging=file
 
+  if [ ! -f "$err_log" ]; then                  # if error log already exists,
+    touch "$err_log"                            # we just append. otherwise,
+    chmod "$fmode" "$err_log"                   # fix the permissions here!
+  fi
+
 else
   if [ -n "$syslog_tag" ]
   then
@@ -626,6 +612,11 @@ then
   then
     USER_OPTION="--user=$user"
   fi
+  # Change the err log to the right user, if it is in use
+  if [ $want_syslog -eq 0 ]; then
+    touch "$err_log"
+    chown $user "$err_log"
+  fi
   if test -n "$open_files"
   then
     ulimit -n $open_files
@@ -644,7 +635,7 @@ if [ ! -d $mysql_unix_port_dir ]
 then
   if ! `mkdir -p $mysql_unix_port_dir`
   then
-    log_error "Fatal error Can't create database directory '$mysql_unix_port'"
+    echo "Fatal error Can't create database directory '$mysql_unix_port'"
     exit 1
   fi
   chown $user $mysql_unix_port_dir
@@ -869,15 +860,6 @@ max_fast_restarts=5
 # flag whether a usable sleep command exists
 have_sleep=1
 
-# close stdout and stderr, everything goes to $logging now
-if expr "${-}" : '.*x' > /dev/null
-then
-  :
-else
-  exec 1>/dev/null
-  exec 2>/dev/null
-fi
-
 while true
 do
   rm -f "$pid_file"	# Some extra safety
@@ -885,6 +867,13 @@ do
   start_time=`date +%M%S`
 
   eval_log_error "$cmd"
+
+  if [ $want_syslog -eq 0 -a ! -f "$err_log" ]; then
+    touch "$err_log"                    # hypothetical: log was renamed but not
+    chown $user "$err_log"              # flushed yet. we'd recreate it with
+    chmod "$fmode" "$err_log"           # wrong owner next time we log, so set
+  fi                                    # it up correctly while we can!
+
   end_time=`date +%M%S`
 
   if test ! -f "$pid_file"		# This is removed if normal shutdown
@@ -948,11 +937,12 @@ do
     done
   fi
   log_notice "mysqld restarted"
-  if test -n "$crash_script"
+  if test -n "$CRASH_SCRIPT"
   then
-    crash_script_output=`$crash_script 2>&1`
+    crash_script_output=`$CRASH_SCRIPT 2>&1`
     log_error "$crash_script_output"
   fi
 done
 
 log_notice "mysqld from pid file $pid_file ended"
+
