@@ -1,5 +1,7 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2018, MariaDB Corporation
+/*
+   Copyright (c) 2000, 2012, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2012, Monty Program Ab
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +27,6 @@
     gives much more speed.
 */
 
-#include <my_global.h>
 #include "sql_priv.h"
 #include "sql_class.h"                          // THD
 #include <m_ctype.h>
@@ -332,7 +333,9 @@ static void do_copy_next_number(Copy_field *copy)
 
 static void do_copy_blob(Copy_field *copy)
 {
-  ((Field_blob*) copy->to_field)->copy_value(((Field_blob*) copy->from_field));
+  ulong length=((Field_blob*) copy->from_field)->get_length();
+  ((Field_blob*) copy->to_field)->store_length(length);
+  memcpy(copy->to_ptr, copy->from_ptr, sizeof(char*));
 }
 
 static void do_conv_blob(Copy_field *copy)
@@ -590,7 +593,7 @@ void Copy_field::set(uchar *to,Field *from)
 {
   from_ptr=from->ptr;
   to_ptr=to;
-  from_length=from->pack_length_in_rec();
+  from_length=from->pack_length();
   if (from->maybe_null())
   {
     from_null_ptr=from->null_ptr;
@@ -641,9 +644,9 @@ void Copy_field::set(Field *to,Field *from,bool save)
   from_field=from;
   to_field=to;
   from_ptr=from->ptr;
-  from_length=from->pack_length_in_rec();
+  from_length=from->pack_length();
   to_ptr=  to->ptr;
-  to_length=to_field->pack_length_in_rec();
+  to_length=to_field->pack_length();
 
   // set up null handling
   from_null_ptr=to_null_ptr=0;
@@ -707,7 +710,12 @@ Copy_field::get_copy_func(Field *to,Field *from)
     if (!(from->flags & BLOB_FLAG) || from->charset() != to->charset())
       return do_conv_blob;
     if (from_length != to_length)
+    {
+      // Correct pointer to point at char pointer
+      to_ptr+=   to_length - portable_sizeof_char_ptr;
+      from_ptr+= from_length - portable_sizeof_char_ptr;
       return do_copy_blob;
+    }
   }
   else
   {
@@ -783,10 +791,6 @@ Copy_field::get_copy_func(Field *to,Field *from)
     else if (to->real_type() != from->real_type() ||
 	     to_length != from_length)
     {
-      if ((to->real_type() == MYSQL_TYPE_ENUM ||
-           to->real_type() == MYSQL_TYPE_SET) &&
-          from->real_type() == MYSQL_TYPE_NEWDECIMAL)
-        return do_field_decimal;
       if (to->real_type() == MYSQL_TYPE_DECIMAL ||
 	  to->result_type() == STRING_RESULT)
 	return do_field_string;
@@ -833,10 +837,7 @@ bool memcpy_field_possible(Field *to,Field *from)
 {
   const enum_field_types to_real_type= to->real_type();
   const enum_field_types from_real_type= from->real_type();
-  /*
-    Warning: Calling from->type() may be unsafe in some (unclear) circumstances
-    related to SPs. See MDEV-6799.
-  */
+  const enum_field_types to_type= from->type();
   return (to_real_type == from_real_type &&
           !(to->flags & BLOB_FLAG && to->table->copy_blobs) &&
           to->pack_length() == from->pack_length() &&
@@ -849,8 +850,8 @@ bool memcpy_field_possible(Field *to,Field *from)
            to->field_length == from->field_length) &&
           from->charset() == to->charset() &&
           (!sql_mode_for_dates(to->table->in_use) ||
-           (from->type()!= MYSQL_TYPE_DATE &&
-            from->type()!= MYSQL_TYPE_DATETIME)) &&
+           (to_type != MYSQL_TYPE_DATE &&
+            to_type != MYSQL_TYPE_DATETIME)) &&
           (from_real_type != MYSQL_TYPE_VARCHAR ||
            ((Field_varstring*)from)->length_bytes ==
            ((Field_varstring*)to)->length_bytes));
@@ -896,15 +897,15 @@ int field_conv_incompatible(Field *to, Field *from)
   {						// Be sure the value is stored
     Field_blob *blob=(Field_blob*) to;
     from->val_str(&blob->value);
-
     /*
-      Copy value if copy_blobs is set, or source is part of the table's
-      writeset.
+      Copy value if copy_blobs is set, or source is not a string and
+      we have a pointer to its internal string conversion buffer.
     */
     if (to->table->copy_blobs ||
-        (!blob->value.is_alloced() && from->is_updatable()))
+        (!blob->value.is_alloced() &&
+         from_real_type != MYSQL_TYPE_STRING &&
+         from_real_type != MYSQL_TYPE_VARCHAR))
       blob->value.copy();
-
     return blob->store(blob->value.ptr(),blob->value.length(),from->charset());
   }
   if (from_real_type == MYSQL_TYPE_ENUM &&
