@@ -1486,8 +1486,6 @@ void check_equality(Dep_analysis_context *ctx, Dep_module_expr **eq_mod,
       left->real_item()->type() == Item::FIELD_ITEM)
   {
     Field *field= ((Item_field*)left->real_item())->field;
-    if (right->cmp_type() == TIME_RESULT && field->cmp_type() != TIME_RESULT)
-      return;
     if (field->result_type() == STRING_RESULT)
     {
       if (right->result_type() != STRING_RESULT)
@@ -1501,9 +1499,7 @@ void check_equality(Dep_analysis_context *ctx, Dep_module_expr **eq_mod,
           We can't assume there's a functional dependency if the effective
           collation of the operation differ from the field collation.
         */
-        if ((field->cmp_type() == STRING_RESULT ||
-            field->real_type() == MYSQL_TYPE_ENUM ||
-            field->real_type() == MYSQL_TYPE_SET) &&
+        if (field->cmp_type() == STRING_RESULT &&
             field->charset() != cond->compare_collation())
           return;
       }
