@@ -16,7 +16,7 @@
 
 /* Definitions for MariaDB global transaction ID (GTID). */
 
-#include <my_global.h>
+
 #include "sql_priv.h"
 #include "my_sys.h"
 #include "unireg.h"
@@ -65,16 +65,16 @@ rpl_slave_state::update_state_hash(uint64 sub_id, rpl_gtid *gtid,
 int
 rpl_slave_state::record_and_update_gtid(THD *thd, rpl_group_info *rgi)
 {
+  uint64 sub_id;
   DBUG_ENTER("rpl_slave_state::record_and_update_gtid");
 
   /*
     Update the GTID position, if we have it and did not already update
     it in a GTID transaction.
   */
-  if (rgi->gtid_pending)
+  if ((sub_id= rgi->gtid_sub_id))
   {
-    uint64 sub_id= rgi->gtid_sub_id;
-    rgi->gtid_pending= false;
+    rgi->gtid_sub_id= 0;
     if (rgi->gtid_ignore_duplicate_state!=rpl_group_info::GTID_DUPLICATE_IGNORE)
     {
       if (record_gtid(thd, &rgi->current_gtid, sub_id, false, false))
@@ -117,12 +117,12 @@ int
 rpl_slave_state::check_duplicate_gtid(rpl_gtid *gtid, rpl_group_info *rgi)
 {
   uint32 domain_id= gtid->domain_id;
-  uint64 seq_no= gtid->seq_no;
+  uint32 seq_no= gtid->seq_no;
   rpl_slave_state::element *elem;
   int res;
-  bool did_enter_cond= false;
+  bool did_enter_cond;
   PSI_stage_info old_stage;
-  THD *UNINIT_VAR(thd);
+  THD *thd;
   Relay_log_info *rli= rgi->rli;
 
   mysql_mutex_lock(&LOCK_slave_state);
@@ -138,6 +138,7 @@ rpl_slave_state::check_duplicate_gtid(rpl_gtid *gtid, rpl_group_info *rgi)
     each lock release and re-take.
   */
 
+  did_enter_cond= false;
   for (;;)
   {
     if (elem->highest_seq_no >= seq_no)
@@ -243,10 +244,8 @@ rpl_slave_state_free_element(void *arg)
 
 
 rpl_slave_state::rpl_slave_state()
-  : last_sub_id(0), loaded(false)
+  : last_sub_id(0), inited(false), loaded(false)
 {
-  mysql_mutex_init(key_LOCK_slave_state, &LOCK_slave_state,
-                   MY_MUTEX_INIT_SLOW);
   my_hash_init(&hash, &my_charset_bin, 32, offsetof(element, domain_id),
                sizeof(uint32), NULL, rpl_slave_state_free_element, HASH_UNIQUE);
 }
@@ -254,9 +253,15 @@ rpl_slave_state::rpl_slave_state()
 
 rpl_slave_state::~rpl_slave_state()
 {
-  truncate_hash();
-  my_hash_free(&hash);
-  mysql_mutex_destroy(&LOCK_slave_state);
+}
+
+
+void
+rpl_slave_state::init()
+{
+  DBUG_ASSERT(!inited);
+  mysql_mutex_init(key_LOCK_slave_state, &LOCK_slave_state, MY_MUTEX_INIT_SLOW);
+  inited= true;
 }
 
 
@@ -281,6 +286,16 @@ rpl_slave_state::truncate_hash()
   my_hash_reset(&hash);
 }
 
+void
+rpl_slave_state::deinit()
+{
+  if (!inited)
+    return;
+  truncate_hash();
+  my_hash_free(&hash);
+  mysql_mutex_destroy(&LOCK_slave_state);
+}
+
 
 int
 rpl_slave_state::update(uint32 domain_id, uint32 server_id, uint64 sub_id,
@@ -501,7 +516,6 @@ rpl_slave_state::record_gtid(THD *thd, const rpl_gtid *gtid, uint64 sub_id,
   element *elem;
   ulonglong thd_saved_option= thd->variables.option_bits;
   Query_tables_list lex_backup;
-  wait_for_commit* suspended_wfc;
   DBUG_ENTER("record_gtid");
 
   if (unlikely(!loaded))
@@ -525,28 +539,6 @@ rpl_slave_state::record_gtid(THD *thd, const rpl_gtid *gtid, uint64 sub_id,
                     DBUG_RETURN(1);
                   } );
 
-  /*
-    If we are applying a non-transactional event group, we will be committing
-    here a transaction, but that does not imply that the event group has
-    completed or has been binlogged. So we should not trigger
-    wakeup_subsequent_commits() here.
-
-    Note: An alternative here could be to put a call to mark_start_commit() in
-    stmt_done() before the call to record_and_update_gtid(). This would
-    prevent later calling mark_start_commit() after we have run
-    wakeup_subsequent_commits() from committing the GTID update transaction
-    (which must be avoided to avoid accessing freed group_commit_orderer
-    object). It would also allow following event groups to start slightly
-    earlier. And in the cases where record_gtid() is called without an active
-    transaction, the current statement should have been binlogged already, so
-    binlog order is preserved.
-
-    But this is rather subtle, and potentially fragile. And it does not really
-    seem worth it; non-transactional loads are unlikely to benefit much from
-    parallel replication in any case. So for now, we go with the simple
-    suspend/resume of wakeup_subsequent_commits() here in record_gtid().
-  */
-  suspended_wfc= thd->suspend_subsequent_commits();
   thd->lex->reset_n_backup_query_tables_list(&lex_backup);
   tlist.init_one_table(STRING_WITH_LEN("mysql"),
                        rpl_gtid_slave_state_table_name.str,
@@ -675,7 +667,7 @@ end:
 
   if (table_opened)
   {
-    if (err || (err= ha_commit_trans(thd, FALSE)))
+    if (err)
     {
       /*
         If error, we need to put any remaining elist back into the HASH so we
@@ -689,8 +681,13 @@ end:
       }
 
       ha_rollback_trans(thd, FALSE);
+      close_thread_tables(thd);
+    }
+    else
+    {
+      ha_commit_trans(thd, FALSE);
+      close_thread_tables(thd);
     }
-    close_thread_tables(thd);
     if (in_transaction)
       thd->mdl_context.release_statement_locks();
     else
@@ -698,12 +695,6 @@ end:
   }
   thd->lex->restore_backup_query_tables_list(&lex_backup);
   thd->variables.option_bits= thd_saved_option;
-  thd->resume_subsequent_commits(suspended_wfc);
-  DBUG_EXECUTE_IF("inject_record_gtid_serverid_100_sleep",
-    {
-      if (gtid->server_id == 100)
-        my_sleep(500000);
-    });
   DBUG_RETURN(err);
 }
 
@@ -1104,27 +1095,6 @@ rpl_binlog_state::load(struct rpl_gtid *list, uint32 count)
 }
 
 
-static int rpl_binlog_state_load_cb(rpl_gtid *gtid, void *data)
-{
-  rpl_binlog_state *self= (rpl_binlog_state *)data;
-  return self->update_nolock(gtid, false);
-}
-
-
-bool
-rpl_binlog_state::load(rpl_slave_state *slave_pos)
-{
-  bool res= false;
-
-  mysql_mutex_lock(&LOCK_binlog_state);
-  reset_nolock();
-  if (slave_pos->iterate(rpl_binlog_state_load_cb, this, NULL, 0))
-    res= true;
-  mysql_mutex_unlock(&LOCK_binlog_state);
-  return res;
-}
-
-
 rpl_binlog_state::~rpl_binlog_state()
 {
   free();
@@ -1884,31 +1854,6 @@ slave_connection_state::get_gtid_list(rpl_gtid *gtid_list, uint32 list_size)
 }
 
 
-/*
-  Check if the GTID position has been reached, for mysql_binlog_send().
-
-  The position has not been reached if we have anything in the state, unless
-  it has either the START_ON_EMPTY_DOMAIN flag set (which means it does not
-  belong to this master at all), or the START_OWN_SLAVE_POS (which means that
-  we start on an old position from when the server was a slave with
-  --log-slave-updates=0).
-*/
-bool
-slave_connection_state::is_pos_reached()
-{
-  uint32 i;
-
-  for (i= 0; i < hash.records; ++i)
-  {
-    entry *e= (entry *)my_hash_element(&hash, i);
-    if (!(e->flags & (START_OWN_SLAVE_POS|START_ON_EMPTY_DOMAIN)))
-      return false;
-  }
-
-  return true;
-}
-
-
 /*
   Execute a MASTER_GTID_WAIT().
   The position to wait for is in gtid_str in string form.
@@ -2083,16 +2028,16 @@ gtid_waiting::wait_for_gtid(THD *thd, rpl_gtid *wait_gtid,
       uint64 wakeup_seq_no;
       queue_element *cur_waiter;
 
-      mysql_mutex_lock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+      mysql_mutex_lock(&rpl_global_gtid_slave_state.LOCK_slave_state);
       /*
         The elements in the gtid_slave_state_hash are never re-allocated once
         they enter the hash, so we do not need to re-do the lookup after releasing
         and re-aquiring the lock.
       */
       if (!slave_state_elem &&
-          !(slave_state_elem= rpl_global_gtid_slave_state->get_element(domain_id)))
+          !(slave_state_elem= rpl_global_gtid_slave_state.get_element(domain_id)))
       {
-        mysql_mutex_unlock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+        mysql_mutex_unlock(&rpl_global_gtid_slave_state.LOCK_slave_state);
         remove_from_wait_queue(he, &elem);
         promote_new_waiter(he);
         if (did_enter_cond)
@@ -2109,7 +2054,7 @@ gtid_waiting::wait_for_gtid(THD *thd, rpl_gtid *wait_gtid,
           We do not have to wait. (We will be removed from the wait queue when
           we call process_wait_hash() below.
         */
-        mysql_mutex_unlock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+        mysql_mutex_unlock(&rpl_global_gtid_slave_state.LOCK_slave_state);
       }
       else if ((cur_waiter= slave_state_elem->gtid_waiter) &&
                slave_state_elem->min_wait_seq_no <= seq_no)
@@ -2121,7 +2066,7 @@ gtid_waiting::wait_for_gtid(THD *thd, rpl_gtid *wait_gtid,
           lock).
         */
         elem.do_small_wait= false;
-        mysql_mutex_unlock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+        mysql_mutex_unlock(&rpl_global_gtid_slave_state.LOCK_slave_state);
       }
       else
       {
@@ -2146,7 +2091,7 @@ gtid_waiting::wait_for_gtid(THD *thd, rpl_gtid *wait_gtid,
         else
           mysql_mutex_unlock(&LOCK_gtid_waiting);
         thd->ENTER_COND(&slave_state_elem->COND_wait_gtid,
-                        &rpl_global_gtid_slave_state->LOCK_slave_state,
+                        &rpl_global_gtid_slave_state.LOCK_slave_state,
                         &stage_master_gtid_wait_primary, &old_stage);
         do
         {
@@ -2156,7 +2101,7 @@ gtid_waiting::wait_for_gtid(THD *thd, rpl_gtid *wait_gtid,
           {
             int err=
               mysql_cond_timedwait(&slave_state_elem->COND_wait_gtid,
-                                   &rpl_global_gtid_slave_state->LOCK_slave_state,
+                                   &rpl_global_gtid_slave_state.LOCK_slave_state,
                                    wait_until);
             if (err == ETIMEDOUT || err == ETIME)
             {
@@ -2166,7 +2111,7 @@ gtid_waiting::wait_for_gtid(THD *thd, rpl_gtid *wait_gtid,
           }
           else
             mysql_cond_wait(&slave_state_elem->COND_wait_gtid,
-                            &rpl_global_gtid_slave_state->LOCK_slave_state);
+                            &rpl_global_gtid_slave_state.LOCK_slave_state);
         } while (slave_state_elem->gtid_waiter == &elem);
         wakeup_seq_no= slave_state_elem->highest_seq_no;
         /*
