@@ -14,7 +14,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <my_global.h>
 #include "sql_priv.h"
 #include "unireg.h"
 #include "rpl_rli.h"
@@ -318,7 +317,6 @@ unpack_row(rpl_group_info *rgi,
         if (!pack_ptr)
         {
           rgi->rli->report(ERROR_LEVEL, ER_SLAVE_CORRUPT_EVENT,
-                      rgi->gtid_info(),
                       "Could not read field '%s' of table '%s.%s'",
                       f->field_name, table->s->db.str,
                       table->s->table_name.str);
