@@ -1,5 +1,5 @@
-/* Copyright (c) 2006, 2017, Oracle and/or its affiliates.
-   Copyright (c) 2011, 2017, MariaDB Corporation
+/* Copyright (c) 2006, 2013, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2013, Monty Program Ab
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,7 +14,6 @@
    along with this program; if not, write to the Free Software Foundation,
    51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
 
-#include <my_global.h>
 #include "sql_priv.h"
 #include "unireg.h"                             // HAVE_*
 #include "rpl_mi.h"
@@ -37,7 +36,7 @@ static int count_relay_log_space(Relay_log_info* rli);
    Current replication state (hash of last GTID executed, per replication
    domain).
 */
-rpl_slave_state *rpl_global_gtid_slave_state;
+rpl_slave_state rpl_global_gtid_slave_state;
 /* Object used for MASTER_GTID_WAIT(). */
 gtid_waiting rpl_global_gtid_waiting;
 
@@ -53,17 +52,16 @@ Relay_log_info::Relay_log_info(bool is_slave_recovery)
    info_fd(-1), cur_log_fd(-1), relay_log(&sync_relaylog_period),
    sync_counter(0), is_relay_log_recovery(is_slave_recovery),
    save_temporary_tables(0), mi(0),
-   inuse_relaylog_list(0), last_inuse_relaylog(0),
-   cur_log_old_open_count(0), error_on_rli_init_info(false),
-   group_relay_log_pos(0), event_relay_log_pos(0),
+   cur_log_old_open_count(0), group_relay_log_pos(0), 
+   event_relay_log_pos(0),
 #if HAVE_valgrind
    is_fake(FALSE),
 #endif
    group_master_log_pos(0), log_space_total(0), ignore_log_space_limit(0),
    last_master_timestamp(0), sql_thread_caught_up(true), slave_skip_counter(0),
    abort_pos_wait(0), slave_run_id(0), sql_driver_thd(),
-   gtid_skip_flag(GTID_SKIP_NOT), inited(0), abort_slave(0), stop_for_until(0),
-   slave_running(MYSQL_SLAVE_NOT_RUN), until_condition(UNTIL_NONE),
+   inited(0), abort_slave(0), stop_for_until(0),
+   slave_running(0), until_condition(UNTIL_NONE),
    until_log_pos(0), retried_trans(0), executed_entries(0),
    m_flags(0)
 {
@@ -102,7 +100,6 @@ Relay_log_info::~Relay_log_info()
 {
   DBUG_ENTER("Relay_log_info::~Relay_log_info");
 
-  reset_inuse_relaylog();
   mysql_mutex_destroy(&run_lock);
   mysql_mutex_destroy(&data_lock);
   mysql_mutex_destroy(&log_space_lock);
@@ -119,7 +116,7 @@ int init_relay_log_info(Relay_log_info* rli,
 			const char* info_fname)
 {
   char fname[FN_REFLEN+128];
-  int info_fd= -1;
+  int info_fd;
   const char* msg = 0;
   int error = 0;
   DBUG_ENTER("init_relay_log_info");
@@ -129,8 +126,6 @@ int init_relay_log_info(Relay_log_info* rli,
     DBUG_RETURN(0);
   fn_format(fname, info_fname, mysql_data_home, "", 4+32);
   mysql_mutex_lock(&rli->data_lock);
-  if (rli->error_on_rli_init_info)
-    goto err;
   info_fd = rli->info_fd;
   rli->cur_log_fd = -1;
   rli->slave_skip_counter=0;
@@ -210,7 +205,6 @@ a file name for --relay-log-index option", opt_relaylog_index_name);
     Master_info* mi= rli->mi;
     char buf_relay_logname[FN_REFLEN], buf_relaylog_index_name_buff[FN_REFLEN];
     char *buf_relaylog_index_name= opt_relaylog_index_name;
-    mysql_mutex_t *log_lock;
 
     create_logfile_name_with_suffix(buf_relay_logname,
                                     sizeof(buf_relay_logname),
@@ -230,18 +224,14 @@ a file name for --relay-log-index option", opt_relaylog_index_name);
       note, that if open() fails, we'll still have index file open
       but a destructor will take care of that
     */
-    log_lock= rli->relay_log.get_log_lock();
-    mysql_mutex_lock(log_lock);
     if (rli->relay_log.open_index_file(buf_relaylog_index_name, ln, TRUE) ||
         rli->relay_log.open(ln, LOG_BIN, 0, SEQ_READ_APPEND,
                             mi->rli.max_relay_log_size, 1, TRUE))
     {
-      mysql_mutex_unlock(log_lock);
       mysql_mutex_unlock(&rli->data_lock);
       sql_print_error("Failed when trying to open logs for '%s' in init_relay_log_info(). Error: %M", ln, my_errno);
       DBUG_RETURN(1);
     }
-    mysql_mutex_unlock(log_lock);
   }
 
   /* if file does not exist */
@@ -256,8 +246,8 @@ a file name for --relay-log-index option", opt_relaylog_index_name);
     if ((info_fd= mysql_file_open(key_file_relay_log_info,
                                   fname, O_CREAT|O_RDWR|O_BINARY, MYF(MY_WME))) < 0)
     {
-      sql_print_error("Failed to create a new relay log info file ("
-                      "file '%s', errno %d)", fname, my_errno);
+      sql_print_error("Failed to create a new relay log info file (\
+file '%s', errno %d)", fname, my_errno);
       msg= current_thd->get_stmt_da()->message();
       goto err;
     }
@@ -315,80 +305,20 @@ Failed to open the existing relay log info file '%s' (errno %d)",
     }
 
     rli->info_fd = info_fd;
-    int relay_log_pos, master_log_pos, lines;
-    char *first_non_digit;
-    /*
-      In MySQL 5.6, there is a MASTER_DELAY option to CHANGE MASTER. This is
-      not yet merged into MariaDB (as of 10.0.13). However, we detect the
-      presense of the new option in relay-log.info, as a placeholder for
-      possible later merge of the feature, and to maintain file format
-      compatibility with MySQL 5.6+.
-    */
-    int dummy_sql_delay;
-
-    /*
-      Starting from MySQL 5.6.x, relay-log.info has a new format.
-      Now, its first line contains the number of lines in the file.
-      By reading this number we can determine which version our master.info
-      comes from. We can't simply count the lines in the file, since
-      versions before 5.6.x could generate files with more lines than
-      needed. If first line doesn't contain a number, or if it
-      contains a number less than LINES_IN_RELAY_LOG_INFO_WITH_DELAY,
-      then the file is treated like a file from pre-5.6.x version.
-      There is no ambiguity when reading an old master.info: before
-      5.6.x, the first line contained the binlog's name, which is
-      either empty or has an extension (contains a '.'), so can't be
-      confused with an integer.
-
-      So we're just reading first line and trying to figure which
-      version is this.
-    */
-
-    /*
-      The first row is temporarily stored in mi->master_log_name, if
-      it is line count and not binlog name (new format) it will be
-      overwritten by the second row later.
-    */
+    int relay_log_pos, master_log_pos;
     if (init_strvar_from_file(rli->group_relay_log_name,
                               sizeof(rli->group_relay_log_name),
-                              &rli->info_file, ""))
-    {
-      msg="Error reading slave log configuration";
-      goto err;
-    }
-
-    lines= strtoul(rli->group_relay_log_name, &first_non_digit, 10);
-
-    if (rli->group_relay_log_name[0] != '\0' &&
-        *first_non_digit == '\0' &&
-        lines >= LINES_IN_RELAY_LOG_INFO_WITH_DELAY)
-    {
-      DBUG_PRINT("info", ("relay_log_info file is in new format."));
-      /* Seems to be new format => read relay log name from next line */
-      if (init_strvar_from_file(rli->group_relay_log_name,
-                                sizeof(rli->group_relay_log_name),
-                                &rli->info_file, ""))
-      {
-        msg="Error reading slave log configuration";
-        goto err;
-      }
-    }
-    else
-      DBUG_PRINT("info", ("relay_log_info file is in old format."));
-
-    if (init_intvar_from_file(&relay_log_pos,
-                              &rli->info_file, BIN_LOG_HEADER_SIZE) ||
-        init_strvar_from_file(rli->group_master_log_name,
-                              sizeof(rli->group_master_log_name),
                               &rli->info_file, "") ||
-        init_intvar_from_file(&master_log_pos, &rli->info_file, 0) ||
-        (lines >= LINES_IN_RELAY_LOG_INFO_WITH_DELAY &&
-         init_intvar_from_file(&dummy_sql_delay, &rli->info_file, 0)))
+       init_intvar_from_file(&relay_log_pos,
+                             &rli->info_file, BIN_LOG_HEADER_SIZE) ||
+       init_strvar_from_file(rli->group_master_log_name,
+                             sizeof(rli->group_master_log_name),
+                             &rli->info_file, "") ||
+       init_intvar_from_file(&master_log_pos, &rli->info_file, 0))
     {
       msg="Error reading slave log configuration";
       goto err;
     }
-
     strmake_buf(rli->event_relay_log_name,rli->group_relay_log_name);
     rli->group_relay_log_pos= rli->event_relay_log_pos= relay_log_pos;
     rli->group_master_log_pos= master_log_pos;
@@ -396,7 +326,6 @@ Failed to open the existing relay log info file '%s' (errno %d)",
     if (rli->is_relay_log_recovery && init_recovery(rli->mi, &msg))
       goto err;
 
-    rli->relay_log_state.load(rpl_global_gtid_slave_state);
     if (init_relay_log_pos(rli,
                            rli->group_relay_log_name,
                            rli->group_relay_log_pos,
@@ -438,21 +367,16 @@ Failed to open the existing relay log info file '%s' (errno %d)",
     goto err;
   }
   rli->inited= 1;
-  rli->error_on_rli_init_info= false;
   mysql_mutex_unlock(&rli->data_lock);
-  DBUG_RETURN(0);
+  DBUG_RETURN(error);
 
 err:
-  rli->error_on_rli_init_info= true;
-  if (msg)
-    sql_print_error("%s", msg);
+  sql_print_error("%s", msg);
   end_io_cache(&rli->info_file);
   if (info_fd >= 0)
     mysql_file_close(info_fd, MYF(0));
   rli->info_fd= -1;
-  mysql_mutex_lock(rli->relay_log.get_log_lock());
   rli->relay_log.close(LOG_CLOSE_INDEX | LOG_CLOSE_STOP_EVENT);
-  mysql_mutex_unlock(rli->relay_log.get_log_lock());
   mysql_mutex_unlock(&rli->data_lock);
   DBUG_RETURN(1);
 }
@@ -522,90 +446,6 @@ void Relay_log_info::clear_until_condition()
 }
 
 
-/*
-  Read the correct format description event for starting to replicate from
-  a given position in a relay log file.
-*/
-Format_description_log_event *
-read_relay_log_description_event(IO_CACHE *cur_log, ulonglong start_pos,
-                                 const char **errmsg)
-{
-  Log_event *ev;
-  Format_description_log_event *fdev;
-  bool found= false;
-
-  /*
-    By default the relay log is in binlog format 3 (4.0).
-    Even if format is 4, this will work enough to read the first event
-    (Format_desc) (remember that format 4 is just lenghtened compared to format
-    3; format 3 is a prefix of format 4).
-  */
-  fdev= new Format_description_log_event(3);
-
-  while (!found)
-  {
-    Log_event_type typ;
-
-    /*
-      Read the possible Format_description_log_event; if position
-      was 4, no need, it will be read naturally.
-    */
-    DBUG_PRINT("info",("looking for a Format_description_log_event"));
-
-    if (my_b_tell(cur_log) >= start_pos)
-      break;
-
-    if (!(ev= Log_event::read_log_event(cur_log, 0, fdev,
-                                        opt_slave_sql_verify_checksum)))
-    {
-      DBUG_PRINT("info",("could not read event, cur_log->error=%d",
-                         cur_log->error));
-      if (cur_log->error) /* not EOF */
-      {
-        *errmsg= "I/O error reading event at position 4";
-        delete fdev;
-        return NULL;
-      }
-      break;
-    }
-    typ= ev->get_type_code();
-    if (typ == FORMAT_DESCRIPTION_EVENT)
-    {
-      DBUG_PRINT("info",("found Format_description_log_event"));
-      delete fdev;
-      fdev= (Format_description_log_event*) ev;
-      /*
-        As ev was returned by read_log_event, it has passed is_valid(), so
-        my_malloc() in ctor worked, no need to check again.
-      */
-      /*
-        Ok, we found a Format_description event. But it is not sure that this
-        describes the whole relay log; indeed, one can have this sequence
-        (starting from position 4):
-        Format_desc (of slave)
-        Rotate (of master)
-        Format_desc (of master)
-        So the Format_desc which really describes the rest of the relay log
-        is the 3rd event (it can't be further than that, because we rotate
-        the relay log when we queue a Rotate event from the master).
-        But what describes the Rotate is the first Format_desc.
-        So what we do is:
-        go on searching for Format_description events, until you exceed the
-        position (argument 'pos') or until you find another event than Rotate
-        or Format_desc.
-      */
-    }
-    else
-    {
-      DBUG_PRINT("info",("found event of another type=%d", typ));
-      found= (typ != ROTATE_EVENT);
-      delete ev;
-    }
-  }
-  return fdev;
-}
-
-
 /*
   Open the given relay log
 
@@ -729,13 +569,68 @@ int init_relay_log_pos(Relay_log_info* rli,const char* log,
   */
   if (pos > BIN_LOG_HEADER_SIZE) /* If pos<=4, we stay at 4 */
   {
-    if (look_for_description_event)
+    Log_event* ev;
+    while (look_for_description_event)
     {
-      Format_description_log_event *fdev;
-      if (!(fdev= read_relay_log_description_event(rli->cur_log, pos, errmsg)))
-        goto err;
-      delete rli->relay_log.description_event_for_exec;
-      rli->relay_log.description_event_for_exec= fdev;
+      /*
+        Read the possible Format_description_log_event; if position
+        was 4, no need, it will be read naturally.
+      */
+      DBUG_PRINT("info",("looking for a Format_description_log_event"));
+
+      if (my_b_tell(rli->cur_log) >= pos)
+        break;
+
+      /*
+        Because of we have rli->data_lock and log_lock, we can safely read an
+        event
+      */
+      if (!(ev= Log_event::read_log_event(rli->cur_log, 0,
+                                          rli->relay_log.description_event_for_exec,
+                                          opt_slave_sql_verify_checksum)))
+      {
+        DBUG_PRINT("info",("could not read event, rli->cur_log->error=%d",
+                           rli->cur_log->error));
+        if (rli->cur_log->error) /* not EOF */
+        {
+          *errmsg= "I/O error reading event at position 4";
+          goto err;
+        }
+        break;
+      }
+      else if (ev->get_type_code() == FORMAT_DESCRIPTION_EVENT)
+      {
+        DBUG_PRINT("info",("found Format_description_log_event"));
+        delete rli->relay_log.description_event_for_exec;
+        rli->relay_log.description_event_for_exec= (Format_description_log_event*) ev;
+        /*
+          As ev was returned by read_log_event, it has passed is_valid(), so
+          my_malloc() in ctor worked, no need to check again.
+        */
+        /*
+          Ok, we found a Format_description event. But it is not sure that this
+          describes the whole relay log; indeed, one can have this sequence
+          (starting from position 4):
+          Format_desc (of slave)
+          Rotate (of master)
+          Format_desc (of master)
+          So the Format_desc which really describes the rest of the relay log
+          is the 3rd event (it can't be further than that, because we rotate
+          the relay log when we queue a Rotate event from the master).
+          But what describes the Rotate is the first Format_desc.
+          So what we do is:
+          go on searching for Format_description events, until you exceed the
+          position (argument 'pos') or until you find another event than Rotate
+          or Format_desc.
+        */
+      }
+      else
+      {
+        DBUG_PRINT("info",("found event of another type=%d",
+                           ev->get_type_code()));
+        look_for_description_event= (ev->get_type_code() == ROTATE_EVENT);
+        delete ev;
+      }
     }
     my_b_seek(rli->cur_log,(off_t)pos);
 #ifndef DBUG_OFF
@@ -990,11 +885,11 @@ void Relay_log_info::inc_group_relay_log_pos(ulonglong log_pos,
   if (rgi->is_parallel_exec)
   {
     /* In case of parallel replication, do not update the position backwards. */
-    int cmp= strcmp(group_relay_log_name, rgi->event_relay_log_name);
+    int cmp= strcmp(group_relay_log_name, event_relay_log_name);
     if (cmp < 0)
     {
       group_relay_log_pos= rgi->future_event_relay_log_pos;
-      strmake_buf(group_relay_log_name, rgi->event_relay_log_name);
+      strmake_buf(group_relay_log_name, event_relay_log_name);
       notify_group_relay_log_name_update();
     } else if (cmp == 0 && group_relay_log_pos < rgi->future_event_relay_log_pos)
       group_relay_log_pos= rgi->future_event_relay_log_pos;
@@ -1009,23 +904,12 @@ void Relay_log_info::inc_group_relay_log_pos(ulonglong log_pos,
       if (cmp < 0)
       {
         strcpy(group_master_log_name, rgi->future_event_master_log_name);
+        notify_group_master_log_name_update();
         group_master_log_pos= log_pos;
       }
       else if (group_master_log_pos < log_pos)
         group_master_log_pos= log_pos;
     }
-
-    /*
-      In the parallel case, we only update the Seconds_Behind_Master at the
-      end of a transaction. In the non-parallel case, the value is updated as
-      soon as an event is read from the relay log; however this would be too
-      confusing for the user, seeing the slave reported as up-to-date when
-      potentially thousands of events are still queued up for worker threads
-      waiting for execution.
-    */
-    if (rgi->last_master_timestamp &&
-        rgi->last_master_timestamp > last_master_timestamp)
-      last_master_timestamp= rgi->last_master_timestamp;
   }
   else
   {
@@ -1108,8 +992,6 @@ int purge_relay_logs(Relay_log_info* rli, THD *thd, bool just_reset,
                      const char** errmsg)
 {
   int error=0;
-  const char *ln;
-  char name_buf[FN_REFLEN];
   DBUG_ENTER("purge_relay_logs");
 
   /*
@@ -1136,37 +1018,13 @@ int purge_relay_logs(Relay_log_info* rli, THD *thd, bool just_reset,
   if (!rli->inited)
   {
     DBUG_PRINT("info", ("rli->inited == 0"));
-    if (rli->error_on_rli_init_info)
-    {
-      ln= rli->relay_log.generate_name(opt_relay_logname, "-relay-bin",
-                                       1, name_buf);
+    DBUG_RETURN(0);
+  }
 
-      if (rli->relay_log.open_index_file(opt_relaylog_index_name, ln, TRUE))
-      {
-        sql_print_error("Unable to purge relay log files. Failed to open relay "
-                        "log index file:%s.", rli->relay_log.get_index_fname());
-        DBUG_RETURN(1);
-      }
-      mysql_mutex_lock(rli->relay_log.get_log_lock());
-      if (rli->relay_log.open(ln, LOG_BIN, 0, SEQ_READ_APPEND,
-                             (rli->max_relay_log_size ? rli->max_relay_log_size :
-                              max_binlog_size), 1, TRUE))
-      {
-        sql_print_error("Unable to purge relay log files. Failed to open relay "
-                        "log file:%s.", rli->relay_log.get_log_fname());
-        mysql_mutex_unlock(rli->relay_log.get_log_lock());
-        DBUG_RETURN(1);
-      }
-      mysql_mutex_unlock(rli->relay_log.get_log_lock());
-    }
-    else
-      DBUG_RETURN(0);
-  }
-  else
-  {
-    DBUG_ASSERT(rli->slave_running == 0);
-    DBUG_ASSERT(rli->mi->slave_running == 0);
-  }
+  DBUG_ASSERT(rli->slave_running == 0);
+  DBUG_ASSERT(rli->mi->slave_running == 0);
+
+  rli->slave_skip_counter=0;
   mysql_mutex_lock(&rli->data_lock);
 
   /*
@@ -1188,7 +1046,6 @@ int purge_relay_logs(Relay_log_info* rli, THD *thd, bool just_reset,
     error=1;
     goto err;
   }
-  rli->relay_log_state.load(rpl_global_gtid_slave_state);
   if (!just_reset)
   {
     /* Save name of used relay log file */
@@ -1213,12 +1070,6 @@ int purge_relay_logs(Relay_log_info* rli, THD *thd, bool just_reset,
     rli->group_relay_log_name[0]= rli->event_relay_log_name[0]= 0;
   }
 
-  if (!rli->inited && rli->error_on_rli_init_info)
-  {
-    mysql_mutex_lock(rli->relay_log.get_log_lock());
-    rli->relay_log.close(LOG_CLOSE_INDEX | LOG_CLOSE_STOP_EVENT);
-    mysql_mutex_unlock(rli->relay_log.get_log_lock());
-  }
 err:
 #ifndef DBUG_OFF
   char buf[22];
@@ -1276,8 +1127,7 @@ bool Relay_log_info::is_until_satisfied(THD *thd, Log_event *ev)
     if (ev && ev->server_id == (uint32) global_system_variables.server_id &&
         !replicate_same_server_id)
       DBUG_RETURN(FALSE);
-    log_name= (mi->using_parallel() ?
-               future_event_master_log_name : group_master_log_name);
+    log_name= group_master_log_name;
     log_pos= ((!ev)? group_master_log_pos :
               (get_flag(IN_TRANSACTION) || !ev->log_pos) ?
               group_master_log_pos : ev->log_pos - ev->data_written);
@@ -1349,10 +1199,13 @@ bool Relay_log_info::is_until_satisfied(THD *thd, Log_event *ev)
 }
 
 
-bool Relay_log_info::stmt_done(my_off_t event_master_log_pos, THD *thd,
+void Relay_log_info::stmt_done(my_off_t event_master_log_pos,
+                               time_t event_creation_time, THD *thd,
                                rpl_group_info *rgi)
 {
-  int error= 0;
+#ifndef DBUG_OFF
+  extern uint debug_not_change_ts_if_art_event;
+#endif
   DBUG_ENTER("Relay_log_info::stmt_done");
 
   DBUG_ASSERT(rgi->rli == this);
@@ -1388,9 +1241,9 @@ bool Relay_log_info::stmt_done(my_off_t event_master_log_pos, THD *thd,
   else
   {
     inc_group_relay_log_pos(event_master_log_pos, rgi);
-    if (rpl_global_gtid_slave_state->record_and_update_gtid(thd, rgi))
+    if (rpl_global_gtid_slave_state.record_and_update_gtid(thd, rgi))
     {
-      report(WARNING_LEVEL, ER_CANNOT_UPDATE_GTID_STATE, rgi->gtid_info(),
+      report(WARNING_LEVEL, ER_CANNOT_UPDATE_GTID_STATE,
              "Failed to update GTID state in %s.%s, slave state may become "
              "inconsistent: %d: %s",
              "mysql", rpl_gtid_slave_state_table_name.str,
@@ -1404,100 +1257,28 @@ bool Relay_log_info::stmt_done(my_off_t event_master_log_pos, THD *thd,
     }
     DBUG_EXECUTE_IF("inject_crash_before_flush_rli", DBUG_SUICIDE(););
     if (mi->using_gtid == Master_info::USE_GTID_NO)
-      if (flush_relay_log_info(this))
-        error= 1;
+      flush_relay_log_info(this);
     DBUG_EXECUTE_IF("inject_crash_after_flush_rli", DBUG_SUICIDE(););
+    /*
+      Note that Rotate_log_event::do_apply_event() does not call this
+      function, so there is no chance that a fake rotate event resets
+      last_master_timestamp.  Note that we update without mutex
+      (probably ok - except in some very rare cases, only consequence
+      is that value may take some time to display in
+      Seconds_Behind_Master - not critical).
+
+      In parallel replication, we take care to not set last_master_timestamp
+      backwards, in case of out-of-order calls here.
+    */
+    if (!(event_creation_time == 0 &&
+          IF_DBUG(debug_not_change_ts_if_art_event > 0, 1)) &&
+        !(rgi->is_parallel_exec && event_creation_time <= last_master_timestamp)
+        )
+        last_master_timestamp= event_creation_time;
   }
-  DBUG_RETURN(error);
+  DBUG_VOID_RETURN;
 }
 
-
-int
-Relay_log_info::alloc_inuse_relaylog(const char *name)
-{
-  inuse_relaylog *ir;
-  uint32 gtid_count;
-  rpl_gtid *gtid_list;
-
-  if (!(ir= (inuse_relaylog *)my_malloc(sizeof(*ir), MYF(MY_WME|MY_ZEROFILL))))
-  {
-    my_error(ER_OUTOFMEMORY, MYF(0), (int)sizeof(*ir));
-    return 1;
-  }
-  gtid_count= relay_log_state.count();
-  if (!(gtid_list= (rpl_gtid *)my_malloc(sizeof(*gtid_list)*gtid_count,
-                                         MYF(MY_WME))))
-  {
-    my_free(ir);
-    my_error(ER_OUTOFMEMORY, MYF(0), (int)sizeof(*gtid_list)*gtid_count);
-    return 1;
-  }
-  if (relay_log_state.get_gtid_list(gtid_list, gtid_count))
-  {
-    my_free(gtid_list);
-    my_free(ir);
-    DBUG_ASSERT(0 /* Should not be possible as we allocated correct length */);
-    my_error(ER_OUT_OF_RESOURCES, MYF(0));
-    return 1;
-  }
-  ir->rli= this;
-  strmake_buf(ir->name, name);
-  ir->relay_log_state= gtid_list;
-  ir->relay_log_state_count= gtid_count;
-
-  if (!inuse_relaylog_list)
-    inuse_relaylog_list= ir;
-  else
-  {
-    last_inuse_relaylog->completed= true;
-    last_inuse_relaylog->next= ir;
-  }
-  last_inuse_relaylog= ir;
-  my_atomic_rwlock_init(&ir->inuse_relaylog_atomic_lock);
-
-  return 0;
-}
-
-
-void
-Relay_log_info::free_inuse_relaylog(inuse_relaylog *ir)
-{
-  my_free(ir->relay_log_state);
-  my_atomic_rwlock_destroy(&ir->inuse_relaylog_atomic_lock);
-  my_free(ir);
-}
-
-
-void
-Relay_log_info::reset_inuse_relaylog()
-{
-  inuse_relaylog *cur= inuse_relaylog_list;
-  while (cur)
-  {
-    DBUG_ASSERT(cur->queued_count == cur->dequeued_count);
-    inuse_relaylog *next= cur->next;
-    free_inuse_relaylog(cur);
-    cur= next;
-  }
-  inuse_relaylog_list= last_inuse_relaylog= NULL;
-}
-
-
-int
-Relay_log_info::update_relay_log_state(rpl_gtid *gtid_list, uint32 count)
-{
-  int res= 0;
-  while (count)
-  {
-    if (relay_log_state.update_nolock(gtid_list, false))
-      res= 1;
-    ++gtid_list;
-    --count;
-  }
-  return res;
-}
-
-
 #if !defined(MYSQL_CLIENT) && defined(HAVE_REPLICATION)
 int
 rpl_load_gtid_slave_state(THD *thd)
@@ -1515,9 +1296,9 @@ rpl_load_gtid_slave_state(THD *thd)
   uint32 i;
   DBUG_ENTER("rpl_load_gtid_slave_state");
 
-  mysql_mutex_lock(&rpl_global_gtid_slave_state->LOCK_slave_state);
-  bool loaded= rpl_global_gtid_slave_state->loaded;
-  mysql_mutex_unlock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+  mysql_mutex_lock(&rpl_global_gtid_slave_state.LOCK_slave_state);
+  bool loaded= rpl_global_gtid_slave_state.loaded;
+  mysql_mutex_unlock(&rpl_global_gtid_slave_state.LOCK_slave_state);
   if (loaded)
     DBUG_RETURN(0);
 
@@ -1617,23 +1398,23 @@ rpl_load_gtid_slave_state(THD *thd)
     }
   }
 
-  mysql_mutex_lock(&rpl_global_gtid_slave_state->LOCK_slave_state);
-  if (rpl_global_gtid_slave_state->loaded)
+  mysql_mutex_lock(&rpl_global_gtid_slave_state.LOCK_slave_state);
+  if (rpl_global_gtid_slave_state.loaded)
   {
-    mysql_mutex_unlock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+    mysql_mutex_unlock(&rpl_global_gtid_slave_state.LOCK_slave_state);
     goto end;
   }
 
   for (i= 0; i < array.elements; ++i)
   {
     get_dynamic(&array, (uchar *)&tmp_entry, i);
-    if ((err= rpl_global_gtid_slave_state->update(tmp_entry.gtid.domain_id,
+    if ((err= rpl_global_gtid_slave_state.update(tmp_entry.gtid.domain_id,
                                                  tmp_entry.gtid.server_id,
                                                  tmp_entry.sub_id,
                                                  tmp_entry.gtid.seq_no,
                                                  NULL)))
     {
-      mysql_mutex_unlock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+      mysql_mutex_unlock(&rpl_global_gtid_slave_state.LOCK_slave_state);
       my_error(ER_OUT_OF_RESOURCES, MYF(0));
       goto end;
     }
@@ -1646,14 +1427,14 @@ rpl_load_gtid_slave_state(THD *thd)
         mysql_bin_log.bump_seq_no_counter_if_needed(entry->gtid.domain_id,
                                                     entry->gtid.seq_no))
     {
-      mysql_mutex_unlock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+      mysql_mutex_unlock(&rpl_global_gtid_slave_state.LOCK_slave_state);
       my_error(ER_OUT_OF_RESOURCES, MYF(0));
       goto end;
     }
   }
 
-  rpl_global_gtid_slave_state->loaded= true;
-  mysql_mutex_unlock(&rpl_global_gtid_slave_state->LOCK_slave_state);
+  rpl_global_gtid_slave_state.loaded= true;
+  mysql_mutex_unlock(&rpl_global_gtid_slave_state.LOCK_slave_state);
 
   err= 0;                                       /* Clear HA_ERR_END_OF_FILE */
 
@@ -1684,20 +1465,16 @@ rpl_group_info::reinit(Relay_log_info *rli)
   tables_to_lock_count= 0;
   trans_retries= 0;
   last_event_start_time= 0;
-  gtid_sub_id= 0;
-  commit_id= 0;
-  gtid_pending= false;
   worker_error= 0;
   row_stmt_start_timestamp= 0;
   long_find_row_note_printed= false;
   did_mark_start_commit= false;
-  last_master_timestamp = 0;
   gtid_ignore_duplicate_state= GTID_DUPLICATE_NULL;
   commit_orderer.reinit();
 }
 
 rpl_group_info::rpl_group_info(Relay_log_info *rli)
-  : thd(0), wait_commit_sub_id(0),
+  : thd(0), gtid_sub_id(0), wait_commit_sub_id(0),
     wait_commit_group_info(0), parallel_entry(0),
     deferred_events(NULL), m_annotate_event(0), is_parallel_exec(false)
 {
@@ -1721,18 +1498,16 @@ rpl_group_info::~rpl_group_info()
 int
 event_group_new_gtid(rpl_group_info *rgi, Gtid_log_event *gev)
 {
-  uint64 sub_id= rpl_global_gtid_slave_state->next_sub_id(gev->domain_id);
+  uint64 sub_id= rpl_global_gtid_slave_state.next_sub_id(gev->domain_id);
   if (!sub_id)
   {
     /* Out of memory caused hash insertion to fail. */
     return 1;
   }
   rgi->gtid_sub_id= sub_id;
-  rgi->current_gtid.domain_id= gev->domain_id;
   rgi->current_gtid.server_id= gev->server_id;
+  rgi->current_gtid.domain_id= gev->domain_id;
   rgi->current_gtid.seq_no= gev->seq_no;
-  rgi->commit_id= gev->commit_id;
-  rgi->gtid_pending= true;
   return 0;
 }
 
@@ -1788,7 +1563,7 @@ delete_or_keep_event_post_apply(rpl_group_info *rgi,
 
 void rpl_group_info::cleanup_context(THD *thd, bool error)
 {
-  DBUG_ENTER("rpl_group_info::cleanup_context");
+  DBUG_ENTER("Relay_log_info::cleanup_context");
   DBUG_PRINT("enter", ("error: %d", (int) error));
   
   DBUG_ASSERT(this->thd == thd);
@@ -1809,11 +1584,6 @@ void rpl_group_info::cleanup_context(THD *thd, bool error)
     trans_rollback_stmt(thd); // if a "statement transaction"
     /* trans_rollback() also resets OPTION_GTID_BEGIN */
     trans_rollback(thd);      // if a "real transaction"
-    /*
-      Now that we have rolled back the transaction, make sure we do not
-      erroneously update the GTID position.
-    */
-    gtid_pending= false;
   }
   m_table_map.clear_tables();
   slave_close_thread_tables(thd);
@@ -1830,13 +1600,6 @@ void rpl_group_info::cleanup_context(THD *thd, bool error)
       rli->clear_flag(Relay_log_info::IN_STMT);
       rli->clear_flag(Relay_log_info::IN_TRANSACTION);
     }
-
-    /*
-      Ensure we always release the domain for others to process, when using
-      --gtid-ignore-duplicates.
-    */
-    if (gtid_ignore_duplicate_state != GTID_DUPLICATE_NULL)
-      rpl_global_gtid_slave_state->release_domain_owner(this);
   }
 
   /*
@@ -1845,6 +1608,13 @@ void rpl_group_info::cleanup_context(THD *thd, bool error)
   thd->variables.option_bits&= ~OPTION_NO_FOREIGN_KEY_CHECKS;
   thd->variables.option_bits&= ~OPTION_RELAXED_UNIQUE_CHECKS;
 
+  /*
+    Ensure we always release the domain for others to process, when using
+    --gtid-ignore-duplicates.
+  */
+  if (gtid_ignore_duplicate_state != GTID_DUPLICATE_NULL)
+    rpl_global_gtid_slave_state.release_domain_owner(this);
+
   /*
     Reset state related to long_find_row notes in the error log:
     - timestamp
@@ -1853,18 +1623,13 @@ void rpl_group_info::cleanup_context(THD *thd, bool error)
   reset_row_stmt_start_timestamp();
   unset_long_find_row_note_printed();
 
-  DBUG_EXECUTE_IF("inject_sleep_gtid_100_x_x", {
-      if (current_gtid.domain_id == 100)
-        my_sleep(50000);
-    };);
-
   DBUG_VOID_RETURN;
 }
 
 
 void rpl_group_info::clear_tables_to_lock()
 {
-  DBUG_ENTER("rpl_group_info::clear_tables_to_lock()");
+  DBUG_ENTER("Relay_log_info::clear_tables_to_lock()");
 #ifndef DBUG_OFF
   /**
     When replicating in RBR and MyISAM Merge tables are involved
@@ -1911,7 +1676,7 @@ void rpl_group_info::clear_tables_to_lock()
 
 void rpl_group_info::slave_close_thread_tables(THD *thd)
 {
-  DBUG_ENTER("rpl_group_info::slave_close_thread_tables(THD *thd)");
+  DBUG_ENTER("Relay_log_info::slave_close_thread_tables(THD *thd)");
   thd->get_stmt_da()->set_overwrite_status(true);
   thd->is_error() ? trans_rollback_stmt(thd) : trans_commit_stmt(thd);
   thd->get_stmt_da()->set_overwrite_status(false);
@@ -1946,20 +1711,11 @@ void rpl_group_info::slave_close_thread_tables(THD *thd)
 
 
 static void
-mark_start_commit_inner(rpl_parallel_entry *e, group_commit_orderer *gco,
-                        rpl_group_info *rgi)
+mark_start_commit_inner(rpl_parallel_entry *e, group_commit_orderer *gco)
 {
-  group_commit_orderer *tmp;
   uint64 count= ++e->count_committing_event_groups;
-  /* Signal any following GCO whose wait_count has been reached now. */
-  tmp= gco;
-  while ((tmp= tmp->next_gco))
-  {
-    uint64 wait_count= tmp->wait_count;
-    if (wait_count > count)
-      break;
-    mysql_cond_broadcast(&tmp->COND_group_commit_orderer);
-  }
+  if (gco->next_gco && gco->next_gco->wait_count == count)
+    mysql_cond_broadcast(&gco->next_gco->COND_group_commit_orderer);
 }
 
 
@@ -1968,8 +1724,8 @@ rpl_group_info::mark_start_commit_no_lock()
 {
   if (did_mark_start_commit)
     return;
+  mark_start_commit_inner(parallel_entry, gco);
   did_mark_start_commit= true;
-  mark_start_commit_inner(parallel_entry, gco, this);
 }
 
 
@@ -1980,60 +1736,12 @@ rpl_group_info::mark_start_commit()
 
   if (did_mark_start_commit)
     return;
+
+  e= this->parallel_entry;
+  mysql_mutex_lock(&e->LOCK_parallel_entry);
+  mark_start_commit_inner(e, gco);
+  mysql_mutex_unlock(&e->LOCK_parallel_entry);
   did_mark_start_commit= true;
-
-  e= this->parallel_entry;
-  mysql_mutex_lock(&e->LOCK_parallel_entry);
-  mark_start_commit_inner(e, gco, this);
-  mysql_mutex_unlock(&e->LOCK_parallel_entry);
-}
-
-
-/*
-  Format the current GTID as a string suitable for printing in error messages.
-
-  The string is stored in a buffer inside rpl_group_info, so remains valid
-  until next call to gtid_info() or until destruction of rpl_group_info.
-
-  If no GTID is available, then NULL is returned.
-*/
-char *
-rpl_group_info::gtid_info()
-{
-  if (!gtid_sub_id || !current_gtid.seq_no)
-    return NULL;
-  my_snprintf(gtid_info_buf, sizeof(gtid_info_buf), "Gtid %u-%u-%llu",
-              current_gtid.domain_id, current_gtid.server_id,
-              current_gtid.seq_no);
-  return gtid_info_buf;
-}
-
-
-/*
-  Undo the effect of a prior mark_start_commit().
-
-  This is only used for retrying a transaction in parallel replication, after
-  we have encountered a deadlock or other temporary error.
-
-  When we get such a deadlock, it means that the current group of transactions
-  did not yet all start committing (else they would not have deadlocked). So
-  we will not yet have woken up anything in the next group, our rgi->gco is
-  still live, and we can simply decrement the counter (to be incremented again
-  later, when the retry succeeds and reaches the commit step).
-*/
-void
-rpl_group_info::unmark_start_commit()
-{
-  rpl_parallel_entry *e;
-
-  if (!did_mark_start_commit)
-    return;
-  did_mark_start_commit= false;
-
-  e= this->parallel_entry;
-  mysql_mutex_lock(&e->LOCK_parallel_entry);
-  --e->count_committing_event_groups;
-  mysql_mutex_unlock(&e->LOCK_parallel_entry);
 }
 
 
