@@ -14,7 +14,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
-#include <my_global.h>
 #include "rpl_utility.h"
 #include "log_event.h"
 
@@ -452,7 +451,7 @@ void show_sql_type(enum_field_types type, uint16 metadata, String *str, CHARSET_
       CHARSET_INFO *cs= str->charset();
       uint32 length=
         cs->cset->snprintf(cs, (char*) str->ptr(), str->alloced_length(),
-                           "decimal(%d,?)/*old*/", metadata);
+                           "decimal(%d,?)", metadata);
       str->length(length);
     }
     break;
@@ -827,7 +826,7 @@ can_convert_field_to(Field *field,
   @retval false Master table is not compatible with slave table.
 */
 bool
-table_def::compatible_with(THD *thd, rpl_group_info *rgi,
+table_def::compatible_with(THD *thd, Relay_log_info *rli,
                            TABLE *table, TABLE **conv_table_var)
   const
 {
@@ -835,7 +834,6 @@ table_def::compatible_with(THD *thd, rpl_group_info *rgi,
     We only check the initial columns for the tables.
   */
   uint const cols_to_check= MY_MIN(table->s->fields, size());
-  Relay_log_info *rli= rgi->rli;
   TABLE *tmp_table= NULL;
 
   for (uint col= 0 ; col < cols_to_check ; ++col)
@@ -859,7 +857,7 @@ table_def::compatible_with(THD *thd, rpl_group_info *rgi,
           This will create the full table with all fields. This is
           necessary to ge the correct field lengths for the record.
         */
-        tmp_table= create_conversion_table(thd, rgi, table);
+        tmp_table= create_conversion_table(thd, rli, table);
         if (tmp_table == NULL)
             return false;
         /*
@@ -887,7 +885,7 @@ table_def::compatible_with(THD *thd, rpl_group_info *rgi,
       String target_type(target_buf, sizeof(target_buf), &my_charset_latin1);
       show_sql_type(type(col), field_metadata(col), &source_type, field->charset());
       field->sql_type(target_type);
-      rli->report(ERROR_LEVEL, ER_SLAVE_CONVERSION_FAILED, rgi->gtid_info(),
+      rli->report(ERROR_LEVEL, ER_SLAVE_CONVERSION_FAILED,
                   ER(ER_SLAVE_CONVERSION_FAILED),
                   col, db_name, tbl_name,
                   source_type.c_ptr_safe(), target_type.c_ptr_safe());
@@ -929,14 +927,12 @@ table_def::compatible_with(THD *thd, rpl_group_info *rgi,
   conversion table.
  */
 
-TABLE *table_def::create_conversion_table(THD *thd, rpl_group_info *rgi,
-                                          TABLE *target_table) const
+TABLE *table_def::create_conversion_table(THD *thd, Relay_log_info *rli, TABLE *target_table) const
 {
   DBUG_ENTER("table_def::create_conversion_table");
 
   List<Create_field> field_list;
   TABLE *conv_table= NULL;
-  Relay_log_info *rli= rgi->rli;
   /*
     At slave, columns may differ. So we should create
     MY_MIN(columns@master, columns@slave) columns in the
@@ -947,7 +943,6 @@ TABLE *table_def::create_conversion_table(THD *thd, rpl_group_info *rgi,
   {
     Create_field *field_def=
       (Create_field*) alloc_root(thd->mem_root, sizeof(Create_field));
-    bool unsigned_flag= 0;
     if (field_list.push_back(field_def))
       DBUG_RETURN(NULL);
 
@@ -957,7 +952,8 @@ TABLE *table_def::create_conversion_table(THD *thd, rpl_group_info *rgi,
     uint32 max_length=
       max_display_length_for_field(type(col), field_metadata(col));
 
-    switch(type(col)) {
+    switch(type(col))
+    {
       int precision;
     case MYSQL_TYPE_ENUM:
     case MYSQL_TYPE_SET:
@@ -996,18 +992,6 @@ TABLE *table_def::create_conversion_table(THD *thd, rpl_group_info *rgi,
       pack_length= field_metadata(col) & 0x00ff;
       break;
 
-    case MYSQL_TYPE_TINY:
-    case MYSQL_TYPE_SHORT:
-    case MYSQL_TYPE_INT24:
-    case MYSQL_TYPE_LONG:
-    case MYSQL_TYPE_LONGLONG:
-      /*
-        As we don't know if the integer was signed or not on the master,
-        assume we have same sign on master and slave.  This is true when not
-        using conversions so it should be true also when using conversions.
-      */
-      unsigned_flag= ((Field_num*) target_table->field[col])->unsigned_flag;
-      break;
     default:
       break;
     }
@@ -1015,13 +999,12 @@ TABLE *table_def::create_conversion_table(THD *thd, rpl_group_info *rgi,
     DBUG_PRINT("debug", ("sql_type: %d, target_field: '%s', max_length: %d, decimals: %d,"
                          " maybe_null: %d, unsigned_flag: %d, pack_length: %u",
                          binlog_type(col), target_table->field[col]->field_name,
-                         max_length, decimals, TRUE, unsigned_flag,
-                         pack_length));
+                         max_length, decimals, TRUE, FALSE, pack_length));
     field_def->init_for_tmp_table(type(col),
                                   max_length,
                                   decimals,
                                   TRUE,         // maybe_null
-                                  unsigned_flag,
+                                  FALSE,        // unsigned_flag
                                   pack_length);
     field_def->charset= target_table->field[col]->charset();
     field_def->interval= interval;
@@ -1031,7 +1014,7 @@ TABLE *table_def::create_conversion_table(THD *thd, rpl_group_info *rgi,
 
 err:
   if (conv_table == NULL)
-    rli->report(ERROR_LEVEL, ER_SLAVE_CANT_CREATE_CONVERSION, rgi->gtid_info(),
+    rli->report(ERROR_LEVEL, ER_SLAVE_CANT_CREATE_CONVERSION,
                 ER(ER_SLAVE_CANT_CREATE_CONVERSION),
                 target_table->s->db.str,
                 target_table->s->table_name.str);
@@ -1187,7 +1170,7 @@ bool event_checksum_test(uchar *event_buf, ulong event_len, uint8 alg)
       compile_time_assert(BINLOG_CHECKSUM_ALG_ENUM_END <= 0x80);
     }
     incoming= uint4korr(event_buf + event_len - BINLOG_CHECKSUM_LEN);
-    computed= 0;
+    computed= my_checksum(0L, NULL, 0);
     /* checksum the event content but the checksum part itself */
     computed= my_checksum(computed, (const uchar*) event_buf, 
                           event_len - BINLOG_CHECKSUM_LEN);
