@@ -1,6 +1,5 @@
 /*
    Copyright (c) 2002, 2011, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2015, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +21,7 @@
 */
 
 
-#include <my_global.h>                         /* NO_EMBEDDED_ACCESS_CHECKS */
+#include "my_global.h"                          /* NO_EMBEDDED_ACCESS_CHECKS */
 #include "sql_priv.h"
 #include "unireg.h"
 #include "sql_derived.h"
@@ -170,9 +169,7 @@ mysql_handle_single_derived(LEX *lex, TABLE_LIST *derived, uint phases)
   uint8 allowed_phases= (derived->is_merged_derived() ? DT_PHASES_MERGE :
                          DT_PHASES_MATERIALIZE);
   DBUG_ENTER("mysql_handle_single_derived");
-  DBUG_PRINT("enter", ("phases: 0x%x  allowed: 0x%x  alias: '%s'",
-                       phases, allowed_phases,
-                       (derived->alias ? derived->alias : "<NULL>")));
+  DBUG_PRINT("enter", ("phases: 0x%x  allowed: 0x%x", phases, allowed_phases));
   if (!lex->derived_tables)
     DBUG_RETURN(FALSE);
 
@@ -362,16 +359,9 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
   SELECT_LEX *parent_lex= derived->select_lex;
   Query_arena *arena, backup;
   DBUG_ENTER("mysql_derived_merge");
-  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
-                       (derived->alias ? derived->alias : "<NULL>"),
-                       derived->get_unit()));
 
   if (derived->merged)
-  {
-
-    DBUG_PRINT("info", ("Irreversibly merged: exit"));
     DBUG_RETURN(FALSE);
-  }
 
   if (dt_select->uncacheable & UNCACHEABLE_RAND)
   {
@@ -454,9 +444,6 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
   {
     Item *expr= derived->on_expr;
     expr= and_conds(expr, dt_select->join ? dt_select->join->conds : 0);
-    if (expr)
-      expr->top_level_item();
-    
     if (expr && (derived->prep_on_expr || expr != derived->on_expr))
     {
       derived->on_expr= expr;
@@ -473,8 +460,7 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
     // Update used tables cache according to new table map
     if (derived->on_expr)
     {
-      derived->on_expr->fix_after_pullout(parent_lex, &derived->on_expr,
-                                          TRUE);
+      derived->on_expr->fix_after_pullout(parent_lex, &derived->on_expr);
       fix_list_after_tbl_changes(parent_lex, &derived->nested_join->join_list);
     }
   }
@@ -517,26 +503,10 @@ unconditional_materialization:
 bool mysql_derived_merge_for_insert(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
   DBUG_ENTER("mysql_derived_merge_for_insert");
-  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
-                       (derived->alias ? derived->alias : "<NULL>"),
-                       derived->get_unit()));
-  DBUG_PRINT("info", ("merged_for_insert: %d  is_materialized_derived: %d  "
-                      "is_multitable: %d  single_table_updatable: %d  "
-                      "merge_underlying_list: %d",
-                      derived->merged_for_insert,
-                      derived->is_materialized_derived(),
-                      derived->is_multitable(),
-                      derived->single_table_updatable(),
-                      derived->merge_underlying_list != 0));
   if (derived->merged_for_insert)
     DBUG_RETURN(FALSE);
-  if (derived->init_derived(thd, FALSE))
-    DBUG_RETURN(TRUE);
   if (derived->is_materialized_derived())
     DBUG_RETURN(mysql_derived_prepare(thd, lex, derived));
-  if ((thd->lex->sql_command == SQLCOM_UPDATE_MULTI ||
-       thd->lex->sql_command == SQLCOM_DELETE_MULTI))
-    DBUG_RETURN(FALSE);
   if (!derived->is_multitable())
   {
     if (!derived->single_table_updatable())
@@ -546,9 +516,8 @@ bool mysql_derived_merge_for_insert(THD *thd, LEX *lex, TABLE_LIST *derived)
       derived->table= derived->merge_underlying_list->table;
       derived->schema_table= derived->merge_underlying_list->schema_table;
       derived->merged_for_insert= TRUE;
-      DBUG_ASSERT(derived->table);
     }
-  }
+  }  
   DBUG_RETURN(FALSE);
 }
 
@@ -575,9 +544,6 @@ bool mysql_derived_init(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
   SELECT_LEX_UNIT *unit= derived->get_unit();
   DBUG_ENTER("mysql_derived_init");
-  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
-                       (derived->alias ? derived->alias : "<NULL>"),
-                       derived->get_unit()));
 
   // Skip already prepared views/DT
   if (!unit || unit->prepared)
@@ -648,9 +614,6 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
   SELECT_LEX_UNIT *unit= derived->get_unit();
   DBUG_ENTER("mysql_derived_prepare");
   bool res= FALSE;
-  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
-                       (derived->alias ? derived->alias : "<NULL>"),
-                       unit));
 
   // Skip already prepared views/DT
   if (!unit || unit->prepared ||
@@ -660,6 +623,9 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
           thd->lex->sql_command == SQLCOM_DELETE_MULTI))))
     DBUG_RETURN(FALSE);
 
+  Query_arena *arena, backup;
+  arena= thd->activate_stmt_arena_if_needed(&backup);
+
   SELECT_LEX *first_select= unit->first_select();
 
   /* prevent name resolving out of derived table */
@@ -682,19 +648,6 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
 
   unit->derived= derived;
 
-  /*
-    Above cascade call of prepare is important for PS protocol, but after it
-    is called we can check if we really need prepare for this derived
-  */
-  if (derived->merged)
-  {
-    DBUG_PRINT("info", ("Irreversibly merged: exit"));
-    DBUG_RETURN(FALSE);
-  }
-
-
-  derived->fill_me= FALSE;
-
   if (!(derived->derived_result= new select_union))
     DBUG_RETURN(TRUE); // out of memory
 
@@ -738,7 +691,6 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
   thd->create_tmp_table_for_derived= FALSE;
 
   derived->table= derived->derived_result->table;
-  DBUG_ASSERT(derived->table);
   if (derived->is_derived() && derived->is_merged_derived())
     first_select->mark_as_belong_to_derived(derived);
 
@@ -791,6 +743,8 @@ exit:
     if (derived->outer_join)
       table->maybe_null= 1;
   }
+  if (arena)
+    thd->restore_active_arena(arena, &backup);
   DBUG_RETURN(res);
 }
 
@@ -819,14 +773,6 @@ bool mysql_derived_optimize(THD *thd, LEX *lex, TABLE_LIST *derived)
 
   bool res= FALSE;
   DBUG_ENTER("mysql_derived_optimize");
-  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
-                       (derived->alias ? derived->alias : "<NULL>"),
-                       derived->get_unit()));
-  if (derived->merged)
-  {
-    DBUG_PRINT("info", ("Irreversibly merged: exit"));
-    DBUG_RETURN(FALSE);
-  }
 
   if (unit->optimized)
     DBUG_RETURN(FALSE);
@@ -892,9 +838,6 @@ err:
 bool mysql_derived_create(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
   DBUG_ENTER("mysql_derived_create");
-  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
-                       (derived->alias ? derived->alias : "<NULL>"),
-                       derived->get_unit()));
   TABLE *table= derived->table;
   SELECT_LEX_UNIT *unit= derived->get_unit();
 
@@ -944,13 +887,9 @@ bool mysql_derived_create(THD *thd, LEX *lex, TABLE_LIST *derived)
 
 bool mysql_derived_fill(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
-  Field_iterator_table field_iterator;
+  DBUG_ENTER("mysql_derived_fill");
   SELECT_LEX_UNIT *unit= derived->get_unit();
   bool res= FALSE;
-  DBUG_ENTER("mysql_derived_fill");
-  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
-                       (derived->alias ? derived->alias : "<NULL>"),
-                       derived->get_unit()));
 
   if (unit->executed && !unit->uncacheable && !unit->describe)
     DBUG_RETURN(FALSE);
@@ -990,27 +929,7 @@ bool mysql_derived_fill(THD *thd, LEX *lex, TABLE_LIST *derived)
     if (derived_result->flush())
       res= TRUE;
     unit->executed= TRUE;
-
-    if (derived->field_translation)
-    {
-      /* reset translation table to materialized table */
-      field_iterator.set_table(derived->table);
-      for (uint i= 0;
-           !field_iterator.end_of_fields();
-           field_iterator.next(), i= i + 1)
-      {
-        Item *item;
-
-        if (!(item= field_iterator.create_item(thd)))
-        {
-          res= TRUE;
-          break;
-        }
-        thd->change_item_tree(&derived->field_translation[i].item, item);
-      }
-    }
   }
-
   if (res || !lex->describe) 
     unit->cleanup();
   lex->current_select= save_current_select;
@@ -1039,12 +958,10 @@ bool mysql_derived_fill(THD *thd, LEX *lex, TABLE_LIST *derived)
 bool mysql_derived_reinit(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
   DBUG_ENTER("mysql_derived_reinit");
-  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
-                       (derived->alias ? derived->alias : "<NULL>"),
-                       derived->get_unit()));
   st_select_lex_unit *unit= derived->get_unit();
 
-  derived->merged_for_insert= FALSE;
+  if (derived->table)
+    derived->merged_for_insert= FALSE;
   unit->unclean();
   unit->types.empty();
   /* for derived tables & PS (which can't be reset by Item_subquery) */
@@ -1052,3 +969,4 @@ bool mysql_derived_reinit(THD *thd, LEX *lex, TABLE_LIST *derived)
   unit->set_thd(thd);
   DBUG_RETURN(FALSE);
 }
+
