@@ -112,7 +112,7 @@ public:
   {
     (void) ptr_arg;
     (void) size;
-    TRASH_FREE(ptr_arg, size);
+    TRASH(ptr_arg, size);
   }
   static void operator delete(void *, MEM_ROOT *)
   { /* never called */ }
@@ -136,7 +136,6 @@ public:
   inline bool is_empty() const { return (str_length == 0); }
   inline void mark_as_const() { Alloced_length= 0;}
   inline const char *ptr() const { return Ptr; }
-  inline const char *end() const { return Ptr + str_length; }
   inline char *c_ptr()
   {
     DBUG_ASSERT(!alloced || !Ptr || !Alloced_length || 
@@ -254,9 +253,7 @@ public:
   */
   inline void chop()
   {
-    str_length--;
-    Ptr[str_length]= '\0';
-    DBUG_ASSERT(strlen(Ptr) == str_length);
+    Ptr[str_length--]= '\0'; 
   }
 
   inline void free()
@@ -354,10 +351,6 @@ public:
   bool set_or_copy_aligned(const char *s, uint32 arg_length, CHARSET_INFO *cs);
   bool copy(const char*s,uint32 arg_length, CHARSET_INFO *csfrom,
 	    CHARSET_INFO *csto, uint *errors);
-  bool copy(const String *str, CHARSET_INFO *tocs, uint *errors)
-  {
-    return copy(str->ptr(), str->length(), str->charset(), tocs, errors);
-  }
   void move(String &s)
   {
     free();
@@ -414,7 +407,7 @@ public:
   friend int stringcmp(const String *a,const String *b);
   friend String *copy_if_not_alloced(String *a,String *b,uint32 arg_length);
   friend class Field;
-  uint32 numchars() const;
+  uint32 numchars();
   int charpos(longlong i,uint32 offset=0);
 
   int reserve(uint32 space_needed)
@@ -505,7 +498,7 @@ public:
     str_length+= arg_length;
     return FALSE;
   }
-  void print(String *print) const;
+  void print(String *print);
 
   bool append_for_single_quote(const char *st, uint len);
   bool append_for_single_quote(const String *s)
@@ -524,12 +517,6 @@ public:
   {
     return (s->alloced && Ptr >= s->Ptr && Ptr < s->Ptr + s->str_length);
   }
-  uint well_formed_length() const
-  {
-    int dummy_error;
-    return charset()->cset->well_formed_len(charset(), ptr(), ptr() + length(),
-                                            length(), &dummy_error);
-  }
   bool is_ascii() const
   {
     if (length() == 0)
@@ -543,15 +530,6 @@ public:
     }
     return TRUE;
   }
-  bool bin_eq(const String *other) const
-  {
-    return length() == other->length() &&
-           !memcmp(ptr(), other->ptr(), length());
-  }
-  bool eq(const String *other, CHARSET_INFO *cs) const
-  {
-    return !sortcmp(this, other, cs);
-  }
 };
 
 
