@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2000, 2011, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2018, MariaDB Corporation
+   Copyright (c) 2009, 2013, Monty Program Ab.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
     str is a (long) to record position where 0 is the first position.
 */
 
-#include <my_global.h>
 #include "sql_priv.h"
 #include "unireg.h"
 #include "sql_partition.h"                      // struct partition_info
@@ -33,6 +32,7 @@
 #include "create_options.h"
 #include "discover.h"
 #include <m_ctype.h>
+#include <assert.h>
 
 #define FCOMP			17		/* Bytes for a packed field */
 
@@ -83,7 +83,7 @@ static uchar *extra2_write(uchar *pos, enum extra2_frm_value_type type,
   return extra2_write(pos, type, reinterpret_cast<LEX_STRING *>(str));
 }
 
-/*
+/**
   Create a frm (table definition) file
 
   @param thd                    Thread handler
@@ -156,8 +156,7 @@ LEX_CUSTRING build_frm_image(THD *thd, const char *table,
   DBUG_PRINT("info", ("Options length: %u", options_len));
 
   if (validate_comment_length(thd, &create_info->comment, TABLE_COMMENT_MAXLEN,
-                              ER_TOO_LONG_TABLE_COMMENT,
-                              table))
+                              ER_TOO_LONG_TABLE_COMMENT, table))
      DBUG_RETURN(frm);
   /*
     If table comment is longer than TABLE_COMMENT_INLINE_MAXLEN bytes,
@@ -212,12 +211,6 @@ LEX_CUSTRING build_frm_image(THD *thd, const char *table,
   filepos= frm.length;
   frm.length+= FRM_FORMINFO_SIZE;               // forminfo
   frm.length+= packed_fields_length(create_fields);
-
-  if (frm.length > FRM_MAX_SIZE)
-  {
-    my_error(ER_TABLE_DEFINITION_TOO_BIG, MYF(0), table);
-    DBUG_RETURN(frm);
-  }
   
   frm_ptr= (uchar*) my_malloc(frm.length, MYF(MY_WME | MY_ZEROFILL |
                                               MY_THREAD_SPECIFIC));
@@ -247,14 +240,6 @@ LEX_CUSTRING build_frm_image(THD *thd, const char *table,
 
   DBUG_ASSERT(pos == frm_ptr + uint2korr(fileinfo+6));
   key_info_length= pack_keys(pos, keys, key_info, data_offset);
-  if (key_info_length > UINT_MAX16)
-  {
-    my_printf_error(ER_CANT_CREATE_TABLE,
-                    "Cannot create table %`s: index information is too long. "
-                    "Decrease number of indexes or use shorter index names or shorter comments.",
-                    MYF(0), table);
-    goto err;
-  }
 
   int2store(forminfo+2, frm.length - filepos);
   int4store(fileinfo+10, frm.length);
