@@ -1,11 +1,11 @@
 /*********** File AM Dbf C++ Program Source Code File (.CPP) ****************/
 /* PROGRAM NAME: FILAMDBF                                                   */
 /* -------------                                                            */
-/*  Version 1.8                                                             */
+/*  Version 1.6                                                             */
 /*                                                                          */
 /* COPYRIGHT:                                                               */
 /* ----------                                                               */
-/*  (C) Copyright to the author Olivier BERTRAND          2005-2017         */
+/*  (C) Copyright to the author Olivier BERTRAND          2005-2013         */
 /*                                                                          */
 /* WHAT THIS PROGRAM DOES:                                                  */
 /* -----------------------                                                  */
@@ -22,12 +22,12 @@
 /*  Include relevant sections of the System header files.              */
 /***********************************************************************/
 #include "my_global.h"
-#if defined(__WIN__)
+#if defined(WIN32)
 #include <io.h>
 #include <fcntl.h>
 //#include <errno.h>
 //#include <windows.h>
-#else   // !__WIN__
+#else   // !WIN32
 #if defined(UNIX)
 #include <errno.h>
 #include <unistd.h>
@@ -35,7 +35,7 @@
 //#include <io.h>
 #endif  // !UNIX
 //#include <fcntl.h>
-#endif  // !__WIN__
+#endif  // !WIN32
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
@@ -63,6 +63,8 @@
 #define DBFTYPE        3            /* value of bits 0 and 1 if .dbf        */
 #define EOH         0x0D            /* end-of-header marker in .dbf file    */
 
+extern "C" int trace;                // The general trace value  
+
 /****************************************************************************/
 /*  First 32 bytes of a .dbf file.                                          */
 /*  Note: some reserved fields are used here to store info (Fields)         */
@@ -74,28 +76,16 @@ typedef struct _dbfheader {
 //uchar  Dbfox :4;                  /*  FoxPro if equal to 3                */
   uchar  Version;                   /*  Version information flags           */
   char   Filedate[3];               /*  date, YYMMDD, binary. YY=year-1900  */
- private:
-  /* The following four members are stored in little-endian format on disk */
-  char   m_RecordsBuf[4];           /*  records in the file                 */
-  char   m_HeadlenBuf[2];           /*  bytes in the header                 */
-  char   m_ReclenBuf[2];            /*  bytes in a record                   */
-  char   m_FieldsBuf[2];            /*  Reserved but used to store fields   */
- public:
+  uint   Records;                   /*  records in the file                 */
+  ushort Headlen;                   /*  bytes in the header                 */
+  ushort Reclen;                    /*  bytes in a record                   */
+  ushort Fields;                    /*  Reserved but used to store fields   */
   char   Incompleteflag;            /*  01 if incomplete, else 00           */
   char   Encryptflag;               /*  01 if encrypted, else 00            */
   char   Reserved2[12];             /*  for LAN use                         */
   char   Mdxflag;                   /*  01 if production .mdx, else 00      */
   char   Language;                  /*  Codepage                            */
   char   Reserved3[2];
-
-  uint   Records(void) const {return uint4korr(m_RecordsBuf);}
-  ushort Headlen(void) const {return uint2korr(m_HeadlenBuf);}
-  ushort Reclen(void)  const {return uint2korr(m_ReclenBuf);}
-  ushort Fields(void)  const {return uint2korr(m_FieldsBuf);}
-
-  void   SetHeadlen(ushort num) {int2store(m_HeadlenBuf, num);}
-  void   SetReclen(ushort num)  {int2store(m_ReclenBuf, num);}
-  void   SetFields(ushort num)  {int2store(m_FieldsBuf, num);}
   } DBFHEADER;
 
 /****************************************************************************/
@@ -155,7 +145,7 @@ static int dbfhead(PGLOBAL g, FILE *file, PSZ fn, DBFHEADER *buf)
     strcpy(g->Message, MSG(DBASE_FILE));
 
   // Check last byte(s) of header
-  if (fseek(file, buf->Headlen() - dbc, SEEK_SET) != 0) {
+  if (fseek(file, buf->Headlen - dbc, SEEK_SET) != 0) {
     sprintf(g->Message, MSG(BAD_HEADER), fn);
     return RC_FX;
     } // endif fseek
@@ -175,7 +165,7 @@ static int dbfhead(PGLOBAL g, FILE *file, PSZ fn, DBFHEADER *buf)
     } // endif endmark
 
   // Calculate here the number of fields while we have the dbc info
-  buf->SetFields((buf->Headlen() - dbc - 1) / 32);
+  buf->Fields = (buf->Headlen - dbc - 1) / 32;
   fseek(file, HEADLEN, SEEK_SET);
   return rc;
   } // end of dbfhead
@@ -186,7 +176,7 @@ static int dbfhead(PGLOBAL g, FILE *file, PSZ fn, DBFHEADER *buf)
 /*  DBFColumns: constructs the result blocks containing the description     */
 /*  of all the columns of a DBF file that will be retrieved by #GetData.    */
 /****************************************************************************/
-PQRYRES DBFColumns(PGLOBAL g, char *dp, const char *fn, bool info)
+PQRYRES DBFColumns(PGLOBAL g, const char *fn, BOOL info)
   {
   int  buftyp[] = {TYPE_STRING, TYPE_SHORT, TYPE_STRING,
                    TYPE_INT,    TYPE_INT,   TYPE_SHORT};
@@ -196,7 +186,7 @@ PQRYRES DBFColumns(PGLOBAL g, char *dp, const char *fn, bool info)
   char       buf[2], filename[_MAX_PATH];
   int        ncol = sizeof(buftyp) / sizeof(int);
   int        rc, type, len, field, fields;
-  bool       bad;
+  BOOL       bad;
   DBFHEADER  mainhead;
   DESCRIPTOR thisfield;
   FILE      *infile = NULL;
@@ -215,7 +205,7 @@ PQRYRES DBFColumns(PGLOBAL g, char *dp, const char *fn, bool info)
     /************************************************************************/
     /*  Open the input file.                                                */
     /************************************************************************/
-    PlugSetPath(filename, fn, dp);
+    PlugSetPath(filename, fn, PlgGetDataPath(g));
 
     if (!(infile= global_fopen(g, MSGID_CANNOT_OPEN, filename, "rb")))
       return NULL;
@@ -231,7 +221,7 @@ PQRYRES DBFColumns(PGLOBAL g, char *dp, const char *fn, bool info)
     /************************************************************************/
     /*  Allocate the structures used to refer to the result set.            */
     /************************************************************************/
-    fields = mainhead.Fields();
+    fields = mainhead.Fields;
   } else
     fields = 0;
 
@@ -248,11 +238,11 @@ PQRYRES DBFColumns(PGLOBAL g, char *dp, const char *fn, bool info)
   if (trace) {
     htrc("Structure of %s\n", filename);
     htrc("headlen=%hd reclen=%hd degree=%d\n",
-          mainhead.Headlen(), mainhead.Reclen(), fields);
+          mainhead.Headlen, mainhead.Reclen, fields);
     htrc("flags(iem)=%d,%d,%d cp=%d\n", mainhead.Incompleteflag,
           mainhead.Encryptflag, mainhead.Mdxflag, mainhead.Language);
     htrc("%hd records, last changed %02d/%02d/%d\n",
-          mainhead.Records(), mainhead.Filedate[1], mainhead.Filedate[2],
+          mainhead.Records, mainhead.Filedate[1], mainhead.Filedate[2],
           mainhead.Filedate[0] + (mainhead.Filedate[0] <= 30) ? 2000 : 1900);
     htrc("Field    Type  Offset  Len  Dec  Set  Mdx\n");
     } // endif trace
@@ -281,33 +271,22 @@ PQRYRES DBFColumns(PGLOBAL g, char *dp, const char *fn, bool info)
     /************************************************************************/
     switch (thisfield.Type) {
       case 'C':                      // Characters
-      case 'L':                      // Logical 'T' or 'F' or space
-				type = TYPE_STRING;
-				break;
-			case 'M':                      // Memo		a .DBT block number
-			case 'B':                      // Binary	a .DBT block number
-			case 'G':                      // Ole			a .DBT block number
-				type = TYPE_STRING;
+      case 'L':                      // Logical 'T' or 'F'
+        type = TYPE_STRING;
         break;
-			//case 'I':											 // Long
-			//case '+':											 // Autoincrement
-			//	type = TYPE_INT;
-			//	break;
       case 'N':
         type = (thisfield.Decimals) ? TYPE_DOUBLE
              : (len > 10) ? TYPE_BIGINT : TYPE_INT;
         break;
-      case 'F':											 // Float
-			//case 'O':											 // Double
-				type = TYPE_DOUBLE;
+      case 'F':
+        type = TYPE_DOUBLE;
         break;
       case 'D':
         type = TYPE_DATE;            // Is this correct ???
         break;
       default:
         if (!info) {
-          sprintf(g->Message, MSG(BAD_DBF_TYPE), thisfield.Type
-                                               , thisfield.Name);
+          sprintf(g->Message, MSG(BAD_DBF_TYPE), thisfield.Type);
           goto err;
           } // endif info
 
@@ -393,7 +372,7 @@ DBFBASE::DBFBASE(DBFBASE *txfp)
 /*  and header length. Set Records, check that Reclen is equal to lrecl and */
 /*  return the header length or 0 in case of error.                         */
 /****************************************************************************/
-int DBFBASE::ScanHeader(PGLOBAL g, PSZ fn, int lrecl, int *rln, char *defpath)
+int DBFBASE::ScanHeader(PGLOBAL g, PSZ fname, int lrecl, char *defpath)
   {
   int       rc;
   char      filename[_MAX_PATH];
@@ -403,7 +382,7 @@ int DBFBASE::ScanHeader(PGLOBAL g, PSZ fn, int lrecl, int *rln, char *defpath)
   /************************************************************************/
   /*  Open the input file.                                                */
   /************************************************************************/
-  PlugSetPath(filename, fn, defpath);
+  PlugSetPath(filename, fname, defpath);
 
   if (!(infile= global_fopen(g, MSGID_CANNOT_OPEN, filename, "rb")))
     return 0;              // Assume file does not exist
@@ -420,9 +399,13 @@ int DBFBASE::ScanHeader(PGLOBAL g, PSZ fn, int lrecl, int *rln, char *defpath)
   } else if (rc == RC_FX)
     return -1;
 
-	*rln = (int)header.Reclen();
-  Records = (int)header.Records();
-  return (int)header.Headlen();
+  if ((int)header.Reclen != lrecl) {
+    sprintf(g->Message, MSG(BAD_LRECL), lrecl, header.Reclen);
+    return -1;
+    } // endif Lrecl
+
+  Records = (int)header.Records;
+  return (int)header.Headlen;
   } // end of ScanHeader
 
 /* ---------------------------- Class DBFFAM ------------------------------ */
@@ -437,28 +420,9 @@ int DBFFAM::Cardinality(PGLOBAL g)
   if (!g)
     return 1;
 
-	if (!Headlen) {
-		int rln = 0;								// Record length in the file header
-
-		Headlen = ScanHeader(g, To_File, Lrecl, &rln, Tdbp->GetPath());
-
-		if (Headlen < 0)
-			return -1;                // Error in ScanHeader
-
-		if (rln && Lrecl != rln) {
-			// This happens always on some Linux platforms
-			sprintf(g->Message, MSG(BAD_LRECL), Lrecl, rln);
-
-			if (Accept) {
-				Lrecl = rln;
-				Blksize = Nrec * rln;
-				PushWarning(g, Tdbp);
-			} else
-				return -1;
-
-		} // endif rln
-
-	}	// endif Headlen
+  if (!Headlen)
+    if ((Headlen = ScanHeader(g, To_File, Lrecl, Tdbp->GetPath())) < 0)
+      return -1;                // Error in ScanHeader
 
   // Set number of blocks for later use
   Block = (Records > 0) ? (Records + Nrec - 1) / Nrec : 0;
@@ -553,7 +517,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
   To_Buf = (char*)PlugSubAlloc(g, NULL, Buflen);
 
   if (mode == MODE_INSERT) {
-#if defined(__WIN__)
+#if defined(WIN32)
     /************************************************************************/
     /*  Now we can revert to binary mode in particular because the eventual */
     /*  writing of a new header must be done in binary mode to avoid        */
@@ -563,7 +527,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
       sprintf(g->Message, MSG(BIN_MODE_FAIL), strerror(errno));
       return true;
       } // endif setmode
-#endif   // __WIN__
+#endif   // WIN32
 
     /************************************************************************/
     /*  If this is a new file, the header must be generated.                */
@@ -590,14 +554,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
 
       if (Lrecl != reclen) {
         sprintf(g->Message, MSG(BAD_LRECL), Lrecl, reclen);
-
-				if (Accept) {
-					Lrecl = reclen;
-					Blksize = Nrec * Lrecl;
-					PushWarning(g, Tdbp);
-				}	else
-					return true;
-
+        return true;
         } // endif Lrecl
 
       hlen = HEADLEN * (n + 1) + 2;
@@ -609,8 +566,8 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
       header->Filedate[0] = datm->tm_year - 100;
       header->Filedate[1] = datm->tm_mon + 1;
       header->Filedate[2] = datm->tm_mday;
-      header->SetHeadlen((ushort)hlen);
-      header->SetReclen(reclen);
+      header->Headlen = (ushort)hlen;
+      header->Reclen = (ushort)reclen;
       descp = (DESCRIPTOR*)header;
 
       // Currently only standard Xbase types are supported
@@ -630,7 +587,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
             case 'D':           // Date
               break;
             default:            // Should never happen
-              sprintf(g->Message, MSG(BAD_DBF_TYPE),
+              sprintf(g->Message, "Unsupported DBF type %c for column %s",
                                   c, cdp->GetName());
               return true;
             } // endswitch c
@@ -671,20 +628,13 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
     DBFHEADER header;
 
     if ((rc = dbfhead(g, Stream, Tdbp->GetFile(g), &header)) == RC_OK) {
-      if (Lrecl != (int)header.Reclen()) {
-        sprintf(g->Message, MSG(BAD_LRECL), Lrecl, header.Reclen());
+      if (Lrecl != (int)header.Reclen) {
+        sprintf(g->Message, MSG(BAD_LRECL), Lrecl, header.Reclen);
+        return true;
+        } // endif Lrecl
 
-				if (Accept) {
-					Lrecl = header.Reclen();
-					Blksize = Nrec * Lrecl;
-					PushWarning(g, Tdbp);
-				} else
-					return true;
-
-			} // endif Lrecl
-
-      Records = (int)header.Records();
-      Headlen = (int)header.Headlen();
+      Records = (int)header.Records;
+      Headlen = (int)header.Headlen;
     } else if (rc == RC_NF) {
       Records = 0;
       Headlen = 0;
@@ -718,9 +668,12 @@ void DBFFAM::ResetBuffer(PGLOBAL g)
   /*********************************************************************/
   /*  If access is random, performances can be much better when the    */
   /*  reads are done on only one row, except for small tables that can */
-  /*  be entirely read in one block.                                   */
+  /*  be entirely read in one block. If the index is just used as a    */
+  /*  bitmap filter, as for Update or delete, reading will be          */
+  /*  sequential and we better keep block reading.                     */
   /*********************************************************************/
-  if (Tdbp->GetKindex() && ReadBlks != 1) {
+  if (Tdbp->GetKindex() && Tdbp->GetMode() == MODE_READ &&
+      ReadBlks != 1) {
     Nrec = 1;                       // Better for random access
     Rbuf = 0;
     Blksize = Lrecl;
@@ -800,36 +753,6 @@ bool DBFFAM::CopyHeader(PGLOBAL g)
   return rc;
   } // end of CopyHeader
 
-#if 0 // Not useful when UseTemp is false.
-/***********************************************************************/
-/*  Mark the line to delete with '*' (soft delete).                    */
-/*  NOTE: this is not ready for UseTemp.                               */
-/***********************************************************************/
-int DBFFAM::InitDelete(PGLOBAL g, int fpos, int spos)
-  {
-  int rc = RC_FX;
-  size_t lrecl = (size_t)Lrecl;
-
-  if (Nrec != 1)
-    strcpy(g->Message, "Cannot delete in block mode");
-  else if (fseek(Stream, Headlen + fpos * Lrecl, SEEK_SET))
-    sprintf(g->Message, MSG(FSETPOS_ERROR), 0);
-  else if (fread(To_Buf, 1, lrecl, Stream) != lrecl)
-    sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(errno));
-  else
-    *To_Buf = '*';
-
-  if (fseek(Stream, Headlen + fpos * Lrecl, SEEK_SET))
-    sprintf(g->Message, MSG(FSETPOS_ERROR), 0);
-  else if (fwrite(To_Buf, 1, lrecl, Stream) != lrecl)
-    sprintf(g->Message, MSG(FWRITE_ERROR), strerror(errno));
-  else
-    rc = RC_NF;     // Ok, Nothing else to do 
-
-  return rc;
-  } // end of InitDelete
-#endif // 0
-
 /***********************************************************************/
 /*  Data Base delete line routine for DBF access methods.              */
 /*  Deleted lines are just flagged in the first buffer character.      */
@@ -868,13 +791,11 @@ void DBFFAM::Rewind(void)
 /***********************************************************************/
 /*  Table file close routine for DBF access method.                    */
 /***********************************************************************/
-void DBFFAM::CloseTableFile(PGLOBAL g, bool abort)
+void DBFFAM::CloseTableFile(PGLOBAL g)
   {
   int rc = RC_OK, wrc = RC_OK;
   MODE mode = Tdbp->GetMode();
 
-  Abort = abort;
-
   // Closing is True if last Write was in error
   if (mode == MODE_INSERT && CurNum && !Closing) {
     // Some more inserted lines remain to be written
@@ -885,21 +806,21 @@ void DBFFAM::CloseTableFile(PGLOBAL g, bool abort)
     if (Modif && !Closing) {
       // Last updated block remains to be written
       Closing = true;
-      wrc = WriteModifiedBlock(g);
+      wrc = ReadBuffer(g);
       } // endif Modif
 
     if (UseTemp && T_Stream && wrc == RC_OK) {
-      if (!Abort) {
-        // Copy any remaining lines
-        bool b;
-    
-        Fpos = Tdbp->Cardinality(g);
-        Abort = MoveIntermediateLines(g, &b) != RC_OK;
-        } // endif Abort
+      // Copy any remaining lines
+      bool b;
+
+      Fpos = Tdbp->Cardinality(g);
+
+      if ((rc = MoveIntermediateLines(g, &b)) == RC_OK) {
+        // Delete the old file and rename the new temp file.
+        RenameTempFile(g);
+        goto fin;
+        } // endif rc
 
-      // Delete the old file and rename the new temp file.
-      RenameTempFile(g);
-      goto fin;
       } // endif UseTemp
 
   } // endif's mode
@@ -919,11 +840,8 @@ void DBFFAM::CloseTableFile(PGLOBAL g, bool abort)
         PlugSetPath(filename, To_File, Tdbp->GetPath());
         if ((Stream= global_fopen(g, MSGID_OPEN_MODE_STRERROR, filename, "r+b")))
         {
-          char nRecords[4];
-          int4store(nRecords, n);
-
           fseek(Stream, 4, SEEK_SET);     // Get header.Records position
-          fwrite(nRecords, sizeof(nRecords), 1, Stream);
+          fwrite(&n, sizeof(int), 1, Stream);
           fclose(Stream);
           Stream= NULL;
           Records= n;                    // Update Records value
@@ -955,28 +873,9 @@ int DBMFAM::Cardinality(PGLOBAL g)
   if (!g)
     return 1;
 
-	if (!Headlen) {
-		int rln = 0;								// Record length in the file header
-
-		Headlen = ScanHeader(g, To_File, Lrecl, &rln, Tdbp->GetPath());
-
-		if (Headlen < 0)
-			return -1;                // Error in ScanHeader
-
-		if (rln && Lrecl != rln) {
-			// This happens always on some Linux platforms
-			sprintf(g->Message, MSG(BAD_LRECL), Lrecl, rln);
-
-			if (Accept) {
-				Lrecl = rln;
-				Blksize = Nrec * Lrecl;
-				PushWarning(g, Tdbp);
-			} else
-				return -1;
-
-		} // endif rln
-
-	}	// endif Headlen
+  if (!Headlen)
+    if ((Headlen = ScanHeader(g, To_File, Lrecl, Tdbp->GetPath())) < 0)
+      return -1;                // Error in ScanHeader
 
   // Set number of blocks for later use
   Block = (Records > 0) ? (Records + Nrec - 1) / Nrec : 0;
@@ -1017,20 +916,13 @@ bool DBMFAM::AllocateBuffer(PGLOBAL g)
     /************************************************************************/
     DBFHEADER *hp = (DBFHEADER*)Memory;
 
-    if (Lrecl != (int)hp->Reclen()) {
-      sprintf(g->Message, MSG(BAD_LRECL), Lrecl, hp->Reclen());
+    if (Lrecl != (int)hp->Reclen) {
+      sprintf(g->Message, MSG(BAD_LRECL), Lrecl, hp->Reclen);
+      return true;
+      } // endif Lrecl
 
-			if (Accept) {
-				Lrecl = hp->Reclen();
-				Blksize = Nrec * Lrecl;
-				PushWarning(g, Tdbp);
-			} else
-				return true;
-
-		} // endif Lrecl
-
-    Records = (int)hp->Records();
-    Headlen = (int)hp->Headlen();
+    Records = (int)hp->Records;
+    Headlen = (int)hp->Headlen;
     } // endif Headlen
 
   /**************************************************************************/
