@@ -1,4 +1,4 @@
-/* Copyright (C) MariaDB Corporation Ab
+/* Copyright (C) Olivier Bertrand 2004 - 2013
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -11,10 +11,9 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /** @file ha_connect.h
-	Author Olivier Bertrand
 
     @brief
   The ha_connect engine is a prototype storage engine to access external data.
@@ -27,11 +26,6 @@
 #pragma interface     /* gcc class implementation */
 #endif
 
-/****************************************************************************/
-/*  mycat.h contains the TOS, PTOS, ha_table_option_struct declarations.    */
-/****************************************************************************/
-#include "mycat.h"
-
 /****************************************************************************/
 /*  Structures used to pass info between CONNECT and ha_connect.            */
 /****************************************************************************/
@@ -56,11 +50,15 @@ typedef struct _xinfo {
 
 class XCHK : public BLOCK {
 public:
-  XCHK(void) {oldsep= newsep= false;
+  XCHK(void) {oldsep= newsep= false; 
               oldopn= newopn= NULL;
               oldpix= newpix= NULL;}
 
-  inline char *SetName(PGLOBAL g, PCSZ name) {return PlugDup(g, name);}
+  inline char *SetName(PGLOBAL g, char *name) {
+    char *nm= NULL;
+    if (name) {nm= (char*)PlugSubAlloc(g, NULL, strlen(name) + 1);
+               strcpy(nm, name);}
+    return nm;}
 
   bool         oldsep;              // Sepindex before create/alter
   bool         newsep;              // Sepindex after create/alter
@@ -72,8 +70,8 @@ public:
 
 typedef class XCHK *PCHK;
 typedef class user_connect *PCONNECT;
-typedef struct ha_field_option_struct FOS, *PFOS;
-typedef struct ha_index_option_struct XOS, *PXOS;
+typedef struct ha_table_option_struct TOS, *PTOS;
+typedef struct ha_field_option_struct FOS, *PFOS; 
 
 extern handlerton *connect_hton;
 
@@ -82,9 +80,38 @@ extern handlerton *connect_hton;
 
   These can be specified in the CREATE TABLE:
   CREATE TABLE ( ... ) {...here...}
-
-	------ Was moved to mycat.h ------
-	*/
+*/
+struct ha_table_option_struct {
+  const char *type;
+  const char *filename;
+  const char *optname;
+  const char *tabname;
+  const char *tablist;
+  const char *dbname;
+  const char *separator;
+//const char *connect;
+  const char *qchar;
+  const char *module;
+  const char *subtype;
+  const char *catfunc;
+  const char *srcdef;
+  const char *colist;
+  const char *oplist;
+  const char *data_charset;
+  ulonglong lrecl;
+  ulonglong elements;
+//ulonglong estimate;
+  ulonglong multiple;
+  ulonglong header;
+  ulonglong quoted;
+  ulonglong ending;
+  ulonglong compressed;
+  bool mapped;
+  bool huge;
+  bool split;
+  bool readonly;
+  bool sepindex;
+  };
 
 /**
   structure for CREATE TABLE options (field options)
@@ -95,27 +122,12 @@ extern handlerton *connect_hton;
 struct ha_field_option_struct
 {
   ulonglong offset;
-  ulonglong freq;
   ulonglong fldlen;
-  uint opt;
   const char *dateformat;
   const char *fieldformat;
   char *special;
 };
 
-/*
-  index options can be declared similarly
-  using the ha_index_option_struct structure.
-
-  Their values can be specified in the CREATE TABLE per index:
-  CREATE TABLE ( field ..., .., INDEX .... *here*, ... )
-*/
-struct ha_index_option_struct
-{
-  bool dynamic;
-  bool mapped;
-};
-
 /** @brief
   CONNECT_SHARE is a structure that will be shared among all open handlers.
   This example implements the minimum of what you will probably need.
@@ -146,74 +158,48 @@ class ha_connect: public handler
   CONNECT_SHARE *share;        ///< Shared lock info
   CONNECT_SHARE *get_share();
 
-protected:
-  char *PlugSubAllocStr(PGLOBAL g, void *memp, const char *str, size_t length)
-  {
-    char *ptr= (char*)PlgDBSubAlloc(g, memp, length + 1);
-
-    if (ptr) {
-      memcpy(ptr, str, length);
-      ptr[length]= '\0';
-      } // endif ptr
-
-    return ptr;
-  } // end of PlugSubAllocStr
-
 public:
   ha_connect(handlerton *hton, TABLE_SHARE *table_arg);
   ~ha_connect();
 
   // CONNECT Implementation
-//static   bool connect_init(void);
-//static   bool connect_end(void);
+  static   bool connect_init(void);
+  static   bool connect_end(void);
   TABTYPE  GetRealType(PTOS pos= NULL);
-  char    *GetRealString(PCSZ s);
-	PCSZ     GetStringOption(PCSZ opname, PCSZ sdef= NULL);
+  char    *GetStringOption(char *opname, char *sdef= NULL);
   PTOS     GetTableOptionStruct(TABLE_SHARE *s= NULL);
-  bool     GetBooleanOption(PCSZ opname, bool bdef);
-  bool     SetBooleanOption(PCSZ opname, bool b);
-  int      GetIntegerOption(PCSZ opname);
-  bool     GetIndexOption(KEY *kp, PCSZ opname);
-  bool     CheckString(PCSZ str1, PCSZ str2);
-  bool     SameString(TABLE *tab, PCSZ opn);
-  bool     SetIntegerOption(PCSZ opname, int n);
-  bool     SameInt(TABLE *tab, PCSZ opn);
-  bool     SameBool(TABLE *tab, PCSZ opn);
-  bool     FileExists(const char *fn, bool bf);
+  bool     GetBooleanOption(char *opname, bool bdef);
+  bool     SetBooleanOption(char *opname, bool b);
+  int      GetIntegerOption(char *opname);
+  bool     CheckString(const char *str1, const char *str2);
+  bool     SameString(TABLE *tab, char *opn);
+  bool     SetIntegerOption(char *opname, int n);
+  bool     SameInt(TABLE *tab, char *opn);
+  bool     SameBool(TABLE *tab, char *opn);
+  bool     FileExists(const char *fn);
   bool     NoFieldOptionChange(TABLE *tab);
   PFOS     GetFieldOptionStruct(Field *fp);
   void    *GetColumnOption(PGLOBAL g, void *field, PCOLINFO pcf);
-  PXOS     GetIndexOptionStruct(KEY *kp);
   PIXDEF   GetIndexInfo(TABLE_SHARE *s= NULL);
-  bool     CheckVirtualIndex(TABLE_SHARE *s);
-  PCSZ     GetDBName(PCSZ name);
-  PCSZ     GetTableName(void);
-  char    *GetPartName(void);
+  const char *GetDBName(const char *name);
+  const char *GetTableName(void);
 //int      GetColNameLen(Field *fp);
 //char    *GetColName(Field *fp);
 //void     AddColName(char *cp, Field *fp);
   TABLE   *GetTable(void) {return table;}
   bool     IsSameIndex(PIXDEF xp1, PIXDEF xp2);
-  bool     IsPartitioned(void);
-  bool     IsUnique(uint n);
-  PCSZ     GetDataPath(void) {return datapath;}
 
-  bool     SetDataPath(PGLOBAL g, PCSZ path);
   PTDB     GetTDB(PGLOBAL g);
   int      OpenTable(PGLOBAL g, bool del= false);
-  bool     CheckColumnList(PGLOBAL g);
-  bool     IsOpened(void);
+  bool     IsOpened(void); 
   int      CloseTable(PGLOBAL g);
   int      MakeRecord(char *buf);
-  int      ScanRecord(PGLOBAL g, const uchar *buf);
-  int      CheckRecord(PGLOBAL g, const uchar *oldbuf, const uchar *newbuf);
-	int      ReadIndexed(uchar *buf, OPVAL op, const key_range *kr= NULL);
-	bool     IsIndexed(Field *fp);
-  bool     MakeKeyWhere(PGLOBAL g, PSTRG qry, OPVAL op, char q,
-                                   const key_range *kr);
-//inline char *Strz(LEX_STRING &ls);
-	key_range start_key;
-
+  int      ScanRecord(PGLOBAL g, uchar *buf);
+  int      CheckRecord(PGLOBAL g, const uchar *oldbuf, uchar *newbuf);
+  int      ReadIndexed(uchar *buf, OPVAL op, const uchar* key= NULL,
+                                             uint key_len= 0);
+  bool     MakeKeyWhere(PGLOBAL g, char *qry, OPVAL op, char *q,
+                                   const void *key, int klen);
 
   /** @brief
     The name that will be used for display purposes.
@@ -229,7 +215,7 @@ public:
   /** @brief
     The file extensions.
    */
-//const char **bas_ext() const;
+  const char **bas_ext() const;
 
  /**
     Check if a storage engine supports a particular alter table in-place
@@ -332,22 +318,17 @@ public:
    @note
    The pushed conditions form a stack (from which one can remove the
    last pushed condition using cond_pop).
-   The table handler filters out rows using (pushed_cond1 AND pushed_cond2
+   The table handler filters out rows using (pushed_cond1 AND pushed_cond2 
    AND ... AND pushed_condN)
    or less restrictive condition, depending on handler's capabilities.
 
    handler->ha_reset() call empties the condition stack.
    Calls to rnd_init/rnd_end, index_init/index_end etc do not affect the
    condition stack.
- */
+ */ 
 virtual const COND *cond_push(const COND *cond);
-PCFIL CheckCond(PGLOBAL g, PCFIL filp, const Item *cond);
+PCFIL CheckCond(PGLOBAL g, PCFIL filp, AMT tty, Item *cond);
 const char *GetValStr(OPVAL vop, bool neg);
-PFIL  CondFilter(PGLOBAL g, Item *cond);
-//PFIL  CheckFilter(PGLOBAL g);
-
-/** admin commands - called from mysql_admin_table */
-virtual int check(THD* thd, HA_CHECK_OPT* check_opt);
 
  /**
    Number of rows in table. It will only be called if
@@ -355,7 +336,7 @@ virtual int check(THD* thd, HA_CHECK_OPT* check_opt);
  */
  virtual ha_rows records();
 
- /**
+ /** 
    Type of table for caching query
    CONNECT should not use caching because its tables are external
    data prone to me modified out of MariaDB
@@ -482,30 +463,8 @@ int index_prev(uchar *buf);
                              enum thr_lock_type lock_type);     ///< required
   int optimize(THD* thd, HA_CHECK_OPT* check_opt);
 
-  /**
-   * Multi Range Read interface
-   */
-  int multi_range_read_init(RANGE_SEQ_IF *seq, void *seq_init_param,
-                            uint n_ranges, uint mode, HANDLER_BUFFER *buf);
-  int multi_range_read_next(range_id_t *range_info);
-  ha_rows multi_range_read_info_const(uint keyno, RANGE_SEQ_IF *seq,
-                                      void *seq_init_param,
-                                      uint n_ranges, uint *bufsz,
-                                      uint *flags, Cost_estimate *cost);
-  ha_rows multi_range_read_info(uint keyno, uint n_ranges, uint keys,
-                                uint key_parts, uint *bufsz,
-                                uint *flags, Cost_estimate *cost);
-  int multi_range_read_explain_info(uint mrr_mode, char *str, size_t size);
-
-  int reset(void) {ds_mrr.dsmrr_close(); return 0;}
-
-  /* Index condition pushdown implementation */
-//  Item *idx_cond_push(uint keyno, Item* idx_cond);
-private:
-  DsMrr_impl ds_mrr;
-
 protected:
-  bool check_privileges(THD *thd, PTOS options, char *dbn, bool quick=false);
+  bool check_privileges(THD *thd, PTOS options, char *dbn);
   MODE CheckMode(PGLOBAL g, THD *thd, MODE newmode, bool *chk, bool *cras);
   char *GetDBfromName(const char *name);
 
@@ -515,30 +474,22 @@ protected:
   ulong         hnum;                 // The number of this handler
   query_id_t    valid_query_id;       // The one when tdbp was allocated
   query_id_t    creat_query_id;       // The one when handler was allocated
-  PCSZ          datapath;             // Is the Path of DB data directory
   PTDB          tdbp;                 // To table class object
-  PVAL          sdvalin1;             // Used to convert date values
-  PVAL          sdvalin2;             // Used to convert date values
-  PVAL          sdvalin3;             // Used to convert date values
-  PVAL          sdvalin4;             // Used to convert date values
+  PVAL          sdvalin;              // Used to convert date values
   PVAL          sdvalout;             // Used to convert date values
   bool          istable;              // True for table handler
-  char          partname[65];         // The partition name
+//char          tname[64];            // The table name
   MODE          xmod;                 // Table mode
   XINFO         xinfo;                // The table info structure
   bool          valid_info;           // True if xinfo is valid
   bool          stop;                 // Used when creating index
   bool          alter;                // True when converting to other engine
-  bool          mrr;                  // True when getting index positions
-  bool          nox;                  // True when index should not be made
-  bool          abort;                // True after error in UPDATE/DELETE
   int           indexing;             // Type of indexing for CONNECT
   int           locked;               // Table lock
-  MY_BITMAP    *part_id;              // Columns used for partition func
   THR_LOCK_DATA lock_data;
 
 public:
-  TABLE_SHARE  *tshp;                 // Used by called tables
+  TABLE_SHARE  *tshp;                 // Used by called tables 
   char   *data_file_name;
   char   *index_file_name;
   uint    int_table_flags;            // Inherited from MyISAM
