@@ -1,11 +1,11 @@
 /********** PlgDBUtl Fpe C++ Program Source Code File (.CPP) ***********/
 /* PROGRAM NAME: PLGDBUTL                                              */
 /* -------------                                                       */
-/*  Version 4.0                                                        */
+/*  Version 3.9                                                        */
 /*                                                                     */
 /* COPYRIGHT:                                                          */
 /* ----------                                                          */
-/*  (C) Copyright to the author Olivier BERTRAND          1998-2017    */
+/*  (C) Copyright to the author Olivier BERTRAND          1998-2014    */
 /*                                                                     */
 /* WHAT THIS PROGRAM DOES:                                             */
 /* -----------------------                                             */
@@ -38,20 +38,20 @@
 /*  Include relevant MariaDB header file.                              */
 /***********************************************************************/
 #include "my_global.h"
-#if defined(__WIN__)
+#if defined(WIN32)
 #include <io.h>
 #include <fcntl.h>
 #include <errno.h>
 #define BIGMEM         1048576            // 1 Megabyte
-#else     // !__WIN__
+#else     // !WIN32
 #include <unistd.h>
 #include <fcntl.h>
-//#if defined(THREAD)
+#if defined(THREAD)
 #include <pthread.h>
-//#endif   // THREAD
+#endif   // THREAD
 #include <stdarg.h>
 #define BIGMEM      2147483647            // Max int value
-#endif    // !__WIN__
+#endif    // !WIN32
 #include <locale.h>
 
 /***********************************************************************/
@@ -68,9 +68,17 @@
 #include "tabcol.h"    // header of XTAB and COLUMN classes
 #include "valblk.h"
 #include "rcmsg.h"
-#ifdef ZIP_SUPPORT
-#include "filamzip.h"
-#endif   // ZIP_SUPPORT
+
+/***********************************************************************/
+/*  Macro or external routine definition                               */
+/***********************************************************************/
+#if defined(THREAD)
+#if defined(WIN32)
+extern CRITICAL_SECTION parsec;      // Used calling the Flex parser
+#else   // !WIN32
+extern pthread_mutex_t parmut;
+#endif  // !WIN32
+#endif  //  THREAD
 
 /***********************************************************************/
 /*  DB static variables.                                               */
@@ -79,13 +87,15 @@ bool  Initdone = false;
 bool  plugin = false;  // True when called by the XDB plugin handler 
 
 extern "C" {
+#if defined(XMSG)
+       char  msglang[16] = "ENGLISH";      // Default language
+#endif
+extern int  trace;
 extern char version[];
 } // extern "C"
 
-extern pthread_mutex_t parmut;
-
 // The debug trace used by the main thread
-FILE *pfile = NULL;
+       FILE *pfile = NULL;
 
 MBLOCK Nmblk = {NULL, false, 0, false, NULL};   // Used to init MBLOCK's
 
@@ -107,17 +117,11 @@ void CloseXMLFile(PGLOBAL, PFBLOCK, bool);
 
 /***********************************************************************/
 /* Routines for file IO with error reporting to g->Message             */
-/* Note: errno and strerror must be called before the message file     */
-/* is read in the case of XMSG compile.                                */
 /***********************************************************************/
-static void global_open_error_msg(GLOBAL *g, int msgid, const char *path, 
-                                                        const char *mode)
+static void
+global_open_error_msg(GLOBAL *g, int msgid, const char *path, const char *mode)
 {
-  int  len, rno= (int)errno;
-  char errmsg[256]= "";
-
-  strncat(errmsg, strerror(errno), 255);
-
+  int len;
   switch (msgid)
   {
     case MSGID_CANNOT_OPEN:
@@ -129,21 +133,19 @@ static void global_open_error_msg(GLOBAL *g, int msgid, const char *path,
     case MSGID_OPEN_MODE_ERROR:
       len= snprintf(g->Message, sizeof(g->Message) - 1,
                     MSG(OPEN_MODE_ERROR), // "Open(%s) error %d on %s"
-                    mode, rno, path);
+                    mode, (int) errno, path);
       break;
 
     case MSGID_OPEN_MODE_STRERROR:
-      {char fmt[256];
-      strcat(strcpy(fmt, MSG(OPEN_MODE_ERROR)), ": %s");
       len= snprintf(g->Message, sizeof(g->Message) - 1,
-                    fmt, // Open(%s) error %d on %s: %s
-                    mode, rno, path, errmsg);
-      }break;
+                    MSG(OPEN_MODE_ERROR) ": %s", // Open(%s) error %d on %s: %s
+                    mode, (int) errno, path, strerror(errno));
+      break;
 
     case MSGID_OPEN_STRERROR:
       len= snprintf(g->Message, sizeof(g->Message) - 1,
                     MSG(OPEN_STRERROR), // "open error: %s"
-                    errmsg);
+                    strerror(errno));
       break;
 
     case MSGID_OPEN_ERROR_AND_STRERROR:
@@ -151,13 +153,13 @@ static void global_open_error_msg(GLOBAL *g, int msgid, const char *path,
                     //OPEN_ERROR does not work, as it wants mode %d (not %s)
                     //MSG(OPEN_ERROR) "%s",// "Open error %d in mode %d on %s: %s"
                     "Open error %d in mode %s on %s: %s",
-                    rno, mode, path, errmsg);
+                    errno, mode, path, strerror(errno));
       break;
 
     case MSGID_OPEN_EMPTY_FILE:
       len= snprintf(g->Message, sizeof(g->Message) - 1,
                     MSG(OPEN_EMPTY_FILE), // "Opening empty file %s: %s"
-                    path, errmsg);
+                    path, strerror(errno));
       break;
 
     default:
@@ -234,84 +236,92 @@ void ptrc(char const *fmt, ...)
 PQRYRES PlgAllocResult(PGLOBAL g, int ncol, int maxres, int ids,
                        int *buftyp, XFLD *fldtyp, 
                        unsigned int *length, bool blank, bool nonull)
-{
+  {
   char     cname[NAM_LEN+1];
   int      i;
   PCOLRES *pcrp, crp;
   PQRYRES  qrp;
 
-	try {
-		/**********************************************************************/
-		/*  Allocate the structure used to contain the result set.            */
-		/**********************************************************************/
-		qrp = (PQRYRES)PlugSubAlloc(g, NULL, sizeof(QRYRES));
-		pcrp = &qrp->Colresp;
-		qrp->Continued = false;
-		qrp->Truncated = false;
-		qrp->Info = false;
-		qrp->Suball = true;
-		qrp->Maxres = maxres;
-		qrp->Maxsize = 0;
-		qrp->Nblin = 0;
-		qrp->Nbcol = 0;                                     // will be ncol
-		qrp->Cursor = 0;
-		qrp->BadLines = 0;
+  // Save stack and allocation environment and prepare error return
+  if (g->jump_level == MAX_JUMP) {
+    strcpy(g->Message, MSG(TOO_MANY_JUMPS));
+    return NULL;
+    } // endif jump_level
 
-		for (i = 0; i < ncol; i++) {
-			*pcrp = (PCOLRES)PlugSubAlloc(g, NULL, sizeof(COLRES));
-			crp = *pcrp;
-			pcrp = &crp->Next;
-			memset(crp, 0, sizeof(COLRES));
-			crp->Colp = NULL;
-			crp->Ncol = ++qrp->Nbcol;
-			crp->Type = buftyp[i];
-			crp->Length = length[i];
-			crp->Clen = GetTypeSize(crp->Type, length[i]);
-			crp->Prec = 0;
+  if (setjmp(g->jumper[++g->jump_level]) != 0) {
+    printf("%s\n", g->Message);
+    qrp = NULL;
+    goto fin;
+    } // endif rc
 
-			if (ids > 0) {
+  /************************************************************************/
+  /*  Allocate the structure used to contain the result set.              */
+  /************************************************************************/
+  qrp = (PQRYRES)PlugSubAlloc(g, NULL, sizeof(QRYRES));
+  pcrp = &qrp->Colresp;
+  qrp->Continued = false;
+  qrp->Truncated = false;
+  qrp->Info = false;
+  qrp->Suball = true;
+  qrp->Maxres = maxres;
+  qrp->Maxsize = 0;
+  qrp->Nblin = 0;
+  qrp->Nbcol = 0;                                     // will be ncol
+  qrp->Cursor = 0;
+  qrp->BadLines = 0;
+
+  for (i = 0; i < ncol; i++) {
+    *pcrp = (PCOLRES)PlugSubAlloc(g, NULL, sizeof(COLRES));
+    crp = *pcrp;
+    pcrp = &crp->Next;
+    memset(crp, 0, sizeof(COLRES));
+    crp->Colp = NULL;
+    crp->Ncol = ++qrp->Nbcol;
+    crp->Type = buftyp[i];
+    crp->Length = length[i];
+    crp->Clen = GetTypeSize(crp->Type, length[i]);
+    crp->Prec = 0;
+
+    if (ids > 0) {
 #if defined(XMSG)
-				// Get header from message file
-				strncpy(cname, PlugReadMessage(g, ids + crp->Ncol, NULL), NAM_LEN);
-				cname[NAM_LEN] = 0;					// for truncated long names
-#else   // !XMSG
-				GetRcString(ids + crp->Ncol, cname, sizeof(cname));
-#endif  // !XMSG
-				crp->Name = (PSZ)PlugDup(g, cname);
-			} else
-				crp->Name = NULL;           // Will be set by caller
+      // Get header from message file
+			strncpy(cname, PlugReadMessage(g, ids + crp->Ncol, NULL), NAM_LEN);
+			cname[NAM_LEN] = 0;					// for truncated long names
+//#elif defined(WIN32)
+      // Get header from ressource file
+//    LoadString(s_hModule, ids + crp->Ncol, cname, sizeof(cname));
+#else   // !WIN32
+      GetRcString(ids + crp->Ncol, cname, sizeof(cname));
+#endif  // !WIN32
+      crp->Name = (PSZ)PlugSubAlloc(g, NULL, strlen(cname) + 1);
+      strcpy(crp->Name, cname);
+    } else
+      crp->Name = NULL;           // Will be set by caller
 
-			if (fldtyp)
-				crp->Fld = fldtyp[i];
-			else
-				crp->Fld = FLD_NO;
+    if (fldtyp)
+      crp->Fld = fldtyp[i];
+    else
+      crp->Fld = FLD_NO;
 
-			// Allocate the Value Block that will contain data
-			if (crp->Length || nonull)
-				crp->Kdata = AllocValBlock(g, NULL, crp->Type, maxres,
-					crp->Length, 0, true, blank, false);
-			else
-				crp->Kdata = NULL;
+    // Allocate the Value Block that will contain data
+    if (crp->Length || nonull)
+      crp->Kdata = AllocValBlock(g, NULL, crp->Type, maxres,
+                                    crp->Length, 0, true, blank, false);
+    else
+      crp->Kdata = NULL;
 
-			if (trace)
-				htrc("Column(%d) %s type=%d len=%d value=%p\n",
-					crp->Ncol, crp->Name, crp->Type, crp->Length, crp->Kdata);
+    if (g->Trace)
+      htrc("Column(%d) %s type=%d len=%d value=%p\n",
+              crp->Ncol, crp->Name, crp->Type, crp->Length, crp->Kdata);
 
-		} // endfor i
+    } // endfor i
 
-		*pcrp = NULL;
+  *pcrp = NULL;
 
-	} catch (int n) {
-  	htrc("Exception %d: %s\n", n, g->Message);
-	  qrp = NULL;
-  } catch (const char *msg) {
-  	strcpy(g->Message, msg);
-	  htrc("%s\n", g->Message);
-	  qrp = NULL;
-  } // end catch
-
-	return qrp;
-} // end of PlgAllocResult
+ fin:
+  g->jump_level--;
+  return qrp;
+  } // end of PlgAllocResult
 
 /***********************************************************************/
 /*  Allocate and initialize the new DB User Block.                     */
@@ -326,8 +336,17 @@ PDBUSER PlgMakeUser(PGLOBAL g)
     } // endif dbuserp
 
   memset(dbuserp, 0, sizeof(DBUSERBLK));
-  dbuserp->Maxbmp = MAXBMP;
-//dbuserp->UseTemp = TMP_AUTO;
+//dbuserp->Act2 = g->Activityp;
+//#if defined(UNIX)
+//  dbuserp->LineLen = 160;
+//#else
+//  dbuserp->LineLen = 78;
+//#endif
+//dbuserp->Maxres = MAXRES;
+//dbuserp->Maxlin = MAXLIN;
+//dbuserp->Maxbmp = MAXBMP;
+//dbuserp->AlgChoice = AMOD_AUTO;
+  dbuserp->UseTemp = TMP_AUTO;
   dbuserp->Check = CHK_ALL;
   strcpy(dbuserp->Server, "CONNECT");
   return dbuserp;
@@ -357,13 +376,12 @@ PCATLG PlgGetCatalog(PGLOBAL g, bool jump)
   if (!cat && jump) {
     // Raise exception so caller doesn't have to check return value
     strcpy(g->Message, MSG(NO_ACTIVE_DB));
-		throw 1;
-	} // endif cat
+    longjmp(g->jumper[g->jump_level], 1);
+    } // endif cat
 
   return cat;
   } // end of PlgGetCatalog
 
-#if 0
 /***********************************************************************/
 /*  PlgGetDataPath: returns the default data path.                     */
 /***********************************************************************/
@@ -373,65 +391,6 @@ char *PlgGetDataPath(PGLOBAL g)
 
   return (cat) ? cat->GetDataPath() : NULL;
   } // end of PlgGetDataPath
-#endif // 0
-
-/***********************************************************************/
-/*  This function returns a database path.                             */
-/***********************************************************************/
-char *SetPath(PGLOBAL g, const char *path)
-{
-  char *buf= NULL;
-
-	if (path) {
-		size_t len = strlen(path) + (*path != '.' ? 4 : 1);
-
-		if (!(buf = (char*)PlgDBSubAlloc(g, NULL, len)))
-			return NULL;
-
-		if (PlugIsAbsolutePath(path)) {
-			strcpy(buf, path);
-			return buf;
-		} // endif path
-
-		if (*path != '.') {
-#if defined(__WIN__)
-			const char *s = "\\";
-#else   // !__WIN__
-			const char *s = "/";
-#endif  // !__WIN__
-			strcat(strcat(strcat(strcpy(buf, "."), s), path), s);
-		} else
-			strcpy(buf, path);
-
-	} // endif path
-
-  return buf;
-} // end of SetPath
-
-/***********************************************************************/
-/*  Extract from a path name the required component.                   */
-/*  This function assumes there is enough space in the buffer.         */
-/***********************************************************************/
-char *ExtractFromPath(PGLOBAL g, char *pBuff, char *FileName, OPVAL op)
-  {
-  char *drive = NULL, *direc = NULL, *fname = NULL, *ftype = NULL;
-
-  switch (op) {           // Determine which part to extract
-#if defined(__WIN__)
-    case OP_FDISK: drive = pBuff; break;
-#endif   // !UNIX
-    case OP_FPATH: direc = pBuff; break;
-    case OP_FNAME: fname = pBuff; break;
-    case OP_FTYPE: ftype = pBuff; break;
-    default:
-      sprintf(g->Message, MSG(INVALID_OPER), op, "ExtractFromPath");
-      return NULL;
-    } // endswitch op
-
-  // Now do the extraction
-  _splitpath(FileName, drive, direc, fname, ftype);
-  return pBuff;
-  } // end of PlgExtractFromPath
 
 /***********************************************************************/
 /*  Check the occurence and matching of a pattern against a string.    */
@@ -441,7 +400,7 @@ char *ExtractFromPath(PGLOBAL g, char *pBuff, char *FileName, OPVAL op)
 static bool PlugCheckPattern(PGLOBAL g, LPCSTR string, LPCSTR pat)
   {
   if (pat && strlen(pat)) {
-    // This leaves 2048 bytes (MAX_STR / 2) for each components
+    // This leaves 512 bytes (MAX_STR / 2) for each components
     LPSTR name = g->Message + MAX_STR / 2;
 
     strlwr(strcpy(name, string));
@@ -469,8 +428,8 @@ bool PlugEvalLike(PGLOBAL g, LPCSTR strg, LPCSTR pat, bool ci)
       tp = g->Message;
     else if (!(tp = new char[strlen(pat) + strlen(strg) + 2])) {
       strcpy(g->Message, MSG(NEW_RETURN_NULL));
-			throw (int)OP_LIKE;
-		} /* endif tp */
+      longjmp(g->jumper[g->jump_level], OP_LIKE);
+      } /* endif tp */
     
     sp = tp + strlen(pat) + 1;
     strlwr(strcpy(tp, pat));      /* Make a lower case copy of pat     */
@@ -480,8 +439,8 @@ bool PlugEvalLike(PGLOBAL g, LPCSTR strg, LPCSTR pat, bool ci)
       tp = g->Message;            /* Use this as temporary work space. */
     else if (!(tp = new char[strlen(pat) + 1])) {
       strcpy(g->Message, MSG(NEW_RETURN_NULL));
-			throw (int)OP_LIKE;
-		} /* endif tp */
+      longjmp(g->jumper[g->jump_level], OP_LIKE);
+      } /* endif tp */
     
     strcpy(tp, pat);                  /* Make a copy to be worked into */
     sp = (char*)strg;
@@ -669,16 +628,15 @@ void PlugConvertConstant(PGLOBAL g, void* & value, short& type)
 /*  format and a Strftime output format. Flag if not 0 indicates that  */
 /*  non quoted blanks are not included in the output format.           */
 /***********************************************************************/
-PDTP MakeDateFormat(PGLOBAL g, PCSZ dfmt, bool in, bool out, int flag)
-{
-	int  rc;
+PDTP MakeDateFormat(PGLOBAL g, PSZ dfmt, bool in, bool out, int flag)
+  {
   PDTP pdp = (PDTP)PlugSubAlloc(g, NULL, sizeof(DATPAR));
 
   if (trace)
     htrc("MakeDateFormat: dfmt=%s\n", dfmt);
 
   memset(pdp, 0, sizeof(DATPAR));
-  pdp->Format = pdp->Curp = PlugDup(g, dfmt);
+  pdp->Format = pdp->Curp = dfmt;
   pdp->Outsize = 2 * strlen(dfmt) + 1;
 
   if (in)
@@ -693,26 +651,35 @@ PDTP MakeDateFormat(PGLOBAL g, PCSZ dfmt, bool in, bool out, int flag)
   /* Call the FLEX generated parser. In multi-threading mode the next  */
   /* instruction is included in an Enter/LeaveCriticalSection bracket. */
   /*********************************************************************/
+#if defined(THREAD)
+#if defined(WIN32)
+  EnterCriticalSection((LPCRITICAL_SECTION)&parsec);
+#else   // !WIN32
   pthread_mutex_lock(&parmut);
-  rc = fmdflex(pdp);
+#endif  // !WIN32
+#endif  //  THREAD
+  /*int rc =*/ fmdflex(pdp);
+#if defined(THREAD)
+#if defined(WIN32)
+  LeaveCriticalSection((LPCRITICAL_SECTION)&parsec);
+#else   // !WIN32
   pthread_mutex_unlock(&parmut);
+#endif  // !WIN32
+#endif  //  THREAD
 
   if (trace)
-    htrc("Done: in=%s out=%s rc=%d\n", SVP(pdp->InFmt), SVP(pdp->OutFmt), rc);
-
+    htrc("Done:  in=%s out=%s\n", SVP(pdp->InFmt), SVP(pdp->OutFmt));           
   return pdp;
-} // end of MakeDateFormat
+  } // end of MakeDateFormat
 
 /***********************************************************************/
 /* Extract the date from a formatted string according to format.       */
 /***********************************************************************/
 int ExtractDate(char *dts, PDTP pdp, int defy, int val[6])
   {
-	PCSZ fmt;
-	char c, d, e, W[8][12];
-  int  i, k, m, numval;
-  int  n, y = 30;
-  bool b = true;           // true for null dates
+  char *fmt, c, d, e, W[8][12];
+  int   i, k, m, numval;
+  int   n, y = 30;
 
   if (pdp)
     fmt = pdp->InFmt;
@@ -746,8 +713,7 @@ int ExtractDate(char *dts, PDTP pdp, int defy, int val[6])
     m = pdp->Num;
 
   for (i = 0; i < m; i++) {
-    if ((n = *(int*)W[i]))
-      b = false;
+    n = *(int*)W[i];
 
     switch (k = pdp->Index[i]) {
       case 0:
@@ -806,7 +772,7 @@ int ExtractDate(char *dts, PDTP pdp, int defy, int val[6])
     htrc("numval=%d val=(%d,%d,%d,%d,%d,%d)\n",
           numval, val[0], val[1], val[2], val[3], val[4], val[5]); 
 
-  return (b) ? 0 : numval;
+  return numval;
   } // end of ExtractDate
 
 /***********************************************************************/
@@ -834,7 +800,8 @@ FILE *PlugOpenFile(PGLOBAL g, LPCSTR fname, LPCSTR ftype)
       htrc(" fp=%p\n", fp);
 
     // fname may be in volatile memory such as stack
-    fp->Fname = PlugDup(g, fname);
+    fp->Fname = (char*)PlugSubAlloc(g, NULL, strlen(fname) + 1);
+    strcpy((char*)fp->Fname, fname);
     fp->Count = 1;
     fp->Type = TYPE_FB_FILE;
     fp->File = fop;
@@ -853,24 +820,7 @@ FILE *PlugOpenFile(PGLOBAL g, LPCSTR fname, LPCSTR ftype)
 /*  Close file routine: the purpose of this routine is to avoid        */
 /*  double closing that freeze the system on some Unix platforms.      */
 /***********************************************************************/
-FILE *PlugReopenFile(PGLOBAL g, PFBLOCK fp, LPCSTR md)
-  {
-  FILE *fop;
-
-  if ((fop = global_fopen(g, MSGID_OPEN_MODE_STRERROR, fp->Fname, md))) {
-    fp->Count = 1;
-    fp->Type = TYPE_FB_FILE;
-    fp->File = fop;
-    } /* endif fop */
-
-  return (fop);
-  } // end of PlugOpenFile
-
-/***********************************************************************/
-/*  Close file routine: the purpose of this routine is to avoid        */
-/*  double closing that freeze the system on some Unix platforms.      */
-/***********************************************************************/
-int PlugCloseFile(PGLOBAL g __attribute__((unused)), PFBLOCK fp, bool all)
+int PlugCloseFile(PGLOBAL g, PFBLOCK fp, bool all)
   {
   int rc = 0;
 
@@ -899,7 +849,7 @@ int PlugCloseFile(PGLOBAL g __attribute__((unused)), PFBLOCK fp, bool all)
 
       fp->Memory = NULL;
       fp->Mode = MODE_ANY;
-      // fall through
+      // Passthru
     case TYPE_FB_HANDLE:
       if (fp->Handle && fp->Handle != INVALID_HANDLE_VALUE)
         if (CloseFileHandle(fp->Handle))
@@ -919,20 +869,7 @@ int PlugCloseFile(PGLOBAL g __attribute__((unused)), PFBLOCK fp, bool all)
       CloseXML2File(g, fp, all);
       break;
 #endif   // LIBXML2_SUPPORT
-#ifdef ZIP_SUPPORT
-		case TYPE_FB_ZIP:
-			if (fp->Mode == MODE_INSERT)
-				((ZIPUTIL*)fp->File)->close();
-			else
-				((UNZIPUTL*)fp->File)->close();
-
-			fp->Memory = NULL;
-			fp->Mode = MODE_ANY;
-			fp->Count = 0;
-			fp->File = NULL;
-			break;
-#endif   // ZIP_SUPPORT
-		default:
+    default:
       rc = RC_FX;
     } // endswitch Type
 
@@ -978,7 +915,7 @@ void PlugCleanup(PGLOBAL g, bool dofree)
     /*  This is the place to reset the pointer on domains.             */
     /*******************************************************************/
     dbuserp->Subcor = false;
-    dbuserp->Step = "New query";     // was STEP(PARSING_QUERY);
+    dbuserp->Step = STEP(PARSING_QUERY);
     dbuserp->ProgMax = dbuserp->ProgCur = dbuserp->ProgSav = 0;
     } // endif dofree
 
@@ -1095,8 +1032,7 @@ char *GetAmName(PGLOBAL g, AMT am, void *memp)
     case TYPE_AM_DOM:   strcpy(amn, "DOM");   break;
     case TYPE_AM_DIR:   strcpy(amn, "DIR");   break;
     case TYPE_AM_ODBC:  strcpy(amn, "ODBC");  break;
-		case TYPE_AM_JDBC:  strcpy(amn, "JDBC");  break;
-		case TYPE_AM_MAC:   strcpy(amn, "MAC");   break;
+    case TYPE_AM_MAC:   strcpy(amn, "MAC");   break;
     case TYPE_AM_OEM:   strcpy(amn, "OEM");   break;
     case TYPE_AM_OUT:   strcpy(amn, "OUT");   break;
     default:           sprintf(amn, "OEM(%d)", am);
@@ -1105,7 +1041,7 @@ char *GetAmName(PGLOBAL g, AMT am, void *memp)
   return amn;
   } // end of GetAmName
 
-#if defined(SE_CATCH)
+#if defined(WIN32) && !defined(NOCATCH)
 /***********************************************************************/
 /*  GetExceptionDesc: return the description of an exception code.     */
 /***********************************************************************/
@@ -1193,7 +1129,7 @@ char *GetExceptionDesc(PGLOBAL g, unsigned int e)
 
   return p;
   } // end of GetExceptionDesc
-#endif   // SE_CATCH
+#endif   // WIN32 && !NOCATCH
 
 /***********************************************************************/
 /*  PlgDBalloc: allocates or suballocates memory conditionally.        */
@@ -1235,9 +1171,9 @@ void *PlgDBalloc(PGLOBAL g, void *area, MBLOCK& mp)
   if (!mp.Sub) {
     // For allocations greater than one fourth of remaining storage
     // in the area, do allocate from virtual storage.
-#if defined(__WIN__)
+#if defined(WIN32)
     if (mp.Size >= BIGMEM)
-      mp.Memp = VirtualAlloc(NULL, mp.Size, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
+      mp.Memp = VirtualAlloc(NULL, mp.Size, MEM_COMMIT, PAGE_READWRITE);
     else
 #endif
       mp.Memp = malloc(mp.Size);
@@ -1332,7 +1268,7 @@ void PlgDBfree(MBLOCK& mp)
     htrc("PlgDBfree: %p sub=%d size=%d\n", mp.Memp, mp.Sub, mp.Size);
 
   if (!mp.Sub && mp.Memp)
-#if defined(__WIN__)
+#if defined(WIN32)
     if (mp.Size >= BIGMEM)
       VirtualFree(mp.Memp, 0, MEM_RELEASE);
     else
@@ -1394,23 +1330,6 @@ void *PlgDBSubAlloc(PGLOBAL g, void *memp, size_t size)
   return (memp);
   } // end of PlgDBSubAlloc
 
-/***********************************************************************/
-/*  Program for sub-allocating and copying a string in a storage area. */
-/***********************************************************************/
-char *PlgDBDup(PGLOBAL g, const char *str)
-  {
-  if (str) {
-    char *sm = (char*)PlgDBSubAlloc(g, NULL, strlen(str) + 1);
-
-    if (sm)
-      strcpy(sm, str);
-
-    return sm;
-  } else
-    return NULL;
-
-  } // end of PlgDBDup
-
 /***********************************************************************/
 /*  PUTOUT: Plug DB object typing routine.                             */
 /***********************************************************************/
@@ -1476,7 +1395,7 @@ void PlugPutOut(PGLOBAL g, FILE *f, short t, void *v, uint n)
     case TYPE_TABLE:
     case TYPE_TDB:
     case TYPE_XOBJECT:
-      ((PBLOCK)v)->Printf(g, f, n-2);
+      ((PBLOCK)v)->Print(g, f, n-2);
       break;
 
     default:
@@ -1502,8 +1421,8 @@ DllExport void NewPointer(PTABS t, void *oldv, void *newv)
       PGLOBAL g = t->G;
 
       sprintf(g->Message, "NewPointer: %s", MSG(MEM_ALLOC_ERROR));
-			throw 3;
-		} else {
+      longjmp(g->jumper[g->jump_level], 3);
+    } else {
       tp->Next = t->P1;
       tp->Num = 0;
       t->P1 = tp;
@@ -1528,26 +1447,26 @@ int FileComp(PGLOBAL g, char *file1, char *file2)
   bp[0] = buff1; bp[1] = buff2;
 
   for (i = 0; i < 2; i++) {
-#if defined(__WIN__)
+#if defined(WIN32)
     h[i]= global_open(g, MSGID_NONE, fn[i], _O_RDONLY | _O_BINARY);
-#else   // !__WIN__
+#else   // !WIN32
     h[i]= global_open(g, MSGOD_NONE, fn[i], O_RDONLY);
-#endif  // !__WIN__
+#endif  // !WIN32
 
     if (h[i] == -1) {
 //      if (errno != ENOENT) {
         sprintf(g->Message, MSG(OPEN_MODE_ERROR),
                 "rb", (int)errno, fn[i]);
         strcat(strcat(g->Message, ": "), strerror(errno));
-				throw 666;
-				//      } else
+        longjmp(g->jumper[g->jump_level], 666);
+//      } else
 //        len[i] = 0;          // File does not exist yet
 
     } else {
       if ((len[i] = _filelength(h[i])) < 0) {
         sprintf(g->Message, MSG(FILELEN_ERROR), "_filelength", fn[i]);
-				throw 666;
-			} // endif len
+        longjmp(g->jumper[g->jump_level], 666);
+        } // endif len
 
     } // endif h
 
