@@ -1,11 +1,11 @@
 /************* RelDef CPP Program Source Code File (.CPP) **************/
-/* PROGRAM NAME: RELDEF                                                */
+/* PROGRAM NAME: REFDEF                                                */
 /* -------------                                                       */
-/*  Version 1.6                                                        */
+/*  Version 1.3                                                        */
 /*                                                                     */
 /* COPYRIGHT:                                                          */
 /* ----------                                                          */
-/*  (C) Copyright to the author Olivier BERTRAND          2004-2016    */
+/*  (C) Copyright to the author Olivier BERTRAND          2004-2012    */
 /*                                                                     */
 /* WHAT THIS PROGRAM DOES:                                             */
 /* -----------------------                                             */
@@ -17,7 +17,7 @@
 /*  Include relevant MariaDB header file.                              */
 /***********************************************************************/
 #include "my_global.h"
-#if defined(__WIN__)
+#if defined(WIN32)
 #include <sqlext.h>
 #else
 #include <dlfcn.h>          // dlopen(), dlclose(), dlsym() ...
@@ -35,32 +35,20 @@
 /***********************************************************************/
 #include "global.h"
 #include "plgdbsem.h"
+#include "mycat.h"
 #include "reldef.h"
 #include "colblk.h"
-#include "tabcol.h"
 #include "filamap.h"
 #include "filamfix.h"
-#if defined(VCT_SUPPORT)
 #include "filamvct.h"
-#endif   // VCT_SUPPORT
-#if defined(GZ_SUPPORT)
-#include "filamgz.h"
-#endif   // GZ_SUPPORT
+#if defined(ZIP_SUPPORT)
+#include "filamzip.h"
+#endif   // ZIP_SUPPORT
 #include "tabdos.h"
 #include "valblk.h"
 #include "tabmul.h"
 #include "ha_connect.h"
-#include "mycat.h"
 
-#if !defined(__WIN__)
-extern handlerton *connect_hton;
-#endif   // !__WIN__
-
-/***********************************************************************/
-/*  External function.                                                 */
-/***********************************************************************/
-USETEMP UseTemp(void);
-char   *GetPluginDir(void);
 
 /* --------------------------- Class RELDEF -------------------------- */
 
@@ -77,18 +65,10 @@ RELDEF::RELDEF(void)
   Hc = NULL;
   } // end of RELDEF constructor
 
-/***********************************************************************/
-/*  This function return a pointer to the Table Option Struct.         */
-/***********************************************************************/
-PTOS RELDEF::GetTopt(void)
-	{
-	return Hc->GetTableOptionStruct();
-	} // end of GetTopt
-
 /***********************************************************************/
 /*  This function sets an integer table information.                   */
 /***********************************************************************/
-bool RELDEF::SetIntCatInfo(PCSZ what, int n)
+bool RELDEF::SetIntCatInfo(PSZ what, int n)
 	{
 	return Hc->SetIntegerOption(what, n);
 	} // end of SetIntCatInfo
@@ -96,7 +76,7 @@ bool RELDEF::SetIntCatInfo(PCSZ what, int n)
 /***********************************************************************/
 /*  This function returns integer table information.                   */
 /***********************************************************************/
-int RELDEF::GetIntCatInfo(PCSZ what, int idef)
+int RELDEF::GetIntCatInfo(PSZ what, int idef)
 	{
 	int n= Hc->GetIntegerOption(what);
 
@@ -106,7 +86,7 @@ int RELDEF::GetIntCatInfo(PCSZ what, int idef)
 /***********************************************************************/
 /*  This function returns Boolean table information.                   */
 /***********************************************************************/
-bool RELDEF::GetBoolCatInfo(PCSZ what, bool bdef)
+bool RELDEF::GetBoolCatInfo(PSZ what, bool bdef)
 	{
 	bool b= Hc->GetBooleanOption(what, bdef);
 
@@ -116,10 +96,9 @@ bool RELDEF::GetBoolCatInfo(PCSZ what, bool bdef)
 /***********************************************************************/
 /*  This function returns size catalog information.                    */
 /***********************************************************************/
-int RELDEF::GetSizeCatInfo(PCSZ what, PCSZ sdef)
+int RELDEF::GetSizeCatInfo(PSZ what, PSZ sdef)
 	{
-	char c;
-	PCSZ s;
+	char * s, c;
   int  i, n= 0;
 
 	if (!(s= Hc->GetStringOption(what)))
@@ -129,7 +108,6 @@ int RELDEF::GetSizeCatInfo(PCSZ what, PCSZ sdef)
     switch (toupper(c)) {
       case 'M':
         n *= 1024;
-        // fall through
       case 'K':
         n *= 1024;
       } // endswitch c
@@ -140,48 +118,33 @@ int RELDEF::GetSizeCatInfo(PCSZ what, PCSZ sdef)
 /***********************************************************************/
 /*  This function sets char table information in buf.                  */
 /***********************************************************************/
-int RELDEF::GetCharCatInfo(PCSZ what, PCSZ sdef, char *buf, int size)
+int RELDEF::GetCharCatInfo(PSZ what, PSZ sdef, char *buf, int size)
 	{
-	PCSZ s= Hc->GetStringOption(what);
+	char *s= Hc->GetStringOption(what);
 
 	strncpy(buf, ((s) ? s : sdef), size);
 	return size;
 	} // end of GetCharCatInfo
 
-/***********************************************************************/
-/*  To be used by any TDB's.                                           */
-/***********************************************************************/
-bool RELDEF::Partitioned(void)
-	{
-	return Hc->IsPartitioned();
-	} // end of Partitioned
-
 /***********************************************************************/
 /*  This function returns string table information.                    */
 /*  Default parameter is "*" to get the handler default.               */
 /***********************************************************************/
-char *RELDEF::GetStringCatInfo(PGLOBAL g, PCSZ what, PCSZ sdef)
+char *RELDEF::GetStringCatInfo(PGLOBAL g, PSZ what, PSZ sdef)
 	{
-	char *sval = NULL;
-	PCSZ  name, s= Hc->GetStringOption(what, sdef);
+	char *sval= NULL, *s= Hc->GetStringOption(what, sdef);
 	
 	if (s) {
-    if (!Hc->IsPartitioned() ||
-        (stricmp(what, "filename") && stricmp(what, "tabname")
-                                   && stricmp(what, "connect")))
-		  sval= PlugDup(g, s);
-    else
-      sval= (char*)s;
-
+		sval= (char*)PlugSubAlloc(g, NULL, strlen(s) + 1);
+		strcpy(sval, s);
   } else if (!stricmp(what, "filename")) {
     // Return default file name
-		PCSZ ftype= Hc->GetStringOption("Type", "*");
-    int  i, n;
+    char *ftype= Hc->GetStringOption("Type", "*");
+    int   i, n;
 
     if (IsFileType(GetTypeID(ftype))) {
-      name= Hc->GetPartName();
-      sval= (char*)PlugSubAlloc(g, NULL, strlen(name) + 12);
-      strcat(strcpy(sval, name), ".");
+      sval= (char*)PlugSubAlloc(g, NULL, strlen(Hc->GetTableName()) + 12);
+      strcat(strcpy(sval, Hc->GetTableName()), ".");
       n= strlen(sval);
   
       // Fold ftype to lower case
@@ -216,22 +179,19 @@ TABDEF::TABDEF(void)
   Degree = 0;
   Pseudo = 0;
   Read_Only = false;
-  m_data_charset = NULL;
-  csname = NULL;
   } // end of TABDEF constructor
 
 /***********************************************************************/
 /*  Define: initialize the table definition block from XDB file.       */
 /***********************************************************************/
-bool TABDEF::Define(PGLOBAL g, PCATLG cat, 
-	                  LPCSTR name, LPCSTR schema, LPCSTR am)
+bool TABDEF::Define(PGLOBAL g, PCATLG cat, LPCSTR name, LPCSTR am)
   {
   int   poff = 0;
 
-	Hc = ((MYCAT*)cat)->GetHandler();
-	Name = (PSZ)name;
-	Schema = (PSZ)Hc->GetDBName(schema);
+  Name = (PSZ)PlugSubAlloc(g, NULL, strlen(name) + 1);
+  strcpy(Name, name);
   Cat = cat;
+  Hc = ((MYCAT*)cat)->GetHandler();
   Catfunc = GetFuncID(GetStringCatInfo(g, "Catfunc", NULL));
   Elemt = GetIntCatInfo("Elements", 0);
   Multiple = GetIntCatInfo("Multiple", 0);
@@ -241,7 +201,6 @@ bool TABDEF::Define(PGLOBAL g, PCATLG cat,
   m_data_charset= data_charset_name ?
                   get_charset_by_csname(data_charset_name, MY_CS_PRIMARY, 0):
                   NULL;
-  csname = GetStringCatInfo(g, "Table_charset", NULL);
 
   // Get The column definitions
   if ((poff = GetColCatInfo(g)) < 0)
@@ -251,22 +210,13 @@ bool TABDEF::Define(PGLOBAL g, PCATLG cat,
   return DefineAM(g, am, poff);
   } // end of Define
 
-/***********************************************************************/
-/*  This function returns the database data path.                      */
-/***********************************************************************/
-PCSZ TABDEF::GetPath(void)
-  {
-  return (Database) ? Database : (Hc) ? Hc->GetDataPath() : NULL;
-  } // end of GetPath
-
 /***********************************************************************/
 /*  This function returns column table information.                    */
 /***********************************************************************/
 int TABDEF::GetColCatInfo(PGLOBAL g)
 	{
 	char		*type= GetStringCatInfo(g, "Type", "*");
-  char     c, fty, eds;
-	int      i, n, loff, poff, nof, nlg;
+	int      i, loff, poff, nof, nlg;
 	void    *field= NULL;
   TABTYPE  tc;
   PCOLDEF  cdp, lcdp= NULL, tocols= NULL;
@@ -280,13 +230,8 @@ int TABDEF::GetColCatInfo(PGLOBAL g)
   // Take care of the column definitions
 	i= poff= nof= nlg= 0;
 
-#if defined(__WIN__)
 	// Offsets of HTML and DIR tables start from 0, DBF at 1
-	loff= (tc == TAB_DBF) ? 1 : (tc == TAB_XML || tc == TAB_DIR) ? -1 : 0;
-#else   // !__WIN__
-	// Offsets of HTML tables start from 0, DIR and DBF at 1
-	loff = (tc == TAB_DBF || tc == TAB_DIR) ? 1 : (tc == TAB_XML) ? -1 : 0;
-#endif  // !__WIN__
+	loff= (tc == TAB_DBF) ? 1 : (tc == TAB_XML || tc == TAB_DIR) ? -1 : 0; 
 
   while (true) {
 		// Default Offset depends on table type
@@ -304,7 +249,7 @@ int TABDEF::GetColCatInfo(PGLOBAL g)
 				nlg+= nof;
       case TAB_DIR:
       case TAB_XML:
-				poff= loff + (pcf->Flags & U_VIRTUAL ? 0 : 1);
+        poff= loff + 1;
         break;
       case TAB_INI:
       case TAB_MAC:
@@ -315,7 +260,7 @@ int TABDEF::GetColCatInfo(PGLOBAL g)
       case TAB_OEM:
         poff = 0;      // Offset represents an independant flag
         break;
-      default:         // VCT PLG ODBC JDBC MYSQL WMI...
+      default:         // VCT PLG ODBC MYSQL WMI...
         poff = 0;			 // NA
         break;
 			} // endswitch tc
@@ -338,7 +283,7 @@ int TABDEF::GetColCatInfo(PGLOBAL g)
 
     if ((nof= cdp->Define(g, NULL, pcf, poff)) < 0)
       return -1;						 // Error, probably unhandled type
-		else
+		else if (nof)
 			loff= cdp->GetOffset();
 
 		switch (tc) {
@@ -346,52 +291,20 @@ int TABDEF::GetColCatInfo(PGLOBAL g)
 				cdp->SetOffset(0);		 // Not to have shift
 			case TAB_BIN:
 				// BIN/VEC are packed by default
-        if (nof) {
+				if (nof)
 					// Field width is the internal representation width
 					// that can also depend on the column format
-          fty = cdp->Decode ? 'C' : 'X';
-          eds = 0;
-          n = 0;
-
-          if (cdp->Fmt && !cdp->Decode) {
-            for (i = 0; cdp->Fmt[i]; i++) {
-              c = toupper(cdp->Fmt[i]);
-
-              if (isdigit(c))
-                n = (n * 10 + (c - '0'));
-              else if (c == 'L' || c == 'B' || c == 'H')
-                eds = c;
-              else
-                fty = c;
-
-              } // endfor i
-
-          } // endif Fmt
-
-          if (n)
-            nof = n;
-          else switch (fty) {
-						case 'X':
-              if (eds && IsTypeChar(cdp->Buf_Type))
-                nof = sizeof(longlong);
-              else
-                nof= cdp->Clen;
-
-              break;
-						case 'C':                         break;
+					switch (cdp->Fmt ? *cdp->Fmt : 'X') {
+						case 'C':         break;
 						case 'R':
-						case 'F': nof = sizeof(float);    break;
-						case 'I':	nof = sizeof(int);      break;
-						case 'D':	nof = sizeof(double);   break;
-						case 'S':	nof = sizeof(short);    break;
-						case 'T':	nof = sizeof(char);     break;
-						case 'G':	nof = sizeof(longlong); break;
-						default:  /* Wrong format */
-              sprintf(g->Message, "Invalid format %c", fty);
-              return -1;
-						} // endswitch fty
-
-          } // endif nof
+						case 'F':
+						case 'L':
+						case 'I':	nof= 4; break;
+						case 'D':	nof= 8; break;
+						case 'S':	nof= 2; break;
+						case 'T':	nof= 1; break;
+						default:  nof= cdp->Clen;
+						} // endswitch Fmt
 
       default:
 				break;
@@ -417,16 +330,20 @@ int TABDEF::GetColCatInfo(PGLOBAL g)
 		// not specified (for instance if quoted is specified)
 //	if ((ending= Hc->GetIntegerOption("Ending")) < 0) {
 		if ((ending= Hc->GetIntegerOption("Ending")) <= 0) {
-      ending= (tc == TAB_BIN || tc == TAB_VEC) ? 0 : CRLF;
+#if defined(WIN32)
+			ending= 2;
+#else
+			ending= 1;
+#endif
 			Hc->SetIntegerOption("Ending", ending);
 			} // endif ending
 
 		// Calculate the default record size
 		switch (tc) {
       case TAB_FIX:
-      case TAB_BIN:
         recln= nlg + ending;     // + length of line ending
         break;
+      case TAB_BIN:
       case TAB_VEC:
         recln= nlg;
 	
@@ -450,11 +367,7 @@ int TABDEF::GetColCatInfo(PGLOBAL g)
       } // endswitch tc
 
 		// lrecl must be at least recln to avoid buffer overflow
-		if (trace(1))
-			htrc("Lrecl: Calculated=%d defined=%d\n", 
-			  recln, Hc->GetIntegerOption("Lrecl"));
-
-		recln = MY_MAX(recln, Hc->GetIntegerOption("Lrecl"));
+		recln= MY_MAX(recln, Hc->GetIntegerOption("Lrecl"));
 		Hc->SetIntegerOption("Lrecl", recln);
 		((PDOSDEF)this)->SetLrecl(recln);
 		} // endif Lrecl
@@ -481,31 +394,20 @@ void TABDEF::SetIndexInfo(void)
 PTABDEF OEMDEF::GetXdef(PGLOBAL g)
   {
   typedef PTABDEF (__stdcall *XGETDEF) (PGLOBAL, void *);
-  char    c, soname[_MAX_PATH], getname[40] = "Get";
+  char    c, getname[40] = "Get";
   PTABDEF xdefp;
   XGETDEF getdef = NULL;
   PCATLG  cat = Cat;
 
-  /*********************************************************************/
-  /*  Ensure that the .dll doesn't have a path.                        */
-  /*  This is done to ensure that only approved dll from the system    */
-  /*  directories are used (to make this even remotely secure).        */
-  /*********************************************************************/
-  if (check_valid_path(Module, strlen(Module))) {
-    strcpy(g->Message, "Module cannot contain a path");
-    return NULL;
-  } else
-    PlugSetPath(soname, Module, GetPluginDir());
-    
-#if defined(__WIN__)
+#if defined(WIN32)
   // Is the DLL already loaded?
-  if (!Hdll && !(Hdll = GetModuleHandle(soname)))
+  if (!Hdll && !(Hdll = GetModuleHandle(Module)))
     // No, load the Dll implementing the function
-    if (!(Hdll = LoadLibrary(soname))) {
+    if (!(Hdll = LoadLibrary(Module))) {
       char  buf[256];
       DWORD rc = GetLastError();
 
-      sprintf(g->Message, MSG(DLL_LOAD_ERROR), rc, soname);
+      sprintf(g->Message, MSG(DLL_LOAD_ERROR), rc, Module);
       FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM |
                     FORMAT_MESSAGE_IGNORE_INSERTS, NULL, rc, 0,
                     (LPTSTR)buf, sizeof(buf), NULL);
@@ -522,42 +424,18 @@ PTABDEF OEMDEF::GetXdef(PGLOBAL g)
 
   // Get the function returning an instance of the external DEF class
   if (!(getdef = (XGETDEF)GetProcAddress((HINSTANCE)Hdll, getname))) {
-		char  buf[256];
-		DWORD rc = GetLastError();
-
-		sprintf(g->Message, MSG(PROCADD_ERROR), rc, getname);
-		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM |
-			FORMAT_MESSAGE_IGNORE_INSERTS, NULL, rc, 0,
-			(LPTSTR)buf, sizeof(buf), NULL);
-		strcat(strcat(g->Message, ": "), buf);
-		FreeLibrary((HMODULE)Hdll);
+    sprintf(g->Message, MSG(PROCADD_ERROR), GetLastError(), getname);
+    FreeLibrary((HMODULE)Hdll);
     return NULL;
     } // endif getdef
-#else   // !__WIN__
+#else   // !WIN32
   const char *error = NULL;
-    
-#if 0  // Don't know what all this stuff does
-	Dl_info dl_info;
-
-	// The OEM lib must retrieve exported CONNECT variables
-  if (dladdr(&connect_hton, &dl_info)) {
-    if (dlopen(dl_info.dli_fname, RTLD_NOLOAD | RTLD_NOW | RTLD_GLOBAL) == 0) {
-      error = dlerror();
-      sprintf(g->Message, "dlopen failed: %s, OEM not supported", SVP(error));
-      return NULL;
-      } // endif dlopen
-    
-  } else {
-    error = dlerror();
-    sprintf(g->Message, "dladdr failed: %s, OEM not supported", SVP(error));
-    return NULL;
-  } // endif dladdr
-#endif // 0
-
+  // Is the library already loaded?
+//  if (!Hdll && !(Hdll = ???))
   // Load the desired shared library
-  if (!Hdll && !(Hdll = dlopen(soname, RTLD_LAZY))) {
+  if (!(Hdll = dlopen(Module, RTLD_LAZY))) {
     error = dlerror();
-    sprintf(g->Message, MSG(SHARED_LIB_ERR), soname, SVP(error));
+    sprintf(g->Message, MSG(SHARED_LIB_ERR), Module, SVP(error));
     return NULL;
     } // endif Hdll
 
@@ -575,7 +453,7 @@ PTABDEF OEMDEF::GetXdef(PGLOBAL g)
     dlclose(Hdll);
     return NULL;
     } // endif getdef
-#endif  // !__WIN__
+#endif  // !WIN32
 
   // Just in case the external Get function does not set error messages
   sprintf(g->Message, MSG(DEF_ALLOC_ERROR), Subtype);
@@ -592,7 +470,7 @@ PTABDEF OEMDEF::GetXdef(PGLOBAL g)
     } // endif Cbuf
 
   // Here "OEM" should be replace by a more useful value
-  if (xdefp->Define(g, cat, Name, Schema, "OEM"))
+  if (xdefp->Define(g, cat, Name, "OEM"))
     return NULL;
 
   // Ok, return external block
@@ -615,7 +493,7 @@ bool OEMDEF::DeleteTableFile(PGLOBAL g)
 /***********************************************************************/
 /*  Define: initialize the table definition block from XDB file.       */
 /***********************************************************************/
-bool OEMDEF::DefineAM(PGLOBAL g, LPCSTR, int)
+bool OEMDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
   {
   Module = GetStringCatInfo(g, "Module", "");
   Subtype = GetStringCatInfo(g, "Subtype", Module);
@@ -623,10 +501,9 @@ bool OEMDEF::DefineAM(PGLOBAL g, LPCSTR, int)
   if (!*Module)
     Module = Subtype;
 
-  char *desc = (char*)PlugSubAlloc(g, NULL, strlen(Module)
-                                          + strlen(Subtype) + 3);
-  sprintf(desc, "%s(%s)", Module, Subtype);
-	Desc = desc;
+  Desc = (char*)PlugSubAlloc(g, NULL, strlen(Module)
+                                    + strlen(Subtype) + 3);
+  sprintf(Desc, "%s(%s)", Module, Subtype);
   return false;
   } // end of DefineAM
 
@@ -635,8 +512,8 @@ bool OEMDEF::DefineAM(PGLOBAL g, LPCSTR, int)
 /***********************************************************************/
 PTDB OEMDEF::GetTable(PGLOBAL g, MODE mode)
   {
-  RECFM rfm;
-  PTDB  tdbp = NULL;
+  RECFM   rfm;
+  PTDBASE tdbp = NULL;
 
   // If define block not here yet, get it now
   if (!Pxdef && !(Pxdef = GetXdef(g)))
@@ -646,7 +523,7 @@ PTDB OEMDEF::GetTable(PGLOBAL g, MODE mode)
   /*  Allocate a TDB of the proper type.                               */
   /*  Column blocks will be allocated only when needed.                */
   /*********************************************************************/
-  if (!(tdbp = Pxdef->GetTable(g, mode)))
+  if (!(tdbp = (PTDBASE)Pxdef->GetTable(g, mode)))
     return NULL;
   else
     rfm = tdbp->GetFtype();
@@ -669,7 +546,7 @@ PTDB OEMDEF::GetTable(PGLOBAL g, MODE mode)
   PTXF    txfp = NULL;
   PDOSDEF defp = (PDOSDEF)Pxdef;
   bool    map = defp->Mapped && mode != MODE_INSERT &&
-                !(UseTemp() == TMP_FORCE &&
+                !(PlgGetUser(g)->UseTemp == TMP_FORCE &&
                 (mode == MODE_UPDATE || mode == MODE_DELETE));
   int     cmpr = defp->Compressed;
 
@@ -679,15 +556,17 @@ PTDB OEMDEF::GetTable(PGLOBAL g, MODE mode)
   /*********************************************************************/
   if (!((PTDBDOS)tdbp)->GetTxfp()) {
     if (cmpr) {
-#if defined(GZ_SUPPORT)
+#if defined(ZIP_SUPPORT)
       if (cmpr == 1)
-        txfp = new(g) GZFAM(defp);
-      else
-        txfp = new(g) ZLBFAM(defp);
-#else   // !GZ_SUPPORT
+        txfp = new(g) ZIPFAM(defp);
+      else {
+        strcpy(g->Message, "Compress 2 not supported yet");
+        return NULL;
+      } // endelse
+#else   // !ZIP_SUPPORT
       strcpy(g->Message, "Compress not supported");
       return NULL;
-#endif  // !GZ_SUPPORT
+#endif  // !ZIP_SUPPORT
     } else if (rfm == RECFM_VAR) {
       if (map)
         txfp = new(g) MAPFAM(defp);
@@ -699,19 +578,16 @@ PTDB OEMDEF::GetTable(PGLOBAL g, MODE mode)
         txfp = new(g) MPXFAM(defp);
       else
         txfp = new(g) FIXFAM(defp);
+
     } else if (rfm == RECFM_VCT) {
-#if defined(VCT_SUPPORT)
-			assert(Pxdef->GetDefType() == TYPE_AM_VCT);
+      assert (Pxdef->GetDefType() == TYPE_AM_VCT);
 
       if (map)
         txfp = new(g) VCMFAM((PVCTDEF)defp);
       else
         txfp = new(g) VCTFAM((PVCTDEF)defp);
-#else   // !VCT_SUPPORT
-			strcpy(g->Message, "VCT no more supported");
-			return NULL;
-#endif  // !VCT_SUPPORT
-		} // endif's
+
+    } // endif's
 
     ((PTDBDOS)tdbp)->SetTxfp(txfp);
     } // endif Txfp
@@ -737,10 +613,8 @@ COLCRT::COLCRT(PSZ name)
   Offset = -1;
   Long = -1;
   Precision = -1;
-  Freq = -1;
   Key = -1;
   Scale = -1;
-  Opt = -1;
   DataType = '*';
   } // end of COLCRT constructor for table creation
 
@@ -754,10 +628,8 @@ COLCRT::COLCRT(void)
   Offset = 0;
   Long = 0;
   Precision = 0;
-  Freq = 0;
   Key = 0;
   Scale = 0;
-  Opt = 0;
   DataType = '*';
   } // end of COLCRT constructor for table & view definition
 
@@ -768,14 +640,6 @@ COLCRT::COLCRT(void)
 /***********************************************************************/
 COLDEF::COLDEF(void) : COLCRT()
   {
-  To_Min = NULL;
-  To_Max = NULL;
-  To_Pos = NULL;
-  Xdb2 = FALSE;
-  To_Bmap = NULL;
-  To_Dval = NULL;
-  Ndv = 0;
-  Nbm = 0;
   Buf_Type = TYPE_ERROR;
   Clen = 0;
   Poff = 0;
@@ -786,15 +650,16 @@ COLDEF::COLDEF(void) : COLCRT()
 /***********************************************************************/
 /*  Define: initialize a column definition from a COLINFO structure.   */
 /***********************************************************************/
-int COLDEF::Define(PGLOBAL g, void *, PCOLINFO cfp, int poff)
+int COLDEF::Define(PGLOBAL g, void *memp, PCOLINFO cfp, int poff)
   {
-  Name = (PSZ)PlugDup(g, cfp->Name);
+  Name = (PSZ)PlugSubAlloc(g, memp, strlen(cfp->Name) + 1);
+  strcpy(Name, cfp->Name);
 
   if (!(cfp->Flags & U_SPECIAL)) {
     Poff = poff;
     Buf_Type = cfp->Type;
 
-    if ((Clen = GetTypeSize(Buf_Type, cfp->Length)) < 0) {
+    if ((Clen = GetTypeSize(Buf_Type, cfp->Length)) <= 0) {
       sprintf(g->Message, MSG(BAD_COL_TYPE), GetTypeName(Buf_Type), Name);
       return -1;
       } // endswitch
@@ -806,21 +671,24 @@ int COLDEF::Define(PGLOBAL g, void *, PCOLINFO cfp, int poff)
     Precision = cfp->Precision;
     Scale = cfp->Scale;
     Long = cfp->Length;
-    Opt = cfp->Opt;
     Key = cfp->Key;
-    Freq = cfp->Freq;
 
-    if (cfp->Remark && *cfp->Remark)
-      Desc = (PSZ)PlugDup(g, cfp->Remark);
+    if (cfp->Remark && *cfp->Remark) {
+      Desc = (PSZ)PlugSubAlloc(g, memp, strlen(cfp->Remark) + 1);
+      strcpy(Desc, cfp->Remark);
+      } // endif Remark
 
-    if (cfp->Datefmt)
-      Decode = (PSZ)PlugDup(g, cfp->Datefmt);
+    if (cfp->Datefmt) {
+      Decode = (PSZ)PlugSubAlloc(g, memp, strlen(cfp->Datefmt) + 1);
+      strcpy(Decode, cfp->Datefmt);
+      } // endif Datefmt
 
-  } else
-    Offset = poff;
+    } // endif special
 
-  if (cfp->Fieldfmt)
-    Fmt = (PSZ)PlugDup(g, cfp->Fieldfmt);
+  if (cfp->Fieldfmt) {
+    Fmt = (PSZ)PlugSubAlloc(g, memp, strlen(cfp->Fieldfmt) + 1);
+    strcpy(Fmt, cfp->Fieldfmt);
+    } // endif Fieldfmt
 
   Flags = cfp->Flags;
   return (Flags & (U_VIRTUAL|U_SPECIAL)) ? 0 : Long;
