@@ -1,7 +1,7 @@
 /*************** TabFmt H Declares Source Code File (.H) ***************/
-/*  Name: TABFMT.H    Version 2.5                                      */
+/*  Name: TABFMT.H    Version 2.3                                      */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          2001-2016    */
+/*  (C) Copyright to the author Olivier BERTRAND          2001-2013    */
 /*                                                                     */
 /*  This file contains the CSV and FMT classes declares.               */
 /***********************************************************************/
@@ -13,7 +13,8 @@ typedef class  TDBFMT    *PTDBFMT;
 /***********************************************************************/
 /*  Functions used externally.                                         */
 /***********************************************************************/
-PQRYRES CSVColumns(PGLOBAL g, char *dp, PTOS topt, bool info);
+PQRYRES CSVColumns(PGLOBAL g, const char *fn, char sep, char q,
+                   int hdr, int mxr, bool info);
 
 /***********************************************************************/
 /*  CSV table.                                                         */
@@ -21,8 +22,7 @@ PQRYRES CSVColumns(PGLOBAL g, char *dp, PTOS topt, bool info);
 class DllExport CSVDEF : public DOSDEF { /* Logical table description  */
   friend class TDBCSV;
   friend class TDBCCL;
-	friend PQRYRES CSVColumns(PGLOBAL, char *, PTOS, bool);
-public:
+ public:
   // Constructor
   CSVDEF(void);
 
@@ -50,10 +50,9 @@ public:
 /*  This is the DOS/UNIX Access Method class declaration for files     */
 /*  that are CSV files with columns separated by the Sep character.    */
 /***********************************************************************/
-class DllExport TDBCSV : public TDBDOS {
+class TDBCSV : public TDBDOS {
   friend class CSVCOL;
-	friend PQRYRES CSVColumns(PGLOBAL, char *, PTOS, bool);
-public:
+ public:
   // Constructor
   TDBCSV(PCSVDEF tdp, PTXF txfp);
   TDBCSV(PGLOBAL g, PTDBCSV tdbp);
@@ -76,13 +75,11 @@ public:
   virtual int  ReadBuffer(PGLOBAL g);        // Physical file read
 
   // Specific routines
-  virtual int  EstimatedLength(void);
+  virtual int  EstimatedLength(PGLOBAL g);
   virtual bool SkipHeader(PGLOBAL g);
   virtual bool CheckErr(void);
 
  protected:
-  virtual bool PrepareWriting(PGLOBAL g);
-
   // Members
   PSZ  *Field;             // Field to write to current line
   int  *Offset;            // Column offsets for current record
@@ -102,7 +99,7 @@ public:
 /*  Class CSVCOL: CSV access method column descriptor.                 */
 /*  This A.M. is used for Comma Separated V(?) files.                  */
 /***********************************************************************/
-class DllExport CSVCOL : public DOSCOL {
+class CSVCOL : public DOSCOL {
   friend class TDBCSV;
   friend class TDBFMT;
  public:
@@ -114,7 +111,6 @@ class DllExport CSVCOL : public DOSCOL {
   virtual int    GetAmType() {return TYPE_AM_CSV;}
 
   // Methods
-  virtual bool   VarSize(void);
   virtual void   ReadColumn(PGLOBAL g);
   virtual void   WriteColumn(PGLOBAL g);
 
@@ -130,7 +126,7 @@ class DllExport CSVCOL : public DOSCOL {
 /*  This is the DOS/UNIX Access Method class declaration for files     */
 /*  whose record format is described by a Format keyword.              */
 /***********************************************************************/
-class DllExport TDBFMT : public TDBCSV {
+class TDBFMT : public TDBCSV {
   friend class CSVCOL;
 //friend class FMTCOL;
  public:
@@ -158,12 +154,9 @@ class DllExport TDBFMT : public TDBCSV {
   virtual int  ReadBuffer(PGLOBAL g);        // Physical file read
 
   // Specific routines
-  virtual int  EstimatedLength(void);
+  virtual int  EstimatedLength(PGLOBAL g);
 
  protected:
-  virtual bool PrepareWriting(PGLOBAL g) 
-       {sprintf(g->Message, MSG(TABLE_READ_ONLY), "FMT"); return true;}
-
   // Members
   PSZ  *FldFormat;                      // Field read format
   void *To_Fld;                         // To field test buffer
@@ -174,17 +167,21 @@ class DllExport TDBFMT : public TDBCSV {
 /***********************************************************************/
 /*  This is the class declaration for the CSV catalog table.           */
 /***********************************************************************/
-class DllExport TDBCCL : public TDBCAT {
+class TDBCCL : public TDBCAT {
  public:
   // Constructor
   TDBCCL(PCSVDEF tdp);
 
  protected:
-  // Specific routines
-  virtual PQRYRES GetResult(PGLOBAL g);
+	// Specific routines
+	virtual PQRYRES GetResult(PGLOBAL g);
 
   // Members
-	PTOS  Topt;
-}; // end of class TDBCCL
+  char   *Fn;                     // The CSV file (path) name
+  bool    Hdr;                    // true if first line contains headers
+  int     Mxr;                    // Maximum number of bad records
+  int     Qtd;                    // Quoting level for quoted fields
+  char    Sep;                    // Separator for standard CSV files 
+  }; // end of class TDBCCL
 
 /* ------------------------- End of TabFmt.H ------------------------- */
