@@ -1,4 +1,4 @@
-// TDBMYSQL.H     Olivier Bertrand    2007-2017
+// TDBMYSQL.H     Olivier Bertrand    2007-2013
 #include "myconn.h"               // MySQL connection declares
 
 typedef class MYSQLDEF *PMYDEF;
@@ -18,7 +18,7 @@ typedef class MYSQLC   *PMYC;
 /***********************************************************************/
 /*  MYSQL table.                                                       */
 /***********************************************************************/
-class MYSQLDEF : public EXTDEF           {/* Logical table description */
+class MYSQLDEF : public TABDEF           {/* Logical table description */
   friend class TDBMYSQL;
   friend class TDBMYEXC;
   friend class TDBMCL;
@@ -27,18 +27,19 @@ class MYSQLDEF : public EXTDEF           {/* Logical table description */
   // Constructor
   MYSQLDEF(void);
 
+
   // Implementation
   virtual const char *GetType(void) {return "MYSQL";}
   inline  PSZ  GetHostname(void) {return Hostname;};
-//inline  PSZ  GetDatabase(void) {return Tabschema;};
-//inline  PSZ  GetTabname(void) {return Tabname;}
-//inline  PSZ  GetSrcdef(void) {return Srcdef;}
-//inline  PSZ  GetUsername(void) {return Username;};
-//inline  PSZ  GetPassword(void) {return Password;};
+  inline  PSZ  GetDatabase(void) {return Database;};
+  inline  PSZ  GetTabname(void) {return Tabname;}
+  inline  PSZ  GetSrcdef(void) {return Srcdef;}
+  inline  PSZ  GetUsername(void) {return Username;};
+  inline  PSZ  GetPassword(void) {return Password;};
   inline  int  GetPortnumber(void) {return Portnumber;}
 
   // Methods
-//virtual int  Indexable(void) {return 2;}
+  virtual int  Indexable(void) {return 2;}
   virtual bool DefineAM(PGLOBAL g, LPCSTR am, int poff);
   virtual PTDB GetTable(PGLOBAL g, MODE m);
           bool ParseURL(PGLOBAL g, char *url, bool b = true);
@@ -47,58 +48,57 @@ class MYSQLDEF : public EXTDEF           {/* Logical table description */
  protected:
   // Members
   PSZ     Hostname;           /* Host machine to use                   */
-//PSZ     Tabschema;          /* Database to be used by server         */
-//PSZ     Tabname;            /* External table name                   */
-//PSZ     Srcdef;             /* The source table SQL definition       */
-//PSZ     Username;           /* User logon name                       */
-//PSZ     Password;           /* Password logon info                   */
+  PSZ     Database;           /* Database to be used by server         */
+  PSZ     Tabname;            /* External table name                   */
+  PSZ     Srcdef;             /* The source table SQL definition       */
+  PSZ     Username;           /* User logon name                       */
+  PSZ     Password;           /* Password logon info                   */
   PSZ     Server;             /* PServerID                             */
-//PSZ     Qrystr;             /* The original query                    */
+  PSZ     Qrystr;             /* The original query                    */
   int     Portnumber;         /* MySQL port number (0 = default)       */
-//int     Maxerr;             /* Maxerr for an Exec table              */
-//int     Quoted;             /* Identifier quoting level              */
+  int     Mxr;                /* Maxerr for an Exec table              */
+  int     Quoted;             /* Identifier quoting level              */
   bool    Isview;             /* true if this table is a MySQL view    */
   bool    Bind;               /* Use prepared statement on insert      */
   bool    Delayed;            /* Delayed insert                        */
-//bool    Xsrc;               /* Execution type                        */
+  bool    Xsrc;               /* Execution type                        */
   bool    Huge;               /* True for big table                    */
   }; // end of MYSQLDEF
 
 /***********************************************************************/
 /*  This is the class declaration for the MYSQL table.                 */
 /***********************************************************************/
-class TDBMYSQL : public TDBEXT {
+class TDBMYSQL : public TDBASE {
   friend class MYSQLCOL;
  public:
   // Constructor
   TDBMYSQL(PMYDEF tdp);
-  TDBMYSQL(PTDBMY tdbp);
+  TDBMYSQL(PGLOBAL g, PTDBMY tdbp);
 
   // Implementation
   virtual AMT  GetAmType(void) {return TYPE_AM_MYSQL;}
-  virtual PTDB Duplicate(PGLOBAL g) {return (PTDB)new(g) TDBMYSQL(this);}
+  virtual PTDB Duplicate(PGLOBAL g) {return (PTDB)new(g) TDBMYSQL(g, this);}
 
   // Methods
-  virtual PTDB Clone(PTABS t);
-//virtual int  GetAffectedRows(void) {return AftRows;}
+  virtual PTDB CopyOne(PTABS t);
+  virtual int  GetAffectedRows(void) {return AftRows;}
   virtual int  GetRecpos(void) {return N;}
   virtual int  GetProgMax(PGLOBAL g);
   virtual void ResetDB(void) {N = 0;}
   virtual int  RowNumber(PGLOBAL g, bool b = false);
   virtual bool IsView(void) {return Isview;}
-  virtual PCSZ GetServer(void) {return Server;}
-          void SetDatabase(LPCSTR db) {Schema = (char*)db;}
+  virtual PSZ  GetServer(void) {return Server;}
+          void SetDatabase(LPCSTR db) {Database = (char*)db;}
 
-  // Schema routines
+  // Database routines
   virtual PCOL MakeCol(PGLOBAL g, PCOLDEF cdp, PCOL cprec, int n);
-  virtual int  Cardinality(PGLOBAL g);
-//virtual int  GetMaxSize(PGLOBAL g);
+  virtual int  GetMaxSize(PGLOBAL g);
   virtual bool OpenDB(PGLOBAL g);
   virtual int  ReadDB(PGLOBAL g);
   virtual int  WriteDB(PGLOBAL g);
   virtual int  DeleteDB(PGLOBAL g, int irc);
   virtual void CloseDB(PGLOBAL g);
-  virtual bool ReadKey(PGLOBAL g, OPVAL op, const key_range *kr);
+  virtual bool ReadKey(PGLOBAL g, OPVAL op, const void *key, int len);
 
   // Specific routines
           bool SetColumnRanks(PGLOBAL g);
@@ -109,8 +109,8 @@ class TDBMYSQL : public TDBEXT {
   // Internal functions
   bool MakeSelect(PGLOBAL g, bool mx);
   bool MakeInsert(PGLOBAL g);
-  int  BindColumns(PGLOBAL g __attribute__((unused)));
-  virtual bool MakeCommand(PGLOBAL g);
+  int  BindColumns(PGLOBAL g);
+  int  MakeCommand(PGLOBAL g);
 //int  MakeUpdate(PGLOBAL g);  
 //int  MakeDelete(PGLOBAL g);
   int  SendCommand(PGLOBAL g);
@@ -118,25 +118,26 @@ class TDBMYSQL : public TDBEXT {
   // Members
   MYSQLC      Myc;            // MySQL connection class
   MYSQL_BIND *Bind;           // To the MySQL bind structure array
-//PSTRG       Query;          // Constructed SQL query
   char       *Host;           // Host machine to use
-//char       *User;           // User logon info
-//char       *Pwd;            // Password logon info
-//char       *Schema;         // Database to be used by server
-//char       *TableName;      // External table name
-//char       *Srcdef;         // The source table SQL definition
+  char       *User;           // User logon info
+  char       *Pwd;            // Password logon info
+  char       *Database;       // Database to be used by server
+  char       *Tabname;        // External table name
+  char       *Srcdef;         // The source table SQL definition
   char       *Server;         // The server ID
-//char       *Qrystr;         // The original query
+  char       *Query;          // Points to SQL query
+  char       *Qbuf;           // Used for not prepared insert
+  char       *Qrystr;         // The original query
   bool        Fetched;        // True when fetch was done
   bool        Isview;         // True if this table is a MySQL view
   bool        Prep;           // Use prepared statement on insert
   bool        Delayed;        // Use delayed insert
   int         m_Rc;           // Return code from command
-//int         AftRows;        // The number of affected rows
+  int         AftRows;        // The number of affected rows
   int         N;              // The current table index
   int         Port;           // MySQL port number (0 = default) 
-//int         Nparm;          // The number of statement parameters
-//int         Quoted;         // The identifier quoting level
+  int         Nparm;          // The number of statement parameters
+  int         Quoted;         // The identifier quoting level
   }; // end of class TDBMYSQL
 
 /***********************************************************************/
@@ -146,8 +147,8 @@ class MYSQLCOL : public COLBLK {
   friend class TDBMYSQL;
  public:
   // Constructors
-  MYSQLCOL(PCOLDEF cdp, PTDB tdbp, PCOL cprec, int i,  PCSZ am = "MYSQL");
-  MYSQLCOL(MYSQL_FIELD *fld, PTDB tdbp, int i,  PCSZ am = "MYSQL");
+  MYSQLCOL(PCOLDEF cdp, PTDB tdbp, PCOL cprec, int i,  PSZ am = "MYSQL");
+  MYSQLCOL(MYSQL_FIELD *fld, PTDB tdbp, int i,  PSZ am = "MYSQL");
   MYSQLCOL(MYSQLCOL *colp, PTDB tdbp); // Constructor used in copy process
 
   // Implementation
@@ -161,6 +162,9 @@ class MYSQLCOL : public COLBLK {
           bool FindRank(PGLOBAL g);
 
  protected:
+  // Default constructor not to be used
+  MYSQLCOL(void) {}
+
   // Members
   MYSQL_BIND   *Bind;            // This column bind structure pointer
   PVAL          To_Val;          // To value used for Update/Insert
@@ -176,14 +180,14 @@ class TDBMYEXC : public TDBMYSQL {
  public:
   // Constructors
   TDBMYEXC(PMYDEF tdp); 
-  TDBMYEXC(PTDBMYX tdbp);
+  TDBMYEXC(PGLOBAL g, PTDBMYX tdbp);
 
   // Implementation
   virtual AMT  GetAmType(void) {return TYPE_AM_MYX;}
-  virtual PTDB Duplicate(PGLOBAL g) {return (PTDB)new(g) TDBMYEXC(this);}
+  virtual PTDB Duplicate(PGLOBAL g) {return (PTDB)new(g) TDBMYEXC(g, this);}
 
   // Methods
-  virtual PTDB Clone(PTABS t);
+  virtual PTDB CopyOne(PTABS t);
   virtual bool IsView(void) {return Isview;}
 
   // Database routines
@@ -215,8 +219,8 @@ class MYXCOL : public MYSQLCOL {
   friend class TDBMYEXC;
  public:
   // Constructors
-  MYXCOL(PCOLDEF cdp, PTDB tdbp, PCOL cprec, int i,  PCSZ am = "MYSQL");
-  MYXCOL(MYSQL_FIELD *fld, PTDB tdbp, int i,  PCSZ am = "MYSQL");
+  MYXCOL(PCOLDEF cdp, PTDB tdbp, PCOL cprec, int i,  PSZ am = "MYSQL");
+  MYXCOL(MYSQL_FIELD *fld, PTDB tdbp, int i,  PSZ am = "MYSQL");
   MYXCOL(MYXCOL *colp, PTDB tdbp);   // Constructor used in copy process
 
   // Methods
@@ -224,6 +228,9 @@ class MYXCOL : public MYSQLCOL {
   virtual void WriteColumn(PGLOBAL g);
 
  protected:
+  // Default constructor not to be used
+  MYXCOL(void) {}
+
   // Members
   char    *Buffer;              // To get returned message
   int      Flag;                // Column content desc
@@ -242,10 +249,10 @@ class TDBMCL : public TDBCAT {
 	virtual PQRYRES GetResult(PGLOBAL g);
 
   // Members
-  PCSZ Host;                      // Host machine to use            
-	PCSZ Db;                        // Database to be used by server  
-	PCSZ Tab;                       // External table name            
-	PCSZ User;                      // User logon name                
-	PCSZ Pwd;                       // Password logon info            
-	int  Port;                      // MySQL port number (0 = default)
+  PSZ     Host;                  // Host machine to use            
+  PSZ     Db;                    // Database to be used by server  
+  PSZ     Tab;                   // External table name            
+  PSZ     User;                  // User logon name                
+  PSZ     Pwd;                   // Password logon info            
+  int     Port;                  // MySQL port number (0 = default)
   }; // end of class TDBMCL
