@@ -1,7 +1,7 @@
 /************ Valblk C++ Functions Source Code File (.CPP) *************/
-/*  Name: VALBLK.CPP  Version 2.3                                      */
+/*  Name: VALBLK.CPP  Version 2.1                                      */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          2005-2017    */
+/*  (C) Copyright to the author Olivier BERTRAND          2005-2014    */
 /*                                                                     */
 /*  This file contains the VALBLK and derived classes functions.       */
 /*  Second family is VALBLK, representing simple suballocated arrays   */
@@ -23,7 +23,7 @@
 /*  Include relevant MariaDB header file.                              */
 /***********************************************************************/
 #include "my_global.h"
-#if defined(__WIN__)
+#if defined(WIN32)
 //#include <windows.h>
 #else
 #include "osutil.h"
@@ -43,6 +43,7 @@
 #define CheckBlanks      assert(!Blanks);
 #define CheckParms(V, N) ChkIndx(N); ChkTyp(V);
 
+extern "C" int  trace;
 extern MBLOCK Nmblk;                /* Used to initialize MBLOCK's     */
 
 /***********************************************************************/
@@ -53,18 +54,17 @@ PVBLK AllocValBlock(PGLOBAL g, void *mp, int type, int nval, int len,
   {
   PVBLK blkp;
 
-  if (trace(1))
+  if (trace)
     htrc("AVB: mp=%p type=%d nval=%d len=%d check=%u blank=%u\n",
          mp, type, nval, len, check, blank);
 
   switch (type) {
     case TYPE_STRING:
-		case TYPE_BIN:
     case TYPE_DECIM:
       if (len)
-        blkp = new(g) CHRBLK(mp, nval, type, len, prec, blank);
+        blkp = new(g) CHRBLK(mp, nval, len, prec, blank);
       else
-        blkp = new(g) STRBLK(g, mp, nval, type);
+        blkp = new(g) STRBLK(g, mp, nval);
 
       break;
     case TYPE_SHORT:
@@ -100,9 +100,6 @@ PVBLK AllocValBlock(PGLOBAL g, void *mp, int type, int nval, int len,
       else
         blkp = new(g) TYPBLK<char>(mp, nval, type);
 
-      break;
-    case TYPE_PCHAR:
-      blkp = new(g) PTRBLK(g, mp, nval);
       break;
     default:
       sprintf(g->Message, MSG(BAD_VALBLK_TYPE), type);
@@ -133,20 +130,20 @@ VALBLK::VALBLK(void *mp, int type, int nval, bool un)
 /***********************************************************************/
 /*  Raise error for numeric types.                                     */
 /***********************************************************************/
-PSZ VALBLK::GetCharValue(int)
+PSZ VALBLK::GetCharValue(int n)
   {
   PGLOBAL& g = Global;
 
   assert(g);
   sprintf(g->Message, MSG(NO_CHAR_FROM), Type);
-	throw Type;
-	return NULL;
+  longjmp(g->jumper[g->jump_level], Type);
+  return NULL;
   } // end of GetCharValue
 
 /***********************************************************************/
 /*  Set format so formatted dates can be converted on input.           */
 /***********************************************************************/
-bool VALBLK::SetFormat(PGLOBAL g, PCSZ, int, int)
+bool VALBLK::SetFormat(PGLOBAL g, PSZ fmt, int len, int year)
   {
   sprintf(g->Message, MSG(NO_DATE_FMT), Type);
   return true;
@@ -207,8 +204,8 @@ void VALBLK::ChkIndx(int n)
   if (n < 0 || n >= Nval) {
     PGLOBAL& g = Global;
     strcpy(g->Message, MSG(BAD_VALBLK_INDX));
-		throw Type;
-	} // endif n
+    longjmp(g->jumper[g->jump_level], Type);
+    } // endif n
 
   } // end of ChkIndx
 
@@ -217,8 +214,8 @@ void VALBLK::ChkTyp(PVAL v)
   if (Check && (Type != v->GetType() || Unsigned != v->IsUnsigned())) {
     PGLOBAL& g = Global;
     strcpy(g->Message, MSG(VALTYPE_NOMATCH));
-		throw Type;
-	} // endif Type
+    longjmp(g->jumper[g->jump_level], Type);
+    } // endif Type
 
   } // end of ChkTyp
 
@@ -227,8 +224,8 @@ void VALBLK::ChkTyp(PVBLK vb)
   if (Check && (Type != vb->GetType() || Unsigned != vb->IsUnsigned())) {
     PGLOBAL& g = Global;
     strcpy(g->Message, MSG(VALTYPE_NOMATCH));
-		throw Type;
-	} // endif Type
+    longjmp(g->jumper[g->jump_level], Type);
+    } // endif Type
 
   } // end of ChkTyp
 
@@ -336,15 +333,15 @@ uchar TYPBLK<uchar>::GetTypedValue(PVAL valp)
 /*  Set one value in a block from a zero terminated string.            */
 /***********************************************************************/
 template <class TYPE>
-void TYPBLK<TYPE>::SetValue(PCSZ p, int n)
+void TYPBLK<TYPE>::SetValue(PSZ p, int n)
   {
   ChkIndx(n);
 
   if (Check) {
     PGLOBAL& g = Global;
     strcpy(g->Message, MSG(BAD_SET_STRING));
-		throw Type;
-	} // endif Check
+    longjmp(g->jumper[g->jump_level], Type);
+    } // endif Check
 
   bool      minus;
   ulonglong maxval = MaxVal();
@@ -386,15 +383,15 @@ template <>
 ulonglong TYPBLK<ulonglong>::MaxVal(void) {return ULONGLONG_MAX;}
 
 template <>
-void TYPBLK<double>::SetValue(PCSZ p, int n)
+void TYPBLK<double>::SetValue(PSZ p, int n)
   {
   ChkIndx(n);
 
   if (Check) {
     PGLOBAL& g = Global;
     strcpy(g->Message, MSG(BAD_SET_STRING));
-		throw Type;
-	} // endif Check
+    longjmp(g->jumper[g->jump_level], Type);
+    } // endif Check
 
   Typp[n] = atof(p);
   SetNull(n, false);
@@ -404,7 +401,7 @@ void TYPBLK<double>::SetValue(PCSZ p, int n)
 /*  Set one value in a block from an array of characters.              */
 /***********************************************************************/
 template <class TYPE>
-void TYPBLK<TYPE>::SetValue(PCSZ sp, uint len, int n)
+void TYPBLK<TYPE>::SetValue(char *sp, uint len, int n)
   {
   PGLOBAL& g = Global;
   PSZ spz = (PSZ)PlugSubAlloc(g, NULL, 0);    // Temporary
@@ -471,36 +468,6 @@ template <>
 uchar TYPBLK<uchar>::GetTypedValue(PVBLK blk, int n)
   {return blk->GetUTinyValue(n);}
 
-/***********************************************************************/
-/*  Set one value in a block if val is less than the current value.    */
-/***********************************************************************/
-template <class TYPE>
-void TYPBLK<TYPE>::SetMin(PVAL valp, int n)
-  {
-  CheckParms(valp, n)
-  TYPE  tval = GetTypedValue(valp);
-  TYPE& tmin = Typp[n];
-
-  if (tval < tmin)
-    tmin = tval;
-
-  } // end of SetMin
-
-/***********************************************************************/
-/*  Set one value in a block if val is greater than the current value. */
-/***********************************************************************/
-template <class TYPE>
-void TYPBLK<TYPE>::SetMax(PVAL valp, int n)
-  {
-  CheckParms(valp, n)
-  TYPE  tval = GetTypedValue(valp);
-  TYPE& tmin = Typp[n];
-
-  if (tval > tmin)
-    tmin = tval;
-
-  } // end of SetMax
-
 #if 0
 /***********************************************************************/
 /*  Set many values in a block from values in another block.           */
@@ -616,8 +583,8 @@ int TYPBLK<TYPE>::GetMaxLength(void)
 /***********************************************************************/
 /*  Constructor.                                                       */
 /***********************************************************************/
-CHRBLK::CHRBLK(void *mp, int nval, int type, int len, int prec, bool blank)
-      : VALBLK(mp, type, nval), Chrp((char*&)Blkp)
+CHRBLK::CHRBLK(void *mp, int nval, int len, int prec, bool blank)
+      : VALBLK(mp, TYPE_STRING, nval), Chrp((char*&)Blkp)
   {
   Valp = NULL;
   Blanks = blank;
@@ -753,7 +720,7 @@ double CHRBLK::GetFloatValue(int n)
 /***********************************************************************/
 /*  STRING GetCharString: get string representation of a char value.   */
 /***********************************************************************/
-char *CHRBLK::GetCharString(char *, int n)
+char *CHRBLK::GetCharString(char *p, int n)
   {
   return (char *)GetValPtrEx(n);
   } // end of GetCharString
@@ -779,7 +746,7 @@ void CHRBLK::SetValue(PVAL valp, int n)
 /***********************************************************************/
 /*  Set one value in a block from a zero terminated string.            */
 /***********************************************************************/
-void CHRBLK::SetValue(PCSZ sp, int n)
+void CHRBLK::SetValue(PSZ sp, int n)
   {
   uint len = (sp) ? strlen(sp) : 0;
   SetValue(sp, len, n);
@@ -788,7 +755,7 @@ void CHRBLK::SetValue(PCSZ sp, int n)
 /***********************************************************************/
 /*  Set one value in a block from an array of characters.              */
 /***********************************************************************/
-void CHRBLK::SetValue(const char *sp, uint len, int n)
+void CHRBLK::SetValue(char *sp, uint len, int n)
   {
   char  *p = Chrp + n * Long;
 
@@ -796,8 +763,8 @@ void CHRBLK::SetValue(const char *sp, uint len, int n)
   if (Check && (signed)len > Long) {
     PGLOBAL& g = Global;
     strcpy(g->Message, MSG(SET_STR_TRUNC));
-		throw Type;
-	} // endif Check
+    longjmp(g->jumper[g->jump_level], Type);
+    } // endif Check
 #endif   // _DEBUG
 
   if (sp)
@@ -824,47 +791,17 @@ void CHRBLK::SetValue(PVBLK pv, int n1, int n2)
   if (Type != pv->GetType() || Long != ((CHRBLK*)pv)->Long) {
     PGLOBAL& g = Global;
     strcpy(g->Message, MSG(BLKTYPLEN_MISM));
-		throw Type;
-	} // endif Type
+    longjmp(g->jumper[g->jump_level], Type);
+    } // endif Type
 
-  if (!(b = pv->IsNull(n2)))
+  if (!(b = pv->IsNull(n2) && Nullable))
     memcpy(Chrp + n1 * Long, ((CHRBLK*)pv)->Chrp + n2 * Long, Long);
   else
     Reset(n1);
 
-  SetNull(n1, b && Nullable);
+  SetNull(n1, b);
   } // end of SetValue
 
-/***********************************************************************/
-/*  Set one value in a block if val is less than the current value.    */
-/***********************************************************************/
-void CHRBLK::SetMin(PVAL valp, int n)
-  {
-  CheckParms(valp, n)
-  CheckBlanks
-  char *vp = valp->GetCharValue();
-  char *bp = Chrp + n * Long;
-
-  if (((Ci) ? strnicmp(vp, bp, Long) : strncmp(vp, bp, Long)) < 0)
-    memcpy(bp, vp, Long);
-
-  } // end of SetMin
-
-/***********************************************************************/
-/*  Set one value in a block if val is greater than the current value. */
-/***********************************************************************/
-void CHRBLK::SetMax(PVAL valp, int n)
-  {
-  CheckParms(valp, n)
-  CheckBlanks
-  char *vp = valp->GetCharValue();
-  char *bp = Chrp + n * Long;
-
-  if (((Ci) ? strnicmp(vp, bp, Long) : strncmp(vp, bp, Long)) > 0)
-    memcpy(bp, vp, Long);
-
-  } // end of SetMax
-
 #if 0
 /***********************************************************************/
 /*  Set many values in a block from values in another block.           */
@@ -875,8 +812,8 @@ void CHRBLK::SetValues(PVBLK pv, int k, int n)
   if (Type != pv->GetType() || Long != ((CHRBLK*)pv)->Long) {
     PGLOBAL& g = Global;
     strcpy(g->Message, MSG(BLKTYPLEN_MISM));
-		throw Type;
-	} // endif Type
+    longjmp(g->jumper[g->jump_level], Type);
+    } // endif Type
 #endif   // _DEBUG
   char *p = ((CHRBLK*)pv)->Chrp;
 
@@ -1009,8 +946,8 @@ int CHRBLK::GetMaxLength(void)
 /***********************************************************************/
 /*  Constructor.                                                       */
 /***********************************************************************/
-STRBLK::STRBLK(PGLOBAL g, void *mp, int nval, int type)
-      : VALBLK(mp, type, nval), Strp((PSZ*&)Blkp)
+STRBLK::STRBLK(PGLOBAL g, void *mp, int nval)
+      : VALBLK(mp, TYPE_STRING, nval), Strp((PSZ*&)Blkp)
   {
   Global = g;
   Nullable = true;
@@ -1153,12 +1090,13 @@ void STRBLK::SetValue(PVAL valp, int n)
 /***********************************************************************/
 /*  Set one value in a block from a zero terminated string.            */
 /***********************************************************************/
-void STRBLK::SetValue(PCSZ p, int n)
+void STRBLK::SetValue(PSZ p, int n)
   {
   if (p) {
-    if (!Sorted || !n || !Strp[n-1] || strcmp(p, Strp[n-1]))
-      Strp[n] = (PSZ)PlugDup(Global, p);
-    else
+    if (!Sorted || !n || !Strp[n-1] || strcmp(p, Strp[n-1])) {
+      Strp[n] = (PSZ)PlugSubAlloc(Global, NULL, strlen(p) + 1);
+      strcpy(Strp[n], p);
+    } else
       Strp[n] = Strp[n-1];
 
   } else
@@ -1169,7 +1107,7 @@ void STRBLK::SetValue(PCSZ p, int n)
 /***********************************************************************/
 /*  Set one value in a block from an array of characters.              */
 /***********************************************************************/
-void STRBLK::SetValue(const char *sp, uint len, int n)
+void STRBLK::SetValue(char *sp, uint len, int n)
   {
   PSZ p;
 
@@ -1188,34 +1126,6 @@ void STRBLK::SetValue(const char *sp, uint len, int n)
   Strp[n] = p;
   } // end of SetValue
 
-/***********************************************************************/
-/*  Set one value in a block if val is less than the current value.    */
-/***********************************************************************/
-void STRBLK::SetMin(PVAL valp, int n)
-  {
-  CheckParms(valp, n)
-  char *vp = valp->GetCharValue();
-  char *bp = Strp[n];
-
-  if (strcmp(vp, bp) < 0)
-    SetValue(valp, n);
-
-  } // end of SetMin
-
-/***********************************************************************/
-/*  Set one value in a block if val is greater than the current value. */
-/***********************************************************************/
-void STRBLK::SetMax(PVAL valp, int n)
-  {
-  CheckParms(valp, n)
-  char *vp = valp->GetCharValue();
-  char *bp = Strp[n];
-
-  if (strcmp(vp, bp) > 0)
-    SetValue(valp, n);
-
-  } // end of SetMax
-
 /***********************************************************************/
 /*  Move one value from i to j.                                        */
 /***********************************************************************/
@@ -1317,7 +1227,7 @@ DATBLK::DATBLK(void *mp, int nval) : TYPBLK<int>(mp, nval, TYPE_INT)
 /***********************************************************************/
 /*  Set format so formatted dates can be converted on input.           */
 /***********************************************************************/
-bool DATBLK::SetFormat(PGLOBAL g, PCSZ fmt, int len, int year)
+bool DATBLK::SetFormat(PGLOBAL g, PSZ fmt, int len, int year)
   {
   if (!(Dvalp = AllocateValue(g, TYPE_DATE, len, year, false, fmt)))
     return true;
@@ -1344,7 +1254,7 @@ char *DATBLK::GetCharString(char *p, int n)
 /***********************************************************************/
 /*  Set one value in a block from a char string.                       */
 /***********************************************************************/
-void DATBLK::SetValue(PCSZ p, int n)
+void DATBLK::SetValue(PSZ p, int n)
   {
   if (Dvalp) {
     // Decode the string according to format
@@ -1355,61 +1265,5 @@ void DATBLK::SetValue(PCSZ p, int n)
 
   } // end of SetValue
 
-
-/* -------------------------- Class PTRBLK --------------------------- */
-
-/***********************************************************************/
-/*  Compare two values of the block.                                   */
-/***********************************************************************/
-int PTRBLK::CompVal(int i1, int i2)
-  {
-  return (Strp[i1] > Strp[i2]) ? 1 : (Strp[i1] < Strp[i2]) ? (-1) : 0;
-  } // end of CompVal
-
-
-/* -------------------------- Class MBVALS --------------------------- */
-
-/***********************************************************************/
-/*  Allocate a value block according to type,len, and nb of values.    */
-/***********************************************************************/
-PVBLK MBVALS::Allocate(PGLOBAL g, int type, int len, int prec,
-																	int n, bool sub)
-  {
-  Mblk.Sub = sub;
-  Mblk.Size = n * GetTypeSize(type, len);
-
-  if (!PlgDBalloc(g, NULL, Mblk)) {
-    sprintf(g->Message, MSG(ALLOC_ERROR), "MBVALS::Allocate");
-    return NULL;
-  } else
-    Vblk = AllocValBlock(g, Mblk.Memp, type, n, len, prec, 
-                            TRUE, TRUE, FALSE);
-
-  return Vblk;
-  } // end of Allocate
-
-/***********************************************************************/
-/*  Reallocate the value block according to the new size.              */
-/***********************************************************************/
-bool MBVALS::ReAllocate(PGLOBAL g, int n)
-  {
-  if (!PlgDBrealloc(g, NULL, Mblk, n * Vblk->GetVlen())) {
-    sprintf(g->Message, MSG(ALLOC_ERROR), "MBVALS::ReAllocate");
-    return TRUE;
-  } else
-    Vblk->ReAlloc(Mblk.Memp, n);
-
-  return FALSE;
-  } // end of ReAllocate
-
-/***********************************************************************/
-/*  Free the value block.                                              */
-/***********************************************************************/
-void MBVALS::Free(void)
-  {
-  PlgDBfree(Mblk);
-  Vblk = NULL;
-  } // end of Free
-
 /* ------------------------- End of Valblk --------------------------- */
 
