@@ -1,7 +1,7 @@
 /************* Value C++ Functions Source Code File (.CPP) *************/
 /*  Name: VALUE.CPP  Version 2.5                                       */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          2001-2015    */
+/*  (C) Copyright to the author Olivier BERTRAND          2001-2014    */
 /*                                                                     */
 /*  This file contains the VALUE and derived classes family functions. */
 /*  These classes contain values of different types. They are used so  */
@@ -30,11 +30,11 @@
 #include "sql_class.h"
 #include "sql_time.h"
 
-#if defined(__WIN__)
+#if defined(WIN32)
 //#include <windows.h>
-#else   // !__WIN__
+#else   // !WIN32
 #include <string.h>
-#endif  // !__WIN__
+#endif  // !WIN32
 
 #include <math.h>
 
@@ -48,6 +48,7 @@
 #include "global.h"
 #include "plgdbsem.h"
 #include "preparse.h"                     // For DATPAR
+//#include "value.h"
 #include "valblk.h"
 #define NO_FUNC                           // Already defined in ODBConn
 #include "plgcnx.h"                       // For DB types
@@ -67,6 +68,12 @@
 
 #define FOURYEARS    126230400    // Four years in seconds (1 leap)
 
+/***********************************************************************/
+/*  Static variables.                                                  */
+/***********************************************************************/
+
+extern "C" int  trace;
+
 /***********************************************************************/
 /*  Initialize the DTVAL static member.                                */
 /***********************************************************************/
@@ -77,12 +84,12 @@ int DTVAL::Shift = 0;
 /***********************************************************************/
 bool PlugEvalLike(PGLOBAL, LPCSTR, LPCSTR, bool);
 
-#if !defined(__WIN__)
+#if !defined(WIN32)
 extern "C" {
 PSZ strupr(PSZ s);
 PSZ strlwr(PSZ s);
 }
-#endif   // !__WIN__
+#endif   // !WIN32
 
 /***********************************************************************/
 /*  Get a long long number from its character representation.          */
@@ -94,7 +101,7 @@ PSZ strlwr(PSZ s);
 /*  OUT minus: Set to true if the number is negative                   */
 /*  Returned val: The resulting number                                 */
 /***********************************************************************/
-ulonglong CharToNumber(char *p, int n, ulonglong maxval,
+ulonglong CharToNumber(char *p, int n, ulonglong maxval, 
                        bool un, bool *minus, bool *rc)
 {
   char     *p2;
@@ -103,8 +110,7 @@ ulonglong CharToNumber(char *p, int n, ulonglong maxval,
 
   if (minus) *minus = false;
   if (rc) *rc = false;
-	if (n <= 0) return 0LL;
-
+  
   // Eliminate leading blanks or 0
   for (p2 = p + n; p < p2 && (*p == ' ' || *p == '0'); p++) ;
 
@@ -152,7 +158,6 @@ PSZ GetTypeName(int type)
     case TYPE_TINY:   name = "TINY";     break;
     case TYPE_DECIM:  name = "DECIMAL";  break;
     case TYPE_BIN:    name = "BINARY";   break;
-    case TYPE_PCHAR:  name = "PCHAR";    break;
     default:          name = "UNKNOWN";  break;
     } // endswitch type
 
@@ -174,7 +179,6 @@ int GetTypeSize(int type, int len)
     case TYPE_DATE:   len = sizeof(int);        break;
     case TYPE_DOUBLE: len = sizeof(double);     break;
     case TYPE_TINY:   len = sizeof(char);       break;
-    case TYPE_PCHAR:  len = sizeof(char*);      break;
     default:          len = 0;
     } // endswitch type
 
@@ -198,7 +202,6 @@ char *GetFormatType(int type)
     case TYPE_TINY:   c = "T"; break;
     case TYPE_DECIM:  c = "M"; break;
     case TYPE_BIN:    c = "B"; break;
-    case TYPE_PCHAR:  c = "P"; break;
     } // endswitch type
 
   return c;
@@ -221,7 +224,6 @@ int GetFormatType(char c)
     case 'T': type = TYPE_TINY;   break;
     case 'M': type = TYPE_DECIM;  break;
     case 'B': type = TYPE_BIN;    break;
-    case 'P': type = TYPE_PCHAR;  break;
     } // endswitch type
 
   return type;
@@ -281,57 +283,10 @@ const char *GetFmt(int type, bool un)
   return fmt;
   } // end of GetFmt
 
-/***********************************************************************/
-/*  ConvertType: what this function does is to determine the type to   */
-/*  which should be converted a value so no precision would be lost.   */
-/*  This can be a numeric type if num is true or non numeric if false. */
-/*  Note: this is an ultra simplified version of this function that    */
-/*  should become more and more complex as new types are added.        */
-/*  Not evaluated types (TYPE_VOID or TYPE_UNDEF) return false from    */
-/*  IsType... functions so match does not prevent correct setting.     */
-/***********************************************************************/
-int ConvertType(int target, int type, CONV kind, bool match)
-  {
-  switch (kind) {
-    case CNV_CHAR:
-      if (match && (!IsTypeChar(target) || !IsTypeChar(type)))
-        return TYPE_ERROR;
-
-      return TYPE_STRING;
-    case CNV_NUM:
-      if (match && (!IsTypeNum(target) || !IsTypeNum(type)))
-        return TYPE_ERROR;
-
-      return (target == TYPE_DOUBLE || type == TYPE_DOUBLE) ? TYPE_DOUBLE
-           : (target == TYPE_DATE   || type == TYPE_DATE)   ? TYPE_DATE
-           : (target == TYPE_BIGINT || type == TYPE_BIGINT) ? TYPE_BIGINT
-           : (target == TYPE_INT    || type == TYPE_INT)    ? TYPE_INT
-           : (target == TYPE_SHORT  || type == TYPE_SHORT)  ? TYPE_SHORT
-                                                            : TYPE_TINY;
-    default:
-      if (target == TYPE_ERROR || target == type)
-        return type;
-
-      if (match && ((IsTypeChar(target) && !IsTypeChar(type)) ||
-                    (IsTypeNum(target) && !IsTypeNum(type))))
-        return TYPE_ERROR;
-
-      return (target == TYPE_DOUBLE || type == TYPE_DOUBLE) ? TYPE_DOUBLE
-           : (target == TYPE_DATE   || type == TYPE_DATE)   ? TYPE_DATE
-           : (target == TYPE_BIGINT || type == TYPE_BIGINT) ? TYPE_BIGINT
-           : (target == TYPE_INT    || type == TYPE_INT)    ? TYPE_INT
-           : (target == TYPE_SHORT  || type == TYPE_SHORT)  ? TYPE_SHORT
-           : (target == TYPE_STRING || type == TYPE_STRING) ? TYPE_STRING
-           : (target == TYPE_TINY   || type == TYPE_TINY)   ? TYPE_TINY
-                                                            : TYPE_ERROR;
-    } // endswitch kind
-
-  } // end of ConvertType
-
 /***********************************************************************/
 /*  AllocateConstant: allocates a constant Value.                      */
 /***********************************************************************/
-PVAL AllocateValue(PGLOBAL g, void *value, short type, short prec)
+PVAL AllocateValue(PGLOBAL g, void *value, short type)
   {
   PVAL valp;
 
@@ -340,19 +295,19 @@ PVAL AllocateValue(PGLOBAL g, void *value, short type, short prec)
 
   switch (type) {
     case TYPE_STRING:
-      valp = new(g) TYPVAL<PSZ>((PSZ)value, prec);
+      valp = new(g) TYPVAL<PSZ>((PSZ)value);
       break;
     case TYPE_SHORT:
       valp = new(g) TYPVAL<short>(*(short*)value, TYPE_SHORT);
       break;
-    case TYPE_INT:
+    case TYPE_INT: 
       valp = new(g) TYPVAL<int>(*(int*)value, TYPE_INT);
       break;
     case TYPE_BIGINT:
       valp = new(g) TYPVAL<longlong>(*(longlong*)value, TYPE_BIGINT);
       break;
     case TYPE_DOUBLE:
-      valp = new(g) TYPVAL<double>(*(double *)value, TYPE_DOUBLE, prec);
+      valp = new(g) TYPVAL<double>(*(double *)value, TYPE_DOUBLE, 2);
       break;
     case TYPE_TINY:
       valp = new(g) TYPVAL<char>(*(char *)value, TYPE_TINY);
@@ -378,10 +333,10 @@ PVAL AllocateValue(PGLOBAL g, int type, int len, int prec,
     case TYPE_STRING:
       valp = new(g) TYPVAL<PSZ>(g, (PSZ)NULL, len, prec);
       break;
-    case TYPE_DATE:
+    case TYPE_DATE: 
       valp = new(g) DTVAL(g, len, prec, fmt);
       break;
-    case TYPE_INT:
+    case TYPE_INT: 
       if (uns)
         valp = new(g) TYPVAL<uint>((uint)0, TYPE_INT, 0, true);
       else
@@ -427,80 +382,6 @@ PVAL AllocateValue(PGLOBAL g, int type, int len, int prec,
   return valp;
   } // end of AllocateValue
 
-/***********************************************************************/
-/*  Allocate a constant Value converted to newtype.                    */
-/*  Can also be used to copy a Value eventually converted.             */
-/***********************************************************************/
-PVAL AllocateValue(PGLOBAL g, PVAL valp, int newtype, int uns)
-  {
-  PSZ  p, sp;
-  bool un = (uns < 0) ? false : (uns > 0) ? true : valp->IsUnsigned();
-  PVAL vp;
-
-  if (!valp)
-    return NULL;
-    
-  if (newtype == TYPE_VOID)  // Means allocate a value of the same type
-    newtype = valp->GetType();
-
-  switch (newtype) {
-    case TYPE_STRING:
-      p = (PSZ)PlugSubAlloc(g, NULL, 1 + valp->GetValLen());
-
-      if ((sp = valp->GetCharString(p)) != p && sp)
-        strcpy(p, sp);
-
-      vp = new(g) TYPVAL<PSZ>(g, p, valp->GetValLen(), valp->GetValPrec());
-      break;
-    case TYPE_SHORT:
-      if (un)
-        vp = new(g) TYPVAL<ushort>(valp->GetUShortValue(),
-                                   TYPE_SHORT, 0, true);
-      else
-        vp = new(g) TYPVAL<short>(valp->GetShortValue(), TYPE_SHORT);
-
-      break;
-    case TYPE_INT:
-      if (un)
-        vp = new(g) TYPVAL<uint>(valp->GetUIntValue(), TYPE_INT, 0, true);
-      else
-        vp = new(g) TYPVAL<int>(valp->GetIntValue(), TYPE_INT);
-
-      break;
-    case TYPE_BIGINT:
-      if (un)
-        vp = new(g) TYPVAL<ulonglong>(valp->GetUBigintValue(),
-                                      TYPE_BIGINT, 0, true);
-      else
-        vp = new(g) TYPVAL<longlong>(valp->GetBigintValue(), TYPE_BIGINT);
-
-      break;
-    case TYPE_DATE:
-      vp = new(g) DTVAL(valp->GetIntValue());
-      break;
-    case TYPE_DOUBLE:
-      vp = new(g) TYPVAL<double>(valp->GetFloatValue(), TYPE_DOUBLE,
-                   (uns) ? uns : valp->GetValPrec());
-      break;
-    case TYPE_TINY:
-      if (un)
-        vp = new(g) TYPVAL<uchar>(valp->GetUTinyValue(),
-                                    TYPE_TINY, 0, true);
-      else
-        vp = new(g) TYPVAL<char>(valp->GetTinyValue(), TYPE_TINY);
-
-      break;
-    default:
-      sprintf(g->Message, MSG(BAD_VALUE_TYPE), newtype);
-      return NULL;
-    } // endswitch type
-  
-  vp->SetNullable(valp->GetNullable());
-  vp->SetNull(valp->IsNull());
-  vp->SetGlobal(g);
-  return vp;
-  } // end of AllocateValue
-
 /* -------------------------- Class VALUE ---------------------------- */
 
 /***********************************************************************/
@@ -537,27 +418,6 @@ const char *VALUE::GetXfmt(void)
   return fmt;
   } // end of GetFmt
 
-/***********************************************************************/
-/*  Returns a BYTE indicating the comparison between two values.       */
-/*  Bit 1 indicates equality, Bit 2 less than, and Bit3 greater than.  */
-/*  More than 1 bit can be set only in the case of TYPE_LIST.          */
-/***********************************************************************/
-BYTE VALUE::TestValue(PVAL vp)
-  {
-  int n = CompareValue(vp);
-
-  return (n > 0) ? 0x04 : (n < 0) ? 0x02 : 0x01;
-  } // end of TestValue
-
-/***********************************************************************/
-/*  Compute a function on a string.                                    */
-/***********************************************************************/
-bool VALUE::Compute(PGLOBAL g, PVAL *, int, OPVAL)
-  {
-  strcpy(g->Message, "Compute not implemented for this value type");
-  return true;
-  } // end of Compute
-
 /* -------------------------- Class TYPVAL ---------------------------- */
 
 /***********************************************************************/
@@ -628,16 +488,13 @@ int TYPVAL<double>::GetValLen(void)
 template <class TYPE>
 bool TYPVAL<TYPE>::SetValue_pval(PVAL valp, bool chktype)
   {
-  if (valp != this) {
-    if (chktype && Type != valp->GetType())
-      return true;
+  if (chktype && Type != valp->GetType())
+    return true;
 
-    if (!(Null = valp->IsNull() && Nullable))
-      Tval = GetTypedValue(valp);
-    else
-      Reset();
-
-    } // endif valp
+  if (!(Null = valp->IsNull() && Nullable))
+    Tval = GetTypedValue(valp);
+  else
+    Reset();
 
   return false;
   } // end of SetValue
@@ -686,8 +543,8 @@ bool TYPVAL<TYPE>::SetValue_char(char *p, int n)
   {
   bool      rc, minus;
   ulonglong maxval = MaxVal();
-  ulonglong val = CharToNumber(p, n, maxval, Unsigned, &minus, &rc);
-
+  ulonglong val = CharToNumber(p, n, maxval, Unsigned, &minus, &rc); 
+    
   if (minus && val < maxval)
     Tval = (TYPE)(-(signed)val);
   else
@@ -706,10 +563,10 @@ bool TYPVAL<TYPE>::SetValue_char(char *p, int n)
 template <>
 bool TYPVAL<double>::SetValue_char(char *p, int n)
   {
-  if (p && n > 0) {
+  if (p) {
     char buf[64];
 
-    for (; n > 0 && *p == ' '; p++)
+    for (; n > 0 && *p == ' '; p++) 
       n--;
 
     memcpy(buf, p, MY_MIN(n, 31));
@@ -931,220 +788,6 @@ bool TYPVAL<TYPE>::IsEqual(PVAL vp, bool chktype)
 
   } // end of IsEqual
 
-/***********************************************************************/
-/*  Compare values and returns 1, 0 or -1 according to comparison.     */
-/*  This function is used for evaluation of numeric filters.           */
-/***********************************************************************/
-template <class TYPE>
-int TYPVAL<TYPE>::CompareValue(PVAL vp)
-  {
-//assert(vp->GetType() == Type);
-
-  // Process filtering on numeric values.
-  TYPE n = GetTypedValue(vp);
-
-//if (trace)
-//  htrc(" Comparing: val=%d,%d\n", Tval, n);
-
-  return (Tval > n) ? 1 : (Tval < n) ? (-1) : 0;
-  } // end of CompareValue
-
-/***********************************************************************/
-/*  Return max type value if b is true, else min type value.           */
-/***********************************************************************/
-template <>
-short TYPVAL<short>::MinMaxVal(bool b)
-  {return (b) ? INT_MAX16 : INT_MIN16;}
-
-template <>
-ushort TYPVAL<ushort>::MinMaxVal(bool b)
-  {return (b) ? UINT_MAX16 : 0;}
-
-template <>
-int TYPVAL<int>::MinMaxVal(bool b)
-  {return (b) ? INT_MAX32 : INT_MIN32;}
-
-template <>
-uint TYPVAL<uint>::MinMaxVal(bool b)
-  {return (b) ? UINT_MAX32 : 0;}
-
-template <>
-longlong TYPVAL<longlong>::MinMaxVal(bool b)
-  {return (b) ? INT_MAX64 : INT_MIN64;}
-
-template <>
-ulonglong TYPVAL<ulonglong>::MinMaxVal(bool b)
-  {return (b) ? 0xFFFFFFFFFFFFFFFFLL : 0;}
-
-template <>
-double TYPVAL<double>::MinMaxVal(bool)
-  {assert(false); return 0.0;}
-
-template <>
-char TYPVAL<char>::MinMaxVal(bool b)
-  {return (b) ? INT_MAX8 : INT_MIN8;}
-
-template <>
-uchar TYPVAL<uchar>::MinMaxVal(bool b)
-  {return (b) ? UINT_MAX8 : 0;}
-
-/***********************************************************************/
-/*  SafeAdd: adds a value and test whether overflow/underflow occured. */
-/***********************************************************************/
-template <class TYPE>
-TYPE TYPVAL<TYPE>::SafeAdd(TYPE n1, TYPE n2)
-  {
-  PGLOBAL& g = Global;
-  TYPE     n = n1 + n2;
-
-  if ((n2 > 0) && (n < n1)) {
-    // Overflow
-    strcpy(g->Message, MSG(FIX_OVFLW_ADD));
-    longjmp(g->jumper[g->jump_level], 138);
-  } else if ((n2 < 0) && (n > n1)) {
-    // Underflow
-    strcpy(g->Message, MSG(FIX_UNFLW_ADD));
-    longjmp(g->jumper[g->jump_level], 138);
-  } // endif's n2
-
-  return n;
-  } // end of SafeAdd
-
-template <>
-inline double TYPVAL<double>::SafeAdd(double n1, double n2)
-  {
-  return n1 + n2;
-  } // end of SafeAdd
-
-/***********************************************************************/
-/*  SafeMult: multiply values and test whether overflow occured.       */
-/***********************************************************************/
-template <class TYPE>
-TYPE TYPVAL<TYPE>::SafeMult(TYPE n1, TYPE n2)
-  {
-  PGLOBAL& g = Global;
-  double   n = (double)n1 * (double)n2;
-
-  if (n > MinMaxVal(true)) {
-    // Overflow
-    strcpy(g->Message, MSG(FIX_OVFLW_TIMES));
-    longjmp(g->jumper[g->jump_level], 138);
-  } else if (n < MinMaxVal(false)) {
-    // Underflow
-    strcpy(g->Message, MSG(FIX_UNFLW_TIMES));
-    longjmp(g->jumper[g->jump_level], 138);
-  } // endif's n2
-
-  return (TYPE)n;
-  } // end of SafeMult
-
-template <>
-inline double TYPVAL<double>::SafeMult(double n1, double n2)
-  {
-  return n1 * n2;
-  } // end of SafeMult
-
-/***********************************************************************/
-/*  Compute defined functions for the type.                            */
-/***********************************************************************/
-template <class TYPE>
-bool TYPVAL<TYPE>::Compute(PGLOBAL g, PVAL *vp, int np, OPVAL op)
-  {
-  bool rc = false;
-  TYPE val[2];
-
-  assert(np == 2);
-
-  for (int i = 0; i < np; i++)
-    val[i] = GetTypedValue(vp[i]);
-
-  switch (op) {
-    case OP_ADD:
-      Tval = SafeAdd(val[0], val[1]);
-      break;
-    case OP_MULT:
-      Tval = SafeMult(val[0], val[1]);
-      break;
-    case OP_DIV:
-      if (!val[1]) {
-        strcpy(g->Message, MSG(ZERO_DIVIDE));
-        return true;
-        } // endif
-
-      Tval = val[0] / val[1];
-      break;
-    default:
-      rc = Compall(g, vp, np, op);
-      break;
-    } // endswitch op
-
-  return rc;
-  } // end of Compute
-
-template <>
-bool TYPVAL<double>::Compute(PGLOBAL g, PVAL *vp, int np, OPVAL op)
-  {
-  bool   rc = false;
-  double val[2];
-
-  assert(np == 2);
-
-  for (int i = 0; i < np; i++)
-    val[i] = vp[i]->GetFloatValue();
-
-  switch (op) {
-    case OP_ADD:
-      Tval = val[0] + val[1];
-      break;
-    case OP_MULT:
-      Tval = val[0] * val[1];
-      break;
-    default:
-      rc = Compall(g, vp, np, op);
-    } // endswitch op
-
-  return rc;
-  } // end of Compute
-
-/***********************************************************************/
-/*  Compute a function for all types.                                  */
-/***********************************************************************/
-template <class TYPE>
-bool TYPVAL<TYPE>::Compall(PGLOBAL g, PVAL *vp, int np, OPVAL op)
-  {
-  TYPE val[2];
-
-  for (int i = 0; i < np; i++)
-    val[i] = GetTypedValue(vp[i]);
-
-  switch (op) {
-    case OP_DIV:
-      if (val[0]) {
-        if (!val[1]) {
-          strcpy(g->Message, MSG(ZERO_DIVIDE));
-          return true;
-          } // endif
-    
-        Tval = val[0] / val[1];
-      } else
-        Tval = 0;
-
-      break;
-    case OP_MIN:
-      Tval = MY_MIN(val[0], val[1]);
-      break;
-    case OP_MAX:
-      Tval = MY_MAX(val[0], val[1]);
-      break;
-    default:
-//    sprintf(g->Message, MSG(BAD_EXP_OPER), op);
-      strcpy(g->Message, "Function not supported");
-      return true;
-    } // endswitch op
-
-  return false;
-  } // end of Compall
-
 /***********************************************************************/
 /*  FormatValue: This function set vp (a STRING value) to the string   */
 /*  constructed from its own value formated using the fmt format.      */
@@ -1209,12 +852,12 @@ void TYPVAL<TYPE>::Print(PGLOBAL g, char *ps, uint z)
 /***********************************************************************/
 /*  STRING  public constructor from a constant string.                 */
 /***********************************************************************/
-TYPVAL<PSZ>::TYPVAL(PSZ s, short c) : VALUE(TYPE_STRING)
+TYPVAL<PSZ>::TYPVAL(PSZ s) : VALUE(TYPE_STRING)
   {
   Strp = s;
   Len = strlen(s);
   Clen = Len;
-  Ci = (c == 1);
+  Ci = false;
   } // end of STRING constructor
 
 /***********************************************************************/
@@ -1223,18 +866,15 @@ TYPVAL<PSZ>::TYPVAL(PSZ s, short c) : VALUE(TYPE_STRING)
 TYPVAL<PSZ>::TYPVAL(PGLOBAL g, PSZ s, int n, int c)
            : VALUE(TYPE_STRING)
   {
-  Len = (g) ? n : (s) ? strlen(s) : 0;
+  Len = (g) ? n : strlen(s);
 
   if (!s) {
     if (g) {
-      if ((Strp = (char *)PlgDBSubAlloc(g, NULL, Len + 1)))
-        memset(Strp, 0, Len + 1);
-      else
-        Len = 0;
-
-    } else
-      assert(false);
-
+	    Strp = (char *)PlugSubAlloc(g, NULL, Len + 1);
+  	  Strp[Len] = '\0';
+  	} else
+  	  assert(false);
+  	  
   } else
     Strp = s;
 
@@ -1248,8 +888,8 @@ TYPVAL<PSZ>::TYPVAL(PGLOBAL g, PSZ s, int n, int c)
 char TYPVAL<PSZ>::GetTinyValue(void)
   {
   bool      m;
-  ulonglong val = CharToNumber(Strp, strlen(Strp), INT_MAX8, false, &m);
-
+  ulonglong val = CharToNumber(Strp, strlen(Strp), INT_MAX8, false, &m); 
+    
   return (m && val < INT_MAX8) ? (char)(-(signed)val) : (char)val;
   } // end of GetTinyValue
 
@@ -1258,7 +898,7 @@ char TYPVAL<PSZ>::GetTinyValue(void)
 /***********************************************************************/
 uchar TYPVAL<PSZ>::GetUTinyValue(void)
   {
-  return (uchar)CharToNumber(Strp, strlen(Strp), UINT_MAX8, true);
+  return (uchar)CharToNumber(Strp, strlen(Strp), UINT_MAX8, true); 
   } // end of GetUTinyValue
 
 /***********************************************************************/
@@ -1267,8 +907,8 @@ uchar TYPVAL<PSZ>::GetUTinyValue(void)
 short TYPVAL<PSZ>::GetShortValue(void)
   {
   bool      m;
-  ulonglong val = CharToNumber(Strp, strlen(Strp), INT_MAX16, false, &m);
-
+  ulonglong val = CharToNumber(Strp, strlen(Strp), INT_MAX16, false, &m); 
+    
   return (m && val < INT_MAX16) ? (short)(-(signed)val) : (short)val;
   } // end of GetShortValue
 
@@ -1277,7 +917,7 @@ short TYPVAL<PSZ>::GetShortValue(void)
 /***********************************************************************/
 ushort TYPVAL<PSZ>::GetUShortValue(void)
   {
-  return (ushort)CharToNumber(Strp, strlen(Strp), UINT_MAX16, true);
+  return (ushort)CharToNumber(Strp, strlen(Strp), UINT_MAX16, true); 
   } // end of GetUshortValue
 
 /***********************************************************************/
@@ -1286,8 +926,8 @@ ushort TYPVAL<PSZ>::GetUShortValue(void)
 int TYPVAL<PSZ>::GetIntValue(void)
   {
   bool      m;
-  ulonglong val = CharToNumber(Strp, strlen(Strp), INT_MAX32, false, &m);
-
+  ulonglong val = CharToNumber(Strp, strlen(Strp), INT_MAX32, false, &m); 
+    
   return (m && val < INT_MAX32) ? (int)(-(signed)val) : (int)val;
   } // end of GetIntValue
 
@@ -1296,7 +936,7 @@ int TYPVAL<PSZ>::GetIntValue(void)
 /***********************************************************************/
 uint TYPVAL<PSZ>::GetUIntValue(void)
   {
-  return (uint)CharToNumber(Strp, strlen(Strp), UINT_MAX32, true);
+  return (uint)CharToNumber(Strp, strlen(Strp), UINT_MAX32, true); 
   } // end of GetUintValue
 
 /***********************************************************************/
@@ -1305,8 +945,8 @@ uint TYPVAL<PSZ>::GetUIntValue(void)
 longlong TYPVAL<PSZ>::GetBigintValue(void)
   {
   bool      m;
-  ulonglong val = CharToNumber(Strp, strlen(Strp), INT_MAX64, false, &m);
-
+  ulonglong val = CharToNumber(Strp, strlen(Strp), INT_MAX64, false, &m); 
+    
   return (m && val < INT_MAX64) ? (-(signed)val) : (longlong)val;
   } // end of GetBigintValue
 
@@ -1315,7 +955,7 @@ longlong TYPVAL<PSZ>::GetBigintValue(void)
 /***********************************************************************/
 ulonglong TYPVAL<PSZ>::GetUBigintValue(void)
   {
-  return CharToNumber(Strp, strlen(Strp), ULONGLONG_MAX, true);
+  return CharToNumber(Strp, strlen(Strp), ULONGLONG_MAX, true); 
   } // end of GetUBigintValue
 
 /***********************************************************************/
@@ -1323,18 +963,15 @@ ulonglong TYPVAL<PSZ>::GetUBigintValue(void)
 /***********************************************************************/
 bool TYPVAL<PSZ>::SetValue_pval(PVAL valp, bool chktype)
   {
-  if (valp != this) {
-    if (chktype && (valp->GetType() != Type || valp->GetSize() > Len))
-      return true;
+  if (chktype && (valp->GetType() != Type || valp->GetSize() > Len))
+    return true;
 
-    char buf[64];
+  char buf[64];
 
-    if (!(Null = valp->IsNull() && Nullable))
-      strncpy(Strp, valp->GetCharString(buf), Len);
-    else
-      Reset();
-
-    } // endif valp
+  if (!(Null = valp->IsNull() && Nullable))
+    strncpy(Strp, valp->GetCharString(buf), Len);
+  else
+    Reset();
 
   return false;
   } // end of SetValue_pval
@@ -1346,24 +983,24 @@ bool TYPVAL<PSZ>::SetValue_char(char *p, int n)
   {
   bool rc;
 
-  if (p && n > 0) {
+  if (p) {
     rc = n > Len;
 
     if ((n = MY_MIN(n, Len))) {
     	strncpy(Strp, p, n);
 
-//    for (p = Strp + n - 1; p >= Strp && (*p == ' ' || *p == '\0'); p--) ;
+//	  for (p = Strp + n - 1; p >= Strp && (*p == ' ' || *p == '\0'); p--) ;
       for (p = Strp + n - 1; p >= Strp; p--)
         if (*p && *p != ' ')
           break;
 
-      *(++p) = '\0';
+	    *(++p) = '\0';
 
-      if (trace > 1)
-        htrc(" Setting string to: '%s'\n", Strp);
-
-    } else
-      Reset();
+	    if (trace > 1)
+	      htrc(" Setting string to: '%s'\n", Strp);
+	      
+	  } else
+	  	Reset();
 
     Null = false;
   } else {
@@ -1567,7 +1204,7 @@ bool TYPVAL<PSZ>::GetBinValue(void *buf, int buflen, bool go)
 /***********************************************************************/
 /*  STRING ShowValue: get string representation of a char value.       */
 /***********************************************************************/
-char *TYPVAL<PSZ>::ShowValue(char *, int)
+char *TYPVAL<PSZ>::ShowValue(char *buf, int len)
   {
   return Strp;
   } // end of ShowValue
@@ -1575,7 +1212,7 @@ char *TYPVAL<PSZ>::ShowValue(char *, int)
 /***********************************************************************/
 /*  STRING GetCharString: get string representation of a char value.   */
 /***********************************************************************/
-char *TYPVAL<PSZ>::GetCharString(char *)
+char *TYPVAL<PSZ>::GetCharString(char *p)
   {
   return Strp;
   } // end of GetCharString
@@ -1601,71 +1238,6 @@ bool TYPVAL<PSZ>::IsEqual(PVAL vp, bool chktype)
 
   } // end of IsEqual
 
-/***********************************************************************/
-/*  Compare values and returns 1, 0 or -1 according to comparison.     */
-/*  This function is used for evaluation of numeric filters.           */
-/***********************************************************************/
-int TYPVAL<PSZ>::CompareValue(PVAL vp)
-  {
-  int n;
-//assert(vp->GetType() == Type);
-
-  if (trace)
-    htrc(" Comparing: val='%s','%s'\n", Strp, vp->GetCharValue());
-
-  // Process filtering on character strings.
-  if (Ci || vp->IsCi())
-    n = stricmp(Strp, vp->GetCharValue());
-  else
-    n = strcmp(Strp, vp->GetCharValue());
-
-#if defined(__WIN__)
-  if (n == _NLSCMPERROR)
-    return n;                        // Here we should raise an error
-#endif   // __WIN__
-
-  return (n > 0) ? 1 : (n < 0) ? -1 : 0;
-  } // end of CompareValue
-
-/***********************************************************************/
-/*  Compute a function on a string.                                    */
-/***********************************************************************/
-bool TYPVAL<PSZ>::Compute(PGLOBAL g, PVAL *vp, int np, OPVAL op)
-  {
-  char *p[2], val[2][32];
-  int   i;
-
-  for (i = 0; i < np; i++)
-    p[i] = vp[i]->GetCharString(val[i]);
-
-  switch (op) {
-    case OP_CNC:
-      assert(np == 1 || np == 2);
-
-      if (np == 2)
-        strncpy(Strp, p[0], Len);
-
-      if ((i = Len - (signed)strlen(Strp)) > 0)
-        strncat(Strp, p[np - 1], i);
-
-      break;
-    case OP_MIN:
-      assert(np == 2);
-      strcpy(Strp, (strcmp(p[0], p[1]) < 0) ? p[0] : p[1]);
-      break;
-    case OP_MAX:
-      assert(np == 2);
-      strcpy(Strp, (strcmp(p[0], p[1]) > 0) ? p[0] : p[1]);
-      break;
-    default:
-//    sprintf(g->Message, MSG(BAD_EXP_OPER), op);
-      strcpy(g->Message, "Function not supported");
-      return true;
-    } // endswitch op
-
-  return false;
-  } // end of Compute
-
 /***********************************************************************/
 /*  FormatValue: This function set vp (a STRING value) to the string   */
 /*  constructed from its own value formated using the fmt format.      */
@@ -1682,7 +1254,7 @@ bool TYPVAL<PSZ>::FormatValue(PVAL vp, char *fmt)
 /***********************************************************************/
 /*  STRING SetFormat function (used to set SELECT output format).      */
 /***********************************************************************/
-bool TYPVAL<PSZ>::SetConstFormat(PGLOBAL, FORMAT& fmt)
+bool TYPVAL<PSZ>::SetConstFormat(PGLOBAL g, FORMAT& fmt)
   {
   fmt.Type[0] = 'C';
   fmt.Length = Len;
@@ -1732,7 +1304,7 @@ bool DECVAL::IsZero(void)
 /***********************************************************************/
 /*  DECIMAL: Reset value to zero.                                      */
 /***********************************************************************/
-void DECVAL::Reset(void)
+void DECVAL::Reset(void) 
 {
   int i = 0;
 
@@ -1805,24 +1377,24 @@ bool DECVAL::SetValue_char(char *p, int n)
   {
   bool rc;
 
-  if (p && n > 0) {
+  if (p) {
     rc = n > Len;
 
     if ((n = MY_MIN(n, Len))) {
     	strncpy(Strp, p, n);
 
-//    for (p = Strp + n - 1; p >= Strp && (*p == ' ' || *p == '\0'); p--) ;
+//	  for (p = Strp + n - 1; p >= Strp && (*p == ' ' || *p == '\0'); p--) ;
       for (p = Strp + n - 1; p >= Strp; p--)
         if (*p && *p != ' ')
           break;
 
-      *(++p) = '\0';
+	    *(++p) = '\0';
 
-      if (trace > 1)
-        htrc(" Setting string to: '%s'\n", Strp);
-
-    } else
-      Reset();
+	    if (trace > 1)
+	      htrc(" Setting string to: '%s'\n", Strp);
+	      
+	  } else
+	  	Reset();
 
     Null = false;
   } else {
@@ -1892,23 +1464,6 @@ bool DECVAL::IsEqual(PVAL vp, bool chktype)
   return !strcmp(Strp, vp->GetCharString(buf));
   } // end of IsEqual
 
-/***********************************************************************/
-/*  Compare values and returns 1, 0 or -1 according to comparison.     */
-/*  This function is used for evaluation of numeric filters.           */
-/***********************************************************************/
-int DECVAL::CompareValue(PVAL vp)
-  {
-//assert(vp->GetType() == Type);
-
-  // Process filtering on numeric values.
-  double f = atof(Strp), n = vp->GetFloatValue();
-
-//if (trace)
-//  htrc(" Comparing: val=%d,%d\n", f, n);
-
-  return (f > n) ? 1 : (f < n) ? (-1) : 0;
-  } // end of CompareValue
-
 #if 0
 /***********************************************************************/
 /*  FormatValue: This function set vp (a STRING value) to the string   */
@@ -2070,21 +1625,18 @@ double BINVAL::GetFloatValue(void)
 /***********************************************************************/
 bool BINVAL::SetValue_pval(PVAL valp, bool chktype)
   {
+  if (chktype && (valp->GetType() != Type || valp->GetSize() > Clen))
+    return true;
+
   bool rc = false;
-      
-  if (valp != this) {
-    if (chktype && (valp->GetType() != Type || valp->GetSize() > Clen))
-      return true;
+    
+  if (!(Null = valp->IsNull() && Nullable)) {
+    if ((rc = (Len = valp->GetSize()) > Clen))
+      Len = Clen;
 
-    if (!(Null = valp->IsNull() && Nullable)) {
-      if ((rc = (Len = valp->GetSize()) > Clen))
-        Len = Clen;
-
-      memcpy(Binp, valp->GetTo_Val(), Len);
-    } else
-      Reset();
-
-    } // endif valp
+    memcpy(Binp, valp->GetTo_Val(), Len);
+  } else
+    Reset();
 
   return rc;
   } // end of SetValue_pval
@@ -2096,7 +1648,7 @@ bool BINVAL::SetValue_char(char *p, int n)
   {
   bool rc;
 
-  if (p && n > 0) {
+  if (p) {
     rc = n > Clen;
     Len = MY_MIN(n, Clen);
     memcpy(Binp, p, Len);
@@ -2301,7 +1853,7 @@ char *BINVAL::ShowValue(char *buf, int len)
 /***********************************************************************/
 /*  BINVAL GetCharString: get string representation of a binary value. */
 /***********************************************************************/
-char *BINVAL::GetCharString(char *)
+char *BINVAL::GetCharString(char *p)
   {
   if (!Chrp)
     Chrp = (char*)PlugSubAlloc(Global, NULL, Clen * 2 + 1);
@@ -2350,7 +1902,7 @@ bool BINVAL::FormatValue(PVAL vp, char *fmt)
 /***********************************************************************/
 /*  BINVAL SetFormat function (used to set SELECT output format).      */
 /***********************************************************************/
-bool BINVAL::SetConstFormat(PGLOBAL, FORMAT& fmt)
+bool BINVAL::SetConstFormat(PGLOBAL g, FORMAT& fmt)
   {
   fmt.Type[0] = 'B';
   fmt.Length = Clen;
@@ -2380,7 +1932,7 @@ DTVAL::DTVAL(PGLOBAL g, int n, int prec, PSZ fmt)
 /***********************************************************************/
 /*  DTVAL  public constructor from int.                                */
 /***********************************************************************/
-DTVAL::DTVAL(int n) : TYPVAL<int>(n, TYPE_DATE)
+DTVAL::DTVAL(PGLOBAL g, int n) : TYPVAL<int>(n, TYPE_DATE)
   {
   Pdtp = NULL;
   Len = 19;
@@ -2447,11 +1999,10 @@ static void TIME_to_localtime(struct tm *tm, const MYSQL_TIME *ltime)
   tm->tm_year= ltime->year - 1900;
   tm->tm_mon=  ltime->month - 1;
   tm->tm_mday= ltime->day;
-  mktime(tm);  // set tm->tm_wday tm->yday fields to get proper day name (OB)
   tm->tm_hour= ltime->hour;
   tm->tm_min=  ltime->minute;
   tm->tm_sec=  ltime->second;
-} // end of TIME_to_localtime
+}
 
 // Added by Alexander Barkov
 static struct tm *gmtime_mysql(const time_t *timep, struct tm *tm)
@@ -2460,7 +2011,7 @@ static struct tm *gmtime_mysql(const time_t *timep, struct tm *tm)
   thd_gmt_sec_to_TIME(current_thd, &ltime, (my_time_t) *timep);
   TIME_to_localtime(tm, &ltime);
   return tm;
-} // end of gmtime_mysql
+}
 
 /***********************************************************************/
 /*  GetGmTime: returns a pointer to a static tm structure obtained     */
@@ -2511,7 +2062,7 @@ bool DTVAL::MakeTime(struct tm *ptm)
   time_t t = mktime_mysql(ptm);
 
   if (trace > 1)
-    htrc("MakeTime from (%d,%d,%d,%d,%d,%d)\n",
+    htrc("MakeTime from (%d,%d,%d,%d,%d,%d)\n", 
           ptm->tm_year, ptm->tm_mon, ptm->tm_mday,
           ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
 
@@ -2534,7 +2085,7 @@ bool DTVAL::MakeTime(struct tm *ptm)
   Tval= (int) t;
 
   if (trace > 1)
-    htrc("MakeTime Ival=%d\n", Tval);
+    htrc("MakeTime Ival=%d\n", Tval); 
 
   return false;
   } // end of MakeTime
@@ -2618,7 +2169,7 @@ bool DTVAL::MakeDate(PGLOBAL g, int *val, int nval)
     } // endfor i
 
   if (trace > 1)
-    htrc("MakeDate datm=(%d,%d,%d,%d,%d,%d)\n",
+    htrc("MakeDate datm=(%d,%d,%d,%d,%d,%d)\n", 
     datm.tm_year, datm.tm_mon, datm.tm_mday,
     datm.tm_hour, datm.tm_min, datm.tm_sec);
 
@@ -2639,24 +2190,21 @@ bool DTVAL::MakeDate(PGLOBAL g, int *val, int nval)
 /***********************************************************************/
 bool DTVAL::SetValue_pval(PVAL valp, bool chktype)
   {
-  if (valp != this) {
-    if (chktype && Type != valp->GetType())
-      return true;
+  if (chktype && Type != valp->GetType())
+    return true;
 
-    if (!(Null = valp->IsNull() && Nullable)) {
-      if (Pdtp && !valp->IsTypeNum()) {
-        int   ndv;
-        int  dval[6];
-
-        ndv = ExtractDate(valp->GetCharValue(), Pdtp, DefYear, dval);
-        MakeDate(NULL, dval, ndv);
-      } else
-        Tval = valp->GetIntValue();
+  if (!(Null = valp->IsNull() && Nullable)) {
+    if (Pdtp && !valp->IsTypeNum()) {
+      int   ndv;
+      int  dval[6];
 
+      ndv = ExtractDate(valp->GetCharValue(), Pdtp, DefYear, dval);
+      MakeDate(NULL, dval, ndv);
     } else
-      Reset();
+      Tval = valp->GetIntValue();
 
-    } // endif valp
+  } else
+    Reset();
 
   return false;
   } // end of SetValue
@@ -2666,23 +2214,20 @@ bool DTVAL::SetValue_pval(PVAL valp, bool chktype)
 /***********************************************************************/
 bool DTVAL::SetValue_char(char *p, int n)
   {
-  bool rc= 0;
+  bool rc;
 
   if (Pdtp) {
     char *p2;
     int   ndv;
     int  dval[6];
 
-		if (n > 0) {
-			// Trim trailing blanks
-			for (p2 = p + n -1; p < p2 && *p2 == ' '; p2--);
+    // Trim trailing blanks
+    for (p2 = p + n -1; p < p2 && *p2 == ' '; p2--) ;
 
-			if ((rc = (n = p2 - p + 1) > Len))
-				n = Len;
-
-			memcpy(Sdate, p, n);
-			} // endif n
+    if ((rc = (n = p2 - p + 1) > Len))
+      n = Len;
 
+    memcpy(Sdate, p, n);
     Sdate[n] = '\0';
 
     ndv = ExtractDate(Sdate, Pdtp, DefYear, dval);
@@ -2691,11 +2236,9 @@ bool DTVAL::SetValue_char(char *p, int n)
     if (trace > 1)
       htrc(" setting date: '%s' -> %d\n", Sdate, Tval);
 
-    Null = (Nullable && ndv == 0);
-  } else {
+    Null = false;
+  } else
     rc = TYPVAL<int>::SetValue_char(p, n);
-    Null = (Nullable && Tval == 0);
-  } // endif Pdtp
 
   return rc;
   } // end of SetValue
@@ -2718,11 +2261,9 @@ void DTVAL::SetValue_psz(PSZ p)
     if (trace > 1)
       htrc(" setting date: '%s' -> %d\n", Sdate, Tval);
 
-    Null = (Nullable && ndv == 0);
-  } else {
+    Null = false;
+  } else
     TYPVAL<int>::SetValue_psz(p);
-    Null = (Nullable && Tval == 0);
-  } // endif Pdtp
 
   } // end of SetValue
 
@@ -2763,7 +2304,7 @@ char *DTVAL::GetCharString(char *p)
   } else
     sprintf(p, "%d", Tval);
 
-//Null = false;                      ??????????????
+  Null = false;
   return p;
   } // end of GetCharString
 
@@ -2774,31 +2315,24 @@ char *DTVAL::ShowValue(char *buf, int len)
   {
   if (Pdtp) {
     char  *p;
+    size_t m, n = 0;
+    struct tm tm, *ptm = GetGmTime(&tm);
 
-    if (!Null) {
-      size_t m, n = 0;
-      struct tm tm, *ptm = GetGmTime(&tm);
+    if (Len < len) {
+      p = buf;
+      m = len;
+    } else {
+      p = Sdate;
+      m = Len + 1;
+    } // endif Len
 
+    if (ptm)
+      n = strftime(p, m, Pdtp->OutFmt, ptm);
 
-  
-      if (Len < len) {
-        p = buf;
-        m = len;
-      } else {
-        p = Sdate;
-        m = Len + 1;
-      } // endif Len
-  
-      if (ptm)
-        n = strftime(p, m, Pdtp->OutFmt, ptm);
-  
-      if (!n) {
-        *p = '\0';
-        strncat(p, "Error", m);
-        } // endif n
-
-    } else
-      p = "";                 // DEFAULT VALUE ???
+    if (!n) {
+      *p = '\0';
+      strncat(p, "Error", m);
+      } // endif n
 
     return p;
   } else
@@ -2865,7 +2399,7 @@ bool DTVAL::WeekNum(PGLOBAL g, int& nval)
 /***********************************************************************/
 bool DTVAL::FormatValue(PVAL vp, char *fmt)
   {
-  char     *buf = (char*)vp->GetTo_Val();       // Should be big enough
+  char      *buf = (char*)vp->GetTo_Val();       // Should be big enough
   struct tm tm, *ptm = GetGmTime(&tm);
 
   if (trace > 1)
