@@ -1,7 +1,7 @@
 /*************** Xindex H Declares Source Code File (.H) ***************/
 /*  Name: XINDEX.H    Version 3.5                                      */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          2004 - 2015  */
+/*  (C) Copyright to the author Olivier BERTRAND          2004 - 2013  */
 /*                                                                     */
 /*  This file contains the XINDEX class declares.                      */
 /***********************************************************************/
@@ -65,11 +65,7 @@ typedef struct index_def : public BLOCK {
 
 typedef struct index_off {
   union {
-#if defined(WORDS_BIGENDIAN)
-    struct {int High; int Low;} v;
-#else   // !WORDS_BIGENDIAN
-    struct {int Low; int High;} v;
-#endif   //!WORDS_BIGENDIAN
+    struct {int Low; int High;};
     longlong Val;                 // File position
     }; // end of union
   } IOFF;
@@ -91,9 +87,7 @@ class DllExport INDEXDEF : public BLOCK { /* Index description block   */
   void    SetNext(PIXDEF pxdf) {Next = pxdf;}
   PSZ     GetName(void) {return (PSZ)Name;}
   bool    IsUnique(void) {return Unique;}
-  bool    IsDynamic(void) {return Dynamic;}
   bool    IsAuto(void) {return AutoInc;}
-  bool    IsValid(void) {return !Invalid;}
   void    SetAuto(bool b) {AutoInc = b;}
   void    SetInvalid(bool b) {Invalid = b;}
   int     GetNparts(void) {return Nparts;}
@@ -121,8 +115,6 @@ class DllExport INDEXDEF : public BLOCK { /* Index description block   */
   bool    Unique;             /* true if defined as unique             */
   bool    Invalid;            /* true if marked as Invalid             */
   bool    AutoInc;            /* true if unique key in auto increment  */
-  bool    Dynamic;            /* KINDEX style                          */
-  bool    Mapped;             /* Use file mapping                      */
   int     Nparts;             /* Number of key parts                   */
   int     ID;                 /* Index ID number                       */
   int     MaxSame;            /* Max number of same values             */
@@ -182,8 +174,6 @@ class DllExport XXBASE : public CSORT, public BLOCK {
   virtual void Reset(void) = 0;
   virtual bool IsMul(void) {return false;}
   virtual bool IsRandom(void) {return true;}
-  virtual bool IsDynamic(void) {return Dynamic;}
-  virtual void SetDynamic(bool dyn) {Dynamic = dyn;}
   virtual bool HaveSame(void) {return false;}
   virtual int  GetCurPos(void) {return Cur_K;}
   virtual void SetNval(int n) {assert(n == 1);}
@@ -196,24 +186,23 @@ class DllExport XXBASE : public CSORT, public BLOCK {
           void SetNth(int n) {Nth = n;}
           int *GetPof(void) {return Pof;}
           int *GetPex(void) {return Pex;}
-          bool IsSorted(void) {return Srtd;}
           void FreeIndex(void) {PlgDBfree(Index);}
 
   // Methods
   virtual void Print(PGLOBAL g, FILE *f, uint n);
   virtual void Print(PGLOBAL g, char *ps, uint z);
   virtual bool Init(PGLOBAL g) = 0;
-  virtual bool Make(PGLOBAL g, PIXDEF sxp) = 0;
 #if defined(XMAP)
   virtual bool MapInit(PGLOBAL g) = 0;
 #endif   // XMAP
   virtual int  MaxRange(void) {return 1;}
   virtual int  Fetch(PGLOBAL g) = 0;
-  virtual bool NextVal(bool) {return true;}
+  virtual bool NextVal(bool eq) {return true;}
   virtual bool PrevVal(void) {return true;}
-  virtual int  FastFind(void) = 0;
-  virtual bool Reorder(PGLOBAL) {return true;}
-  virtual int  Range(PGLOBAL, int = 0, bool = true) {return -1;} // Means error
+  virtual int  FastFind(int nk) = 0;
+  virtual bool Reorder(PGLOBAL g) {return true;}
+  virtual int  Range(PGLOBAL g, int limit = 0, bool incl = true)
+                {return -1;}    // Means error
   virtual int  Qcompare(int *, int *) = 0;
   virtual int  GroupSize(void) {return 1;}
   virtual void Close(void) = 0;
@@ -234,7 +223,6 @@ class DllExport XXBASE : public CSORT, public BLOCK {
   OPVAL   Op;               // Search operator
   bool    Mul;              // true if multiple
   bool    Srtd;             // true for sorted column
-  bool    Dynamic;          // true when dynamically made
   int     Val_K;            // Index of current value
   int     Nblk;             // Number of blocks
   int     Sblk;             // Block size
@@ -269,7 +257,7 @@ class DllExport XINDEX : public XXBASE {
 #endif   // XMAP
   virtual int  Qcompare(int *, int *);
   virtual int  Fetch(PGLOBAL g);
-  virtual int  FastFind(void);
+  virtual int  FastFind(int nk);
   virtual int  GroupSize(void);
   virtual int  Range(PGLOBAL g, int limit = 0, bool incl = true);
   virtual int  MaxRange(void) {return MaxSame;}
@@ -280,10 +268,9 @@ class DllExport XINDEX : public XXBASE {
   virtual bool Make(PGLOBAL g, PIXDEF sxp);
   virtual bool SaveIndex(PGLOBAL g, PIXDEF sxp);
   virtual bool Reorder(PGLOBAL g);
-          bool GetAllSizes(PGLOBAL g,/* int &ndif,*/ int &numk);
+          bool GetAllSizes(PGLOBAL g, int &ndif, int &numk);
 
  protected:
-          bool AddColumns(void);
           bool NextValDif(void);
 
   // Members
@@ -315,7 +302,7 @@ class DllExport XINDXS : public XINDEX {
   // Methods
   virtual int  Qcompare(int *, int *);
   virtual int  Fetch(PGLOBAL g);
-  virtual int  FastFind(void);
+  virtual int  FastFind(int nk);
   virtual bool NextVal(bool eq);
   virtual bool PrevVal(void);
   virtual int  Range(PGLOBAL g, int limit = 0, bool incl = true);
@@ -350,7 +337,7 @@ class DllExport XLOAD : public BLOCK {
 
  protected:
   // Members
-#if defined(__WIN__)
+#if defined(WIN32)
   HANDLE  Hfile;                // Handle to file or map
 #else    // UNIX
   int     Hfile;                // Descriptor to file or map
@@ -394,7 +381,6 @@ class DllExport XHUGE : public XLOAD {
   XHUGE(void) : XLOAD() {}
 
   // Methods
-  using XLOAD::Close;
   virtual bool  Open(PGLOBAL g, char *filename, int id, MODE mode);
   virtual bool  Seek(PGLOBAL g, int low, int high, int origin);
   virtual bool  Read(PGLOBAL g, void *buf, int n, int size);
@@ -424,14 +410,13 @@ class DllExport XXROW : public XXBASE {
   // Methods
   virtual bool Init(PGLOBAL g);
 #if defined(XMAP)
-  virtual bool MapInit(PGLOBAL) {return true;}
+  virtual bool MapInit(PGLOBAL g) {return true;}
 #endif   // XMAP
   virtual int  Fetch(PGLOBAL g);
-  virtual int  FastFind(void);
+  virtual int  FastFind(int nk);
   virtual int  MaxRange(void) {return 1;}
   virtual int  Range(PGLOBAL g, int limit = 0, bool incl = true);
   virtual int  Qcompare(int *, int *) {assert(false); return 0;}
-  virtual bool Make(PGLOBAL, PIXDEF) {return false;}
   virtual void Close(void) {}
 
  protected:
@@ -466,7 +451,7 @@ class KXYCOL: public BLOCK {
   virtual void FreeData(void);
   virtual void FillValue(PVAL valp);
   virtual int  CompVal(int i);
-//        void InitBinFind(void *vp);
+          void InitBinFind(void *vp);
           bool MakeBlockArray(PGLOBAL g, int nb, int size);
           int  Compare(int i1, int i2);
           int  CompBval(int i);
