@@ -1,7 +1,7 @@
 /**************** Table H Declares Source Code File (.H) ***************/
 /*  Name: TABLE.H    Version 2.3                                       */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          1999-2015    */
+/*  (C) Copyright to the author Olivier BERTRAND          1999-2014    */
 /*                                                                     */
 /*  This file contains the TBX, OPJOIN and TDB class definitions.      */
 /***********************************************************************/
@@ -19,13 +19,14 @@
 #include "m_ctype.h"
 
 typedef class CMD *PCMD;
-typedef struct st_key_range key_range;
 
 // Commands executed by XDBC and MYX tables
 class CMD : public BLOCK {
  public:
   // Constructor
-  CMD(PGLOBAL g, char *cmd) {Cmd = PlugDup(g, cmd); Next = NULL;}
+  CMD(PGLOBAL g, char *cmd) {
+    Cmd = (char*)PlugSubAlloc(g, NULL, strlen(cmd) + 1);
+    strcpy(Cmd, cmd); Next = NULL; }
 
   // Members
   PCMD  Next;
@@ -33,24 +34,12 @@ class CMD : public BLOCK {
 }; // end of class CMD
 
 // Condition filter structure
-class CONDFIL : public BLOCK {
- public:
-	// Constructor
-	CONDFIL(const Item *cond, uint idx, AMT type) 
-	{
-		Cond = cond; Idx = idx; Type = type; Body = NULL; Op = OP_XX; Cmds = NULL;
-	}
+typedef struct _cond_filter {
+  char  *Body;
+  OPVAL  Op;
+  PCMD   Cmds;
+} CONDFIL, *PCFIL;
 
-	// Members
-	const Item *Cond;
-	AMT   Type;
-	uint  Idx;
-	char *Body;
-  OPVAL Op;
-  PCMD  Cmds;
-}; // end of class CONDFIL
-
-typedef class CONDFIL *PCFIL;
 typedef class TDBCAT *PTDBCAT;
 typedef class CATCOL *PCATCOL;
 
@@ -73,8 +62,6 @@ class DllExport TDB: public BLOCK {     // Table Descriptor Block.
   inline  PCOL   GetColumns(void) {return Columns;}
   inline  int    GetDegree(void) {return Degree;}
   inline  MODE   GetMode(void) {return Mode;}
-  inline  PFIL   GetFilter(void) {return To_Filter;}
-  inline  void   SetFilter(PFIL fp) {To_Filter = fp;}
   inline  void   SetOrig(PTDB txp) {To_Orig = txp;}
   inline  void   SetUse(TUSE n) {Use = n;}
   inline  void   SetCondFil(PCFIL cfp) {To_CondFil = cfp;}
@@ -85,28 +72,26 @@ class DllExport TDB: public BLOCK {     // Table Descriptor Block.
   inline  void   SetDegree(int degree) {Degree = degree;}
   inline  void   SetMode(MODE mode) {Mode = mode;}
 
-  // Properties
+  //Properties
   virtual AMT    GetAmType(void) {return TYPE_AM_ERROR;}
   virtual int    GetTdb_No(void) {return Tdb_No;}
   virtual PTDB   GetNext(void) {return Next;}
   virtual PCATLG GetCat(void) {return NULL;}
-  virtual void   SetAbort(bool) {;}
 
   // Methods
   virtual bool   IsSame(PTDB tp) {return tp == this;}
-  virtual bool   IsSpecial(PSZ name) = 0;
-  virtual bool   GetBlockValues(PGLOBAL) {return false;}
-  virtual int    Cardinality(PGLOBAL) {return 0;}
+  virtual bool   GetBlockValues(PGLOBAL g) {return false;}
+  virtual int    Cardinality(PGLOBAL g) {return (g) ? -1 : 0;}
   virtual int    GetMaxSize(PGLOBAL) = 0;
   virtual int    GetProgMax(PGLOBAL) = 0;
   virtual int    GetProgCur(void) = 0;
   virtual int    RowNumber(PGLOBAL g, bool b = false);
   virtual bool   IsReadOnly(void) {return true;}
   virtual const CHARSET_INFO *data_charset() {return NULL;}
-  virtual PTDB   Duplicate(PGLOBAL) {return NULL;}
-  virtual PTDB   CopyOne(PTABS) {return this;}
+  virtual PTDB   Duplicate(PGLOBAL g) {return NULL;}
+  virtual PTDB   CopyOne(PTABS t) {return this;}
   virtual PTDB   Copy(PTABS t);
-  virtual void   PrintAM(FILE *f, char *m)
+  virtual void   PrintAM(FILE *f, char *m) 
                   {fprintf(f, "%s AM(%d)\n",  m, GetAmType());}
   virtual void   Print(PGLOBAL g, FILE *f, uint n);
   virtual void   Print(PGLOBAL g, char *ps, uint z);
@@ -121,14 +106,13 @@ class DllExport TDB: public BLOCK {     // Table Descriptor Block.
   virtual int    WriteDB(PGLOBAL) = 0;
   virtual int    DeleteDB(PGLOBAL, int) = 0;
   virtual void   CloseDB(PGLOBAL) = 0;
-  virtual int    CheckWrite(PGLOBAL) {return 0;}
-  virtual bool   ReadKey(PGLOBAL, OPVAL, const key_range *) = 0;
+  virtual int    CheckWrite(PGLOBAL g) {return 0;}
+  virtual bool   ReadKey(PGLOBAL, OPVAL, const void *, int) = 0;
 
  protected:
   // Members
   PTDB   To_Orig;      // Pointer to original if it is a copy
   TUSE   Use;
-  PFIL   To_Filter;
   PCFIL  To_CondFil;   // To condition filter structure
   static int Tnum;     // Used to generate Tdb_no's
   const  int Tdb_No;   // GetTdb_No() is always 0 for OPJOIN
@@ -138,7 +122,6 @@ class DllExport TDB: public BLOCK {     // Table Descriptor Block.
   PCOL   Columns;      // Points to the first column of the table
   MODE   Mode;         // 10 Read, 30 Update, 40 Insert, 50 Delete
   int    Degree;       // Number of columns
-  int    Cardinal;     // Table number of rows
   }; // end of class TDB
 
 /***********************************************************************/
@@ -159,8 +142,6 @@ class DllExport TDBASE : public TDB {
   inline  PKXBASE GetKindex(void) {return To_Kindex;}
   inline  PCOL    GetSetCols(void) {return To_SetCols;}
   inline  void    SetSetCols(PCOL colp) {To_SetCols = colp;}
-  inline  void    SetKey_Col(PCOL *cpp) {To_Key_Col = cpp;}
-  inline  void    SetXdp(PIXDEF xdp) {To_Xdp = xdp;}
   inline  void    SetKindex(PKXBASE kxp) {To_Kindex = kxp;}
 
   // Properties
@@ -168,14 +149,12 @@ class DllExport TDBASE : public TDB {
   PCOL    Key(int i) {return (To_Key_Col) ? To_Key_Col[i] : NULL;}
 
   // Methods
-  virtual bool   IsUsingTemp(PGLOBAL) {return false;}
-  virtual bool   IsIndexed(void) {return false;}
-  virtual bool   IsSpecial(PSZ name);
+  virtual bool   IsUsingTemp(PGLOBAL g) {return false;}
   virtual PCATLG GetCat(void);
   virtual PSZ    GetPath(void);
   virtual void   PrintAM(FILE *f, char *m);
   virtual RECFM  GetFtype(void) {return RECFM_NAF;}
-//virtual int    GetAffectedRows(void) {return -1;}
+  virtual int    GetAffectedRows(void) {return -1;}
   virtual int    GetRecpos(void) = 0;
   virtual bool   SetRecpos(PGLOBAL g, int recpos);
   virtual bool   IsReadOnly(void) {return Read_Only;}
@@ -183,44 +162,39 @@ class DllExport TDBASE : public TDB {
   virtual CHARSET_INFO *data_charset(void);
   virtual int    GetProgMax(PGLOBAL g) {return GetMaxSize(g);}
   virtual int    GetProgCur(void) {return GetRecpos();}
-  virtual PSZ    GetFile(PGLOBAL) {return "Not a file";}
+  virtual PSZ    GetFile(PGLOBAL g) {return "Not a file";}
   virtual int    GetRemote(void) {return 0;}
-  virtual void   SetFile(PGLOBAL, PSZ) {}
+  virtual void   SetFile(PGLOBAL g, PSZ fn) {}
   virtual void   ResetDB(void) {}
   virtual void   ResetSize(void) {MaxSize = -1;}
   virtual void   RestoreNrec(void) {}
-  virtual int    ResetTableOpt(PGLOBAL g, bool dop, bool dox);
+  virtual int    ResetTableOpt(PGLOBAL g, bool dox);
   virtual PSZ    GetServer(void) {return "Current";}
 
   // Database routines
   virtual PCOL ColDB(PGLOBAL g, PSZ name, int num);
   virtual PCOL MakeCol(PGLOBAL, PCOLDEF, PCOL, int)
                       {assert(false); return NULL;}
-  virtual PCOL InsertSpecialColumn(PCOL colp);
+  virtual PCOL InsertSpecialColumn(PGLOBAL g, PCOL colp);
   virtual PCOL InsertSpcBlk(PGLOBAL g, PCOLDEF cdp);
   virtual void MarkDB(PGLOBAL g, PTDB tdb2);
-  virtual int  MakeIndex(PGLOBAL g, PIXDEF, bool)
+  virtual int  MakeIndex(PGLOBAL g, PIXDEF pxdf, bool add)
                 {strcpy(g->Message, "Remote index"); return RC_INFO;}
-  virtual bool ReadKey(PGLOBAL, OPVAL, const key_range *)
-                      {assert(false); return true;}
+  virtual bool ReadKey(PGLOBAL g, OPVAL op, const void *key, int len)
+                      {assert(false); return true;} 
+
 
  protected:
-  virtual bool PrepareWriting(PGLOBAL g) {strcpy(g->Message,
-    "This function should not be called for this table"); return true;}
-
   // Members
   PTABDEF  To_Def;            // Points to catalog description block
   PXOB    *To_Link;           // Points to column of previous relations
   PCOL    *To_Key_Col;        // Points to key columns in current file
   PKXBASE  To_Kindex;         // Points to table key index
-  PIXDEF   To_Xdp;            // To the index definition block
   PCOL     To_SetCols;        // Points to updated columns
-  RECFM    Ftype;             // File type: 0-var 1-fixed 2-binary (VCT)
-  int      MaxSize;           // Max size in number of lines
+  int      MaxSize;            // Max size in number of lines
   int      Knum;              // Size of key arrays
   bool     Read_Only;         // True for read only tables
   const CHARSET_INFO *m_data_charset;
-  const char *csname;         // Table charset name
   }; // end of class TDBASE
 
 /***********************************************************************/
@@ -238,8 +212,8 @@ class DllExport TDBCAT : public TDBASE {
   // Methods
   virtual int  GetRecpos(void) {return N;}
   virtual int  GetProgCur(void) {return N;}
-  virtual int  RowNumber(PGLOBAL, bool = false) {return N + 1;}
-  virtual bool SetRecpos(PGLOBAL g, int recpos);
+  virtual int  RowNumber(PGLOBAL g, bool b = false) {return N + 1;}
+  virtual bool SetRecpos(PGLOBAL g, int recpos); 
 
   // Database routines
   virtual PCOL MakeCol(PGLOBAL g, PCOLDEF cdp, PCOL cprec, int n);
@@ -257,9 +231,9 @@ class DllExport TDBCAT : public TDBASE {
           bool InitCol(PGLOBAL g);
 
   // Members
-  PQRYRES Qrp;
+  PQRYRES Qrp;           
   int     N;                  // Row number
-  bool    Init;
+  bool    Init;          
   }; // end of class TDBCAT
 
 /***********************************************************************/
