@@ -1,7 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, MariaDB Corporation.
+Copyright (c) 1995, 2013, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -58,7 +57,7 @@ is set to TRUE by the page_cleaner thread when it is spawned and is set
 back to FALSE at shutdown by the page_cleaner as well. Therefore no
 need to protect it by a mutex. It is only ever read by the thread
 doing the shutdown */
-UNIV_INTERN bool buf_page_cleaner_is_active;
+UNIV_INTERN ibool buf_page_cleaner_is_active = FALSE;
 
 /** LRU flush batch is further divided into this chunk size to
 reduce the wait time for the threads waiting for a clean block */
@@ -76,15 +75,6 @@ in thrashing. */
 
 /* @} */
 
-/** Handled page counters for a single flush */
-struct flush_counters_t {
-	ulint	flushed;	/*!< number of dirty pages flushed */
-	ulint	evicted;	/*!< number of clean pages evicted, including
-			        evicted uncompressed page images */
-	ulint	unzip_LRU_evicted;/*!< number of uncompressed page images
-				evicted */
-};
-
 /******************************************************************//**
 Increases flush_list size in bytes with zip_size for compressed page,
 UNIV_PAGE_SIZE for uncompressed page in inline function */
@@ -733,7 +723,8 @@ buf_flush_write_complete(
 	flush_type = buf_page_get_flush_type(bpage);
 	buf_pool->n_flush[flush_type]--;
 
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	/* fprintf(stderr, "n pending flush %lu\n",
+	buf_pool->n_flush[flush_type]); */
 
 	if (buf_pool->n_flush[flush_type] == 0
 	    && buf_pool->init_flush[flush_type] == FALSE) {
@@ -837,35 +828,39 @@ buf_flush_init_for_writing(
 	case SRV_CHECKSUM_ALGORITHM_CRC32:
 	case SRV_CHECKSUM_ALGORITHM_STRICT_CRC32:
 		checksum = buf_calc_page_crc32(page);
-		mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
 		break;
 	case SRV_CHECKSUM_ALGORITHM_INNODB:
 	case SRV_CHECKSUM_ALGORITHM_STRICT_INNODB:
 		checksum = (ib_uint32_t) buf_calc_page_new_checksum(page);
-		mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
-		checksum = (ib_uint32_t) buf_calc_page_old_checksum(page);
 		break;
 	case SRV_CHECKSUM_ALGORITHM_NONE:
 	case SRV_CHECKSUM_ALGORITHM_STRICT_NONE:
 		checksum = BUF_NO_CHECKSUM_MAGIC;
-		mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
 		break;
 	/* no default so the compiler will emit a warning if new enum
 	is added and not handled here */
 	}
 
-	/* With the InnoDB checksum, we overwrite the first 4 bytes of
-	the end lsn field to store the old formula checksum. Since it
-	depends also on the field FIL_PAGE_SPACE_OR_CHKSUM, it has to
-	be calculated after storing the new formula checksum.
+	mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
 
-	In other cases we write the same value to both fields.
-	If CRC32 is used then it is faster to use that checksum
-	(calculated above) instead of calculating another one.
-	We can afford to store something other than
-	buf_calc_page_old_checksum() or BUF_NO_CHECKSUM_MAGIC in
-	this field because the file will not be readable by old
-	versions of MySQL/InnoDB anyway (older than MySQL 5.6.3) */
+	/* We overwrite the first 4 bytes of the end lsn field to store
+	the old formula checksum. Since it depends also on the field
+	FIL_PAGE_SPACE_OR_CHKSUM, it has to be calculated after storing the
+	new formula checksum. */
+
+	if (srv_checksum_algorithm == SRV_CHECKSUM_ALGORITHM_STRICT_INNODB
+	    || srv_checksum_algorithm == SRV_CHECKSUM_ALGORITHM_INNODB) {
+
+		checksum = (ib_uint32_t) buf_calc_page_old_checksum(page);
+
+		/* In other cases we use the value assigned from above.
+		If CRC32 is used then it is faster to use that checksum
+		(calculated above) instead of calculating another one.
+		We can afford to store something other than
+		buf_calc_page_old_checksum() or BUF_NO_CHECKSUM_MAGIC in
+		this field because the file will not be readable by old
+		versions of MySQL/InnoDB anyway (older than MySQL 5.6.3) */
+	}
 
 	mach_write_to_4(page + UNIV_PAGE_SIZE - FIL_PAGE_END_LSN_OLD_CHKSUM,
 			checksum);
@@ -938,11 +933,11 @@ buf_flush_write_block_low(
 		break;
 	case BUF_BLOCK_ZIP_DIRTY:
 		frame = bpage->zip.data;
-		mach_write_to_8(frame + FIL_PAGE_LSN,
-				bpage->newest_modification);
 
 		ut_a(page_zip_verify_checksum(frame, zip_size));
 
+		mach_write_to_8(frame + FIL_PAGE_LSN,
+				bpage->newest_modification);
 		memset(frame + FIL_PAGE_FILE_FLUSH_LSN, 0, 8);
 		break;
 	case BUF_BLOCK_FILE_PAGE:
@@ -1439,14 +1434,12 @@ it is a best effort attempt and it is not guaranteed that after a call
 to this function there will be 'max' blocks in the free list.
 @return number of blocks for which the write request was queued. */
 static
-void
+ulint
 buf_flush_LRU_list_batch(
 /*=====================*/
 	buf_pool_t*	buf_pool,	/*!< in: buffer pool instance */
-	ulint		max,		/*!< in: desired number of
+	ulint		max)		/*!< in: desired number of
 					blocks in the free_list */
-	flush_counters_t*	n)	/*!< out: flushed/evicted page
-					counts */
 {
 	buf_page_t*	bpage;
 	ulint		count = 0;
@@ -1456,13 +1449,8 @@ buf_flush_LRU_list_batch(
 
 	ut_ad(buf_pool_mutex_own(buf_pool));
 
-	n->flushed = 0;
-	n->evicted = 0;
-	n->unzip_LRU_evicted = 0;
-
 	bpage = UT_LIST_GET_LAST(buf_pool->LRU);
 	while (bpage != NULL && count < max
-	       && (n->flushed + n->evicted) < max
 	       && free_len < srv_LRU_scan_depth
 	       && lru_len > BUF_LRU_MIN_LEN) {
 
@@ -1490,7 +1478,6 @@ buf_flush_LRU_list_batch(
 			if (buf_LRU_free_page(bpage, true)) {
 				/* buf_pool->mutex was potentially
 				released and reacquired. */
-				n->evicted++;
 				bpage = UT_LIST_GET_LAST(buf_pool->LRU);
 			} else {
 				bpage = UT_LIST_GET_PREV(LRU, bpage);
@@ -1513,7 +1500,7 @@ buf_flush_LRU_list_batch(
 			}
 
 			if (!buf_flush_page_and_try_neighbors(
-				bpage, BUF_FLUSH_LRU, max, &n->flushed)) {
+				bpage, BUF_FLUSH_LRU, max, &count)) {
 
 				bpage = prev_bpage;
 			} else {
@@ -1542,7 +1529,7 @@ buf_flush_LRU_list_batch(
 	/* We keep track of all flushes happening as part of LRU
 	flush. When estimating the desired rate at which flush_list
 	should be flushed, we factor in this value. */
-	buf_lru_flush_page_count += n->flushed;
+	buf_lru_flush_page_count += count;
 
 	ut_ad(buf_pool_mutex_own(buf_pool));
 
@@ -1553,6 +1540,8 @@ buf_flush_LRU_list_batch(
 			MONITOR_LRU_BATCH_SCANNED_PER_CALL,
 			scanned);
 	}
+
+	return(count);
 }
 
 /*******************************************************************//**
@@ -1562,28 +1551,24 @@ Whether LRU or unzip_LRU is used depends on the state of the system.
 or in case of unzip_LRU the number of blocks actually moved to the
 free list */
 static
-void
+ulint
 buf_do_LRU_batch(
 /*=============*/
 	buf_pool_t*	buf_pool,	/*!< in: buffer pool instance */
-	ulint		max,		/*!< in: desired number of
+	ulint		max)		/*!< in: desired number of
 					blocks in the free_list */
-	flush_counters_t*	n)	/*!< out: flushed/evicted page
-					counts */
 {
+	ulint	count = 0;
 
 	if (buf_LRU_evict_from_unzip_LRU(buf_pool)) {
-		n->unzip_LRU_evicted = buf_free_from_unzip_LRU_list_batch(buf_pool, max);
-	} else {
-		n->unzip_LRU_evicted = 0;
+		count += buf_free_from_unzip_LRU_list_batch(buf_pool, max);
 	}
 
-	if (max > n->unzip_LRU_evicted) {
-		buf_flush_LRU_list_batch(buf_pool, max - n->unzip_LRU_evicted, n);
-	} else {
-		n->evicted = 0;
-		n->flushed = 0;
+	if (max > count) {
+		count += buf_flush_LRU_list_batch(buf_pool, max - count);
 	}
+
+	return(count);
 }
 
 /*******************************************************************//**
@@ -1682,7 +1667,7 @@ end up waiting for these latches! NOTE 2: in the case of a flush list flush,
 the calling thread is not allowed to own any latches on pages!
 @return number of blocks for which the write request was queued */
 static
-void
+ulint
 buf_flush_batch(
 /*============*/
 	buf_pool_t*	buf_pool,	/*!< in: buffer pool instance */
@@ -1693,14 +1678,13 @@ buf_flush_batch(
 	ulint		min_n,		/*!< in: wished minimum mumber of blocks
 					flushed (it is not guaranteed that the
 					actual number is that big, though) */
-	lsn_t		lsn_limit,	/*!< in: in the case of BUF_FLUSH_LIST
+	lsn_t		lsn_limit)	/*!< in: in the case of BUF_FLUSH_LIST
 					all blocks whose oldest_modification is
 					smaller than this should be flushed
 					(if their number does not exceed
 					min_n), otherwise ignored */
-	flush_counters_t*	n)	/*!< out: flushed/evicted page
-					counts  */
 {
+	ulint		count	= 0;
 
 	ut_ad(flush_type == BUF_FLUSH_LRU || flush_type == BUF_FLUSH_LIST);
 #ifdef UNIV_SYNC_DEBUG
@@ -1714,11 +1698,10 @@ buf_flush_batch(
 	the flush functions. */
 	switch (flush_type) {
 	case BUF_FLUSH_LRU:
-		buf_do_LRU_batch(buf_pool, min_n, n);
+		count = buf_do_LRU_batch(buf_pool, min_n);
 		break;
 	case BUF_FLUSH_LIST:
-		n->flushed = buf_do_flush_list_batch(buf_pool, min_n, lsn_limit);
-		n->evicted = 0;
+		count = buf_do_flush_list_batch(buf_pool, min_n, lsn_limit);
 		break;
 	default:
 		ut_error;
@@ -1727,13 +1710,15 @@ buf_flush_batch(
 	buf_pool_mutex_exit(buf_pool);
 
 #ifdef UNIV_DEBUG
-	if (buf_debug_prints && n->flushed > 0) {
+	if (buf_debug_prints && count > 0) {
 		fprintf(stderr, flush_type == BUF_FLUSH_LRU
 			? "Flushed %lu pages in LRU flush\n"
 			: "Flushed %lu pages in flush list flush\n",
-			(ulong) n->flushed);
+			(ulong) count);
 	}
 #endif /* UNIV_DEBUG */
+
+	return(count);
 }
 
 /******************************************************************//**
@@ -1862,21 +1847,29 @@ buf_flush_LRU(
 	ulint		min_n,		/*!< in: wished minimum mumber of blocks
 					flushed (it is not guaranteed that the
 					actual number is that big, though) */
-	flush_counters_t	*n)	/*!< out: flushed/evicted page
-					counts */
+	ulint*		n_processed)	/*!< out: the number of pages
+					which were processed is passed
+					back to caller. Ignored if NULL */
 {
+	ulint		page_count;
+
+	if (n_processed) {
+		*n_processed = 0;
+	}
+
 	if (!buf_flush_start(buf_pool, BUF_FLUSH_LRU)) {
-		n->flushed = 0;
-		n->evicted = 0;
-		n->unzip_LRU_evicted = 0;
 		return(false);
 	}
 
-	buf_flush_batch(buf_pool, BUF_FLUSH_LRU, min_n, 0, n);
+	page_count = buf_flush_batch(buf_pool, BUF_FLUSH_LRU, min_n, 0);
 
 	buf_flush_end(buf_pool, BUF_FLUSH_LRU);
 
-	buf_flush_common(BUF_FLUSH_LRU, n->flushed);
+	buf_flush_common(BUF_FLUSH_LRU, page_count);
+
+	if (n_processed) {
+		*n_processed = page_count;
+	}
 
 	return(true);
 }
@@ -1924,7 +1917,7 @@ buf_flush_list(
 	/* Flush to lsn_limit in all buffer pool instances */
 	for (i = 0; i < srv_buf_pool_instances; i++) {
 		buf_pool_t*	buf_pool;
-		flush_counters_t n;
+		ulint		page_count = 0;
 
 		buf_pool = buf_pool_from_array(i);
 
@@ -1944,23 +1937,23 @@ buf_flush_list(
 			continue;
 		}
 
-		buf_flush_batch(
-			buf_pool, BUF_FLUSH_LIST, min_n, lsn_limit, &n);
+		page_count = buf_flush_batch(
+			buf_pool, BUF_FLUSH_LIST, min_n, lsn_limit);
 
 		buf_flush_end(buf_pool, BUF_FLUSH_LIST);
 
-		buf_flush_common(BUF_FLUSH_LIST, n.flushed);
+		buf_flush_common(BUF_FLUSH_LIST, page_count);
 
 		if (n_processed) {
-			*n_processed += n.flushed;
+			*n_processed += page_count;
 		}
 
-		if (n.flushed) {
+		if (page_count) {
 			MONITOR_INC_VALUE_CUMULATIVE(
 				MONITOR_FLUSH_BATCH_TOTAL_PAGE,
 				MONITOR_FLUSH_BATCH_COUNT,
 				MONITOR_FLUSH_BATCH_PAGES,
-				n.flushed);
+				page_count);
 		}
 	}
 
@@ -2098,7 +2091,7 @@ buf_flush_LRU_tail(void)
 		     j < scan_depth;
 		     j += PAGE_CLEANER_LRU_BATCH_CHUNK_SIZE) {
 
-			flush_counters_t	n;
+			ulint	n_flushed = 0;
 
 			/* Currently page_cleaner is the only thread
 			that can trigger an LRU flush. It is possible
@@ -2106,7 +2099,7 @@ buf_flush_LRU_tail(void)
 			still running, */
 			if (buf_flush_LRU(buf_pool,
 					  PAGE_CLEANER_LRU_BATCH_CHUNK_SIZE,
-					  &n)) {
+					  &n_flushed)) {
 
 				/* Allowed only one batch per
 				buffer pool instance. */
@@ -2114,8 +2107,8 @@ buf_flush_LRU_tail(void)
 					buf_pool, BUF_FLUSH_LRU);
 			}
 
-			if (n.flushed) {
-				total_flushed += n.flushed;
+			if (n_flushed) {
+				total_flushed += n_flushed;
 			} else {
 				/* Nothing to flush */
 				break;
@@ -2190,10 +2183,6 @@ af_get_pct_for_dirty()
 {
 	ulint dirty_pct = buf_get_modified_ratio_pct();
 
-	if (dirty_pct > 0 && srv_max_buf_pool_modified_pct == 0) {
-		return(100);
-	}
-
 	ut_a(srv_max_dirty_pages_pct_lwm
 	     <= srv_max_buf_pool_modified_pct);
 
@@ -2284,14 +2273,7 @@ page_cleaner_flush_pages_if_needed(void)
 	ulint			pct_total = 0;
 	int			age_factor = 0;
 
-	cur_lsn = log_get_lsn_nowait();
-
-	/* log_get_lsn_nowait tries to get log_sys->mutex with
-	mutex_enter_nowait, if this does not succeed function
-	returns 0, do not use that value to update stats. */
-	if (cur_lsn == 0) {
-		return(0);
-	}
+	cur_lsn = log_get_lsn();
 
 	if (prev_lsn == 0) {
 		/* First time around. */
@@ -2402,11 +2384,10 @@ extern "C" UNIV_INTERN
 os_thread_ret_t
 DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 /*==========================================*/
-	void*	arg MY_ATTRIBUTE((unused)))
+	void*	arg __attribute__((unused)))
 			/*!< in: a dummy parameter required by
 			os_thread_create */
 {
-	my_thread_init();
 	ulint	next_loop_time = ut_time_ms() + 1000;
 	ulint	n_flushed = 0;
 	ulint	last_activity = srv_get_activity_count();
@@ -2422,18 +2403,29 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 		os_thread_pf(os_thread_get_curr_id()));
 #endif /* UNIV_DEBUG_THREAD_CREATION */
 
+	buf_page_cleaner_is_active = TRUE;
+
 	while (srv_shutdown_state == SRV_SHUTDOWN_NONE) {
 
-		page_cleaner_sleep_if_needed(next_loop_time);
+		/* The page_cleaner skips sleep if the server is
+		idle and there are no pending IOs in the buffer pool
+		and there is work to do. */
+		if (srv_check_activity(last_activity)
+		    || buf_get_n_pending_read_ios()
+		    || n_flushed == 0) {
+			page_cleaner_sleep_if_needed(next_loop_time);
+		}
 
 		next_loop_time = ut_time_ms() + 1000;
 
 		if (srv_check_activity(last_activity)) {
 			last_activity = srv_get_activity_count();
 
+			/* Flush pages from end of LRU if required */
+			n_flushed = buf_flush_LRU_tail();
+
 			/* Flush pages from flush_list if required */
-			page_cleaner_flush_pages_if_needed();
-			n_flushed = 0;
+			n_flushed += page_cleaner_flush_pages_if_needed();
 		} else {
 			n_flushed = page_cleaner_do_flush_batch(
 							PCT_IO(100),
@@ -2447,9 +2439,6 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 					n_flushed);
 			}
 		}
-
-		/* Flush pages from end of LRU if required */
-		n_flushed = buf_flush_LRU_tail();
 	}
 
 	ut_ad(srv_shutdown_state > 0);
@@ -2515,9 +2504,8 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 	/* We have lived our life. Time to die. */
 
 thread_exit:
-	buf_page_cleaner_is_active = false;
+	buf_page_cleaner_is_active = FALSE;
 
-	my_thread_end();
 	/* We count the number of threads in os_thread_exit(). A created
 	thread should always use that to exit and not use return() to exit. */
 	os_thread_exit(NULL);
