@@ -1,9 +1,8 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2017, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 1996, 2014, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2008, Google Inc.
 Copyright (c) 2009, Percona Inc.
-Copyright (c) 2013, 2017, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -110,9 +109,6 @@ UNIV_INTERN ibool	srv_have_fullfsync = FALSE;
 /** TRUE if a raw partition is in use */
 UNIV_INTERN ibool	srv_start_raw_disk_in_use = FALSE;
 
-/** UNDO tablespaces starts with space id. */
-ulint	srv_undo_space_id_start;
-
 /** TRUE if the server is being started, before rolling back any
 incomplete transactions */
 UNIV_INTERN ibool	srv_startup_is_before_trx_rollback_phase = FALSE;
@@ -121,34 +117,20 @@ UNIV_INTERN ibool	srv_is_being_started = FALSE;
 /** TRUE if the server was successfully started */
 UNIV_INTERN ibool	srv_was_started = FALSE;
 /** TRUE if innobase_start_or_create_for_mysql() has been called */
-static ibool	srv_start_has_been_called;
-
-/** Whether any undo log records can be generated */
-UNIV_INTERN bool srv_undo_sources;
+static ibool		srv_start_has_been_called = FALSE;
 
 /** At a shutdown this value climbs from SRV_SHUTDOWN_NONE to
 SRV_SHUTDOWN_CLEANUP and then to SRV_SHUTDOWN_LAST_PHASE, and so on */
 UNIV_INTERN enum srv_shutdown_state	srv_shutdown_state = SRV_SHUTDOWN_NONE;
 
 /** Files comprising the system tablespace */
-static pfs_os_file_t	files[1000];
+static os_file_t	files[1000];
 
 /** io_handler_thread parameters for thread identification */
 static ulint		n[SRV_MAX_N_IO_THREADS + 6];
 /** io_handler_thread identifiers, 32 is the maximum number of purge threads  */
 static os_thread_id_t	thread_ids[SRV_MAX_N_IO_THREADS + 6 + 32];
 
-/** Thead handles */
-static os_thread_t	thread_handles[SRV_MAX_N_IO_THREADS + 6 + 32];
-static os_thread_t	buf_flush_page_cleaner_thread_handle;
-static os_thread_t	buf_dump_thread_handle;
-static os_thread_t	dict_stats_thread_handle;
-/** Status variables, is thread started ?*/
-static bool		thread_started[SRV_MAX_N_IO_THREADS + 6 + 32] = {false};
-static bool		buf_flush_page_cleaner_thread_started = false;
-static bool		buf_dump_thread_started = false;
-static bool		dict_stats_thread_started = false;
-
 /** We use this mutex to test the return value of pthread_mutex_trylock
    on successful locking. HP-UX does NOT return 0, though Linux et al do. */
 static os_fast_mutex_t	srv_os_test_mutex;
@@ -165,9 +147,6 @@ static const ulint SRV_UNDO_TABLESPACE_SIZE_IN_PAGES =
 #define SRV_N_PENDING_IOS_PER_THREAD	OS_AIO_N_PENDING_IOS_PER_THREAD
 #define SRV_MAX_N_PENDING_SYNC_IOS	100
 
-/** The round off to MB is similar as done in srv_parse_megabytes() */
-#define CALC_NUMBER_OF_PAGES(size)  ((size) / (1024 * 1024)) * \
-				  ((1024 * 1024) / (UNIV_PAGE_SIZE))
 #ifdef UNIV_PFS_THREAD
 /* Keys to register InnoDB threads with performance schema */
 UNIV_INTERN mysql_pfs_key_t	io_handler_thread_key;
@@ -243,8 +222,8 @@ srv_file_check_mode(
 
 		/* Note: stat.rw_perm is only valid of files */
 
-		if (stat.type == OS_FILE_TYPE_FILE) {
-
+		if (stat.type == OS_FILE_TYPE_FILE
+		    || stat.type == OS_FILE_TYPE_BLOCK) {
 			if (!stat.rw_perm) {
 
 				ib_logf(IB_LOG_LEVEL_ERROR,
@@ -441,16 +420,14 @@ srv_parse_data_file_paths_and_sizes(
 		    && *(str + 1) == 'e'
 		    && *(str + 2) == 'w') {
 			str += 3;
-			/* Initialize new raw device only during bootstrap */
 			(srv_data_file_is_raw_partition)[i] = SRV_NEW_RAW;
 		}
 
 		if (*str == 'r' && *(str + 1) == 'a' && *(str + 2) == 'w') {
 			str += 3;
 
-			/* Initialize new raw device only during bootstrap */
 			if ((srv_data_file_is_raw_partition)[i] == 0) {
-				(srv_data_file_is_raw_partition)[i] = SRV_NEW_RAW;
+				(srv_data_file_is_raw_partition)[i] = SRV_OLD_RAW;
 			}
 		}
 
@@ -525,7 +502,7 @@ UNIV_INTERN
 void
 srv_normalize_path_for_win(
 /*=======================*/
-	char*	str MY_ATTRIBUTE((unused)))	/*!< in/out: null-terminated
+	char*	str __attribute__((unused)))	/*!< in/out: null-terminated
 						character string */
 {
 #ifdef __WIN__
@@ -542,11 +519,11 @@ srv_normalize_path_for_win(
 /*********************************************************************//**
 Creates a log file.
 @return	DB_SUCCESS or error code */
-static MY_ATTRIBUTE((nonnull, warn_unused_result))
+static __attribute__((nonnull, warn_unused_result))
 dberr_t
 create_log_file(
 /*============*/
-	pfs_os_file_t*	file,	/*!< out: file handle */
+	os_file_t*	file,	/*!< out: file handle */
 	const char*	name)	/*!< in: log file name */
 {
 	ibool		ret;
@@ -585,6 +562,19 @@ create_log_file(
 /** Initial number of the first redo log file */
 #define INIT_LOG_FILE0	(SRV_N_LOG_FILES_MAX + 1)
 
+#ifdef DBUG_OFF
+# define RECOVERY_CRASH(x) do {} while(0)
+#else
+# define RECOVERY_CRASH(x) do {						\
+	if (srv_force_recovery_crash == x) {				\
+		fprintf(stderr, "innodb_force_recovery_crash=%lu\n",	\
+			srv_force_recovery_crash);			\
+		fflush(stderr);						\
+		exit(3);						\
+	}								\
+} while (0)
+#endif
+
 /*********************************************************************//**
 Creates all log files.
 @return	DB_SUCCESS or error code */
@@ -625,14 +615,13 @@ create_log_files(
 			file should be recoverable. The buffer
 			pool was clean, and we can simply create
 			all log files from the scratch. */
-			DBUG_EXECUTE_IF("innodb_log_abort_6",
-					return(DB_ERROR););
+			RECOVERY_CRASH(6);
 		}
 	}
 
 	ut_ad(!buf_pool_check_no_pending_io());
 
-	DBUG_EXECUTE_IF("innodb_log_abort_7", return(DB_ERROR););
+	RECOVERY_CRASH(7);
 
 	for (unsigned i = 0; i < srv_n_log_files; i++) {
 		sprintf(logfilename + dirnamelen,
@@ -645,7 +634,7 @@ create_log_files(
 		}
 	}
 
-	DBUG_EXECUTE_IF("innodb_log_abort_8", return(DB_ERROR););
+	RECOVERY_CRASH(8);
 
 	/* We did not create the first log file initially as
 	ib_logfile0, so that crash recovery cannot find it until it
@@ -690,16 +679,10 @@ create_log_files(
 	return(DB_SUCCESS);
 }
 
-/** Rename the first redo log file.
-@param[in,out]	logfilename	buffer for the log file name
-@param[in]	dirnamelen	length of the directory path
-@param[in]	lsn		FIL_PAGE_FILE_FLUSH_LSN value
-@param[in,out]	logfile0	name of the first log file
-@return	error code
-@retval	DB_SUCCESS	on successful operation */
-MY_ATTRIBUTE((warn_unused_result, nonnull))
+/*********************************************************************//**
+Renames the first log file. */
 static
-dberr_t
+void
 create_log_files_rename(
 /*====================*/
 	char*	logfilename,	/*!< in/out: buffer for log file name */
@@ -710,9 +693,6 @@ create_log_files_rename(
 	/* If innodb_flush_method=O_DSYNC,
 	we need to explicitly flush the log buffers. */
 	fil_flush(SRV_LOG_SPACE_FIRST_ID);
-
-	DBUG_EXECUTE_IF("innodb_log_abort_9", return(DB_ERROR););
-
 	/* Close the log files, so that we can rename
 	the first one. */
 	fil_close_log_files(false);
@@ -721,38 +701,36 @@ create_log_files_rename(
 	checkpoint has been created. */
 	sprintf(logfilename + dirnamelen, "ib_logfile%u", 0);
 
+	RECOVERY_CRASH(9);
+
 	ib_logf(IB_LOG_LEVEL_INFO,
 		"Renaming log file %s to %s", logfile0, logfilename);
 
 	mutex_enter(&log_sys->mutex);
 	ut_ad(strlen(logfile0) == 2 + strlen(logfilename));
-	dberr_t err = os_file_rename(
-		innodb_file_log_key, logfile0, logfilename)
-		? DB_SUCCESS : DB_ERROR;
+	ibool success = os_file_rename(
+		innodb_file_log_key, logfile0, logfilename);
+	ut_a(success);
+
+	RECOVERY_CRASH(10);
 
 	/* Replace the first file with ib_logfile0. */
 	strcpy(logfile0, logfilename);
 	mutex_exit(&log_sys->mutex);
 
-	DBUG_EXECUTE_IF("innodb_log_abort_10", err = DB_ERROR;);
+	fil_open_log_and_system_tablespace_files();
 
-	if (err == DB_SUCCESS) {
-		fil_open_log_and_system_tablespace_files();
-		ib_logf(IB_LOG_LEVEL_WARN,
-			"New log files created, LSN=" LSN_PF, lsn);
-	}
-
-	return(err);
+	ib_logf(IB_LOG_LEVEL_WARN, "New log files created, LSN=" LSN_PF, lsn);
 }
 
 /*********************************************************************//**
 Opens a log file.
 @return	DB_SUCCESS or error code */
-static MY_ATTRIBUTE((nonnull, warn_unused_result))
+static __attribute__((nonnull, warn_unused_result))
 dberr_t
 open_log_file(
 /*==========*/
-	pfs_os_file_t*	file,	/*!< out: file handle */
+	os_file_t*	file,	/*!< out: file handle */
 	const char*	name,	/*!< in: log file name */
 	os_offset_t*	size)	/*!< out: file size */
 {
@@ -776,7 +754,7 @@ open_log_file(
 /*********************************************************************//**
 Creates or opens database data files and closes them.
 @return	DB_SUCCESS or error code */
-static MY_ATTRIBUTE((nonnull, warn_unused_result))
+static __attribute__((nonnull, warn_unused_result))
 dberr_t
 open_or_create_data_files(
 /*======================*/
@@ -868,7 +846,7 @@ open_or_create_data_files(
 				   && os_file_get_last_error(false)
 				   != OS_FILE_ALREADY_EXISTS
 #ifdef UNIV_AIX
-				   /* AIX 5.1 after security patch ML7 may have
+			    	   /* AIX 5.1 after security patch ML7 may have
 			           errno set to 0 here, which causes our
 				   function to return 100; work around that
 				   AIX problem */
@@ -902,24 +880,6 @@ open_or_create_data_files(
 
 				return(DB_ERROR);
 			}
-
-			const char*	check_msg;
-			check_msg = fil_read_first_page(
-				files[i], FALSE, &flags, &space,
-#ifdef UNIV_LOG_ARCHIVE
-				min_arch_log_no, max_arch_log_no,
-#endif /* UNIV_LOG_ARCHIVE */
-				min_flushed_lsn, max_flushed_lsn);
-
-			/* If first page is valid, don't overwrite DB.
-			It prevents overwriting DB when mysql_install_db
-			starts mysqld multiple times during bootstrap. */
-			if (check_msg == NULL) {
-
-				srv_created_new_raw = FALSE;
-				ret = FALSE;
-			}
-
 		} else if (srv_data_file_is_raw_partition[i] == SRV_OLD_RAW) {
 			srv_start_raw_disk_in_use = TRUE;
 
@@ -976,16 +936,10 @@ open_or_create_data_files(
 size_check:
 			size = os_file_get_size(files[i]);
 			ut_a(size != (os_offset_t) -1);
+			/* Round size downward to megabytes */
 
-			/* Under some error conditions like disk full
-			narios or file size reaching filesystem
-			limit the data file could contain an incomplete
-			extent at the end. When we extend a data file
-			and if some failure happens, then also the data
-			file could contain an incomplete extent.  So we
-			need to round the size downward to a megabyte.*/
-
-			rounded_size_pages = (ulint) CALC_NUMBER_OF_PAGES(size);
+			rounded_size_pages = (ulint)
+				(size >> UNIV_PAGE_SIZE_SHIFT);
 
 			if (i == srv_n_data_files - 1
 			    && srv_auto_extend_last_data_file) {
@@ -1171,7 +1125,7 @@ srv_undo_tablespace_create(
 	const char*	name,		/*!< in: tablespace name */
 	ulint		size)		/*!< in: tablespace size in pages */
 {
-	pfs_os_file_t	fh;
+	os_file_t	fh;
 	ibool		ret;
 	dberr_t		err = DB_SUCCESS;
 
@@ -1248,7 +1202,7 @@ srv_undo_tablespace_open(
 	const char*	name,		/*!< in: tablespace name */
 	ulint		space)		/*!< in: tablespace id */
 {
-	pfs_os_file_t	fh;
+	os_file_t	fh;
 	dberr_t		err	= DB_ERROR;
 	ibool		ret;
 	ulint		flags;
@@ -1347,23 +1301,13 @@ srv_undo_tablespaces_init(
 
 	for (i = 0; create_new_db && i < n_conf_tablespaces; ++i) {
 		char	name[OS_FILE_MAX_PATH];
-		ulint	space_id  = i + 1;
-
-		DBUG_EXECUTE_IF("innodb_undo_upgrade",
-				space_id = i + 3;);
 
 		ut_snprintf(
 			name, sizeof(name),
 			"%s%cundo%03lu",
-			srv_undo_dir, SRV_PATH_SEPARATOR, space_id);
-
-		if (i == 0) {
-			srv_undo_space_id_start = space_id;
-			prev_space_id = srv_undo_space_id_start - 1;
-		}
-
-		undo_tablespace_ids[i] = space_id;
+			srv_undo_dir, SRV_PATH_SEPARATOR, i + 1);
 
+		/* Undo space ids start from 1. */
 		err = srv_undo_tablespace_create(
 			name, SRV_UNDO_TABLESPACE_SIZE_IN_PAGES);
 
@@ -1385,16 +1329,14 @@ srv_undo_tablespaces_init(
 	if (!create_new_db) {
 		n_undo_tablespaces = trx_rseg_get_n_undo_tablespaces(
 			undo_tablespace_ids);
-
-		if (n_undo_tablespaces != 0) {
-			srv_undo_space_id_start = undo_tablespace_ids[0];
-			prev_space_id = srv_undo_space_id_start - 1;
-		}
-
 	} else {
 		n_undo_tablespaces = n_conf_tablespaces;
 
-		undo_tablespace_ids[n_conf_tablespaces] = ULINT_UNDEFINED;
+		for (i = 1; i <= n_undo_tablespaces; ++i) {
+			undo_tablespace_ids[i - 1] = i;
+		}
+
+		undo_tablespace_ids[i] = ULINT_UNDEFINED;
 	}
 
 	/* Open all the undo tablespaces that are currently in use. If we
@@ -1418,6 +1360,8 @@ srv_undo_tablespaces_init(
 		ut_a(undo_tablespace_ids[i] != 0);
 		ut_a(undo_tablespace_ids[i] != ULINT_UNDEFINED);
 
+		/* Undo space ids start from 1. */
+
 		err = srv_undo_tablespace_open(name, undo_tablespace_ids[i]);
 
 		if (err != DB_SUCCESS) {
@@ -1452,23 +1396,11 @@ srv_undo_tablespaces_init(
 			break;
 		}
 
-		/** Note the first undo tablespace id in case of
-		no active undo tablespace. */
-		if (n_undo_tablespaces == 0) {
-			srv_undo_space_id_start = i;
-		}
-
 		++n_undo_tablespaces;
 
 		++*n_opened;
 	}
 
-	/** Explictly specify the srv_undo_space_id_start
-	as zero when there are no undo tablespaces. */
-	if (n_undo_tablespaces == 0) {
-		srv_undo_space_id_start = 0;
-	}
-
 	/* If the user says that there are fewer than what we find we
 	tolerate that discrepancy but not the inverse. Because there could
 	be unused undo tablespaces for future use. */
@@ -1513,11 +1445,10 @@ srv_undo_tablespaces_init(
 		mtr_start(&mtr);
 
 		/* The undo log tablespace */
-		for (i = 0; i < n_undo_tablespaces; ++i) {
+		for (i = 1; i <= n_undo_tablespaces; ++i) {
 
 			fsp_header_init(
-				undo_tablespace_ids[i],
-				SRV_UNDO_TABLESPACE_SIZE_IN_PAGES, &mtr);
+				i, SRV_UNDO_TABLESPACE_SIZE_IN_PAGES, &mtr);
 		}
 
 		mtr_commit(&mtr);
@@ -1568,7 +1499,8 @@ are not found and the user wants.
 @return	DB_SUCCESS or error code */
 UNIV_INTERN
 dberr_t
-innobase_start_or_create_for_mysql()
+innobase_start_or_create_for_mysql(void)
+/*====================================*/
 {
 	ibool		create_new_db;
 	lsn_t		min_flushed_lsn;
@@ -1578,6 +1510,8 @@ innobase_start_or_create_for_mysql()
 	ulint		max_arch_log_no;
 #endif /* UNIV_LOG_ARCHIVE */
 	ulint		sum_of_new_sizes;
+	ulint		sum_of_data_file_sizes;
+	ulint		tablespace_size_in_header;
 	dberr_t		err;
 	unsigned	i;
 	ulint		srv_n_log_files_found = srv_n_log_files;
@@ -1588,28 +1522,11 @@ innobase_start_or_create_for_mysql()
 	char		logfilename[10000];
 	char*		logfile0	= NULL;
 	size_t		dirnamelen;
-	bool		sys_datafiles_created = false;
 
-	/* Check that os_fast_mutexes work as expected */
-	os_fast_mutex_init(PFS_NOT_INSTRUMENTED, &srv_os_test_mutex);
-
-	ut_a(0 == os_fast_mutex_trylock(&srv_os_test_mutex));
-
-	os_fast_mutex_unlock(&srv_os_test_mutex);
-
-	os_fast_mutex_lock(&srv_os_test_mutex);
-
-	os_fast_mutex_unlock(&srv_os_test_mutex);
-
-	os_fast_mutex_free(&srv_os_test_mutex);
-
-	if (srv_force_recovery == SRV_FORCE_NO_LOG_REDO) {
-		srv_read_only_mode = 1;
+	if (srv_force_recovery > SRV_FORCE_NO_TRX_UNDO) {
+		srv_read_only_mode = true;
 	}
 
-	high_level_read_only = srv_read_only_mode
-		|| srv_force_recovery > SRV_FORCE_NO_TRX_UNDO;
-
 	if (srv_read_only_mode) {
 		ib_logf(IB_LOG_LEVEL_INFO, "Started in read only mode");
 	}
@@ -1649,7 +1566,22 @@ innobase_start_or_create_for_mysql()
 #endif /* PAGE_ATOMIC_REF_COUNT */
 	);
 
-	compile_time_assert(sizeof(ulint) == sizeof(void*));
+
+	if (sizeof(ulint) != sizeof(void*)) {
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			" InnoDB: Error: size of InnoDB's ulint is %lu, "
+			"but size of void*\n", (ulong) sizeof(ulint));
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			" InnoDB: is %lu. The sizes should be the same "
+			"so that on a 64-bit\n",
+			(ulong) sizeof(void*));
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			" InnoDB: platforms you can allocate more than 4 GB "
+			"of memory.\n");
+	}
 
 #ifdef UNIV_DEBUG
 	ut_print_timestamp(stderr);
@@ -1714,19 +1646,6 @@ innobase_start_or_create_for_mysql()
 	ib_logf(IB_LOG_LEVEL_INFO,
 		"" IB_ATOMICS_STARTUP_MSG "");
 
-	ib_logf(IB_LOG_LEVEL_INFO,
-		"" IB_MEMORY_BARRIER_STARTUP_MSG "");
-
-#ifndef HAVE_MEMORY_BARRIER
-#if defined __i386__ || defined __x86_64__ || defined _M_IX86 || defined _M_X64 || defined __WIN__
-#else
-	ib_logf(IB_LOG_LEVEL_WARN,
-		"MySQL was built without a memory barrier capability on this"
-		" architecture, which might allow a mutex/rw_lock violation"
-		" under high thread concurrency. This may cause a hang.");
-#endif /* IA32 or AMD64 */
-#endif /* HAVE_MEMORY_BARRIER */
-
 	ib_logf(IB_LOG_LEVEL_INFO,
 		"Compressed tables use zlib " ZLIB_VERSION
 #ifdef UNIV_ZIP_DEBUG
@@ -1922,7 +1841,7 @@ innobase_start_or_create_for_mysql()
 			}
 		} else {
 			srv_monitor_file_name = NULL;
-			srv_monitor_file = os_file_create_tmpfile(NULL);
+			srv_monitor_file = os_file_create_tmpfile();
 
 			if (!srv_monitor_file) {
 				return(DB_ERROR);
@@ -1932,7 +1851,7 @@ innobase_start_or_create_for_mysql()
 		mutex_create(srv_dict_tmpfile_mutex_key,
 			     &srv_dict_tmpfile_mutex, SYNC_DICT_OPERATION);
 
-		srv_dict_tmpfile = os_file_create_tmpfile(NULL);
+		srv_dict_tmpfile = os_file_create_tmpfile();
 
 		if (!srv_dict_tmpfile) {
 			return(DB_ERROR);
@@ -1941,7 +1860,7 @@ innobase_start_or_create_for_mysql()
 		mutex_create(srv_misc_tmpfile_mutex_key,
 			     &srv_misc_tmpfile_mutex, SYNC_ANY_LATCH);
 
-		srv_misc_tmpfile = os_file_create_tmpfile(NULL);
+		srv_misc_tmpfile = os_file_create_tmpfile();
 
 		if (!srv_misc_tmpfile) {
 			return(DB_ERROR);
@@ -2050,8 +1969,7 @@ innobase_start_or_create_for_mysql()
 
 		n[i] = i;
 
-		thread_handles[i] = os_thread_create(io_handler_thread, n + i, thread_ids + i);
-		thread_started[i] = true;
+		os_thread_create(io_handler_thread, n + i, thread_ids + i);
 	}
 
 #ifdef UNIV_LOG_ARCHIVE
@@ -2227,18 +2145,14 @@ innobase_start_or_create_for_mysql()
 						dirnamelen, max_flushed_lsn,
 						logfile0);
 
-					if (err == DB_SUCCESS) {
-						err = create_log_files_rename(
-							logfilename,
-							dirnamelen,
-							max_flushed_lsn,
-							logfile0);
-					}
-
 					if (err != DB_SUCCESS) {
 						return(err);
 					}
 
+					create_log_files_rename(
+						logfilename, dirnamelen,
+						max_flushed_lsn, logfile0);
+
 					/* Suppress the message about
 					crash recovery. */
 					max_flushed_lsn = min_flushed_lsn
@@ -2255,10 +2169,6 @@ innobase_start_or_create_for_mysql()
 				break;
 			}
 
-			if (stat_info.type != OS_FILE_TYPE_FILE) {
-				break;
-			}
-
 			if (!srv_file_check_mode(logfilename)) {
 				return(DB_ERROR);
 			}
@@ -2287,9 +2197,9 @@ innobase_start_or_create_for_mysql()
 			} else if (size != srv_log_file_size) {
 				ib_logf(IB_LOG_LEVEL_ERROR,
 					"Log file %s is"
-					" of different size " UINT64PF " bytes"
+					" of different size "UINT64PF" bytes"
 					" than other log"
-					" files " UINT64PF " bytes!",
+					" files "UINT64PF" bytes!",
 					logfilename,
 					size << UNIV_PAGE_SIZE_SHIFT,
 					(os_offset_t) srv_log_file_size
@@ -2368,6 +2278,7 @@ files_checked:
 	trx_sys_create();
 
 	if (create_new_db) {
+
 		ut_a(!srv_read_only_mode);
 
 		mtr_start(&mtr);
@@ -2410,12 +2321,8 @@ files_checked:
 
 		fil_flush_file_spaces(FIL_TABLESPACE);
 
-		err = create_log_files_rename(logfilename, dirnamelen,
-					      max_flushed_lsn, logfile0);
-
-		if (err != DB_SUCCESS) {
-			return(err);
-		}
+		create_log_files_rename(logfilename, dirnamelen,
+					max_flushed_lsn, logfile0);
 #ifdef UNIV_LOG_ARCHIVE
 	} else if (srv_archive_recovery) {
 
@@ -2485,89 +2392,24 @@ files_checked:
 			LOG_CHECKPOINT, LSN_MAX,
 			min_flushed_lsn, max_flushed_lsn);
 
-		if (err == DB_SUCCESS) {
-			/* Initialize the change buffer. */
-			err = dict_boot();
+		if (err != DB_SUCCESS) {
+
+			return(DB_ERROR);
 		}
 
+		/* Since the insert buffer init is in dict_boot, and the
+		insert buffer is needed in any disk i/o, first we call
+		dict_boot(). Note that trx_sys_init_at_db_start() only needs
+		to access space 0, and the insert buffer at this stage already
+		works for space 0. */
+
+		err = dict_boot();
+
 		if (err != DB_SUCCESS) {
 			return(err);
 		}
 
-		if (!srv_read_only_mode) {
-			if (sum_of_new_sizes > 0) {
-				/* New data file(s) were added */
-				mtr_start(&mtr);
-				fsp_header_inc_size(0, sum_of_new_sizes, &mtr);
-				mtr_commit(&mtr);
-				/* Immediately write the log record about
-				increased tablespace size to disk, so that it
-				is durable even if mysqld would crash
-				quickly */
-				log_buffer_flush_to_disk();
-			}
-		}
-
-		const ulint	tablespace_size_in_header
-			= fsp_header_get_tablespace_size();
-
-#ifdef UNIV_DEBUG
-		/* buf_debug_prints = TRUE; */
-#endif /* UNIV_DEBUG */
-		ulint		sum_of_data_file_sizes = 0;
-
-		for (ulint d = 0; d < srv_n_data_files; d++) {
-			sum_of_data_file_sizes += srv_data_file_sizes[d];
-		}
-
-		/* Compare the system tablespace file size to what is
-		stored in FSP_SIZE. In open_or_create_data_files()
-		we already checked that the file sizes match the
-		innodb_data_file_path specification. */
-		if (srv_read_only_mode
-		    || sum_of_data_file_sizes == tablespace_size_in_header) {
-			/* Do not complain about the size. */
-		} else if (!srv_auto_extend_last_data_file
-			   || sum_of_data_file_sizes
-			   < tablespace_size_in_header) {
-			ib_logf(IB_LOG_LEVEL_ERROR,
-				"Tablespace size stored in header is " ULINTPF
-				" pages, but the sum of data file sizes is "
-				ULINTPF " pages",
-				tablespace_size_in_header,
-				sum_of_data_file_sizes);
-
-			if (srv_force_recovery == 0
-			    && sum_of_data_file_sizes
-			    < tablespace_size_in_header) {
-				ib_logf(IB_LOG_LEVEL_ERROR,
-					"Cannot start InnoDB. The tail of"
-					" the system tablespace is"
-					" missing. Have you edited"
-					" innodb_data_file_path in my.cnf"
-					" in an inappropriate way, removing"
-					" data files from there?"
-					" You can set innodb_force_recovery=1"
-					" in my.cnf to force"
-					" a startup if you are trying to"
-					" recover a badly corrupt database.");
-
-				return(DB_ERROR);
-			}
-		}
-
-		/* This must precede recv_apply_hashed_log_recs(true). */
 		ib_bh = trx_sys_init_at_db_start();
-
-		if (srv_force_recovery < SRV_FORCE_NO_LOG_REDO) {
-			/* Apply the hashed log records to the
-			respective file pages, for the last batch of
-			recv_group_scan_log_recs(). */
-
-			recv_apply_hashed_log_recs(true);
-			DBUG_PRINT("ib_log", ("apply completed"));
-		}
-
 		n_recovered_trx = UT_LIST_GET_LEN(trx_sys->rw_trx_list);
 
 		/* The purge system needs to create the purge view and
@@ -2606,15 +2448,6 @@ files_checked:
 				dict_check = DICT_CHECK_NONE_LOADED;
 			}
 
-			/* Create the SYS_TABLESPACES and SYS_DATAFILES system table */
-			err = dict_create_or_check_sys_tablespace();
-			if (err != DB_SUCCESS) {
-				return(err);
-			}
-
-			sys_datafiles_created = true;
-
-			/* This function assumes that SYS_DATAFILES exists */
 			dict_check_tablespaces_and_store_max_id(dict_check);
 		}
 
@@ -2636,8 +2469,7 @@ files_checked:
 				ULINT_MAX, LSN_MAX, NULL);
 			ut_a(success);
 
-			DBUG_EXECUTE_IF("innodb_log_abort_1",
-					return(DB_ERROR););
+			RECOVERY_CRASH(1);
 
 			min_flushed_lsn = max_flushed_lsn = log_get_lsn();
 
@@ -2652,6 +2484,8 @@ files_checked:
 
 			buf_flush_wait_batch_end(NULL, BUF_FLUSH_LIST);
 
+			RECOVERY_CRASH(2);
+
 			/* Flush the old log files. */
 			log_buffer_flush_to_disk();
 			/* If innodb_flush_method=O_DSYNC,
@@ -2666,27 +2500,21 @@ files_checked:
 			ut_d(recv_no_log_write = TRUE);
 			ut_ad(!buf_pool_check_no_pending_io());
 
-			DBUG_EXECUTE_IF("innodb_log_abort_3",
-					return(DB_ERROR););
+			RECOVERY_CRASH(3);
 
 			/* Stamp the LSN to the data files. */
 			fil_write_flushed_lsn_to_data_files(
 				max_flushed_lsn, 0);
 
-			DBUG_EXECUTE_IF("innodb_log_abort_4", err = DB_ERROR;);
-
-			if (err != DB_SUCCESS) {
-				return(err);
-			}
-
 			fil_flush_file_spaces(FIL_TABLESPACE);
 
+			RECOVERY_CRASH(4);
+
 			/* Close and free the redo log files, so that
 			we can replace them. */
 			fil_close_log_files(true);
 
-			DBUG_EXECUTE_IF("innodb_log_abort_5",
-					return(DB_ERROR););
+			RECOVERY_CRASH(5);
 
 			/* Free the old log file space. */
 			log_group_close_all();
@@ -2700,19 +2528,16 @@ files_checked:
 					       dirnamelen, max_flushed_lsn,
 					       logfile0);
 
-			if (err == DB_SUCCESS) {
-				err = create_log_files_rename(
-					logfilename, dirnamelen,
-					max_flushed_lsn, logfile0);
-			}
-
 			if (err != DB_SUCCESS) {
 				return(err);
 			}
+
+			create_log_files_rename(logfilename, dirnamelen,
+						max_flushed_lsn, logfile0);
 		}
 
-		recv_recovery_rollback_active();
 		srv_startup_is_before_trx_rollback_phase = FALSE;
+		recv_recovery_rollback_active();
 
 		/* It is possible that file_format tag has never
 		been set. In this case we initialize it to minimum
@@ -2722,8 +2547,20 @@ files_checked:
 		trx_sys_file_format_tag_init();
 	}
 
-	ut_ad(err == DB_SUCCESS);
-	ut_a(sum_of_new_sizes != ULINT_UNDEFINED);
+	if (!create_new_db && sum_of_new_sizes > 0) {
+		/* New data file(s) were added */
+		mtr_start(&mtr);
+
+		fsp_header_inc_size(0, sum_of_new_sizes, &mtr);
+
+		mtr_commit(&mtr);
+
+		/* Immediately write the log record about increased tablespace
+		size to disk, so that it is durable even if mysqld would crash
+		quickly */
+
+		log_buffer_flush_to_disk();
+	}
 
 #ifdef UNIV_LOG_ARCHIVE
 	/* Archiving is always off under MySQL */
@@ -2784,25 +2621,29 @@ files_checked:
 		srv_undo_logs = ULONG_UNDEFINED;
 	}
 
+	/* Flush the changes made to TRX_SYS_PAGE by trx_sys_create_rsegs()*/
+	if (!srv_force_recovery && !srv_read_only_mode) {
+		bool success = buf_flush_list(ULINT_MAX, LSN_MAX, NULL);
+		ut_a(success);
+		buf_flush_wait_batch_end(NULL, BUF_FLUSH_LIST);
+	}
+
 	if (!srv_read_only_mode) {
 		/* Create the thread which watches the timeouts
 		for lock waits */
-		thread_handles[2 + SRV_MAX_N_IO_THREADS] = os_thread_create(
+		os_thread_create(
 			lock_wait_timeout_thread,
 			NULL, thread_ids + 2 + SRV_MAX_N_IO_THREADS);
-		thread_started[2 + SRV_MAX_N_IO_THREADS] = true;
 
 		/* Create the thread which warns of long semaphore waits */
-		thread_handles[3 + SRV_MAX_N_IO_THREADS] = os_thread_create(
+		os_thread_create(
 			srv_error_monitor_thread,
 			NULL, thread_ids + 3 + SRV_MAX_N_IO_THREADS);
-		thread_started[3 + SRV_MAX_N_IO_THREADS] = true;
 
 		/* Create the thread which prints InnoDB monitor info */
-		thread_handles[4 + SRV_MAX_N_IO_THREADS] = os_thread_create(
+		os_thread_create(
 			srv_monitor_thread,
 			NULL, thread_ids + 4 + SRV_MAX_N_IO_THREADS);
-		thread_started[4 + SRV_MAX_N_IO_THREADS] = true;
 	}
 
 	/* Create the SYS_FOREIGN and SYS_FOREIGN_COLS system tables */
@@ -2811,13 +2652,10 @@ files_checked:
 		return(err);
 	}
 
-	/* Create the SYS_TABLESPACES and SYS_DATAFILES system tables if we
-	have not done that already on crash recovery. */
-	if (sys_datafiles_created == false) {
-		err = dict_create_or_check_sys_tablespace();
-		if (err != DB_SUCCESS) {
-			return(err);
-		}
+	/* Create the SYS_TABLESPACES system table */
+	err = dict_create_or_check_sys_tablespace();
+	if (err != DB_SUCCESS) {
+		return(err);
 	}
 
 	srv_is_being_started = FALSE;
@@ -2829,40 +2667,26 @@ files_checked:
 
 	if (!srv_read_only_mode) {
 
-		thread_handles[1 + SRV_MAX_N_IO_THREADS] = os_thread_create(
+		os_thread_create(
 			srv_master_thread,
 			NULL, thread_ids + (1 + SRV_MAX_N_IO_THREADS));
-		thread_started[1 + SRV_MAX_N_IO_THREADS] = true;
-
-		srv_undo_sources = true;
-		/* Create the dict stats gathering thread */
-		srv_dict_stats_thread_active = true;
-		dict_stats_thread_handle = os_thread_create(
-			dict_stats_thread, NULL, NULL);
-		dict_stats_thread_started = true;
-
-		/* Create the thread that will optimize the FTS sub-system. */
-		fts_optimize_init();
 	}
 
 	if (!srv_read_only_mode
 	    && srv_force_recovery < SRV_FORCE_NO_BACKGROUND) {
 
-		thread_handles[5 + SRV_MAX_N_IO_THREADS] = os_thread_create(
+		os_thread_create(
 			srv_purge_coordinator_thread,
 			NULL, thread_ids + 5 + SRV_MAX_N_IO_THREADS);
 
-		thread_started[5 + SRV_MAX_N_IO_THREADS] = true;
-
 		ut_a(UT_ARR_SIZE(thread_ids)
 		     > 5 + srv_n_purge_threads + SRV_MAX_N_IO_THREADS);
 
 		/* We've already created the purge coordinator thread above. */
 		for (i = 1; i < srv_n_purge_threads; ++i) {
-			thread_handles[5 + i + SRV_MAX_N_IO_THREADS] = os_thread_create(
+			os_thread_create(
 				srv_worker_thread, NULL,
 				thread_ids + 5 + i + SRV_MAX_N_IO_THREADS);
-			thread_started[5 + i + SRV_MAX_N_IO_THREADS] = true;
 		}
 
 		srv_start_wait_for_purge_to_start();
@@ -2872,11 +2696,128 @@ files_checked:
 	}
 
 	if (!srv_read_only_mode) {
-		buf_page_cleaner_is_active = true;
-		buf_flush_page_cleaner_thread_handle = os_thread_create(buf_flush_page_cleaner_thread, NULL, NULL);
-		buf_flush_page_cleaner_thread_started = true;
+		os_thread_create(buf_flush_page_cleaner_thread, NULL, NULL);
 	}
 
+#ifdef UNIV_DEBUG
+	/* buf_debug_prints = TRUE; */
+#endif /* UNIV_DEBUG */
+	sum_of_data_file_sizes = 0;
+
+	for (i = 0; i < srv_n_data_files; i++) {
+		sum_of_data_file_sizes += srv_data_file_sizes[i];
+	}
+
+	tablespace_size_in_header = fsp_header_get_tablespace_size();
+
+	if (!srv_read_only_mode
+	    && !srv_auto_extend_last_data_file
+	    && sum_of_data_file_sizes != tablespace_size_in_header) {
+
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			" InnoDB: Error: tablespace size"
+			" stored in header is %lu pages, but\n",
+			(ulong) tablespace_size_in_header);
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			"InnoDB: the sum of data file sizes is %lu pages\n",
+			(ulong) sum_of_data_file_sizes);
+
+		if (srv_force_recovery == 0
+		    && sum_of_data_file_sizes < tablespace_size_in_header) {
+			/* This is a fatal error, the tail of a tablespace is
+			missing */
+
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: Cannot start InnoDB."
+				" The tail of the system tablespace is\n");
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: missing. Have you edited"
+				" innodb_data_file_path in my.cnf in an\n");
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: inappropriate way, removing"
+				" ibdata files from there?\n");
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: You can set innodb_force_recovery=1"
+				" in my.cnf to force\n");
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: a startup if you are trying"
+				" to recover a badly corrupt database.\n");
+
+			return(DB_ERROR);
+		}
+	}
+
+	if (!srv_read_only_mode
+	    && srv_auto_extend_last_data_file
+	    && sum_of_data_file_sizes < tablespace_size_in_header) {
+
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			" InnoDB: Error: tablespace size stored in header"
+			" is %lu pages, but\n",
+			(ulong) tablespace_size_in_header);
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			" InnoDB: the sum of data file sizes"
+			" is only %lu pages\n",
+			(ulong) sum_of_data_file_sizes);
+
+		if (srv_force_recovery == 0) {
+
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: Cannot start InnoDB. The tail of"
+				" the system tablespace is\n");
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: missing. Have you edited"
+				" innodb_data_file_path in my.cnf in an\n");
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: inappropriate way, removing"
+				" ibdata files from there?\n");
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: You can set innodb_force_recovery=1"
+				" in my.cnf to force\n");
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+				" InnoDB: a startup if you are trying to"
+				" recover a badly corrupt database.\n");
+
+			return(DB_ERROR);
+		}
+	}
+
+	/* Check that os_fast_mutexes work as expected */
+	os_fast_mutex_init(PFS_NOT_INSTRUMENTED, &srv_os_test_mutex);
+
+	if (0 != os_fast_mutex_trylock(&srv_os_test_mutex)) {
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			" InnoDB: Error: pthread_mutex_trylock returns"
+			" an unexpected value on\n");
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			" InnoDB: success! Cannot continue.\n");
+		exit(1);
+	}
+
+	os_fast_mutex_unlock(&srv_os_test_mutex);
+
+	os_fast_mutex_lock(&srv_os_test_mutex);
+
+	os_fast_mutex_unlock(&srv_os_test_mutex);
+
+	os_fast_mutex_free(&srv_os_test_mutex);
+
 	if (srv_print_verbose_log) {
 		ib_logf(IB_LOG_LEVEL_INFO,
 			"%s started; log sequence number " LSN_PF "",
@@ -2900,8 +2841,13 @@ files_checked:
 
 	if (!srv_read_only_mode) {
 		/* Create the buffer pool dump/load thread */
-		buf_dump_thread_handle = os_thread_create(buf_dump_thread, NULL, NULL);
-		buf_dump_thread_started = true;
+		os_thread_create(buf_dump_thread, NULL, NULL);
+
+		/* Create the dict stats gathering thread */
+		os_thread_create(dict_stats_thread, NULL, NULL);
+
+		/* Create the thread that will optimize the FTS sub-system. */
+		fts_optimize_init();
 	}
 
 	srv_was_started = TRUE;
@@ -2939,10 +2885,13 @@ srv_fts_close(void)
 }
 #endif
 
-/** Shut down InnoDB. */
+/****************************************************************//**
+Shuts down the InnoDB database.
+@return	DB_SUCCESS or error code */
 UNIV_INTERN
-void
-innodb_shutdown()
+dberr_t
+innobase_shutdown_for_mysql(void)
+/*=============================*/
 {
 	ulint	i;
 
@@ -2952,20 +2901,15 @@ innodb_shutdown()
 				"Shutting down an improperly started, "
 				"or created database!");
 		}
+
+		return(DB_SUCCESS);
 	}
 
-	if (srv_undo_sources) {
-		ut_ad(!srv_read_only_mode);
+	if (!srv_read_only_mode) {
 		/* Shutdown the FTS optimize sub system. */
 		fts_optimize_start_shutdown();
 
 		fts_optimize_end();
-		dict_stats_shutdown();
-		while (row_get_background_drop_list_len_low()) {
-			srv_wake_master_thread();
-			os_thread_yield();
-		}
-		srv_undo_sources = false;
 	}
 
 	/* 1. Flush the buffer pool to disk, write the current lsn to
@@ -3069,43 +3013,15 @@ innodb_shutdown()
 		dict_stats_thread_deinit();
 	}
 
-#ifdef __WIN__
-	/* MDEV-361: ha_innodb.dll leaks handles on Windows
-	MDEV-7403: should not pass recv_writer_thread_handle to
-	CloseHandle().
-
-	On Windows we should call CloseHandle() for all
-	open thread handles. */
-	if (os_thread_count == 0) {
-		for (i = 0; i < SRV_MAX_N_IO_THREADS + 6 + 32; ++i) {
-			if (thread_started[i]) {
-				CloseHandle(thread_handles[i]);
-			}
-		}
-
-		if (buf_flush_page_cleaner_thread_started) {
-			CloseHandle(buf_flush_page_cleaner_thread_handle);
-		}
-
-		if (buf_dump_thread_started) {
-			CloseHandle(buf_dump_thread_handle);
-		}
-
-		if (dict_stats_thread_started) {
-			CloseHandle(dict_stats_thread_handle);
-		}
-	}
-#endif /* __WIN __ */
-
 	/* This must be disabled before closing the buffer pool
 	and closing the data dictionary.  */
 	btr_search_disable();
 
 	ibuf_close();
 	log_shutdown();
+	lock_sys_close();
 	trx_sys_file_format_close();
 	trx_sys_close();
-	lock_sys_close();
 
 	/* We don't create these mutexes in RO mode because we don't create
 	the temp files that the cover. */
@@ -3128,7 +3044,7 @@ innodb_shutdown()
 	srv_free();
 	fil_close();
 
-	/* 4. Free all os_events and os_mutexes */
+	/* 4. Free the os_conc_mutex and all os_events and os_mutexes */
 
 	os_sync_free();
 
@@ -3168,9 +3084,89 @@ innodb_shutdown()
 
 	srv_was_started = FALSE;
 	srv_start_has_been_called = FALSE;
+
+	return(DB_SUCCESS);
 }
 #endif /* !UNIV_HOTBACKUP */
 
+
+/********************************************************************
+Signal all per-table background threads to shutdown, and wait for them to do
+so. */
+UNIV_INTERN
+void
+srv_shutdown_table_bg_threads(void)
+/*===============================*/
+{
+	dict_table_t*	table;
+	dict_table_t*	first;
+	dict_table_t*	last = NULL;
+
+	mutex_enter(&dict_sys->mutex);
+
+	/* Signal all threads that they should stop. */
+	table = UT_LIST_GET_FIRST(dict_sys->table_LRU);
+	first = table;
+	while (table) {
+		dict_table_t*	next;
+		fts_t*		fts = table->fts;
+
+		if (fts != NULL) {
+			fts_start_shutdown(table, fts);
+		}
+
+		next = UT_LIST_GET_NEXT(table_LRU, table);
+
+		if (!next) {
+			last = table;
+		}
+
+		table = next;
+	}
+
+	/* We must release dict_sys->mutex here; if we hold on to it in the
+	loop below, we will deadlock if any of the background threads try to
+	acquire it (for example, the FTS thread by calling que_eval_sql).
+
+	Releasing it here and going through dict_sys->table_LRU without
+	holding it is safe because:
+
+	 a) MySQL only starts the shutdown procedure after all client
+	 threads have been disconnected and no new ones are accepted, so no
+	 new tables are added or old ones dropped.
+
+	 b) Despite its name, the list is not LRU, and the order stays
+	 fixed.
+
+	To safeguard against the above assumptions ever changing, we store
+	the first and last items in the list above, and then check that
+	they've stayed the same below. */
+
+	mutex_exit(&dict_sys->mutex);
+
+	/* Wait for the threads of each table to stop. This is not inside
+	the above loop, because by signaling all the threads first we can
+	overlap their shutting down delays. */
+	table = UT_LIST_GET_FIRST(dict_sys->table_LRU);
+	ut_a(first == table);
+	while (table) {
+		dict_table_t*	next;
+		fts_t*		fts = table->fts;
+
+		if (fts != NULL) {
+			fts_shutdown(table, fts);
+		}
+
+		next = UT_LIST_GET_NEXT(table_LRU, table);
+
+		if (table == last) {
+			ut_a(!next);
+		}
+
+		table = next;
+	}
+}
+
 /*****************************************************************//**
 Get the meta-data filename from the table name. */
 UNIV_INTERN
