@@ -140,7 +140,7 @@ int main(int argc, char **argv)
   {
     if (init_pagecache(maria_log_pagecache,
                        TRANSLOG_PAGECACHE_SIZE, 0, 0,
-                       TRANSLOG_PAGE_SIZE, 0, MY_WME) == 0 ||
+                       TRANSLOG_PAGE_SIZE, MY_WME) == 0 ||
         translog_init(opt_log_dir, TRANSLOG_FILE_SIZE,
                       0, 0, maria_log_pagecache,
                       TRANSLOG_DEFAULT_FLAGS, 0))
@@ -1054,7 +1054,7 @@ static int maria_chk(HA_CHECK *param, char *filename)
       error= 1;
       goto end2;
     }
-    /* We can't do parallel repair with BLOCK_RECORD yet */
+    /* We can't do parallell repair with BLOCK_RECORD yet */
     if (param->testflag & T_REP_PARALLEL)
     {
       param->testflag&= ~T_REP_PARALLEL;
@@ -1178,7 +1178,7 @@ static int maria_chk(HA_CHECK *param, char *filename)
   maria_lock_database(info, F_EXTRA_LCK);
   datafile= info->dfile.file;
   if (init_pagecache(maria_pagecache, (size_t) param->use_buffers, 0, 0,
-                     maria_block_size, 0, MY_WME) == 0)
+                     maria_block_size, MY_WME) == 0)
   {
     _ma_check_print_error(param, "Can't initialize page cache with %lu memory",
                           (ulong) param->use_buffers);
@@ -1275,7 +1275,7 @@ static int maria_chk(HA_CHECK *param, char *filename)
         mysql_file_close(info->dfile.file, MYF(MY_WME)); /* Close new file */
         error|=maria_change_to_newfile(filename,MARIA_NAME_DEXT,DATA_TMP_EXT,
                                        0, MYF(0));
-        if (_ma_open_datafile(info, info->s))
+        if (_ma_open_datafile(info,info->s, NullS, -1))
           error=1;
         param->out_flag&= ~O_NEW_DATA; /* We are using new datafile */
         param->read_cache.file= info->dfile.file;
