@@ -25,11 +25,11 @@ COPYING CONDITIONS NOTICE:
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-  02110-1335  USA.
+  02110-1301, USA.
 
 COPYRIGHT NOTICE:
 
-  TokuFT, Tokutek Fractal Tree Indexing Library.
+  TokuDB, Tokutek Fractal Tree Indexing Library.
   Copyright (C) 2007-2013 Tokutek, Inc.
 
 DISCLAIMER:
@@ -97,32 +97,31 @@ PATENT RIGHTS GRANT:
  *   For each nonleaf node:  All the messages have keys that are between the associated pivot keys ( left_pivot_key < message <= right_pivot_key)
  */
 
-#include <config.h>
-
-#include "ft/serialize/block_table.h"
-#include "ft/ft.h"
-#include "ft/ft-cachetable-wrappers.h"
-#include "ft/ft-internal.h"
-#include "ft/node.h"
+#include "ft-cachetable-wrappers.h"
+#include "ft-internal.h"
+#include "ft.h"
 
 static int 
 compare_pairs (FT_HANDLE ft_handle, const DBT *a, const DBT *b) {
-    return ft_handle->ft->cmp(a, b);
+    FAKE_DB(db, &ft_handle->ft->cmp_descriptor);
+    int cmp = ft_handle->ft->compare_fun(&db, a, b);
+    return cmp;
 }
 
 static int 
-compare_pair_to_key (FT_HANDLE ft_handle, const DBT *a, const void *key, uint32_t keylen) {
+compare_pair_to_key (FT_HANDLE ft_handle, const DBT *a, bytevec key, ITEMLEN keylen) {
     DBT y;
-    return ft_handle->ft->cmp(a, toku_fill_dbt(&y, key, keylen));
+    FAKE_DB(db, &ft_handle->ft->cmp_descriptor);
+    int cmp = ft_handle->ft->compare_fun(&db, a, toku_fill_dbt(&y, key, keylen));
+    return cmp;
 }
 
 static int
-verify_msg_in_child_buffer(FT_HANDLE ft_handle, enum ft_msg_type type, MSN msn, const void *key, uint32_t keylen, const void *UU(data), uint32_t UU(datalen), XIDS UU(xids), const DBT *lesser_pivot, const DBT *greatereq_pivot)
+verify_msg_in_child_buffer(FT_HANDLE ft_handle, enum ft_msg_type type, MSN msn, bytevec key, ITEMLEN keylen, bytevec UU(data), ITEMLEN UU(datalen), XIDS UU(xids), const DBT *lesser_pivot, const DBT *greatereq_pivot)
     __attribute__((warn_unused_result));
 
-UU()
 static int
-verify_msg_in_child_buffer(FT_HANDLE ft_handle, enum ft_msg_type type, MSN msn, const void *key, uint32_t keylen, const void *UU(data), uint32_t UU(datalen), XIDS UU(xids), const DBT *lesser_pivot, const DBT *greatereq_pivot) {
+verify_msg_in_child_buffer(FT_HANDLE ft_handle, enum ft_msg_type type, MSN msn, bytevec key, ITEMLEN keylen, bytevec UU(data), ITEMLEN UU(datalen), XIDS UU(xids), const DBT *lesser_pivot, const DBT *greatereq_pivot) {
     int result = 0;
     if (msn.msn == ZERO_MSN.msn)
         result = EINVAL;
@@ -160,14 +159,9 @@ get_ith_key_dbt (BASEMENTNODE bn, int i) {
 
 #define VERIFY_ASSERTION(predicate, i, string) ({                                                                              \
     if(!(predicate)) {                                                                                                         \
-        fprintf(stderr, "%s:%d: Looking at child %d of block %" PRId64 ": %s\n", __FILE__, __LINE__, i, blocknum.b, string);   \
-        result = TOKUDB_NEEDS_REPAIR;                                                                                          \
-        if (!keep_going_on_failure) goto done;                                                                                 \
-    }})
-
-#define VERIFY_ASSERTION_BASEMENT(predicate, bn, entry, string) ({           \
-    if(!(predicate)) {                                                                                                         \
-        fprintf(stderr, "%s:%d: Looking at block %" PRId64 " bn %d entry %d: %s\n", __FILE__, __LINE__, blocknum.b, bn, entry, string); \
+        if (verbose) {                                                                                                         \
+            fprintf(stderr, "%s:%d: Looking at child %d of block %" PRId64 ": %s\n", __FILE__, __LINE__, i, blocknum.b, string); \
+        }                                                                                                                      \
         result = TOKUDB_NEEDS_REPAIR;                                                                                          \
         if (!keep_going_on_failure) goto done;                                                                                 \
     }})
@@ -175,7 +169,7 @@ get_ith_key_dbt (BASEMENTNODE bn, int i) {
 struct count_msgs_extra {
     int count;
     MSN msn;
-    message_buffer *msg_buffer;
+    FIFO fifo;
 };
 
 // template-only function, but must be extern
@@ -183,16 +177,15 @@ int count_msgs(const int32_t &offset, const uint32_t UU(idx), struct count_msgs_
     __attribute__((nonnull(3)));
 int count_msgs(const int32_t &offset, const uint32_t UU(idx), struct count_msgs_extra *const e)
 {
-    MSN msn;
-    e->msg_buffer->get_message_key_msn(offset, nullptr, &msn);
-    if (msn.msn == e->msn.msn) {
+    const struct fifo_entry *entry = toku_fifo_get_entry(e->fifo, offset);
+    if (entry->msn.msn == e->msn.msn) {
         e->count++;
     }
     return 0;
 }
 
 struct verify_message_tree_extra {
-    message_buffer *msg_buffer;
+    FIFO fifo;
     bool broadcast;
     bool is_fresh;
     int i;
@@ -205,25 +198,24 @@ struct verify_message_tree_extra {
 int verify_message_tree(const int32_t &offset, const uint32_t UU(idx), struct verify_message_tree_extra *const e) __attribute__((nonnull(3)));
 int verify_message_tree(const int32_t &offset, const uint32_t UU(idx), struct verify_message_tree_extra *const e)
 {
+    int verbose = e->verbose;
     BLOCKNUM blocknum = e->blocknum;
     int keep_going_on_failure = e->keep_going_on_failure;
     int result = 0;
-    DBT k, v;
-    ft_msg msg = e->msg_buffer->get_message(offset, &k, &v);
-    bool is_fresh = e->msg_buffer->get_freshness(offset);
+    const struct fifo_entry *entry = toku_fifo_get_entry(e->fifo, offset);
     if (e->broadcast) {
-        VERIFY_ASSERTION(ft_msg_type_applies_all((enum ft_msg_type) msg.type()) || ft_msg_type_does_nothing((enum ft_msg_type) msg.type()),
+        VERIFY_ASSERTION(ft_msg_type_applies_all((enum ft_msg_type) entry->type) || ft_msg_type_does_nothing((enum ft_msg_type) entry->type),
                          e->i, "message found in broadcast list that is not a broadcast");
     } else {
-        VERIFY_ASSERTION(ft_msg_type_applies_once((enum ft_msg_type) msg.type()),
+        VERIFY_ASSERTION(ft_msg_type_applies_once((enum ft_msg_type) entry->type),
                          e->i, "message found in fresh or stale message tree that does not apply once");
         if (e->is_fresh) {
             if (e->messages_have_been_moved) {
-                VERIFY_ASSERTION(is_fresh,
+                VERIFY_ASSERTION(entry->is_fresh,
                                  e->i, "message found in fresh message tree that is not fresh");
             }
         } else {
-            VERIFY_ASSERTION(!is_fresh,
+            VERIFY_ASSERTION(!entry->is_fresh,
                              e->i, "message found in stale message tree that is fresh");
         }
     }
@@ -239,18 +231,19 @@ int error_on_iter(const int32_t &UU(offset), const uint32_t UU(idx), void *UU(e)
 int verify_marked_messages(const int32_t &offset, const uint32_t UU(idx), struct verify_message_tree_extra *const e) __attribute__((nonnull(3)));
 int verify_marked_messages(const int32_t &offset, const uint32_t UU(idx), struct verify_message_tree_extra *const e)
 {
+    int verbose = e->verbose;
     BLOCKNUM blocknum = e->blocknum;
     int keep_going_on_failure = e->keep_going_on_failure;
     int result = 0;
-    bool is_fresh = e->msg_buffer->get_freshness(offset);
-    VERIFY_ASSERTION(!is_fresh, e->i, "marked message found in the fresh message tree that is fresh");
+    const struct fifo_entry *entry = toku_fifo_get_entry(e->fifo, offset);
+    VERIFY_ASSERTION(!entry->is_fresh, e->i, "marked message found in the fresh message tree that is fresh");
  done:
     return result;
 }
 
 template<typename verify_omt_t>
 static int
-verify_sorted_by_key_msn(FT_HANDLE ft_handle, message_buffer *msg_buffer, const verify_omt_t &mt) {
+verify_sorted_by_key_msn(FT_HANDLE ft_handle, FIFO fifo, const verify_omt_t &mt) {
     int result = 0;
     size_t last_offset = 0;
     for (uint32_t i = 0; i < mt.size(); i++) {
@@ -258,8 +251,12 @@ verify_sorted_by_key_msn(FT_HANDLE ft_handle, message_buffer *msg_buffer, const
         int r = mt.fetch(i, &offset);
         assert_zero(r);
         if (i > 0) {
-            struct toku_msg_buffer_key_msn_cmp_extra extra(ft_handle->ft->cmp, msg_buffer);
-            if (toku_msg_buffer_key_msn_cmp(extra, last_offset, offset) >= 0) {
+            struct toku_fifo_entry_key_msn_cmp_extra extra;
+            ZERO_STRUCT(extra);
+            extra.desc = &ft_handle->ft->cmp_descriptor;
+            extra.cmp = ft_handle->ft->compare_fun;
+            extra.fifo = fifo;
+            if (toku_fifo_entry_key_msn_cmp(extra, last_offset, offset) >= 0) {
                 result = TOKUDB_NEEDS_REPAIR;
                 break;
             }
@@ -271,9 +268,15 @@ verify_sorted_by_key_msn(FT_HANDLE ft_handle, message_buffer *msg_buffer, const
 
 template<typename count_omt_t>
 static int
-count_eq_key_msn(FT_HANDLE ft_handle, message_buffer *msg_buffer, const count_omt_t &mt, const DBT *key, MSN msn) {
-    struct toku_msg_buffer_key_msn_heaviside_extra extra(ft_handle->ft->cmp, msg_buffer, key, msn);
-    int r = mt.template find_zero<struct toku_msg_buffer_key_msn_heaviside_extra, toku_msg_buffer_key_msn_heaviside>(extra, nullptr, nullptr);
+count_eq_key_msn(FT_HANDLE ft_handle, FIFO fifo, const count_omt_t &mt, const DBT *key, MSN msn) {
+    struct toku_fifo_entry_key_msn_heaviside_extra extra;
+    ZERO_STRUCT(extra);
+    extra.desc = &ft_handle->ft->cmp_descriptor;
+    extra.cmp = ft_handle->ft->compare_fun;
+    extra.fifo = fifo;
+    extra.key = key;
+    extra.msn = msn;
+    int r = mt.template find_zero<struct toku_fifo_entry_key_msn_heaviside_extra, toku_fifo_entry_key_msn_heaviside>(extra, nullptr, nullptr);
     int count;
     if (r == 0) {
         count = 1;
@@ -292,8 +295,8 @@ toku_get_node_for_verify(
     )
 {
     uint32_t fullhash = toku_cachetable_hash(ft_handle->ft->cf, blocknum);
-    ftnode_fetch_extra bfe;
-    bfe.create_for_full_read(ft_handle->ft);
+    struct ftnode_fetch_extra bfe;
+    fill_bfe_for_full_read(&bfe, ft_handle->ft);
     toku_pin_ftnode(
         ft_handle->ft,
         blocknum,
@@ -305,83 +308,9 @@ toku_get_node_for_verify(
         );
 }
 
-struct verify_msg_fn {
-    FT_HANDLE ft_handle;
-    NONLEAF_CHILDINFO bnc;
-    const DBT *curr_less_pivot;
-    const DBT *curr_geq_pivot;
-    BLOCKNUM blocknum;
-    MSN this_msn;
-    int verbose;
-    int keep_going_on_failure;
-    bool messages_have_been_moved;
-
-    MSN last_msn;
-    int msg_i;
-    int result = 0; // needed by VERIFY_ASSERTION
-
-    verify_msg_fn(FT_HANDLE handle, NONLEAF_CHILDINFO nl, const DBT *less, const DBT *geq,
-                  BLOCKNUM b, MSN tmsn, int v, int k, bool m) :
-        ft_handle(handle), bnc(nl), curr_less_pivot(less), curr_geq_pivot(geq),
-        blocknum(b), this_msn(tmsn), verbose(v), keep_going_on_failure(k), messages_have_been_moved(m), last_msn(ZERO_MSN), msg_i(0) {
-    }
-
-    int operator()(const ft_msg &msg, bool is_fresh) {
-        enum ft_msg_type type = (enum ft_msg_type) msg.type();
-        MSN msn = msg.msn();
-        XIDS xid = msg.xids();
-        const void *key = msg.kdbt()->data;
-        const void *data = msg.vdbt()->data;
-        uint32_t keylen = msg.kdbt()->size;
-        uint32_t datalen = msg.vdbt()->size;
-
-        int r = verify_msg_in_child_buffer(ft_handle, type, msn, key, keylen, data, datalen, xid,
-                                           curr_less_pivot,
-                                           curr_geq_pivot);
-        VERIFY_ASSERTION(r == 0, msg_i, "A message in the buffer is out of place");
-        VERIFY_ASSERTION((msn.msn > last_msn.msn), msg_i, "msn per msg must be monotonically increasing toward newer messages in buffer");
-        VERIFY_ASSERTION((msn.msn <= this_msn.msn), msg_i, "all messages must have msn within limit of this node's max_msn_applied_to_node_in_memory");
-        if (ft_msg_type_applies_once(type)) {
-            int count;
-            DBT keydbt;
-            toku_fill_dbt(&keydbt, key, keylen);
-            int total_count = 0;
-            count = count_eq_key_msn(ft_handle, &bnc->msg_buffer, bnc->fresh_message_tree, toku_fill_dbt(&keydbt, key, keylen), msn);
-            total_count += count;
-            if (is_fresh) {
-                VERIFY_ASSERTION(count == 1, msg_i, "a fresh message was not found in the fresh message tree");
-            } else if (messages_have_been_moved) {
-                VERIFY_ASSERTION(count == 0, msg_i, "a stale message was found in the fresh message tree");
-            }
-            VERIFY_ASSERTION(count <= 1, msg_i, "a message was found multiple times in the fresh message tree");
-            count = count_eq_key_msn(ft_handle, &bnc->msg_buffer, bnc->stale_message_tree, &keydbt, msn);
-
-            total_count += count;
-            if (is_fresh) {
-                VERIFY_ASSERTION(count == 0, msg_i, "a fresh message was found in the stale message tree");
-            } else if (messages_have_been_moved) {
-                VERIFY_ASSERTION(count == 1, msg_i, "a stale message was not found in the stale message tree");
-            }
-            VERIFY_ASSERTION(count <= 1, msg_i, "a message was found multiple times in the stale message tree");
-
-            VERIFY_ASSERTION(total_count <= 1, msg_i, "a message was found in both message trees (or more than once in a single tree)");
-            VERIFY_ASSERTION(total_count >= 1, msg_i, "a message was not found in either message tree");
-        } else {
-            VERIFY_ASSERTION(ft_msg_type_applies_all(type) || ft_msg_type_does_nothing(type), msg_i, "a message was found that does not apply either to all or to only one key");
-            struct count_msgs_extra extra = { .count = 0, .msn = msn, .msg_buffer = &bnc->msg_buffer };
-            bnc->broadcast_list.iterate<struct count_msgs_extra, count_msgs>(&extra);
-            VERIFY_ASSERTION(extra.count == 1, msg_i, "a broadcast message was not found in the broadcast list");
-        }
-        last_msn = msn;
-        msg_i++;
-done:
-        return result;
-    }
-};
-
 static int
 toku_verify_ftnode_internal(FT_HANDLE ft_handle,
-                            MSN rootmsn, MSN parentmsn_with_messages, bool messages_exist_above,
+                            MSN rootmsn, MSN parentmsn, bool messages_exist_above,
                             FTNODE node, int height,
                             const DBT *lesser_pivot,               // Everything in the subtree should be > lesser_pivot.  (lesser_pivot==NULL if there is no lesser pivot.)
                             const DBT *greatereq_pivot,            // Everything in the subtree should be <= lesser_pivot.  (lesser_pivot==NULL if there is no lesser pivot.)
@@ -389,54 +318,88 @@ toku_verify_ftnode_internal(FT_HANDLE ft_handle,
 {
     int result=0;
     MSN   this_msn;
-    BLOCKNUM blocknum = node->blocknum;
+    BLOCKNUM blocknum = node->thisnodename;
 
     //printf("%s:%d pin %p\n", __FILE__, __LINE__, node_v);
-    toku_ftnode_assert_fully_in_memory(node);
+    toku_assert_entire_node_in_memory(node);
     this_msn = node->max_msn_applied_to_node_on_disk;
 
     if (height >= 0) {
         invariant(height == node->height);   // this is a bad failure if wrong
     }
     if (node->height > 0 && messages_exist_above) {
-        VERIFY_ASSERTION((parentmsn_with_messages.msn >= this_msn.msn), 0, "node msn must be descending down tree, newest messages at top");
+        VERIFY_ASSERTION((parentmsn.msn >= this_msn.msn), 0, "node msn must be descending down tree, newest messages at top");
     }
     // Verify that all the pivot keys are in order.
     for (int i = 0; i < node->n_children-2; i++) {
-        DBT x, y;
-        int compare = compare_pairs(ft_handle, node->pivotkeys.fill_pivot(i, &x), node->pivotkeys.fill_pivot(i + 1, &y));
+        int compare = compare_pairs(ft_handle, &node->childkeys[i], &node->childkeys[i+1]);
         VERIFY_ASSERTION(compare < 0, i, "Value is >= the next value");
     }
     // Verify that all the pivot keys are lesser_pivot < pivot <= greatereq_pivot
     for (int i = 0; i < node->n_children-1; i++) {
-        DBT x;
         if (lesser_pivot) {
-            int compare = compare_pairs(ft_handle, lesser_pivot, node->pivotkeys.fill_pivot(i, &x));
+            int compare = compare_pairs(ft_handle, lesser_pivot, &node->childkeys[i]);
             VERIFY_ASSERTION(compare < 0, i, "Pivot is >= the lower-bound pivot");
         }
         if (greatereq_pivot) {
-            int compare = compare_pairs(ft_handle, greatereq_pivot, node->pivotkeys.fill_pivot(i, &x));
+            int compare = compare_pairs(ft_handle, greatereq_pivot, &node->childkeys[i]);
             VERIFY_ASSERTION(compare >= 0, i, "Pivot is < the upper-bound pivot");
         }
     }
 
     for (int i = 0; i < node->n_children; i++) {
-        DBT x, y;
-        const DBT *curr_less_pivot = (i==0) ? lesser_pivot : node->pivotkeys.fill_pivot(i - 1, &x);
-        const DBT *curr_geq_pivot = (i==node->n_children-1) ? greatereq_pivot : node->pivotkeys.fill_pivot(i, &y);
+        const DBT *curr_less_pivot = (i==0) ? lesser_pivot : &node->childkeys[i-1];
+        const DBT *curr_geq_pivot = (i==node->n_children-1) ? greatereq_pivot : &node->childkeys[i];
         if (node->height > 0) {
-            NONLEAF_CHILDINFO bnc = BNC(node, i);
+            MSN last_msn = ZERO_MSN;
             // Verify that messages in the buffers are in the right place.
-            VERIFY_ASSERTION(verify_sorted_by_key_msn(ft_handle, &bnc->msg_buffer, bnc->fresh_message_tree) == 0, i, "fresh_message_tree");
-            VERIFY_ASSERTION(verify_sorted_by_key_msn(ft_handle, &bnc->msg_buffer, bnc->stale_message_tree) == 0, i, "stale_message_tree");
+            NONLEAF_CHILDINFO bnc = BNC(node, i);
+            VERIFY_ASSERTION(verify_sorted_by_key_msn(ft_handle, bnc->buffer, bnc->fresh_message_tree) == 0, i, "fresh_message_tree");
+            VERIFY_ASSERTION(verify_sorted_by_key_msn(ft_handle, bnc->buffer, bnc->stale_message_tree) == 0, i, "stale_message_tree");
+            FIFO_ITERATE(bnc->buffer, key, keylen, data, datalen, itype, msn, xid, is_fresh,
+                         ({
+                             enum ft_msg_type type = (enum ft_msg_type) itype;
+                             int r = verify_msg_in_child_buffer(ft_handle, type, msn, key, keylen, data, datalen, xid,
+                                                                curr_less_pivot,
+                                                                curr_geq_pivot);
+                             VERIFY_ASSERTION(r==0, i, "A message in the buffer is out of place");
+                             VERIFY_ASSERTION((msn.msn > last_msn.msn), i, "msn per msg must be monotonically increasing toward newer messages in buffer");
+                             VERIFY_ASSERTION((msn.msn <= this_msn.msn), i, "all messages must have msn within limit of this node's max_msn_applied_to_node_in_memory");
+                             if (ft_msg_type_applies_once(type)) {
+                                 int count;
+                                 DBT keydbt;
+                                 toku_fill_dbt(&keydbt, key, keylen);
+                                 int total_count = 0;
+                                 count = count_eq_key_msn(ft_handle, bnc->buffer, bnc->fresh_message_tree, toku_fill_dbt(&keydbt, key, keylen), msn);
+                                 total_count += count;
+                                 if (is_fresh) {
+                                     VERIFY_ASSERTION(count == 1, i, "a fresh message was not found in the fresh message tree");
+                                 } else if (messages_have_been_moved) {
+                                     VERIFY_ASSERTION(count == 0, i, "a stale message was found in the fresh message tree");
+                                 }
+                                 VERIFY_ASSERTION(count <= 1, i, "a message was found multiple times in the fresh message tree");
+                                 count = count_eq_key_msn(ft_handle, bnc->buffer, bnc->stale_message_tree, &keydbt, msn);
 
-            verify_msg_fn verify_msg(ft_handle, bnc, curr_less_pivot, curr_geq_pivot,
-                                     blocknum, this_msn, verbose, keep_going_on_failure, messages_have_been_moved);
-            int r = bnc->msg_buffer.iterate(verify_msg);
-            if (r != 0) { result = r; goto done; }
+                                 total_count += count;
+                                 if (is_fresh) {
+                                     VERIFY_ASSERTION(count == 0, i, "a fresh message was found in the stale message tree");
+                                 } else if (messages_have_been_moved) {
+                                     VERIFY_ASSERTION(count == 1, i, "a stale message was not found in the stale message tree");
+                                 }
+                                 VERIFY_ASSERTION(count <= 1, i, "a message was found multiple times in the stale message tree");
 
-            struct verify_message_tree_extra extra = { .msg_buffer = &bnc->msg_buffer, .broadcast = false, .is_fresh = true, .i = i, .verbose = verbose, .blocknum = node->blocknum, .keep_going_on_failure = keep_going_on_failure, .messages_have_been_moved = messages_have_been_moved };
-            r = bnc->fresh_message_tree.iterate<struct verify_message_tree_extra, verify_message_tree>(&extra);
+                                 VERIFY_ASSERTION(total_count <= 1, i, "a message was found in both message trees (or more than once in a single tree)");
+                                 VERIFY_ASSERTION(total_count >= 1, i, "a message was not found in either message tree");
+                             } else {
+                                 VERIFY_ASSERTION(ft_msg_type_applies_all(type) || ft_msg_type_does_nothing(type), i, "a message was found that does not apply either to all or to only one key");
+                                 struct count_msgs_extra extra = { .count = 0, .msn = msn, .fifo = bnc->buffer };
+                                 bnc->broadcast_list.iterate<struct count_msgs_extra, count_msgs>(&extra);
+                                 VERIFY_ASSERTION(extra.count == 1, i, "a broadcast message was not found in the broadcast list");
+                             }
+                             last_msn = msn;
+                         }));
+            struct verify_message_tree_extra extra = { .fifo = bnc->buffer, .broadcast = false, .is_fresh = true, .i = i, .verbose = verbose, .blocknum = node->thisnodename, .keep_going_on_failure = keep_going_on_failure, .messages_have_been_moved = messages_have_been_moved };
+            int r = bnc->fresh_message_tree.iterate<struct verify_message_tree_extra, verify_message_tree>(&extra);
             if (r != 0) { result = r; goto done; }
             extra.is_fresh = false;
             r = bnc->stale_message_tree.iterate<struct verify_message_tree_extra, verify_message_tree>(&extra);
@@ -464,16 +427,16 @@ toku_verify_ftnode_internal(FT_HANDLE ft_handle,
                 DBT kdbt = get_ith_key_dbt(bn, j);
                 if (curr_less_pivot) {
                     int compare = compare_pairs(ft_handle, curr_less_pivot, &kdbt);
-                    VERIFY_ASSERTION_BASEMENT(compare < 0, i, j, "The leafentry is >= the lower-bound pivot");
+                    VERIFY_ASSERTION(compare < 0, j, "The leafentry is >= the lower-bound pivot");
                 }
                 if (curr_geq_pivot) {
                     int compare = compare_pairs(ft_handle, curr_geq_pivot, &kdbt);
-                    VERIFY_ASSERTION_BASEMENT(compare >= 0, i, j, "The leafentry is < the upper-bound pivot");
+                    VERIFY_ASSERTION(compare >= 0, j, "The leafentry is < the upper-bound pivot");
                 }
                 if (0 < j) {
                     DBT prev_key_dbt = get_ith_key_dbt(bn, j-1);
                     int compare = compare_pairs(ft_handle, &prev_key_dbt, &kdbt);
-                    VERIFY_ASSERTION_BASEMENT(compare < 0, i, j, "Adjacent leafentries are out of order");
+                    VERIFY_ASSERTION(compare < 0, j, "Adjacent leafentries are out of order");
                 }
             }
         }
@@ -487,7 +450,7 @@ done:
 // input is a pinned node, on exit, node is unpinned
 int
 toku_verify_ftnode (FT_HANDLE ft_handle,
-                    MSN rootmsn, MSN parentmsn_with_messages, bool messages_exist_above,
+                    MSN rootmsn, MSN parentmsn, bool messages_exist_above,
                      FTNODE node, int height,
                      const DBT *lesser_pivot,               // Everything in the subtree should be > lesser_pivot.  (lesser_pivot==NULL if there is no lesser pivot.)
                      const DBT *greatereq_pivot,            // Everything in the subtree should be <= lesser_pivot.  (lesser_pivot==NULL if there is no lesser pivot.)
@@ -497,7 +460,7 @@ toku_verify_ftnode (FT_HANDLE ft_handle,
     MSN   this_msn;
 
     //printf("%s:%d pin %p\n", __FILE__, __LINE__, node_v);
-    toku_ftnode_assert_fully_in_memory(node);
+    toku_assert_entire_node_in_memory(node);
     this_msn = node->max_msn_applied_to_node_on_disk;
 
     int result = 0;
@@ -506,7 +469,7 @@ toku_verify_ftnode (FT_HANDLE ft_handle,
         // Otherwise we'll just do the next call
 
         result = toku_verify_ftnode_internal(
-                ft_handle, rootmsn, parentmsn_with_messages, messages_exist_above, node, height, lesser_pivot, greatereq_pivot,
+                ft_handle, rootmsn, parentmsn, messages_exist_above, node, height, lesser_pivot, greatereq_pivot,
                 verbose, keep_going_on_failure, false);
         if (result != 0 && (!keep_going_on_failure || result != TOKUDB_NEEDS_REPAIR)) goto done;
     }
@@ -514,7 +477,7 @@ toku_verify_ftnode (FT_HANDLE ft_handle,
         toku_move_ftnode_messages_to_stale(ft_handle->ft, node);
     }
     result2 = toku_verify_ftnode_internal(
-            ft_handle, rootmsn, parentmsn_with_messages, messages_exist_above, node, height, lesser_pivot, greatereq_pivot,
+            ft_handle, rootmsn, parentmsn, messages_exist_above, node, height, lesser_pivot, greatereq_pivot,
             verbose, keep_going_on_failure, true);
     if (result == 0) {
         result = result2;
@@ -526,17 +489,12 @@ toku_verify_ftnode (FT_HANDLE ft_handle,
         for (int i = 0; i < node->n_children; i++) {
             FTNODE child_node;
             toku_get_node_for_verify(BP_BLOCKNUM(node, i), ft_handle, &child_node);
-            DBT x, y;
-            int r = toku_verify_ftnode(ft_handle, rootmsn,
-                                       (toku_bnc_n_entries(BNC(node, i)) > 0
-                                        ? this_msn
-                                        : parentmsn_with_messages),
-                                       messages_exist_above || toku_bnc_n_entries(BNC(node, i)) > 0,
-                                       child_node, node->height-1,
-                                       (i==0)                  ? lesser_pivot        : node->pivotkeys.fill_pivot(i - 1, &x),
-                                       (i==node->n_children-1) ? greatereq_pivot     : node->pivotkeys.fill_pivot(i, &y),
-                                       progress_callback, progress_extra,
-                                       recurse, verbose, keep_going_on_failure);
+            int r = toku_verify_ftnode(ft_handle, rootmsn, this_msn, messages_exist_above || toku_bnc_n_entries(BNC(node, i)) > 0,
+                                        child_node, node->height-1,
+                                        (i==0)                  ? lesser_pivot        : &node->childkeys[i-1],
+                                        (i==node->n_children-1) ? greatereq_pivot     : &node->childkeys[i],
+                                        progress_callback, progress_extra,
+                                        recurse, verbose, keep_going_on_failure);
             if (r) {
                 result = r;
                 if (!keep_going_on_failure || result != TOKUDB_NEEDS_REPAIR) goto done;
