@@ -50,6 +50,7 @@ UNIVERSITY PATENT NOTICE:
 PATENT MARKING NOTICE:
 
   This software is covered by US Patent No. 8,185,551.
+  This software is covered by US Patent No. 8,489,638.
 
 PATENT RIGHTS GRANT:
 
@@ -112,14 +113,14 @@ static void test_multiple_ft_cursor_dbts(int n) {
 
     int r;
     CACHETABLE ct;
-    FT_HANDLE brt;
+    FT_HANDLE ft;
     FT_CURSOR cursors[n];
 
     unlink(fname);
 
     toku_cachetable_create(&ct, 0, ZERO_LSN, NULL_LOGGER);
 
-    r = toku_open_ft_handle(fname, 1, &brt, 1<<12, 1<<9, TOKU_DEFAULT_COMPRESSION_METHOD, ct, null_txn, toku_builtin_compare_fun);
+    r = toku_open_ft_handle(fname, 1, &ft, 1<<12, 1<<9, TOKU_DEFAULT_COMPRESSION_METHOD, ct, null_txn, toku_builtin_compare_fun);
     assert(r==0);
 
     int i;
@@ -128,14 +129,14 @@ static void test_multiple_ft_cursor_dbts(int n) {
 	char key[10],val[10];
 	snprintf(key, sizeof key, "k%04d", i);
 	snprintf(val, sizeof val, "v%04d", i);
-	toku_ft_insert(brt,
+	toku_ft_insert(ft,
                        toku_fill_dbt(&kbt, key, 1+strlen(key)),
                        toku_fill_dbt(&vbt, val, 1+strlen(val)),
                        0);
     }
 
     for (i=0; i<n; i++) {
-        r = toku_ft_cursor(brt, &cursors[i], NULL, false, false);
+        r = toku_ft_cursor(ft, &cursors[i], NULL, false, false);
         assert(r == 0);
     }
 
@@ -165,7 +166,7 @@ static void test_multiple_ft_cursor_dbts(int n) {
 	toku_free(ptrs[i]);
     }
 
-    r = toku_close_ft_handle_nolsn(brt, 0);
+    r = toku_close_ft_handle_nolsn(ft, 0);
     assert(r==0);
 
     toku_cachetable_close(&ct);
