@@ -50,6 +50,7 @@ UNIVERSITY PATENT NOTICE:
 PATENT MARKING NOTICE:
 
   This software is covered by US Patent No. 8,185,551.
+  This software is covered by US Patent No. 8,489,638.
 
 PATENT RIGHTS GRANT:
 
@@ -91,7 +92,7 @@ PATENT RIGHTS GRANT:
 #include <string.h>
 #include <db.h>
 
-#include <toku_include/memory.h>
+#include <portability/memory.h>
 
 namespace toku {
 
