@@ -1,34 +1,98 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil -*- */
 // vim: ft=cpp:expandtab:ts=8:sw=4:softtabstop=4:
 #ident "$Id$"
-/*======
-This file is part of TokuDB
+/*
+COPYING CONDITIONS NOTICE:
 
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of version 2 of the GNU General Public License as
+  published by the Free Software Foundation, and provided that the
+  following conditions are met:
 
-Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
+      * Redistributions of source code must retain this COPYING
+        CONDITIONS NOTICE, the COPYRIGHT NOTICE (below), the
+        DISCLAIMER (below), the UNIVERSITY PATENT NOTICE (below), the
+        PATENT MARKING NOTICE (below), and the PATENT RIGHTS
+        GRANT (below).
 
-    TokuDBis is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License, version 2,
-    as published by the Free Software Foundation.
+      * Redistributions in binary form must reproduce this COPYING
+        CONDITIONS NOTICE, the COPYRIGHT NOTICE (below), the
+        DISCLAIMER (below), the UNIVERSITY PATENT NOTICE (below), the
+        PATENT MARKING NOTICE (below), and the PATENT RIGHTS
+        GRANT (below) in the documentation and/or other materials
+        provided with the distribution.
 
-    TokuDB is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+  02110-1301, USA.
 
-    You should have received a copy of the GNU General Public License
-    along with TokuDB.  If not, see <http://www.gnu.org/licenses/>.
+COPYRIGHT NOTICE:
 
-======= */
+  TokuDB, Tokutek Fractal Tree Indexing Library.
+  Copyright (C) 2007-2013 Tokutek, Inc.
 
-#ident "Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved."
+DISCLAIMER:
 
-#ifndef _HA_TOKUDB_H
-#define _HA_TOKUDB_H
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-#include "hatoku_hton.h"
+UNIVERSITY PATENT NOTICE:
+
+  The technology is licensed by the Massachusetts Institute of
+  Technology, Rutgers State University of New Jersey, and the Research
+  Foundation of State University of New York at Stony Brook under
+  United States of America Serial No. 11/760379 and to the patents
+  and/or patent applications resulting from it.
+
+PATENT MARKING NOTICE:
+
+  This software is covered by US Patent No. 8,185,551.
+  This software is covered by US Patent No. 8,489,638.
+
+PATENT RIGHTS GRANT:
+
+  "THIS IMPLEMENTATION" means the copyrightable works distributed by
+  Tokutek as part of the Fractal Tree project.
+
+  "PATENT CLAIMS" means the claims of patents that are owned or
+  licensable by Tokutek, both currently or in the future; and that in
+  the absence of this license would be infringed by THIS
+  IMPLEMENTATION or by using or running THIS IMPLEMENTATION.
+
+  "PATENT CHALLENGE" shall mean a challenge to the validity,
+  patentability, enforceability and/or non-infringement of any of the
+  PATENT CLAIMS or otherwise opposing any of the PATENT CLAIMS.
+
+  Tokutek hereby grants to you, for the term and geographical scope of
+  the PATENT CLAIMS, a non-exclusive, no-charge, royalty-free,
+  irrevocable (except as stated in this section) patent license to
+  make, have made, use, offer to sell, sell, import, transfer, and
+  otherwise run, modify, and propagate the contents of THIS
+  IMPLEMENTATION, where such license applies only to the PATENT
+  CLAIMS.  This grant does not include claims that would be infringed
+  only as a consequence of further modifications of THIS
+  IMPLEMENTATION.  If you or your agent or licensee institute or order
+  or agree to the institution of patent litigation against any entity
+  (including a cross-claim or counterclaim in a lawsuit) alleging that
+  THIS IMPLEMENTATION constitutes direct or contributory patent
+  infringement, or inducement of patent infringement, then any rights
+  granted to you under this License shall terminate as of the date
+  such litigation is filed.  If you or your agent or exclusive
+  licensee institute or order or agree to the institution of a PATENT
+  CHALLENGE, then Tokutek may terminate any rights granted to you
+  under this License.
+*/
+
+#ident "Copyright (c) 2007-2013 Tokutek Inc.  All rights reserved."
+#ident "The technology is licensed by the Massachusetts Institute of Technology, Rutgers State University of New Jersey, and the Research Foundation of State University of New York at Stony Brook under United States of America Serial No. 11/760379 and to the patents and/or patent applications resulting from it."
+#if !defined(HA_TOKUDB_H)
+#define HA_TOKUDB_H
+
+#include <db.h>
 #include "hatoku_cmp.h"
-#include "tokudb_background.h"
 
 #define HA_TOKU_ORIG_VERSION 4
 #define HA_TOKU_VERSION 4
@@ -45,377 +109,91 @@ typedef struct loader_context {
     ha_tokudb* ha;
 } *LOADER_CONTEXT;
 
+typedef struct hot_optimize_context {
+    THD *thd;
+    char* write_status_msg;
+    ha_tokudb *ha;
+    uint progress_stage;
+    uint current_table;
+    uint num_tables;
+} *HOT_OPTIMIZE_CONTEXT;
+
 //
-// This class stores table information that is to be shared
+// This object stores table information that is to be shared
 // among all ha_tokudb objects.
-// There is one instance per table, shared among handlers.
+// There is one instance per table, shared among threads.
 // Some of the variables here are the DB* pointers to indexes,
 // and auto increment information.
 //
-// When the last user releases it's reference on the share,
-// it closes all of its database handles and releases all info
-// The share instance stays around though so some data can be transiently
-// kept across open-close-open-close cycles. These data will be explicitly
-// noted below.
-//
 class TOKUDB_SHARE {
 public:
-    enum share_state_t {
-        CLOSED = 0,
-        OPENED = 1,
-        ERROR = 2
-    };
-
-    // one time, start up init
-    static void static_init();
-
-    // one time, shutdown destroy
-    static void static_destroy();
-
-    // retuns a locked, properly reference counted share
-    // callers must check to ensure share is in correct state for callers use
-    // and unlock the share.
-    // if create_new is set, a new "CLOSED" share will be created if one
-    // doesn't exist, otherwise will return NULL if an existing is not found.
-    static TOKUDB_SHARE* get_share(
-        const char* table_name,
-        THR_LOCK_DATA* data,
-        bool create_new);
-
-    // removes a share entirely from the pool, call to rename/deleta a table
-    // caller must hold ddl_mutex on this share and the share MUST have
-    // exactly 0 _use_count
-    static void drop_share(TOKUDB_SHARE* share);
-
-    // returns state string for logging/reporting
-    static const char* get_state_string(share_state_t state);
-
-    void* operator new(size_t sz);
-    void operator delete(void* p);
-
-    TOKUDB_SHARE();
-
-    // increases the ref count and waits for any currently executing state
-    // transition to complete
-    // returns current state and leaves share locked
-    // callers must check to ensure share is in correct state for callers use
-    // and unlock the share.
-    share_state_t addref();
-
-    // decreases the ref count and potentially closes the share
-    // caller must not have ownership of mutex, will lock and release
-    int release();
-
-    // returns the current use count
-    // no locking requirements
-    inline int use_count() const;
-
-    // locks the share
-    inline void lock() const;
-
-    // unlocks the share
-    inline void unlock() const;
-
-    // returns the current state of the share
-    // no locking requirements
-    inline share_state_t state() const;
-
-    // sets the state of the share
-    // caller must hold mutex on this share
-    inline void set_state(share_state_t state);
-
-    // returns the full MySQL table name of the table ex:
-    // ./database/table
-    // no locking requirements
-    inline const char* full_table_name() const;
-
-    // returns the strlen of the full table name
-    // no locking requirements
-    inline uint full_table_name_length() const;
-
-    // returns the parsed database name this table resides in
-    // no locking requirements
-    inline const char* database_name() const;
-
-    // returns the strlen of the database name
-    // no locking requirements
-    inline uint database_name_length() const;
-
-    // returns the parsed table name of this table
-    // no locking requirements
-    inline const char* table_name() const;
-
-    // returns the strlen of the the table name
-    // no locking requirements
-    inline uint table_name_length() const;
-
-    // sets the estimated number of rows in the table
-    // should be called only during share initialization and info call
-    // caller must hold mutex on this share unless specified by 'locked'
-    inline void set_row_count(uint64_t rows, bool locked);
-
-    // updates tracked row count and ongoing table change delta tracking
-    // called from any ha_tokudb operation that inserts/modifies/deletes rows
-    // may spawn background analysis if enabled, allowed and threshold hit
-    // caller must not have ownership of mutex, will lock and release
-    void update_row_count(
-        THD* thd,
-        uint64_t added,
-        uint64_t deleted,
-        uint64_t updated);
-
-    // returns the current row count estimate
-    // no locking requirements
-    inline ha_rows row_count() const;
-
-    // initializes cardinality statistics, takes ownership of incoming buffer
-    // caller must hold mutex on this share
-    inline void init_cardinality_counts(
-        uint32_t rec_per_keys,
-        uint64_t* rec_per_key);
-
-    // update the cardinality statistics. number of records must match
-    // caller must hold mutex on this share
-    inline void update_cardinality_counts(
-        uint32_t rec_per_keys,
-        const uint64_t* rec_per_key);
-
-    // disallow any auto analysis from taking place
-    // caller must hold mutex on this share
-    inline void disallow_auto_analysis();
-
-    // allow any auto analysis to take place
-    // pass in true for 'reset_deltas' to reset delta counting to 0
-    // caller must hold mutex on this share
-    inline void allow_auto_analysis(bool reset_deltas);
-
-    // cancels all background jobs for this share
-    // no locking requirements
-    inline void cancel_background_jobs() const;
-
-    // copies cardinality statistics into TABLE counter set
-    // caller must not have ownership of mutex, will lock and release
-    void set_cardinality_counts_in_table(TABLE* table);
-
-    // performs table analysis on underlying indices and produces estimated
-    // cardinality statistics.
-    // on success updates cardinality counts in status database and this share
-    // MUST pass a valid THD to access session variables.
-    // MAY pass txn. If txn is passed, assumes an explicit user scheduled
-    // ANALYZE and not an auto ANALYZE resulting from delta threshold
-    // uses session variables:
-    //  tokudb_analyze_in_background, tokudb_analyze_throttle,
-    //  tokudb_analyze_time, and tokudb_analyze_delete_fraction
-    // caller must hold mutex on this share
-    int analyze_standard(THD* thd, DB_TXN* txn);
-
-    // performs table scan and updates the internal FT logical row count value
-    // on success also updates share row count estimate.
-    // MUST pass a valid THD to access session variables.
-    // MAY pass txn. If txn is passed, assumes an explicit user scheduled
-    // uses session variables:
-    //  tokudb_analyze_in_background, and tokudb_analyze_throttle
-    // caller must not have ownership of mutex, will lock and release
-    int analyze_recount_rows(THD* thd, DB_TXN* txn);
+    void init(void);
+    void destroy(void);
 
 public:
-    //*********************************
-    // Destroyed and recreated on open-close-open
+    char *table_name;
+    uint table_name_length, use_count;
+    pthread_mutex_t mutex;
+    THR_LOCK lock;
+
     ulonglong auto_ident;
     ulonglong last_auto_increment, auto_inc_create_value;
-
+    //
+    // estimate on number of rows in table
+    //
+    ha_rows rows;
+    //
     // estimate on number of rows added in the process of a locked tables
     // this is so we can better estimate row count during a lock table
+    //
     ha_rows rows_from_locked_table;
-    DB* status_block;
-
+    DB *status_block;
+    //
     // DB that is indexed on the primary key
-    DB* file;
-
+    //
+    DB *file;
+    //
     // array of all DB's that make up table, includes DB that
     // is indexed on the primary key, add 1 in case primary
     // key is hidden
-    DB* key_file[MAX_KEY + 1];
+    //
+    DB *key_file[MAX_KEY +1];
+    rw_lock_t key_file_lock;
     uint status, version, capabilities;
     uint ref_length;
-
+    //
     // whether table has an auto increment column
+    //
     bool has_auto_inc;
-
+    //
     // index of auto increment column in table->field, if auto_inc exists
+    //
     uint ai_field_index;
-
+    //
     // whether the primary key has a string
+    //
     bool pk_has_string;
 
     KEY_AND_COL_INFO kc_info;
-
-    // key info copied from TABLE_SHARE, used by background jobs that have no
-    // access to a handler instance
-    uint _keys;
-    uint _max_key_parts;
-    struct key_descriptor_t {
-        uint _parts;
-        bool _is_unique;
-        char* _name;
-    };
-    key_descriptor_t* _key_descriptors;
-
+    
+    // 
     // we want the following optimization for bulk loads, if the table is empty, 
     // attempt to grab a table lock. emptiness check can be expensive, 
     // so we try it once for a table. After that, we keep this variable around 
-    // to tell us to not try it again.
-    bool try_table_lock;
+    // to tell us to not try it again. 
+    // 
+    bool try_table_lock; 
 
     bool has_unique_keys;
     bool replace_into_fast;
-    tokudb::thread::rwlock_t _num_DBs_lock;
+    rw_lock_t num_DBs_lock;
     uint32_t num_DBs;
 
-private:
-    static std::unordered_map<std::string, TOKUDB_SHARE*> _open_tables;
-    static tokudb::thread::mutex_t* _open_tables_mutex;
-
-    //*********************************
-    // Spans open-close-open
-    mutable tokudb::thread::mutex_t _mutex;
-    mutable tokudb::thread::mutex_t _ddl_mutex;
-    uint _use_count;
-
-    share_state_t _state;
-
-    ulonglong _row_delta_activity;
-    bool _allow_auto_analysis;
-
-    String _full_table_name;
-    String _database_name;
-    String _table_name;
-
-    //*********************************
-    // Destroyed and recreated on open-close-open
-    THR_LOCK _thr_lock;
-
-    // estimate on number of rows in table
-    ha_rows _rows;
-
-    // cardinality counts
-    uint32_t _rec_per_keys;
-    uint64_t* _rec_per_key;
-
-    void init(const char* table_name);
-    void destroy();
+    pthread_cond_t m_openclose_cond;
+    enum { CLOSED, OPENING, OPENED, CLOSING, ERROR } m_state;
+    int m_error;
+    int m_initialize_count;
 };
-inline int TOKUDB_SHARE::use_count() const {
-    return _use_count;
-}
-inline void TOKUDB_SHARE::lock() const {
-    TOKUDB_SHARE_DBUG_ENTER("file[%s]:state[%s]:use_count[%d]",
-                            _full_table_name.ptr(),
-                            get_state_string(_state),
-                            _use_count);
-    mutex_t_lock(_mutex);
-    TOKUDB_SHARE_DBUG_VOID_RETURN();
-}
-inline void TOKUDB_SHARE::unlock() const {
-    TOKUDB_SHARE_DBUG_ENTER("file[%s]:state[%s]:use_count[%d]",
-                            _full_table_name.ptr(),
-                            get_state_string(_state),
-                            _use_count);
-    mutex_t_unlock(_mutex);
-    TOKUDB_SHARE_DBUG_VOID_RETURN();
-}
-inline TOKUDB_SHARE::share_state_t TOKUDB_SHARE::state() const {
-    return _state;
-}
-inline void TOKUDB_SHARE::set_state(TOKUDB_SHARE::share_state_t state) {
-    TOKUDB_SHARE_DBUG_ENTER("file[%s]:state[%s]:use_count[%d]:new_state[%s]",
-        _full_table_name.ptr(),
-        get_state_string(_state),
-        _use_count,
-        get_state_string(state));
-
-    assert_debug(_mutex.is_owned_by_me());
-    _state = state;
-    TOKUDB_SHARE_DBUG_VOID_RETURN();
-}
-inline const char* TOKUDB_SHARE::full_table_name() const {
-    return _full_table_name.ptr();
-}
-inline uint TOKUDB_SHARE::full_table_name_length() const {
-    return _full_table_name.length();
-}
-inline const char* TOKUDB_SHARE::database_name() const {
-    return _database_name.ptr();
-}
-inline uint TOKUDB_SHARE::database_name_length() const {
-    return _database_name.length();
-}
-inline const char* TOKUDB_SHARE::table_name() const {
-    return _table_name.ptr();
-}
-inline uint TOKUDB_SHARE::table_name_length() const {
-    return _table_name.length();
-}
-inline void TOKUDB_SHARE::set_row_count(uint64_t rows, bool locked) {
-    TOKUDB_SHARE_DBUG_ENTER("file[%s]:state[%s]:use_count[%d]:rows[%" PRIu64 "]:locked[%d]",
-        _full_table_name.ptr(),
-        get_state_string(_state),
-        _use_count,
-        rows,
-        locked);
-
-    if (!locked) {
-        lock();
-    } else {
-        assert_debug(_mutex.is_owned_by_me());
-    }
-    if (_rows && rows == 0)
-        _row_delta_activity = 0;
-
-    _rows = rows;
-    if (!locked) {
-        unlock();
-    }
-    TOKUDB_SHARE_DBUG_VOID_RETURN();
-}
-inline ha_rows TOKUDB_SHARE::row_count() const {
-    return _rows;
-}
-inline void TOKUDB_SHARE::init_cardinality_counts(
-    uint32_t rec_per_keys,
-    uint64_t* rec_per_key) {
-
-    assert_debug(_mutex.is_owned_by_me());
-    // can not change number of keys live
-    assert_always(_rec_per_key == NULL && _rec_per_keys == 0);
-    _rec_per_keys = rec_per_keys;
-    _rec_per_key = rec_per_key;
-}
-inline void TOKUDB_SHARE::update_cardinality_counts(
-    uint32_t rec_per_keys,
-    const uint64_t* rec_per_key) {
-
-    assert_debug(_mutex.is_owned_by_me());
-    // can not change number of keys live
-    assert_always(rec_per_keys == _rec_per_keys);
-    assert_always(rec_per_key != NULL);
-    memcpy(_rec_per_key, rec_per_key, _rec_per_keys * sizeof(uint64_t));
-}
-inline void TOKUDB_SHARE::disallow_auto_analysis() {
-    assert_debug(_mutex.is_owned_by_me());
-    _allow_auto_analysis = false;
-}
-inline void TOKUDB_SHARE::allow_auto_analysis(bool reset_deltas) {
-    assert_debug(_mutex.is_owned_by_me());
-    _allow_auto_analysis = true;
-    if (reset_deltas)
-        _row_delta_activity = 0;
-}
-inline void TOKUDB_SHARE::cancel_background_jobs() const {
-    tokudb::background::_job_manager->cancel_job(full_table_name());
-}
-
-
 
 typedef struct st_filter_key_part_info {
     uint offset;
@@ -482,7 +260,6 @@ private:
     uint64_t bulk_fetch_iteration;
     uint64_t rows_fetched_using_bulk_fetch;
     bool doing_bulk_fetch;
-    bool maybe_index_scan;
 
     //
     // buffer used to temporarily store a "packed key" 
@@ -571,7 +348,6 @@ private:
     // 
     ulonglong added_rows;
     ulonglong deleted_rows;
-    ulonglong updated_rows;
 
 
     uint last_dup_key;
@@ -649,11 +425,11 @@ private:
     DBT *create_dbt_key_from_table(DBT * key, uint keynr, uchar * buff, const uchar * record, bool* has_null, int key_length = MAX_KEY_LENGTH);
     DBT* create_dbt_key_for_lookup(DBT * key, KEY* key_info, uchar * buff, const uchar * record, bool* has_null, int key_length = MAX_KEY_LENGTH);
     DBT *pack_key(DBT * key, uint keynr, uchar * buff, const uchar * key_ptr, uint key_length, int8_t inf_byte);
-#if defined(TOKU_INCLUDE_EXTENDED_KEYS) && TOKU_INCLUDE_EXTENDED_KEYS
+#if TOKU_INCLUDE_EXTENDED_KEYS
     DBT *pack_ext_key(DBT * key, uint keynr, uchar * buff, const uchar * key_ptr, uint key_length, int8_t inf_byte);
-#endif  // defined(TOKU_INCLUDE_EXTENDED_KEYS) && TOKU_INCLUDE_EXTENDED_KEYS
+#endif
     bool key_changed(uint keynr, const uchar * old_row, const uchar * new_row);
-    int handle_cursor_error(int error, int err_to_return);
+    int handle_cursor_error(int error, int err_to_return, uint keynr);
     DBT *get_pos(DBT * to, uchar * pos);
  
     int open_main_dictionary(const char* name, bool is_read_only, DB_TXN* txn);
@@ -662,11 +438,9 @@ private:
     int estimate_num_rows(DB* db, uint64_t* num_rows, DB_TXN* txn);
     bool has_auto_increment_flag(uint* index);
 
-#if defined(TOKU_INCLUDE_WRITE_FRM_DATA) && TOKU_INCLUDE_WRITE_FRM_DATA
     int write_frm_data(DB* db, DB_TXN* txn, const char* frm_name);
     int verify_frm_data(const char* frm_name, DB_TXN* trans);
     int remove_frm_data(DB *db, DB_TXN *txn);
-#endif  // defined(TOKU_INCLUDE_WRITE_FRM_DATA) && TOKU_INCLUDE_WRITE_FRM_DATA
 
     int write_to_status(DB* db, HA_METADATA_KEY curr_key_data, void* data, uint size, DB_TXN* txn);
     int remove_from_status(DB* db, HA_METADATA_KEY curr_key_data, DB_TXN* txn);
@@ -700,12 +474,12 @@ private:
         toku_compression_method compression_method
         );
     int create_main_dictionary(const char* name, TABLE* form, DB_TXN* txn, KEY_AND_COL_INFO* kc_info, toku_compression_method compression_method);
-    void trace_create_table_info(TABLE* form);
-    int is_index_unique(bool* is_unique, DB_TXN* txn, DB* db, KEY* key_info, int lock_flags);
+    void trace_create_table_info(const char *name, TABLE * form);
+    int is_index_unique(bool* is_unique, DB_TXN* txn, DB* db, KEY* key_info);
     int is_val_unique(bool* is_unique, uchar* record, KEY* key_info, uint dict_index, DB_TXN* txn);
     int do_uniqueness_checks(uchar* record, DB_TXN* txn, THD* thd);
     void set_main_dict_put_flags(THD* thd, bool opt_eligible, uint32_t* put_flags);
-    int insert_row_to_main_dictionary(DBT* pk_key, DBT* pk_val, DB_TXN* txn);
+    int insert_row_to_main_dictionary(uchar* record, DBT* pk_key, DBT* pk_val, DB_TXN* txn);
     int insert_rows_to_dictionaries_mult(DBT* pk_key, DBT* pk_val, DB_TXN* txn, THD* thd);
     void test_row_packing(uchar* record, DBT* pk_key, DBT* pk_val);
     uint32_t fill_row_mutator(
@@ -734,7 +508,7 @@ public:
     // Returns a bit mask of capabilities of storage engine. Capabilities 
     // defined in sql/handler.h
     //
-    ulonglong table_flags() const;
+    ulonglong table_flags(void) const;
     
     ulong index_flags(uint inx, uint part, bool all_parts) const;
 
@@ -778,7 +552,7 @@ public:
     double index_only_read_time(uint keynr, double records);
 
     int open(const char *name, int mode, uint test_if_locked);
-    int close();
+    int close(void);
     void update_create_info(HA_CREATE_INFO* create_info);
     int create(const char *name, TABLE * form, HA_CREATE_INFO * create_info);
     int delete_table(const char *name);
@@ -788,24 +562,12 @@ public:
     int write_row(uchar * buf);
     int update_row(const uchar * old_data, uchar * new_data);
     int delete_row(const uchar * buf);
+
 #if MYSQL_VERSION_ID >= 100000
     void start_bulk_insert(ha_rows rows, uint flags);
 #else
     void start_bulk_insert(ha_rows rows);
 #endif
-    static int bulk_insert_poll(void* extra, float progress);
-    static void loader_add_index_err(DB* db,
-                                     int i,
-                                     int err,
-                                     DBT* key,
-                                     DBT* val,
-                                     void* error_extra);
-    static void loader_dup(DB* db,
-                           int i,
-                           int err,
-                           DBT* key,
-                           DBT* val,
-                           void* error_extra);
     int end_bulk_insert();
     int end_bulk_insert(bool abort);
 
@@ -823,8 +585,6 @@ public:
     int index_first(uchar * buf);
     int index_last(uchar * buf);
 
-    bool has_gap_locks() const { return true; }
-
     int rnd_init(bool scan);
     int rnd_end();
     int rnd_next(uchar * buf);
@@ -839,7 +599,7 @@ public:
     void position(const uchar * record);
     int info(uint);
     int extra(enum ha_extra_function operation);
-    int reset();
+    int reset(void);
     int external_lock(THD * thd, int lock_type);
     int start_stmt(THD * thd, thr_lock_type lock_type);
 
@@ -851,17 +611,12 @@ public:
     int get_status(DB_TXN* trans);
     void init_hidden_prim_key_info(DB_TXN *txn);
     inline void get_auto_primary_key(uchar * to) {
-        share->lock();
+        tokudb_pthread_mutex_lock(&share->mutex);
         share->auto_ident++;
         hpk_num_to_char(to, share->auto_ident);
-        share->unlock();
+        tokudb_pthread_mutex_unlock(&share->mutex);
     }
-    virtual void get_auto_increment(
-        ulonglong offset,
-        ulonglong increment,
-        ulonglong nb_desired_values,
-        ulonglong* first_value,
-        ulonglong* nb_reserved_values);
+    virtual void get_auto_increment(ulonglong offset, ulonglong increment, ulonglong nb_desired_values, ulonglong * first_value, ulonglong * nb_reserved_values);
     bool is_optimize_blocking();
     bool is_auto_inc_singleton();
     void print_error(int error, myf errflag);
@@ -871,6 +626,9 @@ public:
     bool primary_key_is_clustered() {
         return true;
     }
+    bool supports_clustered_keys() {
+        return true;
+    }
     int cmp_ref(const uchar * ref1, const uchar * ref2);
     bool check_if_incompatible_data(HA_CREATE_INFO * info, uint table_changes);
 
@@ -912,18 +670,19 @@ public:
 
 #endif
 
+    // ICP introduced in MariaDB 5.5
     Item* idx_cond_push(uint keyno, class Item* idx_cond);
-    void cancel_pushed_idx_cond();
 
-#if defined(TOKU_INCLUDE_ALTER_56) && TOKU_INCLUDE_ALTER_56
+
+#if TOKU_INCLUDE_ALTER_56
  public:
     enum_alter_inplace_result check_if_supported_inplace_alter(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
     bool prepare_inplace_alter_table(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
     bool inplace_alter_table(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
     bool commit_inplace_alter_table(TABLE *altered_table, Alter_inplace_info *ha_alter_info, bool commit);
  private:
-    int alter_table_add_index(Alter_inplace_info* ha_alter_info);
-    int alter_table_drop_index(Alter_inplace_info* ha_alter_info);
+    int alter_table_add_index(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
+    int alter_table_drop_index(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
     int alter_table_add_or_drop_column(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
     int alter_table_expand_varchar_offsets(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
     int alter_table_expand_columns(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
@@ -931,36 +690,31 @@ public:
     int alter_table_expand_blobs(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
     void print_alter_info(TABLE *altered_table, Alter_inplace_info *ha_alter_info);
     int setup_kc_info(TABLE *altered_table, KEY_AND_COL_INFO *kc_info);
-    int new_row_descriptor(TABLE* altered_table,
-                           Alter_inplace_info* ha_alter_info,
-                           uint32_t idx,
-                           DBT* row_descriptor);
+    int new_row_descriptor(TABLE *table, TABLE *altered_table, Alter_inplace_info *ha_alter_info, uint32_t idx, DBT *row_descriptor);
 
  public:
-#endif  // defined(TOKU_INCLUDE_ALTER_56) && TOKU_INCLUDE_ALTER_56
-#if defined(TOKU_INCLUDE_ALTER_55) && TOKU_INCLUDE_ALTER_55
+#endif
+#if TOKU_INCLUDE_ALTER_55
 public:
     // Returns true of the 5.6 inplace alter table interface is used.
     bool try_hot_alter_table();
 
     // Used by the partition storage engine to provide new frm data for the table.
     int new_alter_table_frm_data(const uchar *frm_data, size_t frm_len);
-#endif  // defined(TOKU_INCLUDE_ALTER_55) && TOKU_INCLUDE_ALTER_55
+#endif
 
  private:
-  int tokudb_add_index(TABLE* table_arg,
-                       KEY* key_info,
-                       uint num_of_keys,
-                       DB_TXN* txn,
-                       bool* inc_num_DBs,
-                       bool* modified_DB);
-  static int tokudb_add_index_poll(void *extra, float progress);
-  void restore_add_index(TABLE* table_arg,
-                         uint num_of_keys,
-                         bool incremented_numDBs,
-                         bool modified_DBs);
-  int drop_indexes(uint* key_num, uint num_of_keys, KEY* key_info, DB_TXN* txn);
-  void restore_drop_indexes(uint* key_num, uint num_of_keys);
+    int tokudb_add_index(
+        TABLE *table_arg, 
+        KEY *key_info, 
+        uint num_of_keys, 
+        DB_TXN* txn, 
+        bool* inc_num_DBs,
+        bool* modified_DB
+        ); 
+    void restore_add_index(TABLE* table_arg, uint num_of_keys, bool incremented_numDBs, bool modified_DBs);
+    int drop_indexes(TABLE *table_arg, uint *key_num, uint num_of_keys, KEY *key_info, DB_TXN* txn);
+    void restore_drop_indexes(TABLE *table_arg, uint *key_num, uint num_of_keys);
 
  public:
     // delete all rows from the table
@@ -1004,60 +758,41 @@ public:
 
     int fill_range_query_buf(
         bool need_val, 
-        DBT const* key,
-        DBT const* row,
+        DBT const *key, 
+        DBT  const *row, 
         int direction,
         THD* thd,
         uchar* buf,
-        DBT* key_to_compare);
+        DBT* key_to_compare
+        );
 
-#if defined(TOKU_INCLUDE_ROW_TYPE_COMPRESSION) && \
-    TOKU_INCLUDE_ROW_TYPE_COMPRESSION
-    enum row_type get_row_type() const;
-#endif  // defined(TOKU_INCLUDE_ROW_TYPE_COMPRESSION) &&
-        // TOKU_INCLUDE_ROW_TYPE_COMPRESSION
 private:
     int read_full_row(uchar * buf);
     int __close();
     int get_next(uchar* buf, int direction, DBT* key_to_compare, bool do_key_read);
     int read_data_from_range_query_buff(uchar* buf, bool need_val, bool do_key_read);
     // for ICP, only in MariaDB and MySQL 5.6
+#if defined(MARIADB_BASE_VERSION) || (50600 <= MYSQL_VERSION_ID && MYSQL_VERSION_ID <= 50699)
     enum icp_result toku_handler_index_cond_check(Item* pushed_idx_cond);
+#endif
     void invalidate_bulk_fetch();
     void invalidate_icp();
     int delete_all_rows_internal();
     void close_dsmrr();
     void reset_dsmrr();
     
-#if defined(TOKU_INCLUDE_WRITE_FRM_DATA) && TOKU_INCLUDE_WRITE_FRM_DATA
+#if TOKU_INCLUDE_WRITE_FRM_DATA
     int write_frm_data(const uchar *frm_data, size_t frm_len);
-#endif  // defined(TOKU_INCLUDE_WRITE_FRM_DATA) && TOKU_INCLUDE_WRITE_FRM_DATA
-
+#endif
+#if TOKU_INCLUDE_UPSERT
 private:
-#if defined(TOKU_INCLUDE_UPSERT) && TOKU_INCLUDE_UPSERT
-    MY_NODISCARD int fast_update(THD *thd,
-                                 List<Item> &update_fields,
-                                 List<Item> &update_values,
-                                 Item *conds);
-    MY_NODISCARD bool check_fast_update(THD *thd,
-                                        List<Item> &update_fields,
-                                        List<Item> &update_values,
-                                        Item *conds);
-    MY_NODISCARD int send_update_message(List<Item> &update_fields,
-                                         List<Item> &update_values,
-                                         Item *conds,
-                                         DB_TXN *txn);
-    MY_NODISCARD int upsert(THD *thd,
-                            List<Item> &update_fields,
-                            List<Item> &update_values);
-    MY_NODISCARD bool check_upsert(THD *thd,
-                                   List<Item> &update_fields,
-                                   List<Item> &update_values);
-    MY_NODISCARD int send_upsert_message(List<Item> &update_fields,
-                                         List<Item> &update_values,
-                                         DB_TXN *txn);
-#endif  // defined(TOKU_INCLUDE_UPSERT) && TOKU_INCLUDE_UPSERT
-
+    int fast_update(THD *thd, List<Item> &update_fields, List<Item> &update_values, Item *conds);
+    bool check_fast_update(THD *thd, List<Item> &update_fields, List<Item> &update_values, Item *conds);
+    int send_update_message(List<Item> &update_fields, List<Item> &update_values, Item *conds, DB_TXN *txn);
+    int upsert(THD *thd, List<Item> &update_fields, List<Item> &update_values);
+    bool check_upsert(THD *thd, List<Item> &update_fields, List<Item> &update_values);
+    int send_upsert_message(THD *thd, List<Item> &update_fields, List<Item> &update_values, DB_TXN *txn);
+#endif
 public:
     // mysql sometimes retires a txn before a cursor that references the txn is closed.
     // for example, commit is sometimes called before index_end.  the following methods
@@ -1069,24 +804,28 @@ private:
     void remove_from_trx_handler_list();
 
 private:
-    int do_optimize(THD *thd);
     int map_to_handler_error(int error);
-
-#if defined(TOKU_INCLUDE_RFR) && TOKU_INCLUDE_RFR
-public:
-    void rpl_before_write_rows();
-    void rpl_after_write_rows();
-    void rpl_before_delete_rows();
-    void rpl_after_delete_rows();
-    void rpl_before_update_rows();
-    void rpl_after_update_rows();
-    bool rpl_lookup_rows();
-private:
-    bool in_rpl_write_rows;
-    bool in_rpl_delete_rows;
-    bool in_rpl_update_rows;
-#endif // defined(TOKU_INCLUDE_RFR) && TOKU_INCLUDE_RFR
 };
 
-#endif // _HA_TOKUDB_H
+#if TOKU_INCLUDE_OPTION_STRUCTS
+struct ha_table_option_struct {
+    uint row_format;
+};
+
+struct ha_index_option_struct {
+    bool clustering;
+};
+
+static inline bool key_is_clustering(const KEY *key) {
+    return (key->flags & HA_CLUSTERING) || (key->option_struct && key->option_struct->clustering);
+}
+
+#else
+
+static inline bool key_is_clustering(const KEY *key) {
+    return key->option_struct && key->option_struct->clustering;
+}
+#endif
+
+#endif
 
