@@ -1,7 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, MariaDB Corporation.
+Copyright (c) 1995, 2013, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -58,16 +57,22 @@ is set to TRUE by the page_cleaner thread when it is spawned and is set
 back to FALSE at shutdown by the page_cleaner as well. Therefore no
 need to protect it by a mutex. It is only ever read by the thread
 doing the shutdown */
-UNIV_INTERN bool buf_page_cleaner_is_active;
+UNIV_INTERN ibool buf_page_cleaner_is_active = FALSE;
 
 /** Flag indicating if the lru_manager is in active state. */
-UNIV_INTERN bool buf_lru_manager_is_active;
+UNIV_INTERN bool buf_lru_manager_is_active = false;
 
 #ifdef UNIV_PFS_THREAD
 UNIV_INTERN mysql_pfs_key_t buf_page_cleaner_thread_key;
 UNIV_INTERN mysql_pfs_key_t buf_lru_manager_thread_key;
 #endif /* UNIV_PFS_THREAD */
 
+/** If LRU list of a buf_pool is less than this size then LRU eviction
+should not happen. This is because when we do LRU flushing we also put
+the blocks on free list. If LRU list is very small then we can end up
+in thrashing. */
+#define BUF_LRU_MIN_LEN		256
+
 /* @} */
 
 /** Handled page counters for a single flush */
@@ -310,8 +315,6 @@ buf_flush_init_flush_rbt(void)
 
 		buf_flush_list_mutex_enter(buf_pool);
 
-		ut_ad(buf_pool->flush_rbt == NULL);
-
 		/* Create red black tree for speedy insertions in flush list. */
 		buf_pool->flush_rbt = rbt_create(
 			sizeof(buf_page_t*), buf_flush_block_cmp);
@@ -357,7 +360,6 @@ buf_flush_insert_into_flush_list(
 	buf_block_t*	block,		/*!< in/out: block which is modified */
 	lsn_t		lsn)		/*!< in: oldest modification */
 {
-	ut_ad(srv_shutdown_state != SRV_SHUTDOWN_FLUSH_PHASE);
 	ut_ad(log_flush_order_mutex_own());
 	ut_ad(mutex_own(&block->mutex));
 
@@ -416,7 +418,6 @@ buf_flush_insert_sorted_into_flush_list(
 	buf_page_t*	prev_b;
 	buf_page_t*	b;
 
-	ut_ad(srv_shutdown_state != SRV_SHUTDOWN_FLUSH_PHASE);
 	ut_ad(log_flush_order_mutex_own());
 	ut_ad(mutex_own(&block->mutex));
 	ut_ad(buf_block_get_state(block) == BUF_BLOCK_FILE_PAGE);
@@ -545,7 +546,7 @@ buf_flush_ready_for_flush(
 	ut_ad(flush_type < BUF_FLUSH_N_TYPES);
 	ut_ad(mutex_own(buf_page_get_mutex(bpage))
 	      || flush_type == BUF_FLUSH_LIST);
-	ut_a(buf_page_in_file(bpage) || buf_page_get_state(bpage) == BUF_BLOCK_REMOVE_HASH);
+	ut_a(buf_page_in_file(bpage));
 
 	if (bpage->oldest_modification == 0
 	    || buf_page_get_io_fix_unlocked(bpage) != BUF_IO_NONE) {
@@ -556,7 +557,6 @@ buf_flush_ready_for_flush(
 
 	switch (flush_type) {
 	case BUF_FLUSH_LIST:
-		return(buf_page_get_state(bpage) != BUF_BLOCK_REMOVE_HASH);
 	case BUF_FLUSH_LRU:
 	case BUF_FLUSH_SINGLE_PAGE:
 		return(true);
@@ -722,7 +722,9 @@ buf_flush_write_complete(
 	buf_page_set_io_fix(bpage, BUF_IO_NONE);
 
 	buf_pool->n_flush[flush_type]--;
-	ut_ad(buf_pool->n_flush[flush_type] != ULINT_MAX);
+
+	/* fprintf(stderr, "n pending flush %lu\n",
+	buf_pool->n_flush[flush_type]); */
 
 	if (buf_pool->n_flush[flush_type] == 0
 	    && buf_pool->init_flush[flush_type] == FALSE) {
@@ -828,35 +830,39 @@ buf_flush_init_for_writing(
 	case SRV_CHECKSUM_ALGORITHM_CRC32:
 	case SRV_CHECKSUM_ALGORITHM_STRICT_CRC32:
 		checksum = buf_calc_page_crc32(page);
-		mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
 		break;
 	case SRV_CHECKSUM_ALGORITHM_INNODB:
 	case SRV_CHECKSUM_ALGORITHM_STRICT_INNODB:
 		checksum = (ib_uint32_t) buf_calc_page_new_checksum(page);
-		mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
-		checksum = (ib_uint32_t) buf_calc_page_old_checksum(page);
 		break;
 	case SRV_CHECKSUM_ALGORITHM_NONE:
 	case SRV_CHECKSUM_ALGORITHM_STRICT_NONE:
 		checksum = BUF_NO_CHECKSUM_MAGIC;
-		mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
 		break;
 	/* no default so the compiler will emit a warning if new enum
 	is added and not handled here */
 	}
 
-	/* With the InnoDB checksum, we overwrite the first 4 bytes of
-	the end lsn field to store the old formula checksum. Since it
-	depends also on the field FIL_PAGE_SPACE_OR_CHKSUM, it has to
-	be calculated after storing the new formula checksum.
+	mach_write_to_4(page + FIL_PAGE_SPACE_OR_CHKSUM, checksum);
 
-	In other cases we write the same value to both fields.
-	If CRC32 is used then it is faster to use that checksum
-	(calculated above) instead of calculating another one.
-	We can afford to store something other than
-	buf_calc_page_old_checksum() or BUF_NO_CHECKSUM_MAGIC in
-	this field because the file will not be readable by old
-	versions of MySQL/InnoDB anyway (older than MySQL 5.6.3) */
+	/* We overwrite the first 4 bytes of the end lsn field to store
+	the old formula checksum. Since it depends also on the field
+	FIL_PAGE_SPACE_OR_CHKSUM, it has to be calculated after storing the
+	new formula checksum. */
+
+	if (srv_checksum_algorithm == SRV_CHECKSUM_ALGORITHM_STRICT_INNODB
+	    || srv_checksum_algorithm == SRV_CHECKSUM_ALGORITHM_INNODB) {
+
+		checksum = (ib_uint32_t) buf_calc_page_old_checksum(page);
+
+		/* In other cases we use the value assigned from above.
+		If CRC32 is used then it is faster to use that checksum
+		(calculated above) instead of calculating another one.
+		We can afford to store something other than
+		buf_calc_page_old_checksum() or BUF_NO_CHECKSUM_MAGIC in
+		this field because the file will not be readable by old
+		versions of MySQL/InnoDB anyway (older than MySQL 5.6.3) */
+	}
 
 	mach_write_to_4(page + UNIV_PAGE_SIZE - FIL_PAGE_END_LSN_OLD_CHKSUM,
 			checksum);
@@ -928,11 +934,11 @@ buf_flush_write_block_low(
 		break;
 	case BUF_BLOCK_ZIP_DIRTY:
 		frame = bpage->zip.data;
-		mach_write_to_8(frame + FIL_PAGE_LSN,
-				bpage->newest_modification);
 
 		ut_a(page_zip_verify_checksum(frame, zip_size));
 
+		mach_write_to_8(frame + FIL_PAGE_LSN,
+				bpage->newest_modification);
 		memset(frame + FIL_PAGE_FILE_FLUSH_LSN, 0, 8);
 		break;
 	case BUF_BLOCK_FILE_PAGE:
@@ -1057,7 +1063,6 @@ buf_flush_page(
 		}
 
 		++buf_pool->n_flush[flush_type];
-		ut_ad(buf_pool->n_flush[flush_type] != 0);
 
 		mutex_exit(&buf_pool->flush_state_mutex);
 
@@ -1091,8 +1096,8 @@ buf_flush_page(
 # if defined UNIV_DEBUG || defined UNIV_IBUF_DEBUG
 /********************************************************************//**
 Writes a flushable page asynchronously from the buffer pool to a file.
-NOTE: block and LRU list mutexes must be held upon entering this function, and
-they will be released by this function after flushing. This is loosely based on
+NOTE: block->mutex must be held upon entering this function, and it will be
+released by this function after flushing.  This is loosely based on
 buf_flush_batch() and buf_flush_page().
 @return TRUE if the page was flushed and the mutexes released */
 UNIV_INTERN
@@ -1358,8 +1363,7 @@ buf_flush_page_and_try_neighbors(
 	}
 
 	ut_a(buf_page_in_file(bpage)
-	     || (buf_page_get_state(bpage) == BUF_BLOCK_REMOVE_HASH
-		 ));
+	     || buf_page_get_state(bpage) == BUF_BLOCK_REMOVE_HASH);
 
 	if (buf_flush_ready_for_flush(bpage, flush_type)) {
 		buf_pool_t*	buf_pool;
@@ -1484,7 +1488,7 @@ It attempts to make 'max' blocks available in the free list. Note that
 it is a best effort attempt and it is not guaranteed that after a call
 to this function there will be 'max' blocks in the free list.
 @return number of blocks for which the write request was queued. */
-MY_ATTRIBUTE((nonnull))
+__attribute__((nonnull))
 static
 void
 buf_flush_LRU_list_batch(
@@ -1630,7 +1634,7 @@ Whether LRU or unzip_LRU is used depends on the state of the system.
 @return number of blocks for which either the write request was queued
 or in case of unzip_LRU the number of blocks actually moved to the
 free list */
-MY_ATTRIBUTE((nonnull))
+__attribute__((nonnull))
 static
 void
 buf_do_LRU_batch(
@@ -1643,11 +1647,9 @@ buf_do_LRU_batch(
 	flush_counters_t*	n)	/*!< out: flushed/evicted page
 					counts */
 {
-	ut_ad(mutex_own(&buf_pool->LRU_list_mutex));
-
 	if (buf_LRU_evict_from_unzip_LRU(buf_pool)) {
 		n->unzip_LRU_evicted
-			= buf_free_from_unzip_LRU_list_batch(buf_pool, max);
+			+= buf_free_from_unzip_LRU_list_batch(buf_pool, max);
 	} else {
 		n->unzip_LRU_evicted = 0;
 	}
@@ -1750,7 +1752,7 @@ pages: to avoid deadlocks, this function must be written so that it cannot
 end up waiting for these latches! NOTE 2: in the case of a flush list flush,
 the calling thread is not allowed to own any latches on pages!
 @return number of blocks for which the write request was queued */
-MY_ATTRIBUTE((nonnull))
+__attribute__((nonnull))
 static
 void
 buf_flush_batch(
@@ -1928,7 +1930,7 @@ list.
 NOTE: The calling thread is not allowed to own any latches on pages!
 @return true if a batch was queued successfully. false if another batch
 of same type was already running. */
-MY_ATTRIBUTE((nonnull))
+__attribute__((nonnull))
 static
 bool
 buf_flush_LRU(
@@ -1946,7 +1948,6 @@ buf_flush_LRU(
 	if (!buf_flush_start(buf_pool, BUF_FLUSH_LRU)) {
 		n->flushed = 0;
 		n->evicted = 0;
-		n->unzip_LRU_evicted = 0;
 		return(false);
 	}
 
@@ -2200,14 +2201,13 @@ Clears up tail of the LRU lists:
 * Flush dirty pages at the tail of LRU to the disk
 The depth to which we scan each buffer pool is controlled by dynamic
 config parameter innodb_LRU_scan_depth.
-@return number of flushed and evicted pages */
+@return total pages flushed */
 UNIV_INTERN
 ulint
 buf_flush_LRU_tail(void)
 /*====================*/
 {
 	ulint	total_flushed = 0;
-	ulint	total_evicted = 0;
 	ulint	start_time = ut_time_ms();
 	ulint	scan_depth[MAX_BUFFER_POOLS];
 	ulint	requested_pages[MAX_BUFFER_POOLS];
@@ -2273,7 +2273,6 @@ buf_flush_LRU_tail(void)
 				limited_scan[i]
 					= (previous_evicted[i] > n.evicted);
 				previous_evicted[i] = n.evicted;
-				total_evicted += n.evicted;
 
 				requested_pages[i] += lru_chunk_size;
 
@@ -2306,7 +2305,8 @@ buf_flush_LRU_tail(void)
 			MONITOR_LRU_BATCH_PAGES,
 			total_flushed);
 	}
-	return(total_flushed + total_evicted);
+
+	return(total_flushed);
 }
 
 /*********************************************************************//**
@@ -2365,10 +2365,6 @@ af_get_pct_for_dirty()
 {
 	ulint dirty_pct = buf_get_modified_ratio_pct();
 
-	if (dirty_pct > 0 && srv_max_buf_pool_modified_pct == 0) {
-		return(100);
-	}
-
 	ut_a(srv_max_dirty_pages_pct_lwm
 	     <= srv_max_buf_pool_modified_pct);
 
@@ -2471,14 +2467,7 @@ page_cleaner_flush_pages_if_needed(void)
 	ulint			pct_total = 0;
 	int			age_factor = 0;
 
-	cur_lsn = log_get_lsn_nowait();
-
-	/* log_get_lsn_nowait tries to get log_sys->mutex with
-	mutex_enter_nowait, if this does not succeed function
-	returns 0, do not use that value to update stats. */
-	if (cur_lsn == 0) {
-		return(0);
-	}
+	cur_lsn = log_get_lsn();
 
 	if (prev_lsn == 0) {
 		/* First time around. */
@@ -2571,11 +2560,6 @@ page_cleaner_sleep_if_needed(
 	ulint	next_loop_time)	/*!< in: time when next loop iteration
 				should start */
 {
-	/* No sleep if we are cleaning the buffer pool during the shutdown
-	with everything else finished */
-	if (srv_shutdown_state == SRV_SHUTDOWN_FLUSH_PHASE)
-		return;
-
 	ulint	cur_time = ut_time_ms();
 
 	if (next_loop_time > cur_time) {
@@ -2591,7 +2575,7 @@ page_cleaner_sleep_if_needed(
 /*********************************************************************//**
 Returns the aggregate free list length over all buffer pool instances.
 @return total free list length. */
-MY_ATTRIBUTE((warn_unused_result))
+__attribute__((warn_unused_result))
 static
 ulint
 buf_get_total_free_list_length(void)
@@ -2607,50 +2591,27 @@ buf_get_total_free_list_length(void)
 	return result;
 }
 
-/** Returns the aggregate LRU list length over all buffer pool instances.
-@return total LRU list length. */
-MY_ATTRIBUTE((warn_unused_result))
-static
-ulint
-buf_get_total_LRU_list_length(void)
-{
-        ulint result = 0;
-
-        for (ulint i = 0; i < srv_buf_pool_instances; i++) {
-
-                result += UT_LIST_GET_LEN(buf_pool_from_array(i)->LRU);
-        }
-
-        return result;
-}
-
 /*********************************************************************//**
 Adjust the desired page cleaner thread sleep time for LRU flushes.  */
-MY_ATTRIBUTE((nonnull))
+__attribute__((nonnull))
 static
 void
 page_cleaner_adapt_lru_sleep_time(
 /*==============================*/
-	ulint*	lru_sleep_time,	/*!< in/out: desired page cleaner thread sleep
+	ulint*	lru_sleep_time)	/*!< in/out: desired page cleaner thread sleep
 				time for LRU flushes  */
-	ulint	lru_n_flushed) /*!< in: number of flushed in previous batch */
-
 {
-        ulint free_len = buf_get_total_free_list_length();
-        ulint max_free_len = ut_min(buf_get_total_LRU_list_length(),
-                        srv_LRU_scan_depth * srv_buf_pool_instances);
+	ulint free_len = buf_get_total_free_list_length();
+	ulint max_free_len = srv_LRU_scan_depth * srv_buf_pool_instances;
 
-	if (free_len < max_free_len / 100 && lru_n_flushed) {
+	if (free_len < max_free_len / 100) {
 
-		/* Free lists filled less than 1%
-		and iteration was able to flush, no sleep */
+		/* Free lists filled less than 1%, no sleep */
 		*lru_sleep_time = 0;
-	} else if (free_len > max_free_len / 5
-		   || (free_len < max_free_len / 100 && lru_n_flushed == 0)) {
+	} else if (free_len > max_free_len / 5) {
 
-		/* Free lists filled more than 20%
-		or no pages flushed in previous batch, sleep a bit more */
-		*lru_sleep_time += 1;
+		/* Free lists filled more than 20%, sleep a bit more */
+		*lru_sleep_time += 50;
 		if (*lru_sleep_time > srv_cleaner_max_lru_time)
 			*lru_sleep_time = srv_cleaner_max_lru_time;
 	} else if (free_len < max_free_len / 20 && *lru_sleep_time >= 50) {
@@ -2666,7 +2627,7 @@ page_cleaner_adapt_lru_sleep_time(
 /*********************************************************************//**
 Get the desired page cleaner thread sleep time for flush list flushes.
 @return desired sleep time */
-MY_ATTRIBUTE((warn_unused_result))
+__attribute__((warn_unused_result))
 static
 ulint
 page_cleaner_adapt_flush_sleep_time(void)
@@ -2693,11 +2654,10 @@ extern "C" UNIV_INTERN
 os_thread_ret_t
 DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 /*==========================================*/
-	void*	arg MY_ATTRIBUTE((unused)))
+	void*	arg __attribute__((unused)))
 			/*!< in: a dummy parameter required by
 			os_thread_create */
 {
-	my_thread_init();
 	ulint	next_loop_time = ut_time_ms() + 1000;
 	ulint	n_flushed = 0;
 	ulint	last_activity = srv_get_activity_count();
@@ -2718,6 +2678,8 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 		os_thread_pf(os_thread_get_curr_id()));
 #endif /* UNIV_DEBUG_THREAD_CREATION */
 
+	buf_page_cleaner_is_active = TRUE;
+
 	while (srv_shutdown_state == SRV_SHUTDOWN_NONE) {
 
 		ulint	page_cleaner_sleep_time;
@@ -2725,7 +2687,14 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 
 		srv_current_thread_priority = srv_cleaner_thread_priority;
 
-		page_cleaner_sleep_if_needed(next_loop_time);
+		/* The page_cleaner skips sleep if the server is
+		idle and there are no pending IOs in the buffer pool
+		and there is work to do. */
+		if (srv_check_activity(last_activity)
+		    || buf_get_n_pending_read_ios()
+		    || n_flushed == 0) {
+			page_cleaner_sleep_if_needed(next_loop_time);
+		}
 
 		page_cleaner_sleep_time
 			= page_cleaner_adapt_flush_sleep_time();
@@ -2733,7 +2702,6 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 		next_loop_time = ut_time_ms() + page_cleaner_sleep_time;
 
 		server_active = srv_check_activity(last_activity);
-
 		if (server_active
 		    || ut_time_ms() - last_activity_time < 1000) {
 
@@ -2744,8 +2712,7 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 			}
 
 			/* Flush pages from flush_list if required */
-			page_cleaner_flush_pages_if_needed();
-			n_flushed = 0;
+			n_flushed += page_cleaner_flush_pages_if_needed();
 		} else {
 			n_flushed = page_cleaner_do_flush_batch(
 							PCT_IO(100),
@@ -2759,9 +2726,6 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 					n_flushed);
 			}
 		}
-
-		/* Flush pages from end of LRU if required */
-		n_flushed = buf_flush_LRU_tail();
 	}
 
 	ut_ad(srv_shutdown_state > 0);
@@ -2781,10 +2745,7 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 	when SRV_SHUTDOWN_CLEANUP is set other threads like the master
 	and the purge threads may be working as well. We start flushing
 	the buffer pool but can't be sure that no new pages are being
-	dirtied until we enter SRV_SHUTDOWN_FLUSH_PHASE phase. Because
-	the LRU manager thread is also flushing at SRV_SHUTDOWN_CLEANUP
-	but not SRV_SHUTDOWN_FLUSH_PHASE, we only leave the
-	SRV_SHUTDOWN_CLEANUP loop when the LRU manager quits. */
+	dirtied until we enter SRV_SHUTDOWN_FLUSH_PHASE phase. */
 
 	do {
 		n_flushed = page_cleaner_do_flush_batch(PCT_IO(100), LSN_MAX);
@@ -2793,10 +2754,7 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 		if (n_flushed == 0) {
 			os_thread_sleep(100000);
 		}
-
-		os_rmb;
-	} while (srv_shutdown_state == SRV_SHUTDOWN_CLEANUP
-		 || buf_lru_manager_is_active);
+	} while (srv_shutdown_state == SRV_SHUTDOWN_CLEANUP);
 
 	/* At this point all threads including the master and the purge
 	thread must have been suspended. */
@@ -2813,11 +2771,6 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 	buf_flush_wait_batch_end(NULL, BUF_FLUSH_LIST);
 	buf_flush_wait_LRU_batch_end();
 
-#ifdef UNIV_DEBUG
-	os_rmb;
-	ut_ad(!buf_lru_manager_is_active);
-#endif
-
 	bool	success;
 
 	do {
@@ -2838,9 +2791,8 @@ DECLARE_THREAD(buf_flush_page_cleaner_thread)(
 	/* We have lived our life. Time to die. */
 
 thread_exit:
-	buf_page_cleaner_is_active = false;
+	buf_page_cleaner_is_active = FALSE;
 
-	my_thread_end();
 	/* We count the number of threads in os_thread_exit(). A created
 	thread should always use that to exit and not use return() to exit. */
 	os_thread_exit(NULL);
@@ -2857,13 +2809,12 @@ extern "C" UNIV_INTERN
 os_thread_ret_t
 DECLARE_THREAD(buf_flush_lru_manager_thread)(
 /*==========================================*/
-	void*	arg MY_ATTRIBUTE((unused)))
+	void*	arg __attribute__((unused)))
 			/*!< in: a dummy parameter required by
 			os_thread_create */
 {
 	ulint	next_loop_time = ut_time_ms() + 1000;
 	ulint	lru_sleep_time = srv_cleaner_max_lru_time;
-	ulint	lru_n_flushed = 1;
 
 #ifdef UNIV_PFS_THREAD
 	pfs_register_thread(buf_lru_manager_thread_key);
@@ -2879,24 +2830,36 @@ DECLARE_THREAD(buf_flush_lru_manager_thread)(
 		os_thread_pf(os_thread_get_curr_id()));
 #endif /* UNIV_DEBUG_THREAD_CREATION */
 
+	buf_lru_manager_is_active = true;
+
 	/* On server shutdown, the LRU manager thread runs through cleanup
 	phase to provide free pages for the master and purge threads.  */
 	while (srv_shutdown_state == SRV_SHUTDOWN_NONE
 	       || srv_shutdown_state == SRV_SHUTDOWN_CLEANUP) {
 
+		ulint n_flushed_lru;
+
 		srv_current_thread_priority = srv_cleaner_thread_priority;
 
 		page_cleaner_sleep_if_needed(next_loop_time);
 
-		page_cleaner_adapt_lru_sleep_time(&lru_sleep_time, lru_n_flushed);
+		page_cleaner_adapt_lru_sleep_time(&lru_sleep_time);
 
 		next_loop_time = ut_time_ms() + lru_sleep_time;
 
-		lru_n_flushed = buf_flush_LRU_tail();
+		n_flushed_lru = buf_flush_LRU_tail();
+
+		if (n_flushed_lru) {
+
+			MONITOR_INC_VALUE_CUMULATIVE(
+				MONITOR_FLUSH_BACKGROUND_TOTAL_PAGE,
+				MONITOR_FLUSH_BACKGROUND_COUNT,
+				MONITOR_FLUSH_BACKGROUND_PAGES,
+				n_flushed_lru);
+		}
 	}
 
 	buf_lru_manager_is_active = false;
-	os_wmb;
 
 	/* We count the number of threads in os_thread_exit(). A created
 	thread should always use that to exit and not use return() to exit. */
