@@ -1,6 +1,5 @@
 /* Copyright (c) 2002-2007 MySQL AB & tommy@valley.ne.jp
-   Copyright (c) 2002, 2014, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2014, SkySQL Ab.
+   Copyright (c) 2009-2011, Monty Program Ab
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -220,11 +219,11 @@ static size_t my_case_str_bin(CHARSET_INFO *cs __attribute__((unused)),
 
 
 static size_t my_case_bin(CHARSET_INFO *cs __attribute__((unused)),
-                          const char *src, size_t srclen,
-                          char *dst, size_t dstlen)
+                          char *src __attribute__((unused)),
+                          size_t srclen,
+                          char *dst __attribute__((unused)),
+                          size_t dstlen __attribute__((unused)))
 {
-  DBUG_ASSERT(srclen <= dstlen);
-  memcpy(dst, src, srclen);
   return srclen;
 }
 
@@ -288,7 +287,9 @@ void my_hash_sort_8bit_bin(CHARSET_INFO *cs __attribute__((unused)),
 
   for (; key < end ; key++)
   {
-    MY_HASH_ADD(tmp1, tmp2, (uint) *key);
+    tmp1^= (ulong) ((((uint) tmp1 & 63) + tmp2) *
+                    ((uint) *key)) + (tmp1 << 8);
+    tmp2+= 3;
   }
 
   *nr1= tmp1;
@@ -305,7 +306,9 @@ void my_hash_sort_bin(CHARSET_INFO *cs __attribute__((unused)),
 
   for (; key < end ; key++)
   {
-    MY_HASH_ADD(tmp1, tmp2, (uint) *key);
+    tmp1^= (ulong) ((((uint) tmp1 & 63) + tmp2) *
+                    ((uint) *key)) + (tmp1 << 8);
+    tmp2+= 3;
   }
 
   *nr1= tmp1;
@@ -395,7 +398,7 @@ int my_wildcmp_bin_impl(CHARSET_INFO *cs,
 	  if (tmp <= 0)
 	    return(tmp);
 	}
-      } while (str != str_end);
+      } while (str != str_end && wildstr[0] != w_many);
       return(-1);
     }
   }
