@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2014, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2013, Oracle and/or its affiliates.
    Copyright (c) 2009, 2014, SkySQL Ab.
 
    This program is free software; you can redistribute it and/or modify
@@ -71,8 +71,81 @@ get_case_info_for_ch(const CHARSET_INFO *cs, uint page, uint offs)
 
 
 /*
-  Case folding functions for CJK character set.
-  Case conversion can optionally reduce string octet length.
+  For character sets which don't change octet length in case conversion.
+*/
+size_t my_caseup_mb(CHARSET_INFO * cs, char *src, size_t srclen,
+                    char *dst __attribute__((unused)),
+                    size_t dstlen __attribute__((unused)))
+{
+  register uint32 l;
+  register char *srcend= src + srclen;
+  register const uchar *map= cs->to_upper;
+
+  DBUG_ASSERT(cs->caseup_multiply == 1);
+  DBUG_ASSERT(src == dst && srclen == dstlen);
+  DBUG_ASSERT(cs->mbmaxlen == 2);
+  
+  while (src < srcend)
+  {
+    if ((l=my_ismbchar(cs, src, srcend)))
+    {
+      MY_UNICASE_CHARACTER *ch;
+      if ((ch= get_case_info_for_ch(cs, (uchar) src[0], (uchar) src[1])))
+      {
+        *src++= ch->toupper >> 8;
+        *src++= ch->toupper & 0xFF;
+      }
+      else
+        src+= l;
+    }
+    else 
+    {
+      *src=(char) map[(uchar) *src];
+      src++;
+    }
+  }
+  return srclen;
+}
+
+
+size_t my_casedn_mb(CHARSET_INFO * cs, char *src, size_t srclen,
+                    char *dst __attribute__((unused)),
+                    size_t dstlen __attribute__((unused)))
+{
+  register uint32 l;
+  register char *srcend= src + srclen;
+  register const uchar *map=cs->to_lower;
+
+  DBUG_ASSERT(cs->casedn_multiply == 1);
+  DBUG_ASSERT(src == dst && srclen == dstlen);  
+  DBUG_ASSERT(cs->mbmaxlen == 2);
+  
+  while (src < srcend)
+  {
+    if ((l= my_ismbchar(cs, src, srcend)))
+    {
+      MY_UNICASE_CHARACTER *ch;
+      if ((ch= get_case_info_for_ch(cs, (uchar) src[0], (uchar) src[1])))
+      {
+        *src++= ch->tolower >> 8;
+        *src++= ch->tolower & 0xFF;
+      }
+      else
+        src+= l;
+    }
+    else
+    {
+      *src= (char) map[(uchar)*src];
+      src++;
+    }
+  }
+  return srclen;
+}
+
+
+/*
+  Case folding functions for character set
+  where case conversion can change string octet length.
   For example, in EUCKR,
     _euckr 0xA9A5 == "LATIN LETTER DOTLESS I" (Turkish letter)
   is upper-cased to to
@@ -80,14 +153,13 @@ get_case_info_for_ch(const CHARSET_INFO *cs, uint page, uint offs)
   Length is reduced in this example from two bytes to one byte.
 */
 static size_t
-my_casefold_mb(CHARSET_INFO *cs,
-               const char *src, size_t srclen,
-               char *dst, size_t dstlen __attribute__((unused)),
-               const uchar *map,
-               size_t is_upper)
+my_casefold_mb_varlen(CHARSET_INFO *cs,
+                      char *src, size_t srclen,
+                      char *dst, size_t dstlen __attribute__((unused)),
+                      const uchar *map,
+                      size_t is_upper)
 {
-  const char *srcend= src + srclen;
-  char *dst0= dst;
+  char *srcend= src + srclen, *dst0= dst;
 
   DBUG_ASSERT(cs->mbmaxlen == 2);
 
@@ -121,22 +193,22 @@ my_casefold_mb(CHARSET_INFO *cs,
 
 
 size_t
-my_casedn_mb(CHARSET_INFO * cs, const char *src, size_t srclen,
+my_casedn_mb_varlen(CHARSET_INFO * cs, char *src, size_t srclen,
                     char *dst, size_t dstlen)
 {
   DBUG_ASSERT(dstlen >= srclen * cs->casedn_multiply); 
   DBUG_ASSERT(src != dst || cs->casedn_multiply == 1);
-  return my_casefold_mb(cs, src, srclen, dst, dstlen, cs->to_lower, 0);
+  return my_casefold_mb_varlen(cs, src, srclen, dst, dstlen, cs->to_lower, 0);
 }
 
 
 size_t
-my_caseup_mb(CHARSET_INFO * cs, const char *src, size_t srclen,
-             char *dst, size_t dstlen)
+my_caseup_mb_varlen(CHARSET_INFO * cs, char *src, size_t srclen,
+                    char *dst, size_t dstlen)
 {
   DBUG_ASSERT(dstlen >= srclen * cs->caseup_multiply);
   DBUG_ASSERT(src != dst || cs->caseup_multiply == 1);
-  return my_casefold_mb(cs, src, srclen, dst, dstlen, cs->to_upper, 1);
+  return my_casefold_mb_varlen(cs, src, srclen, dst, dstlen, cs->to_upper, 1);
 }
 
 
@@ -282,7 +354,7 @@ int my_wildcmp_mb_impl(CHARSET_INFO *cs,
 	  if (tmp <= 0)
 	    return (tmp);
 	}
-      } while (str != str_end);
+      } while (str != str_end && wildstr[0] != w_many);
       return(-1);
     }
   }
@@ -608,8 +680,6 @@ void
 my_hash_sort_mb_bin(CHARSET_INFO *cs __attribute__((unused)),
                     const uchar *key, size_t len,ulong *nr1, ulong *nr2)
 {
-  register ulong m1= *nr1, m2= *nr2;
-
   /*
      Remove trailing spaces. We have to do this to be able to compare
     'A ' and 'A' as identical
@@ -618,10 +688,10 @@ my_hash_sort_mb_bin(CHARSET_INFO *cs __attribute__((unused)),
   
   for (; key < end ; key++)
   {
-    MY_HASH_ADD(m1, m2, (uint)*key);
+    nr1[0]^=(ulong) ((((uint) nr1[0] & 63)+nr2[0]) * 
+	     ((uint)*key)) + (nr1[0] << 8);
+    nr2[0]+=3;
   }
-  *nr1= m1;
-  *nr2= m2;
 }
 
 
@@ -1062,7 +1132,7 @@ static int my_wildcmp_mb_bin_impl(CHARSET_INFO *cs,
     }
     if (*wildstr == w_many)
     {						/* Found w_many */
-      int cmp;
+      uchar cmp;
       const char* mb = wildstr;
       int mb_len=0;
       
@@ -1120,7 +1190,7 @@ static int my_wildcmp_mb_bin_impl(CHARSET_INFO *cs,
 	  if (tmp <= 0)
 	    return (tmp);
 	}
-      } while (str != str_end);
+      } while (str != str_end && wildstr[0] != w_many);
       return(-1);
     }
   }
